Package: mapsf
Title: Thematic Cartography
Version: 1.2.2
Authors@R: 
    c(person(given = "Timothée",
             family = "Giraud",
             email = "timothee.giraud@cnrs.fr",
             role = c("cre","aut"),
             comment = c(ORCID = "0000-0002-1932-3323")),
      person(given = "Hugues",
             family = "Pecout",
             role = "ctb",
             comment = c(ORCID = "0000-0002-0246-0954", "Logo")),
      person(given = "Ronan",
             family = "Ysebaert",
             role = "ctb",
             comment = c(ORCID = "0000-0002-7344-5911", "Cheat sheet")),
      person(given = "Elina",
             family = "Marveaux",
             role = "ctb",
             comment = c(ORCID = "0009-0000-8667-3019", "Themes")),
      person(given = "Ian",
             family = "Fellows",
             role = "cph",
             comment = "No overlap algorithm for labels, from wordcloud package"),
      person(given = "Danielle",
             family = "Navarro",
             role = "cph",
             comment = c(ORCID = "0000-0001-7648-6578", "Bézier curve algorithm for text annotations")))
Description: Create and integrate thematic maps in your workflow. This package
    helps to design various cartographic representations such as proportional
    symbols, choropleth or typology maps. It also offers several functions to
    display layout elements that improve the graphic presentation of maps
    (e.g. scale bar, north arrow, title, labels). 'mapsf' maps 'sf' objects on
    'base' graphics.
License: GPL (>= 3)
URL: https://riatelab.github.io/mapsf/
BugReports: https://github.com/riatelab/mapsf/issues/
Depends: R (>= 3.6.0)
Imports: classInt, graphics, maplegend (>= 0.6.3), s2, sf, stats,
        utils, grDevices
Suggests: terra, Ckmeans.1d.dp, png, jpeg, knitr, rmarkdown, svglite,
        tinytest, covr
Encoding: UTF-8
VignetteBuilder: knitr
Language: en-US
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-09-02 11:48:58 UTC; tim
Author: Timothée Giraud [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-1932-3323>),
  Hugues Pecout [ctb] (ORCID: <https://orcid.org/0000-0002-0246-0954>,
    Logo),
  Ronan Ysebaert [ctb] (ORCID: <https://orcid.org/0000-0002-7344-5911>,
    Cheat sheet),
  Elina Marveaux [ctb] (ORCID: <https://orcid.org/0009-0000-8667-3019>,
    Themes),
  Ian Fellows [cph] (No overlap algorithm for labels, from wordcloud
    package),
  Danielle Navarro [cph] (ORCID: <https://orcid.org/0000-0001-7648-6578>,
    Bézier curve algorithm for text annotations)
Maintainer: Timothée Giraud <timothee.giraud@cnrs.fr>
Repository: CRAN
Date/Publication: 2026-09-02 12:40:02 UTC
Built: R 4.6.0; ; 2026-09-02 14:50:42 UTC; unix

Package: mapsf
Title: Thematic Cartography
Version: 1.2.1
Authors@R: 
    c(person(given = "Timothée",
             family = "Giraud",
             email = "timothee.giraud@cnrs.fr",
             role = c("cre","aut"),
             comment = c(ORCID = "0000-0002-1932-3323")),
      person(given = "Hugues",
             family = "Pecout",
             role = "ctb",
             comment = c(ORCID = "0000-0002-0246-0954", "Logo")),
      person(given = "Ronan",
             family = "Ysebaert",
             role = "ctb",
             comment = c(ORCID = "0000-0002-7344-5911", "Cheat sheet")),
      person(given = "Elina",
             family = "Marveaux",
             role = "ctb",
             comment = c(ORCID = "0009-0000-8667-3019", "Themes")),
      person(given = "Ian",
             family = "Fellows",
             role = "cph",
             comment = "No overlap algorithm for labels, from wordcloud package"),
      person(given = "Jim",
             family = "Lemon",
             role = "cph",
             comment = "Arc drawing algorithm for annotations, from plotrix package"), 
      person(given = "Danielle",
             family = "Navarro",
             role = "cph",
             comment = c(ORCID = "0000-0001-7648-6578", "Bézier curve algorithm for text annotations")))
Description: Create and integrate thematic maps in your workflow. This package
    helps to design various cartographic representations such as proportional
    symbols, choropleth or typology maps. It also offers several functions to
    display layout elements that improve the graphic presentation of maps
    (e.g. scale bar, north arrow, title, labels). 'mapsf' maps 'sf' objects on
    'base' graphics.
License: GPL (>= 3)
URL: https://riatelab.github.io/mapsf/
BugReports: https://github.com/riatelab/mapsf/issues/
Depends: R (>= 3.6.0)
Imports: classInt, graphics, maplegend (>= 0.6.3), s2, sf, stats,
        utils, grDevices
Suggests: terra, Ckmeans.1d.dp, png, jpeg, knitr, rmarkdown, svglite,
        tinytest, covr
Encoding: UTF-8
VignetteBuilder: knitr
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-29 12:30:10 UTC; tim
Author: Timothée Giraud [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-1932-3323>),
  Hugues Pecout [ctb] (ORCID: <https://orcid.org/0000-0002-0246-0954>,
    Logo),
  Ronan Ysebaert [ctb] (ORCID: <https://orcid.org/0000-0002-7344-5911>,
    Cheat sheet),
  Elina Marveaux [ctb] (ORCID: <https://orcid.org/0009-0000-8667-3019>,
    Themes),
  Ian Fellows [cph] (No overlap algorithm for labels, from wordcloud
    package),
  Jim Lemon [cph] (Arc drawing algorithm for annotations, from plotrix
    package),
  Danielle Navarro [cph] (ORCID: <https://orcid.org/0000-0001-7648-6578>,
    Bézier curve algorithm for text annotations)
Maintainer: Timothée Giraud <timothee.giraud@cnrs.fr>
Repository: CRAN
Date/Publication: 2026-05-29 17:00:02 UTC
Built: R 4.6.0; ; 2026-05-29 17:37:48 UTC; unix

# Generated by roxygen2: do not edit by hand

export(mf_annotation)
export(mf_arrow)
export(mf_background)
export(mf_base)
export(mf_choro)
export(mf_credits)
export(mf_distr)
export(mf_export)
export(mf_frame)
export(mf_get_borders)
export(mf_get_breaks)
export(mf_get_leg_pos)
export(mf_get_links)
export(mf_get_mtq)
export(mf_get_pal)
export(mf_get_pencil)
export(mf_get_ratio)
export(mf_grad)
export(mf_graticule)
export(mf_init)
export(mf_inset_off)
export(mf_inset_on)
export(mf_label)
export(mf_layout)
export(mf_legend)
export(mf_logo)
export(mf_map)
export(mf_png)
export(mf_prop)
export(mf_prop_choro)
export(mf_prop_typo)
export(mf_raster)
export(mf_scale)
export(mf_shadow)
export(mf_svg)
export(mf_symb)
export(mf_symb_choro)
export(mf_text)
export(mf_theme)
export(mf_title)
export(mf_typo)
export(mf_worldmap)
importFrom(classInt,classIntervals)
importFrom(grDevices,
  col2rgb,
  colorRampPalette,
  dev.list,
  dev.size,
  hcl.colors,
  hcl.pals,
  png,
  recordGraphics,
  rgb,
  svg,
  xy.coords
)
importFrom(graphics,
  arrows,
  axis,
  box,
  boxplot,
  grconvertX,
  grconvertY,
  hist,
  lines,
  locator,
  par,
  plot.new,
  plot.window,
  points,
  polygon,
  rasterImage,
  rect,
  segments,
  strheight,
  strwidth,
  symbols,
  text,
  title,
  xinch,
  xyinch,
  yinch
)
importFrom(maplegend,
  leg,
  leg_comp,
  leg_draw
)
importFrom(s2,
  as_s2_geography,
  s2_buffer_cells,
  s2_data_countries,
  s2_difference,
  s2_intersection,
  s2_union_agg
)
importFrom(sf,
  "st_crs<-",
  "st_geometry<-",
  st_agr,
  st_area,
  st_as_sf,
  st_as_sfc,
  st_bbox,
  st_buffer,
  st_cast,
  st_centroid,
  st_collection_extract,
  st_combine,
  st_convex_hull,
  st_coordinates,
  st_crs,
  st_geometry,
  st_geometry_type,
  st_graticule,
  st_intersection,
  st_intersects,
  st_is_empty,
  st_is_longlat,
  st_make_grid,
  st_multilinestring,
  st_multipolygon,
  st_polygon,
  st_read,
  st_sample,
  st_set_agr,
  st_set_crs,
  st_set_geometry,
  st_sf,
  st_sfc,
  st_transform,
  st_union
)
importFrom(stats,
  aggregate,
  bw.SJ,
  density,
  median,
  na.omit,
  quantile,
  runif,
  sd
)
importFrom(utils,globalVariables)

# mapsf <img src="man/figures/logo.png" align="right" width="120"/>

[![CRAN](https://www.r-pkg.org/badges/version-ago/mapsf)](https://cran.r-project.org/package=mapsf)
[![R-CMD-check](https://github.com/riatelab/mapsf/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/riatelab/mapsf/actions/workflows/R-CMD-check.yaml)
[![downloads](https://cranlogs.r-pkg.org/badges/mapsf?color=brightgreen)](https://cran.r-project.org/package=mapsf)
[![Codecov test
coverage](https://codecov.io/gh/riatelab/mapsf/branch/master/graph/badge.svg)](https://app.codecov.io/gh/riatelab/mapsf?branch=master)
[![status](https://tinyverse.netlify.app/badge/mapsf)](https://CRAN.R-project.org/package=mapsf)

**Create thematic maps and integrate them into your R workflow.**

This package helps you design different cartographic representations
such as proportional symbols, choropleths, or typology maps. It also
provides several functions for displaying layout elements that enhance
the graphical presentation of maps (e.g., scale bar, north arrow, title,
labels). `mapsf` maps `sf` objects on `base` graphics.

## Installation

You can install the released version of `mapsf` from
[CRAN](https://cran.r-project.org/package=mapsf) with:

``` r
install.packages("mapsf")
```

Alternatively, you can install the development version of `mapsf` (the
**dev** branch) from [r-universe](https://riatelab.r-universe.dev/mapsf)
with:

``` r
install.packages("mapsf", repos = c("https://riatelab.r-universe.dev", "https://cloud.r-project.org"))
```

## Usage

This is a basic example which shows how to create a map with `mapsf`.

``` r
library(mapsf)
# Import the sample dataset
mtq <- mf_get_mtq()
# Plot the base map
mf_map(x = mtq)
# Plot proportional symbols
mf_map(x = mtq, var = "POP", type = "prop", leg_pos = "topright")
# Plot a map layout
mf_layout(
  title = "Population in Martinique",
  credits = "T. Giraud; Sources: INSEE & IGN, 2018"
)
```

![](man/figures/example1-1.png)<!-- -->

You can find more detailed examples on the [“Get
Started”](https://riatelab.github.io/mapsf/articles/mapsf) section of
the [package website](https://riatelab.github.io/mapsf/) and you can
also download its [cheat
sheet](https://raw.githubusercontent.com/riatelab/mapsf/master/vignettes/fig/mapsf_cheatsheet.pdf).  
[![Cheat
Sheet](man/figures/mapsf_cheatsheet.png)](https://raw.githubusercontent.com/riatelab/mapsf/master/vignettes/fig/mapsf_cheatsheet.pdf)

## Alternatives

- [tmap](https://github.com/r-tmap/tmap)  
- [ggplot2](https://github.com/tidyverse/ggplot2) +
  [ggspatial](https://github.com/paleolimbot/ggspatial)

## Community Guidelines

One can contribute to the package through [pull
requests](https://github.com/riatelab/mapsf/pulls) and report issues or
ask questions [here](https://github.com/riatelab/mapsf/issues). See the
[CONTRIBUTING.md](https://github.com/riatelab/mapsf/blob/master/CONTRIBUTING.md)
file for detailed instructions on how to contribute.  
This project uses [conventional
commits](https://www.conventionalcommits.org/en/v1.0.0-beta.3/) and
[semantic versioning](https://semver.org/).

# mapsf <img src="man/figures/logo.png" align="right" width="120"/>

[![CRAN](https://www.r-pkg.org/badges/version-ago/mapsf)](https://cran.r-project.org/package=mapsf)
[![R-CMD-check](https://github.com/riatelab/mapsf/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/riatelab/mapsf/actions/workflows/R-CMD-check.yaml)
[![downloads](https://cranlogs.r-pkg.org/badges/mapsf?color=brightgreen)](https://cran.r-project.org/package=mapsf)
[![Codecov test
coverage](https://codecov.io/gh/riatelab/mapsf/branch/master/graph/badge.svg)](https://app.codecov.io/gh/riatelab/mapsf?branch=master)
[![status](https://tinyverse.netlify.app/badge/mapsf)](https://CRAN.R-project.org/package=mapsf)

**Create thematic maps and integrate them into your R workflow.**

This package helps you design different cartographic representations
such as proportional symbols, choropleths, or typology maps. It also
provides several functions for displaying layout elements that enhance
the graphical presentation of maps (e.g., scale bar, north arrow, title,
labels). `mapsf` maps `sf` objects on `base` graphics.

## Installation

You can install the released version of `mapsf` from
[CRAN](https://cran.r-project.org/package=mapsf) with:

``` r
install.packages("mapsf")
```

Alternatively, you can install the development version of `mapsf` (the
**dev** branch) from [r-universe](https://riatelab.r-universe.dev/mapsf)
with:

``` r
install.packages("mapsf", repos = c("https://riatelab.r-universe.dev", "https://cloud.r-project.org"))
```

## Usage

This is a basic example which shows how to create a map with `mapsf`.

``` r
library(mapsf)
# Import the sample dataset
mtq <- mf_get_mtq()
# Plot the base map
mf_map(x = mtq)
# Plot proportional symbols
mf_map(x = mtq, var = "POP", type = "prop", leg_pos = "topright")
# Plot a map layout
mf_layout(
  title = "Population in Martinique",
  credits = "T. Giraud; Sources: INSEE & IGN, 2018"
)
```

![](man/figures/example1-1.png)<!-- -->

You can find more detailed examples on the [“Get
Started”](https://riatelab.github.io/mapsf/articles/mapsf) section of
the [package website](https://riatelab.github.io/mapsf/) and you can
also download its [cheat
sheet](https://raw.githubusercontent.com/riatelab/mapsf/master/vignettes/fig/mapsf_cheatsheet.pdf).  
[![Cheat
Sheet](man/figures/mapsf_cheatsheet.png)](https://raw.githubusercontent.com/riatelab/mapsf/master/vignettes/fig/mapsf_cheatsheet.pdf)

## mapsf.gui <img src="man/figures/mapsf_gui_logo.png" align="right" width="60"/>

The [`mapsf.gui` package](https://codeberg.org/riatelab/mapsf.gui)
provides a Shiny interface for creating thematic maps using `mapsf`.
Users of the application can **build thematic maps interactively and
automatically generate the corresponding R code** to reproduce the map
identically.

## Alternatives

- [`tmap`](https://github.com/r-tmap/tmap)  
- [`ggplot2`](https://github.com/tidyverse/ggplot2) +
  [`ggspatial`](https://github.com/paleolimbot/ggspatial)

## Community Guidelines

One can contribute to the package through [pull
requests](https://github.com/riatelab/mapsf/pulls) and report issues or
ask questions [here](https://github.com/riatelab/mapsf/issues). See the
[CONTRIBUTING.md](https://github.com/riatelab/mapsf/blob/master/CONTRIBUTING.md)
file for detailed instructions on how to contribute.  
This project uses [conventional
commits](https://www.conventionalcommits.org/en/v1.0.0-beta.3/) and
[semantic versioning](https://semver.org/).

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 5,
  fig.height = 6
)

## ----mf_basemap, message=FALSE, warning=FALSE---------------------------------
library(mapsf)
# import the sample data set
mtq <- mf_get_mtq()
# plot municipalities
mf_map(mtq, type = "base")
# layout elements
credits <- paste0("Sources: IGN, 2018\n", "mapsf ", packageVersion("mapsf"))
mf_title("Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()

## ----mf_prop, message=FALSE, warning=FALSE------------------------------------
# plot municipalities
mf_map(mtq)
# plot population
mf_map(
  x = mtq,
  var = "POP",
  type = "prop",
  inches = 0.25,
  col = "brown4",
  leg_pos = "topright",
  leg_adj = c(0, -3),
  leg_title = "Total population"
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()

## ----mf_map_c-----------------------------------------------------------------
# population density (inhab./km2) using sf::st_area()
mtq$POPDENS <- 1e6 * mtq$POP / sf::st_area(mtq)
# plot population density
mf_map(
  x = mtq,
  var = "POPDENS",
  type = "choro",
  breaks = "geom",
  nbreaks = 5,
  pal = "Teal",
  border = "white",
  lwd = 0.5,
  leg_pos = "bottomleft",
  leg_adj = c(0, 6),
  leg_title = "Population Density\n(inh. / km2)"
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()

## ----mf_map_t-----------------------------------------------------------------
# plot administrative status
mf_map(
  x = mtq,
  var = "STATUS",
  type = "typo",
  pal = c("aquamarine4", "yellow3", "wheat"),
  lwd = .5,
  val_order = c(
    "Prefecture",
    "Sub-prefecture",
    "Simple municipality"
  ),
  leg_pos = "topright",
  leg_adj = c(0, 1),
  leg_title = ""
)
# labels for a few  municipalities
mf_label(
  x = mtq[mtq$STATUS != "Simple municipality", ], var = "LIBGEO",
  cex = 0.9, halo = TRUE, r = 0.15
)
# layout elements
mf_title("Administrative Status")
mf_credits(credits)
mf_arrow()
mf_scale()

## ----mf_map_pc, fig.width=5---------------------------------------------------
# Plot the municipalities and expand the map space on the right
mf_map(x = mtq, expandBB = c(0, 0, 0, .15))
# Plot symbols with choropleth coloration
mf_map(
  x = mtq,
  var = c("POP", "MED"),
  type = "prop_choro",
  border = "grey50",
  lwd = 1,
  leg_pos = c("topright"),
  leg_title = c("Population", "Median Income\n(in euros)"),
  breaks = "equal",
  nbreaks = 4,
  pal = "Greens",
  leg_val_rnd = c(0, -2),
  leg_frame = FALSE
)
# layout elements
mf_title("Population & Wealth in Martinique, 2015")
mf_credits(credits)
mf_arrow()
mf_scale()

## ----mf_map_pt, fig.width=5---------------------------------------------------
# plot the municipalities and expand the map space on the right
mf_map(x = mtq, expandBB = c(0, 0, 0, .15))
# plot symbols with choropleth coloration
mf_map(
  x = mtq,
  var = c("POP", "STATUS"),
  type = "prop_typo",
  symbol = "square",
  border = "white",
  lwd = .5,
  leg_pos = "topright",
  leg_title = c("Population", "Administrative\nStatus"),
  val_order = c(
    "Prefecture", "Sub-prefecture",
    "Simple municipality"
  )
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()

## ----mf_label-----------------------------------------------------------------
# plot municipalities
mf_map(mtq, col = "#e4e9de", border = "darkseagreen4")
# plot labels
mf_label(
  x = mtq,
  var = "LIBGEO",
  cex = 0.7,
  font = 4,
  halo = TRUE,
  r = 0.1,
  overlap = FALSE,
  q = 3,
  lines = FALSE
)
# layout elements
mf_title("Municipalities of Martinique")
mf_credits(credits)
mf_arrow(pos = "topright")
mf_scale()

## ----mf_grad------------------------------------------------------------------
# import the csv file embedded in mapsf
mob <- read.csv(system.file("csv/mob.csv", package = "mapsf"))
# Select links from Fort-de-France (97209))
mob_97209 <- mob[mob$i == 97209, ]
# create an sf object of links
mob_links <- mf_get_links(x = mtq, df = mob_97209)
# Plot the municipalities
mf_map(mtq)
# plot graduated links
mf_map(
  x = mob_links,
  var = "fij",
  type = "grad",
  breaks = c(100, 500, 1000, 4679.0),
  lwd = c(1, 4, 8),
  leg_pos = "topright",
  leg_title = "Nb. of\nCommuters",
  leg_val_rnd = 0
)
# layout elements
mf_title("Commuting to Fort-de-France")
mf_credits(credits)
mf_arrow()
mf_scale()

pos <- c('center', 'topleft', 'top', 'topright', 'right', 'bottomright',
         'bottom',  'left', 'bottomleft')
# input
expect_silent({
  mf_map(mtq)
  mf_text(x = mtq[1, ])
  mf_text(x = sf::st_geometry(mtq[2, ]))
  mf_text(x = c(731029.56079219, 1626948.29177051))
  for (i in pos[-1]) {
    mf_text(x = i)
  }
})

# position
expect_silent({
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, offset = 5)
  }
})

# alignement
expect_silent({
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[5, ], pos = i, txt = "2 lines of\ntext", align = "left", offset = 1)
  }
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[26, ], pos = i, txt = "2 lines of\ntext", align = "center", offset = 1)
  }
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[27, ], pos = i, txt = "2 lines of\ntext", align = "right", offset = 1)
  }
})

# lines
expect_silent({
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[5, ], pos = i, txt = "2 lines of\ntext", align = "left", offset = 10, line = 1)
  }
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, txt = "2 lines of\ntext", align = "center", offset = 15, line = 2, clockwise = TRUE)
  }
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, txt = "2 lines of\ntext", align = "center", offset = 15, line = 2, clockwise = FALSE)
  }
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, txt = "2 lines of\ntext", align = "center", offset = 15, line = 3, clockwise = TRUE)
  }
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, txt = "2 lines of\ntext", align = "center", offset = 15, line = 3, clockwise = FALSE)
  }
})

# no arrow
expect_silent({
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, offset = 15, line = 1, arrow = FALSE)
  }
})

# wrong pos
expect_warning({
  mf_map(mtq)
  mf_text(x = mtq[4, ], pos = "West", offset = 15, line = 1, arrow = FALSE)
})


# halo
expect_silent({
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, offset = 5, halo = TRUE, col_halo = "green")
  }
})

# box
expect_silent({
  mf_map(mtq)
  for (i in pos) {
    mf_text(x = mtq[4, ], pos = i, offset = 15, box = TRUE, col_box = "green")
  }
})

expect_silent(mf_map(mtq, c("POP", "STATUS"), type = "prop_typo", add = FALSE))

mf_map(mtq)
expect_silent(mf_map(mtq, c("POP", "STATUS"), type = "prop_typo"))

mtq[6, "STATUS"] <- NA
expect_silent(
  mf_map(
    x = mtq, var = c("POP", "STATUS"), type = "prop_typo",
    inches = .35, border = "tomato4",
    val_max = 90000, symbol = "square", col_na = "grey", pal = "Dynamic",
    lwd = 2,
    leg_pos = c("bottomright"),
    leg_title = c("Population", "Municipality\nstatus"),
    leg_title_cex = c(0.9, 0.9),
    leg_val_cex = c(.7, .7),
    val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
    leg_no_data = "No dada",
    leg_frame = c(TRUE, TRUE),
    add = TRUE)
)


mf_map(mtq)
expect_silent(mf_map(mtq_l, c("fij", "sj"), "prop_typo", add = TRUE))
mtq_l[2, "sj"] <- NA
expect_silent(mf_map(mtq_l, c("fij", "sj"), "prop_typo",
                     val_max = 5000, add = FALSE, col_na = "red"))


expect_message(mf_map(mtq, c("POP", "STATUS"), type = "prop_typo",
                      leg_pos = c("top", "bottom")))
expect_message(mf_map(mtq_l, c("fij", "sj"), type = "prop_typo",
                      leg_pos = c("top", "bottom")))

# init
mf_map(mtq)

## mf_credits
for (i in  c("bottomleft", "bottomright", "rightbottom")){
  expect_silent(mf_credits(txt = "credits line with \n a jump",
                           pos = i, col = "red"))
}
expect_silent(mf_credits(txt = "credits line with \n a jump\n\n."))

mf_map(mtq)
expect_silent(mf_inset_on(x = mtq))
mf_map(mtq)
expect_silent(mf_inset_off())
expect_silent(mf_inset_on(fig = c(0, .5, 0, .5)))
mf_map(mtq)
expect_silent(mf_inset_off())


expect_silent(mf_inset_on(x = "worldmap", pos = "topleft"))
mf_worldmap(mtq)
expect_silent(mf_inset_off())

expect_silent(
  mf_map(
    x = mtq, var = "STATUS", type = "typo",
    pal = c("red", "blue", "yellow"), lwd = 1.1,
    val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
    col_na = "green", border = "brown",
    leg_pos = "bottomleft",
    leg_title = "Status", leg_title_cex = 1.1,
    leg_val_cex = 1, leg_no_data = "No data",
    leg_frame = TRUE, add = FALSE)
)

mtq[6, "STATUS"] <- NA
expect_silent(mf_map(x = mtq, var = "STATUS", type = "typo"))
expect_silent(mf_map(x = mtq_p, var = "STATUS", type = "typo"))
expect_silent(mf_map(x = mtq_l, var = "sj", type = "typo"))

mf_map(mtq)
expect_silent(mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro",
                            leg_pos = c("top")))
expect_message(mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro",
                     leg_pos = c("top", "topright")))
expect_message(mf_map(mtq_l, var = c("sj", "sj"), type = "symb_choro"))
expect_message(mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro",
                      pch = 1:2, cex = 1:2))
expect_silent(mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro",
                     leg_pos = "top", add = FALSE))

mf_map(mtq, expandBB = rep(.2, 4))
expect_silent(
  mf_graticule(
    mtq,
    pos = c("top", "left", "bottom", "right"),
    add = TRUE
  )
)
expect_silent(
  mf_graticule(
    mtq,
    pos = c("top", "left", "bottom", "right"),
    label = FALSE,
    add = FALSE
  )
)

g <- mf_graticule(mtq, add = TRUE)
expect_inherits(g, "sf")
expect_length(sf::st_geometry(g), 9)

mf_map(mtq)
expect_silent(mf_background(system.file("img/background.jpg",
                                        package = "mapsf")))
expect_silent(mf_background(system.file("help/figures/logo.png",
                                        package = "mapsf")))

# init
expect_silent(mf_init(mtq))
expect_silent(mf_init(mtq, expandBB = c(0, 0, 0, .4)))
expect_silent(mf_init(mtq, bgc = "red"))
expect_silent(mf_init(mtq, extent = mtq[3, ]))
expect_silent(mf_init(mtq, extent = mtq[3, ], bgc = "red",
                      expandBB = c(0, 0, 0, .4)))
mf_theme(frame = "map")
expect_silent(mf_init(mtq))



b <- terra::rast(system.file("ex/elev.tif", package = "terra"))
expect_silent(mf_init(b))
mf_raster(b, add = TRUE)
expect_silent(mf_init(b, expandBB = c(0, 0, 0, .4)))
expect_silent(mf_init(b, bgc = "red"))
expect_silent(mf_init(b, extent = b))
expect_silent(mf_init(b, extent = b, bgc = "red",
                      expandBB = c(0, 0, 0, .4)))

expect_silent(mf_png(mtq, height = 600,
                     filename = paste0(tempfile(), ".png")))
dev.off()

expect_silent(mf_png(mtq, filename = paste0(tempfile(), ".png")))
dev.off()

expect_silent(mf_png(mtq, paste0(tempfile(), ".png"),
                     width = 800, height = 800))
dev.off()

expect_silent(mf_svg(mtq, filename = paste0(tempfile(), ".svg"),
                     width = 6, height = 6))
dev.off()

expect_silent(mf_svg(mtq, filename = paste0(tempfile(), ".svg")))
dev.off()

expect_message(mf_svg(mtq, width = 51,
                      filename = paste0(tempfile(), ".svg")))
dev.off()

expect_silent(mf_svg(mtq, height = 7,
                     filename = paste0(tempfile(), ".svg")))
dev.off()

expect_silent(mf_svg(mtq, height = 7,
                     filename = paste0(tempfile(), ".svg"),
                     svglite = FALSE))
dev.off()

r <- terra::rast(system.file("ex/elev.tif", package = "terra"))
expect_silent(mf_png(r))
mf_raster(r)
dev.off()

expect_silent(mf_png(st_transform(mtq, "EPSG:4326"),
                     filename = paste0(tempfile(), ".png")))
dev.off()


### mf_export....
expect_warning(mf_export(mtq, height = 600,
                        filename = paste0(tempfile(), ".png")))
dev.off()
expect_warning(mf_export(mtq, filename = paste0(tempfile(), ".png")))
dev.off()

expect_warning(mf_export(mtq, paste0(tempfile(), ".png"),
                        width = 800, height = 800))
dev.off()

expect_warning(mf_export(mtq, filename = paste0(tempfile(), ".svg"),
                        width = 6, height = 6))
dev.off()

expect_warning(mf_export(mtq, filename = paste0(tempfile(), ".svg")))
dev.off()

expect_message(mf_export(mtq, width = 51,
                         filename = paste0(tempfile(), ".svg")))
dev.off()

expect_warning(mf_export(mtq, height = 7,
                        filename = paste0(tempfile(), ".svg")))
dev.off()

expect_warning(mf_export(sf::st_transform(mtq, "epsg:4326"),
                         filename = paste0(tempfile(), ".png")))
dev.off()

mf_map(mtq)
expect_silent(mf_label(x = mtq[1,], var = "LIBGEO", halo = FALSE, cex = 0.8,
                       overlap = FALSE, lines = TRUE))
expect_silent(mf_label(x = mtq, var = "LIBGEO", halo = TRUE, cex = 0.8,
         overlap = FALSE, lines = TRUE))
expect_silent(mf_label(x = mtq, var = "LIBGEO", halo = TRUE,
                       cex = 0.8, col = sample(1:5, nrow(mtq), replace = TRUE),
                       overlap = FALSE, lines = TRUE))
expect_silent(mf_label(x = mtq, var = "LIBGEO", halo = TRUE,
                       cex = 0.8, col = sample(1:5, nrow(mtq), replace = TRUE),
                       overlap = FALSE, lines = TRUE, pos = 3, font = 3))
expect_silent(mf_label(x = mtq, var = "LIBGEO", add = FALSE))

# init
mf_map(mtq)
## mf_arrow
for (i in list("topleft", "top", "topright", "right", "bottomright", "bottom",
               "bottomleft", "left", c(701012.7, 1613554))){
  expect_silent(mf_arrow(pos = i, col = "red"))
}
expect_silent(mf_arrow(pos = "top", adj = c(-5, 0), cex = 2))
expect_silent(mf_arrow(pos = "top", adj = c(-5, 0), cex = 2, align = mtq))

expect_silent(mf_map(
  x = mtq, var = "MED", type = "choro", col_na = "grey", pal = "Cividis",
  breaks = "quantile", nbreaks = 4, border = "white",
  lwd = .5, leg_pos = "bottomleft",
  leg_title = "Median Income", leg_title_cex = 1.1,
  leg_val_cex = 1, leg_val_rnd = -2, leg_no_data = "No data",
  leg_frame = TRUE
))

mtq[6, "MED"] <- NA
expect_silent(mf_map(x = mtq, var = "MED", type = "choro"))

expect_message(mf_map(x = mtq, var = "MED", type = "choro", breaks = c(12000, 15000, 35000)))
expect_message(mf_map(x = mtq, var = "MED", type = "choro", breaks = c(12000, 15000, 19000)))

expect_silent(mf_map(mtq_p, "MED", "choro"))
expect_silent(mf_map(mtq_l, "fij", "choro"))
expect_silent(mf_map(mtq_p, "MED", "choro", pch = 4))

mf_map(mtq)
expect_warning(mf_annotation(x = c(711167.8, 1614764),
              txt = "Look!\nImportant feature\nhere!",
              pos = "bottomleft", cex = 1.2, font = 2,
              halo = TRUE, s = 1.5))
expect_warning(mf_annotation(x = mtq[20, ],
              txt = "This is less\nimportant",
              cex = .7, font = 3, s = 1.3))
expect_warning(mf_annotation(x = mtq[20, ], pos = "topleft", s = 0.9,
                            txt = "This"))
expect_warning(mf_annotation(x = mtq[20, ], pos = "bottomright",
                            txt = "This"))


expect_warning(mf_annotation(x = mtq[20, ], pos = "topright",
                            txt = "This", halo = TRUE))

expect_warning(mf_annotation(x = mtq[20, ], pos = "topleft",
                            txt = "This", halo = TRUE))
expect_warning(mf_annotation(x = mtq[20, ], pos = "bottomleft",
                            txt = "This", halo = FALSE))
expect_warning(mf_annotation(x = mtq[20, ], pos = "bottomright",
                            txt = "This", halo = TRUE))
expect_warning(mf_annotation(x = mtq[20, ], pos = "center",
                            txt = "This", halo = TRUE))

mf_map(mtq)
expect_silent(mf_map(x = mtq, var = "STATUS", type = "symb",add = FALSE))
expect_message(mf_map(x = mtq, var = "STATUS", type = "symb",
                      pch = 20:21, cex = c(2, 2)))
expect_message(mf_map(x = mtq_l, var = "sj", type = "symb"))

#' @title Set a theme
#' @description
#' A theme is a set of graphical parameters that are applied to maps created
#' with `mapsf`. These parameters are:
#' - figure margins and frames,
#' - background, foreground and highlight colors,
#' - default sequential and qualitative palettes,
#' - title options (position, size, banner...).
#'
#' `mapsf` offers some builtin themes. It's possible to modify an existing theme
#' or to start a theme from scratch. It is also possible to set a custom theme
#' using a list of arguments
#'
#' Themes are persistent across maps produced by `mapsf`
#' (e.g. they survive a `dev.off()` call).
#'
#' Current theme parameters are set
#' in `mapsf` options and nammed according to the following convention:
#' "mapsf.mf_theme_arg_name".
#' Use `getOption()` to return the value of a specific argument of
#' the current theme (see examples).
#'
#' Use `mf_theme(NULL)` or `mf_theme('base')` to reset to default theme
#' settings.
#'
#' @eval paste0("@param x name of a map theme. One of '",
#' paste0(get_themes_names(), collapse= "', '" ),"'.")
#' @param mar a numeral vector of the form c(bottom, left, top, right)
#' which gives the margin size specified in number of lines
#' @param title_pos title position, one of 'left', 'center', 'right'
#' @param title_tab if TRUE the title is displayed as a 'tab'
#' @param title_cex cex of the title
#' @param title_font font of the title
#' @param title_line number of lines used for the title
#' @param title_inner if TRUE the title is displayed inside the plot area;
#' if FALSE the title is displayed in the top margin
#' @param title_banner if TRUE the title is displayed as a banner
#' @param foreground foreground color
#' @param background background color
#' @param highlight highlight color
#' @param pal_quali default qualitative color palette (name or function)
#' @param pal_seq default sequential color palette (name or function)
#' @param ... deprecated arguments ('bg', 'fg', 'tab', 'pos', 'inner',
#' 'line', 'cex' and 'font'). See the Note section.
#' @param frame either "none", "map" or "figure"; plot a frame around the map
#' or the figure.
#' @param frame_lwd line width for the frame
#' @param frame_lty line type for the frame
#' @note
#' The following themes are deprecated: "default", "brutal",
#' "ink", "dark", "agolalight", "candy", "darkula", "iceberg", "green",
#' "nevermind", "jsk" and "barcelona".\cr
#' The following arguments are deprecated: "bg", "fg", "tab",
#' "pos", "inner", "line", "cex" and "font".\cr
#'
#' Although the map theming system has been radically changed in version 1.0.0
#' of the package, you can still use the old themes by referencing them by name.
#' If you need to use the *pre* v1.0.0 default theme, set `x` to "default".\cr
#' If an old theme is set, only deprecated arguments are used and others are
#' ignored.\cr
#' If current and deprecated arguments are mixed, only deprecated arguments are
#' used and others are ignored.\cr
#' All references and usages of the old theming system will be removed in the
#' next major version.
#' @return `mf_theme` (invisibly) returns the list of current theme parameters.
#' @export
#' @examples
#' mtq <- mf_get_mtq()
#'
#' # Choosing a theme by name:
#' mf_theme("base")
#' mf_map(mtq)
#' mf_title()
#'
#' # Specifying some values directly:
#' mf_theme(title_banner = TRUE)
#' mf_map(mtq)
#' mf_title()
#'
#' # Using a mix of the above:
#' mf_theme("sol_dark", title_tab = TRUE, title_font = 1)
#' mf_map(mtq)
#' mf_title()
#'
#' # Specifying a list with theme values:
#' theme <- list(
#'   mar = c(1, 1, 3, 1),
#'   title_tab = FALSE,
#'   title_pos = "left",
#'   title_inner = FALSE,
#'   title_line = 2,
#'   title_cex = 1.5,
#'   title_font = 2,
#'   title_banner = FALSE,
#'   frame = "figure",
#'   frame_lwd = 1,
#'   frame_lty = 1,
#'   foreground = "#fbfbfb",
#'   background = "grey75",
#'   highlight = "#0f5027",
#'   pal_quali = "Dark 3",
#'   pal_seq = "Greens"
#' )
#' mf_theme(theme)
#' mf_map(mtq, "MED", "choro")
#' mf_title()
#'
#' # Obtaining a list of parameters for the current theme:
#' current_theme <- mf_theme()
#'
#' # Obtaining individual parameters for the current theme:
#' getOption("mapsf.highlight")
#' getOption("mapsf.pal_seq")
#'
#'
#' # Use default theme:
#' mf_theme(NULL)
#' # or
#' mf_theme("base")
mf_theme <- function(x,
                     mar,
                     foreground,
                     background,
                     highlight,
                     title_tab,
                     title_pos,
                     title_inner,
                     title_line,
                     title_cex,
                     title_font,
                     title_banner,
                     frame,
                     frame_lwd,
                     frame_lty,
                     pal_quali,
                     pal_seq, ...) {
  # current theme
  theme <- list(
    mar          = getOption("mapsf.mar"),
    title_tab    = getOption("mapsf.title_tab"),
    title_pos    = getOption("mapsf.title_pos"),
    title_inner  = getOption("mapsf.title_inner"),
    title_line   = getOption("mapsf.title_line"),
    title_cex    = getOption("mapsf.title_cex"),
    title_font   = getOption("mapsf.title_font"),
    title_banner = getOption("mapsf.title_banner"),
    foreground   = getOption("mapsf.foreground"),
    background   = getOption("mapsf.background"),
    highlight    = getOption("mapsf.highlight"),
    pal_quali    = getOption("mapsf.pal_quali"),
    pal_seq      = getOption("mapsf.pal_seq"),
    frame        = getOption("mapsf.frame"),
    frame_lty    = getOption("mapsf.frame_lty"),
    frame_lwd    = getOption("mapsf.frame_lwd"),
    legacy       = getOption("mapsf.legacy")
  )

  # if no arg input => return param list
  argx <- as.list(match.call()[-1])
  if (length(argx) == 0) {
    return(invisible(theme))
  }


  # input a theme name
  if (!missing(x)) {
    # if is.null(x) => set base theme
    if (is.null(x)) {
      x <- "base"
    }
    # if x is a list of args
    if (is.list(x)) {
      argx <- theme <- x
    } else {
      if (!x %in% names(.gmapsf$themes)) {
        stop("x is not a theme name.", call. = FALSE)
      } else {
        theme <- .gmapsf$themes[[x]]
        if (isTRUE(theme$legacy)) {
          message(
            paste0(
              "The following themes are deprecated:\n",
              "'default', 'brutal', 'ink', 'dark', 'agolalight', 'candy', 'darkula',\n",
              "'iceberg', 'green', 'nevermind', 'jsk', and 'barcelona'.\n",
              "See the Note section in the help page (?mf_theme)."
            )
          )
        }
      }
    }
  }


  # update theme params
  # legacy themes & param
  legacy_argx <- c(
    argx$bg, argx$fg, argx$tab, argx$pos, argx$inner, argx$line,
    argx$cex, argx$font
  )
  if (!is.null(legacy_argx)) {
    message(paste0(
      "The following arguments are deprecated:\n",
      "'bg', 'fg', 'tab', 'pos', 'inner', 'line', 'cex'",
      " and 'font'.\n",
      "See the Note section in the help page (?mf_theme)."
    ))
    theme$legacy <- TRUE
    theme$title_banner <- TRUE
    theme$pal_quali <- "Dynamic"
    theme$pal_seq <- "Mint"
    theme$frame <- "none"
    if (!is.null(argx$bg)) theme$background <- argx$bg
    if (!is.null(argx$fg)) theme$foreground <- argx$fg
    if (!is.null(argx$fg)) theme$highlight <- argx$fg
    if (!is.null(argx$tab)) theme$title_tab <- argx$tab
    if (!is.null(argx$pos)) theme$title_pos <- argx$pos
    if (!is.null(argx$inner)) theme$title_inner <- argx$inner
    if (!is.null(argx$line)) theme$title_line <- argx$line
    if (!is.null(argx$cex)) theme$title_cex <- argx$cex
    if (!is.null(argx$font)) theme$title_font <- argx$font
  } else {
    if (!missing(title_tab)) theme$title_tab <- title_tab
    if (!missing(title_pos)) theme$title_pos <- title_pos
    if (!missing(title_inner)) theme$title_inner <- title_inner
    if (!missing(title_line)) theme$title_line <- title_line
    if (!missing(title_cex)) theme$title_cex <- title_cex
    if (!missing(title_font)) theme$title_font <- title_font
    if (!missing(title_banner)) theme$title_banner <- title_banner
    if (!missing(foreground)) theme$foreground <- foreground
    if (!missing(background)) theme$background <- background
    if (!missing(highlight)) theme$highlight <- highlight
    if (!missing(pal_quali)) theme$pal_quali <- pal_quali
    if (!missing(pal_seq)) theme$pal_seq <- pal_seq
    if (!missing(frame)) theme$frame <- frame
    if (!missing(frame_lwd)) theme$frame_lwd <- frame_lwd
    if (!missing(frame_lty)) theme$frame_lty <- frame_lty
  }
  if (!missing(mar)) theme$mar <- mar


  # set theme options
  options(
    mapsf.mar          = theme$mar,
    mapsf.title_tab    = theme$title_tab,
    mapsf.title_pos    = theme$title_pos,
    mapsf.title_inner  = theme$title_inner,
    mapsf.title_line   = theme$title_line,
    mapsf.title_cex    = theme$title_cex,
    mapsf.title_font   = theme$title_font,
    mapsf.title_banner = theme$title_banner,
    mapsf.foreground   = theme$foreground,
    mapsf.background   = theme$background,
    mapsf.highlight    = theme$highlight,
    mapsf.pal_quali    = theme$pal_quali,
    mapsf.pal_seq      = theme$pal_seq,
    mapsf.legacy       = theme$legacy,
    mapsf.frame        = theme$frame,
    mapsf.frame_lwd    = theme$frame_lwd,
    mapsf.frame_lty    = theme$frame_lty
  )


  return(invisible(as.list(theme)))
}

## mf_theme
expect_silent(mf_theme())
expect_message(mf_theme("iceberg"))
expect_silent(mf_theme("sol_light"))
expect_error(mf_theme("NoT a thEmE"))
expect_message(mf_theme(x = list(
  name = "custom",
  bg = "black",
  fg = "yellow",
  mar = c(5, 5, 5, 5),
  tab = TRUE,
  pos = "right",
  inner = FALSE,
  line = 4,
  cex = 2,
  font = 3
)))
expect_message(mf_theme(bg = "darkslategrey", fg = "cornsilk3",
                       mar = c(2, 2, 4, 2),
                       tab = FALSE, pos = "center", inner = FALSE,
                       line = 2, cex = 2, font = 4)
)
expect_silent(mf_map(mtq, "POP", "prop", leg_frame = TRUE, add = FALSE))
mf_map(mtq)
expect_silent(mf_theme(NULL))

#' @name mf_get_links
#' @title Get a link layer from a data.frame of links
#' @description Create a link layer from a data.frame of links and an sf object.
#' @param x an sf object, a simple feature collection.
#' @param df a data.frame that contains identifiers of starting and ending
#' points.
#' @param x_id name of the identifier variable in x, default to the first
#' column (optional)
#' @param df_id names of the identifier variables in df, character vector of
#' length 2, default to
#' the two first columns. (optional)
#' @return An sf object is returned, it is composed of df and the sfc
#' (LINESTRING) of links.
#' @examples
#' mtq <- mf_get_mtq()
#' mob <- read.csv(system.file("csv/mob.csv", package = "mapsf"))
#' # Select links from Fort-de-France (97209))
#' mob_97209 <- mob[mob$i == 97209, ]
#' # Create a link layer
#' mob_links <- mf_get_links(x = mtq, df = mob_97209)
#' # Plot the links
#' mf_map(mtq)
#' mf_map(mob_links, col = "red4", lwd = 2, add = TRUE)
#' @export
mf_get_links <- function(x, df, x_id, df_id) {
  if (missing(x_id)) {
    x_id <- names(x)[1]
  }
  if (missing(df_id)) {
    df_id <- names(df)[1:2]
  }
  # extract x centroids
  x2 <- data.frame(
    id = x[[x_id]],
    st_coordinates(st_centroid(
      x = st_geometry(x),
      of_largest_polygon = TRUE
    )),
    stringsAsFactors = FALSE
  )
  link <- merge(df, x2, by.x = df_id[2], by.y = "id", all.x = TRUE)
  link <- merge(link, x2, by.x = df_id[1], by.y = "id", all.x = TRUE)
  # watch for NAs
  d1 <- nrow(link)
  link <- link[!is.na(link$X.x) & !is.na(link$X.y), ]
  d2 <- nrow(link)
  if (d2 == 0) {
    stop("No links were created. df_id and x_id do not match.", call. = FALSE)
  }
  if ((d1 - d2) > 0) {
    warning(
      paste0(
        (d1 - d2),
        " links were not created. Some ids from df were not found in x."
      ),
      call. = FALSE
    )
  }
  # build the link
  stringo <- paste0(
    "LINESTRING(", link$X.y, " ", link$Y.y, ", ",
    link$X.x, " ", link$Y.x, ")"
  )
  link <- st_sf(link[, seq_len(ncol(df))],
    geometry = st_as_sfc(stringo, crs = st_crs(x))
  )
  return(link)
}

mob <- read.csv(system.file("csv/mob.csv", package = "mapsf"))
# Select links from Fort-de-France (97209))
mob_97209 <- mob[mob$i == 97209, ]
# Create a link layer
expect_silent(
  {
    mob_links <- mf_get_links(x = mtq, df = mob_97209)
  }
)
expect_equal(nrow(mob_links), 10)

expect_error(mf_get_links(x = mtq, df = mob_97209, x_id = "LIBGEO"))
expect_warning(mf_get_links(x = mtq[1:10, ], df = mob_97209))

.gmapsf$themes <- list(
  base = list(
    mar          = c(0.5, 0.5, 2.25, 0.5),
    title_tab    = FALSE,
    title_pos    = "center",
    title_inner  = FALSE,
    title_line   = 1.75,
    title_cex    = 1.25,
    title_font   = 2,
    title_banner = FALSE,
    frame        = "none",
    frame_lwd    = 1,
    frame_lty    = 1,
    foreground   = "#BFBFBF",
    background   = "#fbfbfb",
    highlight    = "#500f27",
    pal_quali    = "Dark 3",
    pal_seq      = "BrwnYl"
  ),
  grey = list(
    mar          = c(0.5, 0.5, 2.25, 0.5),
    title_tab    = FALSE,
    title_pos    = "center",
    title_inner  = FALSE,
    title_line   = 1.75,
    title_cex    = 1.25,
    title_font   = 2,
    title_banner = TRUE,
    frame        = "none",
    frame_lwd    = 1,
    frame_lty    = 1,
    background   = "#F2F2F2",
    foreground   = "#B3B3B3",
    highlight    = "#1A1A1A",
    pal_seq      = "Grays",
    pal_quali    = "Pastel 1"
  ),
  sol_light = list(
    mar          = c(0.5, 0.5, 2.25, 0.5),
    title_tab    = FALSE,
    title_pos    = "left",
    title_inner  = FALSE,
    title_line   = 1.75,
    title_cex    = 1.25,
    title_font   = 2,
    title_banner = FALSE,
    frame        = "none",
    frame_lwd    = 1,
    frame_lty    = 1,
    background   = "#fdf6e3",
    foreground   = "#93a1a1",
    highlight    = "#1A1A1A",
    pal_seq      = "BurgYl",
    pal_quali    = "Pastel 1"
  ),
  sol_dark = list(
    mar = c(0.5, 0.5, 2.25, 0.5),
    title_tab = FALSE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 1.75,
    title_cex = 1.25,
    title_font = 2,
    title_banner = FALSE,
    frame = "none",
    frame_lwd = 1,
    frame_lty = 1,
    background = "#073642",
    foreground = "#93a1a1",
    highlight = "#F2F2F2",
    pal_seq = colorRampPalette(c(
      "#002b36", "#657b83",
      "#839496", "#eee8d5"
    )),
    pal_quali = "Pastel 1"
  ),
  mint = list(
    mar          = c(0.25, 0.25, 2, 0.25) + .5,
    title_tab    = FALSE,
    title_pos    = "center",
    title_inner  = FALSE,
    title_line   = 1.75,
    title_cex    = 1.25,
    title_font   = 2,
    title_banner = FALSE,
    frame        = "map",
    frame_lwd    = 1,
    frame_lty    = "5252",
    background   = "#FAFAFA",
    foreground   = "#C0EBA6",
    highlight    = "#7d698c",
    pal_seq      = "Emrld",
    pal_quali    = "Pastel 1"
  ),
  dracula = list(
    mar          = c(0.5, 0.5, 2.25, 0.5) - .5,
    title_tab    = FALSE,
    title_pos    = "center",
    title_inner  = FALSE,
    title_line   = 1.75,
    title_cex    = 1.25,
    title_font   = 2,
    title_banner = FALSE,
    frame        = "none",
    frame_lwd    = 1,
    frame_lty    = "5252",
    background   = "#22212c",
    foreground   = "#7359f8",
    highlight    = "#fefe7f",
    pal_seq      = "Purples 3",
    pal_quali    = "Warm"
  ),
  rzine = list(
    mar          = c(0, 0, 1.21, 0),
    title_tab    = FALSE,
    title_pos    = "center",
    title_inner  = FALSE,
    title_line   = 1.2,
    title_cex    = 1,
    title_font   = 2,
    title_banner = FALSE,
    frame        = "figure",
    frame_lwd    = 1,
    frame_lty    = "5252",
    background   = "#b2c3e6",
    foreground   = "#4d77ba",
    highlight    = "#FFFEF3",
    pal_seq      = "Blues 3",
    pal_quali    = "Warm"
  ),
  pistachio = list(
    mar          = c(0, 0, 1.21, 0),
    title_tab    = FALSE,
    title_pos    = "center",
    title_inner  = FALSE,
    title_line   = 1.2,
    title_cex    = 1,
    title_font   = 2,
    title_banner = FALSE,
    frame        = "figure",
    frame_lwd    = 1,
    frame_lty    = "5252",
    foreground   = "#D2E2C3",
    background   = "#EFF3EA",
    highlight    = "#73607A",
    pal_seq      = "Purples 3",
    pal_quali    = "Warm"
  ),
  # legacy themes
  default = list(
    mar = c(.5, .5, 1.7, .5),
    title_tab = TRUE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 1.2,
    title_cex = 1,
    title_font = 1,
    title_banner = TRUE,
    foreground = "#333333",
    background = "#f7f7f7",
    highlight = "#333333",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  brutal = list(
    background = "#FFFFFF",
    foreground = "#3b4252",
    mar = c(5.1, 4.1, 4.1, 2.1),
    title_tab = TRUE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 2,
    title_cex = 1.5,
    title_font = 2,
    highlight = "#3b4252",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  ink = list(
    background = "#FFDEAD",
    foreground = "#0000FF",
    mar = c(0, 0, 1.2, 0),
    title_tab = FALSE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 1.2,
    title_cex = .9,
    title_font = 2,
    highlight = "#0000FF",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  dark = list(
    background = "#2E3947",
    foreground = "#7E848C",
    mar = c(0.5, 0.5, 2, 0.5),
    title_tab = FALSE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 1.5,
    title_cex = 1,
    title_font = 1,
    highlight = "#7E848C",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  agolalight = list(
    background = "#EDF4F5",
    foreground = "#82888A",
    mar = c(0, 0, 2, 0),
    title_tab = FALSE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 2,
    title_cex = 1.5,
    title_font = 3,
    highlight = "#82888A",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  candy = list(
    background = "#FDFCFE",
    foreground = "#6B1767",
    mar = c(0, 0, 2, 0),
    title_tab = FALSE,
    title_pos = "center",
    title_inner = FALSE,
    title_line = 2,
    title_cex = 1.5,
    title_font = 2,
    highlight = "#6B1767",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  darkula = list(
    background = "#232525",
    foreground = "#A9B7C6",
    mar = c(0.5, 0.5, 0.5, 0.5),
    title_tab = TRUE,
    title_pos = "right",
    title_inner = TRUE,
    title_line = 1.5,
    title_cex = 1,
    title_font = 4,
    highlight = "#A9B7C6",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  iceberg = list(
    background = "#0B0E0E",
    foreground = "#BDD6DB",
    mar = c(0.5, 0.5, 0.5, 0.5),
    title_tab = TRUE,
    title_pos = "right",
    title_inner = TRUE,
    title_line = 1.5,
    title_cex = 1,
    title_font = 4,
    highlight = "#BDD6DB",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  green = list(
    background = "#1B1D16",
    foreground = "#D7FF68",
    mar = c(0.5, 0.5, 2, 0.5),
    title_tab = FALSE,
    title_pos = "center",
    title_inner = FALSE,
    title_line = 1.5,
    title_cex = 1,
    title_font = 2,
    highlight = "#D7FF68",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  nevermind = list(
    background = "#4DB8DA",
    foreground = "#121725",
    mar = c(2, 2, 3.5, 2),
    title_tab = FALSE,
    title_pos = "center",
    title_inner = FALSE,
    title_line = 1.5,
    title_cex = 1.4,
    title_font = 1,
    highlight = "#121725",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  jsk = list(
    background = "#ffdc11",
    foreground = "#0c973c",
    mar = c(0, 0, 1.5, 0),
    title_tab = FALSE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 1.5,
    title_cex = 1,
    title_font = 2,
    highlight = "#0c973c",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  ),
  barcelona = list(
    background = "#160808",
    foreground = "#d73e23",
    mar = c(0, 0, 1.2, 0),
    title_tab = TRUE,
    title_pos = "left",
    title_inner = FALSE,
    title_line = 1.2,
    title_cex = 1,
    title_font = 2,
    highlight = "#d73e23",
    pal_quali = "Set 2",
    pal_seq = "Mint",
    legacy = TRUE,
    frame = "none"
  )
)


.onLoad <- function(libname, pkgname) {
  theme <- .gmapsf$themes$base
  options(
    mapsf.mar          = theme$mar,
    mapsf.title_tab    = theme$title_tab,
    mapsf.title_pos    = theme$title_pos,
    mapsf.title_inner  = theme$title_inner,
    mapsf.title_line   = theme$title_line,
    mapsf.title_cex    = theme$title_cex,
    mapsf.title_font   = theme$title_font,
    mapsf.title_banner = theme$title_banner,
    mapsf.foreground   = theme$foreground,
    mapsf.background   = theme$background,
    mapsf.highlight    = theme$highlight,
    mapsf.pal_quali    = theme$pal_quali,
    mapsf.pal_seq      = theme$pal_seq,
    mapsf.legacy       = theme$legacy,
    mapsf.frame        = theme$frame,
    mapsf.frame_lwd    = theme$frame_lwd,
    mapsf.frame_lty    = theme$frame_lty
  )
}

#' @title Deprecated - Plot a choropleth map
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "choro"`
#' instead.
#'
#' Plot a choropleth map
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'border',
#' 'lwd',
#' 'add' ,
#' 'col_na',
#' 'pal',
#' 'alpha',
#' 'rev',
#' 'breaks',
#' 'nbreaks',
#' 'leg_pos',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'leg_val_rnd',
#' 'leg_val_dec',
#' 'leg_val_big',
#' 'leg_no_data',
#' 'leg_frame',
#' 'leg_adj',
#' 'leg_horiz',
#' 'leg_size',
#' 'leg_box_border',
#' 'leg_box_cex',
#' 'leg_fg',
#' 'leg_bg',
#' 'leg_frame_border'))
#' @param cex cex cex of the symbols if x is a POINT layer
#' @param pch pch type of pch if x is a POINT layer
#' @details
#' Breaks defined by a numeric vector or a classification method are
#' left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\[, \[5 - 10\[, \[10 - 15\[, \[15 - 20\].
#' The "jenks" method is an exception and has to be right-closed.
#' Jenks breaks computed as \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\], \]5 - 10\], \]10 - 15\], \]15 - 20\].
#' @keywords internal
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq, var = "MED", type = "choro")
#'
#' mtq[6, "MED"] <- NA
#' mf_map(
#'   x = mtq, var = "MED", type = "choro",
#'   col_na = "grey", pal = "Cividis",
#'   breaks = "quantile", nbreaks = 4, border = "white",
#'   lwd = .5, leg_pos = "topleft",
#'   leg_title = "Median Income", leg_title_cex = 1.1,
#'   leg_val_cex = 1, leg_val_rnd = -2, leg_no_data = "No data",
#'   leg_frame = TRUE
#' )
mf_choro <- function(x, var,
                     pal,
                     alpha = NULL,
                     rev = FALSE,
                     breaks = "quantile",
                     nbreaks,
                     border,
                     pch = 21,
                     cex = 2,
                     lwd = .7,
                     col_na = "white",
                     extent = x,
                     bg,
                     expandBB = rep(.04, 4),
                     leg_pos = mf_get_leg_pos(x),
                     leg_title = var,
                     leg_title_cex = .8,
                     leg_val_cex = .6,
                     leg_val_rnd = 2,
                     leg_val_dec = ".",
                     leg_val_big = "",
                     leg_no_data = "No data",
                     leg_frame = FALSE,
                     leg_horiz = FALSE,
                     leg_adj = c(0, 0),
                     leg_size = 1,
                     leg_box_border,
                     leg_box_cex = c(1, 1),
                     leg_fg,
                     leg_bg,
                     leg_frame_border,
                     add = FALSE) {
  deprecate_direct_calls_to("mf_choro")
  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")
  pal <- go(pal, "pal_seq", "Mint")
  leg_box_border <- go(leg_box_border, "highlight")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )

  # jenks
  jen <- FALSE
  if (any(breaks %in% "jenks")) {
    jen <- TRUE
  }

  # get the breaks
  breaks <- mf_get_breaks(x = x[[var]], nbreaks = nbreaks, breaks = breaks)
  nbreaks <- length(breaks) - 1
  # get the cols
  pal <- get_the_pal(pal = pal, nbreaks = nbreaks, alpha = alpha, rev = !rev)
  # get the color vector
  mycols <- get_col_vec(x = x[[var]], breaks = breaks, pal = pal, jen = jen)

  no_data <- FALSE
  if (max(is.na(mycols)) == 1) {
    no_data <- TRUE
  }
  mycols[is.na(mycols)] <- col_na


  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  xtype <- get_geom_type(x)

  if (xtype == "LINE") {
    plot(st_geometry(x), col = mycols, lwd = lwd, add = TRUE)
    leg(
      type = "choro_line",
      pos = leg_pos,
      val = breaks,
      title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex,
      val_rnd = leg_val_rnd,
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      col_na = col_na,
      no_data = no_data,
      no_data_txt = leg_no_data,
      frame = leg_frame,
      pal = pal,
      bg = leg_bg,
      fg = leg_fg,
      size = leg_size,
      lwd = lwd,
      box_cex = leg_box_cex,
      frame_border = leg_frame_border,
      adj = leg_adj
    )
  }

  if (xtype == "POLYGON") {
    border <- go(border, opt = "highlight")
    plot(
      st_geometry(x),
      col = mycols, border = border, lwd = lwd,
      add = TRUE
    )
    leg(
      type = "choro",
      pos = leg_pos,
      val = breaks,
      title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex,
      val_rnd = leg_val_rnd,
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      col_na = col_na,
      no_data = no_data,
      no_data_txt = leg_no_data,
      horiz = leg_horiz,
      frame = leg_frame,
      pal = pal,
      bg = leg_bg,
      fg = leg_fg,
      size = leg_size,
      box_border = leg_box_border,
      box_cex = leg_box_cex,
      frame_border = leg_frame_border,
      adj = leg_adj
    )
  }
  if (xtype == "POINT") {
    border <- go(border, "background")
    if (pch %in% 21:25) {
      mycolspt <- border
    } else {
      mycolspt <- mycols
    }
    mycolsptbg <- mycols
    plot(
      st_geometry(x),
      col = mycolspt, bg = mycolsptbg, cex = cex, pch = pch,
      lwd = lwd, add = TRUE
    )
    leg(
      type = "choro_symb",
      pos = leg_pos,
      val = breaks,
      pch = pch,
      cex = cex,
      lwd = lwd,
      border = border,
      title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex,
      val_rnd = leg_val_rnd,
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      col_na = col_na,
      no_data = no_data,
      no_data_txt = leg_no_data,
      horiz = leg_horiz,
      frame = leg_frame,
      pal = pal,
      bg = leg_bg,
      fg = leg_fg,
      size = leg_size,
      box_cex = leg_box_cex * c(.75, .85),
      frame_border = leg_frame_border,
      adj = leg_adj
    )
  }

  return(invisible(x))
}

#' @title Deprecated - Plot an sf object
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "base"`
#' instead.
#'
#' Plot an sf object. This is mostly a wrapper around
#' \code{plot(st_geometry(x), ...)}.
#' @param x	object of class \code{sf}, \code{sfc} or \code{sfg}
#' @eval my_params(c(
#' 'col',
#' 'border',
#' 'lwd',
#' 'pch',
#' 'alpha',
#' 'add'))
#' @param cex point size
#' @param lty line or border type
#' @param ... ignored
#' @keywords internal
#' @return x is (invisibly) returned.
#' @export
#' @examples
#' library(sf)
#' mtq <- mf_get_mtq()
#' mf_map(mtq, type = "base")
#' mf_map(mtq, type = "base", col = "blue")
mf_base <- function(x,
                    col,
                    border,
                    alpha = NULL,
                    cex = 1,
                    pch = 20,
                    lwd = .7,
                    lty = 1,
                    add = FALSE,
                    extent = x,
                    bg,
                    expandBB = rep(.04, 4),
                    ...) {
  deprecate_direct_calls_to("mf_base")
  # margins mgmt
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")

  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  xtype <- get_geom_type(x)

  if (xtype == "LINE") {
    col <- go(col, "highlight", "grey20")
    if (!is.null(alpha)) {
      col <- get_hex_pal(col, alpha)
    }
    plot(
      st_geometry(x),
      col = col, lwd = lwd, lty = lty,
      add = TRUE
    )
  }

  if (xtype == "POLYGON") {
    col <- go(col, "foreground", "grey80")
    border <- go(border, "highlight", "grey20")
    if (!is.null(alpha)) {
      col <- get_hex_pal(col, alpha)
    }
    plot(
      st_geometry(x),
      col = col, border = border, lwd = lwd, lty = lty,
      add = TRUE
    )
  }

  if (xtype == "POINT") {
    col <- go(col, "highlight", "grey20")
    if (!is.null(alpha)) {
      col <- get_hex_pal(col, alpha)
    }
    if (pch %in% 21:25) {
      if (missing(border)) {
        border <- go(border, "foreground", "grey80")
      }
      mycolspt <- border
    } else {
      mycolspt <- col
    }
    mycolsptbg <- col
    plot(
      st_geometry(x),
      col = mycolspt, bg = mycolsptbg, cex = cex, pch = pch,
      lwd = lwd, add = TRUE
    )
  }

  return(invisible(x))
}

#' @title Deprecated - Plot symbols using choropleth coloration
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "symb_choro"`
#' instead.
#'
#' Plot symbols with colors based on a quantitative
#' data classification.
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'border',
#' 'lwd',
#' 'add' ,
#' 'col_na',
#' 'pal',
#' 'cexs',
#' 'pch',
#' 'pch_na',
#' 'cex_na',
#' 'val_order',
#' 'alpha',
#' 'rev',
#' 'breaks',
#' 'nbreaks',
#' 'leg_pos2',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'leg_val_rnd',
#' 'leg_val_dec',
#' 'leg_val_big',
#' 'leg_no_data',
#' 'leg_frame',
#' 'leg_horiz',
#' 'leg_frame_border',
#' 'leg_fg',
#' 'leg_bg',
#' 'leg_size',
#' 'leg_box_border',
#' 'leg_box_cex',
#' 'leg_adj'))
#' @details
#' Breaks defined by a numeric vector or a classification method are
#' left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\[, \[5 - 10\[, \[10 - 15\[, \[15 - 20\].
#' The "jenks" method is an exception and has to be right-closed.
#' Jenks breaks computed as \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\], \]5 - 10\], \]10 - 15\], \]15 - 20\].
#' @keywords internal
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_map(mtq, c("STATUS", "MED"), "symb_choro")
#'
#' mf_map(mtq)
#' mtq$STATUS[30] <- NA
#' mtq$MED[5] <- NA
#' mf_map(mtq, c("STATUS", "MED"),
#'   type = "symb_choro",
#'   pal = "Reds 3", breaks = "quantile", nbreaks = 4,
#'   pch = 21:23, cex = c(3, 2, 1),
#'   pch_na = 25, cex_na = 1.5, col_na = "blue",
#'   val_order = c(
#'     "Prefecture",
#'     "Sub-prefecture",
#'     "Simple municipality"
#'   )
#' )
mf_symb_choro <- function(x, var,
                          pal,
                          alpha = NULL,
                          rev = FALSE,
                          breaks = "quantile",
                          nbreaks,
                          border,
                          pch,
                          cex = 2,
                          lwd = .7,
                          pch_na = 4,
                          cex_na = 1,
                          col_na = "white",
                          val_order,
                          extent = x,
                          bg,
                          expandBB = rep(.04, 4),
                          leg_pos = mf_get_leg_pos(x, 1),
                          leg_title = var,
                          leg_title_cex = c(.8, .8),
                          leg_val_cex = c(.6, .6),
                          leg_val_rnd = 2,
                          leg_val_dec = ".",
                          leg_val_big = "",
                          leg_no_data = c("No data", "No data"),
                          leg_frame = c(FALSE, FALSE),
                          leg_frame_border,
                          leg_horiz = FALSE,
                          leg_adj = c(0, 0),
                          leg_fg,
                          leg_bg,
                          leg_size = 1,
                          leg_box_border,
                          leg_box_cex = c(1, 1),
                          add = TRUE) {
  deprecate_direct_calls_to("mf_symb_choro")
  xtype <- get_geom_type(x)
  # linestring special case
  if (xtype == "LINE") {
    message("This map type is not available for lines.")
    return(invisible(NULL))
  }

  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")
  border <- go(border, "background")
  leg_box_border <- go(leg_box_border, "highlight")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )
  pal <- go(pal, "pal_seq", "Mint")


  xout <- x
  var2 <- var[2]
  var1 <- var[1]

  # Transform to point
  st_geometry(x) <- st_centroid(st_geometry(x), of_largest_polygon = TRUE)

  # jenks
  jen <- FALSE
  if (any(breaks %in% "jenks")) {
    jen <- TRUE
  }
  # get the breaks
  breaks <- mf_get_breaks(x = x[[var2]], nbreaks = nbreaks, breaks = breaks)
  nbreaks <- length(breaks) - 1
  # get the cols
  pal <- get_the_pal(pal = pal, nbreaks = nbreaks, alpha = alpha, rev = !rev)
  # get the color vector
  mycols <- get_col_vec(x = x[[var2]], breaks = breaks, pal = pal, jen = jen)

  no_data <- c(FALSE, FALSE)
  if (max(is.na(mycols)) == 1) {
    no_data[2] <- TRUE
  }
  mycols[is.na(mycols)] <- col_na

  # get modalities
  val_order <- get_modalities(
    x = x[[var1]],
    val_order = val_order
  )

  if (missing(pch)) {
    pchs <- c(21:25, 0:20, 32:127)
    pch <- pchs[seq_along(val_order)]
  }
  if (length(pch) != length(val_order)) {
    message(paste0(
      "The length of pch does not match the number of ",
      "modalities. The first pch is used for all modalities."
    ))
    pch <- rep(pch[1], length(val_order))
  }
  if (length(cex) != length(val_order)) {
    if (length(cex) != 1) {
      message(paste0(
        "The length of cex does not match the number of ",
        "modalities. The first cex is used for all modalities."
      ))
    }
    cex <- rep(cex[1], length(val_order))
  }

  # get symbol list and association
  mysym <- get_sym_typo(
    x = x[[var1]], pch = pch,
    val_order = val_order
  )

  mycex <- get_sym_typo(
    x = x[[var1]], pch = cex,
    val_order = val_order
  )

  if (max(is.na(mysym)) == 1) {
    no_data[1] <- TRUE
  }
  mysym[is.na(mysym)] <- pch_na
  mycex[is.na(mycex)] <- cex_na

  border <- border[[1]]
  lwd <- lwd[[1]]

  mycolspt <- mycols
  mycolspt[mysym %in% 21:25] <- border
  mycolsptbg <- mycols

  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  plot(st_geometry(x),
    col = mycolspt, bg = mycolsptbg, cex = mycex, pch = mysym,
    lwd = lwd, add = TRUE
  )

  leg_pos <- split_leg(leg_pos)
  if (is.null(getOption("mapsf.legacy"))) {
    ccol <- getOption("mapsf.foreground")
  } else {
    ccol <- "grey80"
  }
  if (all(leg_frame, !leg_horiz, is.null(getOption("mapsf.legacy")))) {
    ccol <- getOption("mapsf.highlight")
  }
  border <- rep(border, sum(pch %in% 21:25))

  if (length(leg_pos) == 1) {
    leg_title <- test2args(leg_title)
    leg_no_data <- test2args(leg_no_data)

    la1 <- list(
      type = "symb",
      val = val_order,
      title = leg_title[1],
      col_na = ccol,
      no_data = no_data[1],
      no_data_txt = leg_no_data[1],
      pal = rep(ccol, length(val_order)),
      border = border,
      cex = cex,
      pch = pch,
      lwd = lwd,
      cex_na = cex_na,
      pch_na = pch_na
    )
    lg <- do.call(leg_comp, la1)
    la2 <- list(
      leg = lg,
      type = "choro",
      val = breaks,
      title = leg_title[2],
      val_rnd = leg_val_rnd,
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      col_na = col_na,
      no_data = no_data[2],
      no_data_txt = leg_no_data[2],
      pal = pal,
      horiz = leg_horiz,
      box_border = leg_box_border,
      box_cex = leg_box_cex
    )
    lg <- do.call(leg_comp, la2)
    leg_draw(lg,
      pos = leg_pos[[1]], bg = leg_bg, fg = leg_fg, size = leg_size,
      frame = leg_frame[1], title_cex = leg_title_cex[1],
      val_cex = leg_val_cex[1], mar = getOption("mapsf.mar"),
      adj = leg_adj, frame_border = leg_frame_border
    )
  } else {
    message(paste0(
      "The use of separated legends for this map type is deprecated.\n",
      "Please, use only one value for leg_pos",
      " or use mf_legend() to display two legends."
    ))

    leg(
      type = "symb",
      pos = leg_pos[[1]],
      val = val_order,
      title = leg_title[1],
      title_cex = leg_title_cex[1],
      val_cex = leg_val_cex[1],
      col_na = "grey",
      no_data = no_data[1],
      no_data_txt = leg_no_data[1],
      frame = leg_frame[1],
      border = border,
      pal = rep("grey", length(val_order)),
      cex = cex,
      pch = pch,
      cex_na = cex_na,
      pch_na = pch_na,
      lwd = lwd,
      bg = leg_bg,
      fg = leg_fg,
      size = leg_size,
      frame_border = leg_frame_border
    )
    leg(
      type = "choro",
      pos = leg_pos[[2]],
      val = breaks,
      title = leg_title[2],
      title_cex = leg_title_cex[2],
      val_cex = leg_val_cex[2],
      val_rnd = leg_val_rnd,
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      col_na = col_na,
      no_data = no_data[2],
      no_data_txt = leg_no_data[2],
      frame = leg_frame[2],
      frame_border = leg_frame_border,
      box_cex = leg_box_cex,
      box_border = leg_box_border,
      pal = pal,
      bg = leg_bg,
      fg = leg_fg,
      horiz = leg_horiz
    )
  }
  return(invisible(xout))
}

#' @title Deprecated - Plot graduated symbols
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "grad"`
#' instead.
#'
#' Plot graduated symbols based on quantitative data.
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'border',
#' 'lwd',
#' 'cexs',
#' 'pch',
#' 'add' ,
#' 'col',
#' 'alpha',
#' 'leg_pos',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'leg_val_rnd',
#' 'leg_val_dec',
#' 'leg_val_big',
#' 'leg_frame',
#' 'breaks',
#' 'nbreaks',
#' 'leg_adj',
#' 'leg_size',
#' 'leg_box_cex',
#' 'leg_fg',
#' 'leg_bg',
#' 'leg_frame_border'))
#' @details
#' Breaks defined by a numeric vector or a classification method are
#' left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\[, \[5 - 10\[, \[10 - 15\[, \[15 - 20\].
#' The "jenks" method is an exception and has to be right-closed.
#' Jenks breaks computed as \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\], \]5 - 10\], \]10 - 15\], \]15 - 20\].
#' @keywords internal
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_map(mtq, "POP", "grad", pch = 22)
mf_grad <- function(x,
                    var,
                    breaks = "quantile",
                    nbreaks = 3,
                    col,
                    alpha = NULL,
                    border,
                    pch = 21,
                    cex,
                    lwd,
                    extent = x,
                    bg,
                    expandBB = rep(.04, 4),
                    leg_pos = mf_get_leg_pos(x),
                    leg_title = var,
                    leg_title_cex = .8,
                    leg_val_cex = .6,
                    leg_val_rnd = 0,
                    leg_val_dec = ".",
                    leg_val_big = "",
                    leg_frame = FALSE,
                    leg_adj = c(0, 0),
                    leg_size = 1,
                    leg_box_cex = c(1, 1),
                    leg_fg,
                    leg_bg,
                    leg_frame_border,
                    add = TRUE) {
  deprecate_direct_calls_to("mf_grad")

  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))
  xout <- x

  bgc <- go(bg, "background")
  col <- go(col, "highlight", "tomato4")
  border <- go(border, "background")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )

  if (!is.null(alpha)) {
    col <- get_hex_pal(col, alpha)
  }

  # data prep
  x <- x[!is.na(x = x[[var]]), ]
  x <- x[order(x[[var]], decreasing = TRUE), ]
  # jenks
  jen <- FALSE
  if (any(breaks %in% "jenks")) {
    jen <- TRUE
  }
  breaks <- mf_get_breaks(x = x[[var]], nbreaks = nbreaks, breaks = breaks)
  nbreaks <- length(breaks) - 1

  xtype <- get_geom_type(x)

  if (xtype == "LINE") {
    # lwd mgmt
    if (missing(lwd)) {
      lwd <- seq(1, 4, length.out = nbreaks)
    }
    if (length(lwd) != nbreaks) {
      stop("The length of lwd does not match the number of breaks.",
        call. = FALSE
      )
    }
    mylwd <- get_col_vec(
      x = x[[var]], breaks = breaks, pal = lwd, jen = jen
    )

    if (add == FALSE) {
      mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
    }
    # map
    plot(st_geometry(x), col = col, lwd = mylwd, add = TRUE)
    # legend
    leg(
      type = "grad_line",
      pos = leg_pos,
      val = breaks,
      title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex,
      val_rnd = leg_val_rnd,
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      lwd = lwd,
      col = col,
      bg = leg_bg,
      fg = leg_fg,
      size = leg_size,
      frame = leg_frame,
      box_cex = leg_box_cex,
      frame_border = leg_frame_border,
      adj = leg_adj
    )
    return(invisible(xout))
  }

  if (missing(lwd)) lwd <- .7

  # Transform to point
  st_geometry(x) <- st_centroid(st_geometry(x), of_largest_polygon = TRUE)
  # cex mgmt
  if (missing(cex)) {
    cex <- seq(1, 4, length.out = nbreaks)
  }
  if (length(cex) != nbreaks) {
    stop("The length of cex does not match the number of breaks.",
      call. = FALSE
    )
  }
  mycex <- get_col_vec(
    x = x[[var]], breaks = breaks, pal = cex, jen = jen
  )
  # color mgmt
  pch <- pch[1]
  mycolspt <- col[1]
  if (pch %in% 21:25) mycolspt <- border
  mycolsptbg <- col[1]


  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }
  # display
  plot(st_geometry(x),
    col = mycolspt, bg = mycolsptbg, cex = mycex, pch = pch,
    lwd = lwd, add = TRUE
  )
  # legend
  bks <- get_val_rnd(breaks, leg_val_rnd, leg_val_dec, leg_val_big)
  lb <- length(bks)
  lab <- paste0(bks[1:(lb - 1)], rep(" - ", lb - 1), bks[2:lb])
  leg(
    type = "symb",
    pos = leg_pos,
    val = rev(lab),
    title = leg_title,
    title_cex = leg_title_cex,
    val_cex = leg_val_cex,
    frame = leg_frame,
    border = rep(border, length(lab)),
    pal = col,
    lwd = lwd,
    cex = rev(cex),
    pch = rep(pch, lb - 1),
    bg = leg_bg,
    fg = leg_fg,
    size = leg_size,
    frame_border = leg_frame_border,
    adj = leg_adj
  )
  return(invisible(xout))
}

#' @title Get color palettes
#' @description `mf_get_pal` builds sequential, diverging and
#' qualitative color palettes.
#' Diverging color palettes can be dissymmetric (different number of colors in
#' each of the two gradients).
#' @name mf_get_pal
#' @param n the number of colors (>= 1) to be in the palette
#' @param palette a valid palette name. See [hcl.pals] to get available
#' palette names. The name is matched
#' to the list of available palettes, ignoring upper vs. lower case, spaces,
#' dashes, etc. in the matching.
#' @param alpha opacity, in the range \[0,1\] (0 means
#' transparent and 1 means opaque). Default is set to 1.
#' @param rev	logical indicating whether the ordering of the colors should be
#' reversed
#' @param breaks a vector of class limit
#' @param mid a numeric value use to divide the palette in two colors
#' @param neutral a color, if two gradients are used, the 'neutral' color can be
#' added between them
#' @return A vector of colors.
#' @export
#' @examples
#' cls <- mf_get_pal(n = c(3, 7), palette = c("Reds 2", "Greens"))
#' plot(1:10, rep(1, 10), bg = cls, pch = 22, cex = 4)
#' mtq <- mf_get_mtq()
#' bks <- mf_get_breaks(mtq$MED, breaks = "equal", nbreaks = 8)
#' pal <- mf_get_pal(
#'   breaks = bks, mid = 15000,
#'   palette = c("Dark Mint", "Burg"), neutral = "grey90"
#' )
#' mf_map(mtq, "MED", "choro", breaks = bks, pal = pal)
#' pal <- mf_get_pal(breaks = bks, mid = bks[4], palette = c("Dark Mint", "Burg"))
#' mf_map(mtq, "MED", "choro", breaks = bks, pal = pal)
mf_get_pal <- function(n, palette, alpha = NULL,
                       rev = c(FALSE, FALSE), neutral,
                       breaks, mid) {
  if (missing(n) && !missing(breaks) && !missing(mid)) {
    pal <- mf_get_divpal(breaks, mid,
      palette = palette, neutral = neutral,
      alpha = alpha, rev = rev
    )
  } else {
    if (length(n) == 1) {
      pal <- hcl.colors(
        n = n, palette = palette,
        alpha = alpha, rev = rev[1]
      )
    }
    if (length(n) == 2) {
      nmax <- max(n)
      if (length(alpha) == 1) {
        alpha <- c(alpha, alpha)
      }
      pal1 <- hcl.colors(
        n = nmax, palette = palette[1],
        alpha = alpha[1], rev = rev[1]
      )
      pal2 <- hcl.colors(
        n = nmax, palette = palette[2],
        alpha = alpha[2], rev = rev[2]
      )

      if (n[1] < nmax) {
        pal1 <- pal1[(nmax - n[1] + 1):nmax]
      } else {
        pal1 <- pal1[1:nmax]
      }
      pal2 <- rev(pal2)
      pal2 <- pal2[1:n[2]]
      pal <- c(pal1, pal2)
      if (!missing(neutral)) {
        if (!is.null(alpha)) {
          neutral <- get_hex_pal(neutral, alpha[1])
        }
        pal <- c(pal1, neutral, pal2)
      }
    }
  }
  return(pal)
}


mf_get_divpal <- function(bks, mid, palette,
                          neutral,
                          alpha = NULL,
                          rev = c(FALSE, FALSE)) {
  if (missing(neutral)) {
    neutral <- "grey80"
  }
  if (length(palette) != 2) {
    stop("2 palette names must be provided.", call. = FALSE)
  }
  if (is.null(alpha)) {
    alpha <- 1
  }
  if (length(alpha) == 1) {
    alpha <- c(alpha, alpha)
  }
  bks <- sort(unique(bks))
  ncol <- length(bks) - 1
  if (mid %in% bks) {
    neg <- which(bks == mid) - 1
    pos <- ncol - neg
    if (neg == 0) {
      pal <- mf_get_pal(
        n = pos,
        palette = palette[2],
        alpha = alpha[2],
        rev = rev[2]
      )
    } else if (pos == 0) {
      pal <- mf_get_pal(
        n = neg,
        palette = palette[1],
        alpha = alpha[1],
        rev = rev[1]
      )
    } else {
      pal <- mf_get_pal(
        n = c(neg, pos),
        palette = palette,
        alpha = alpha,
        rev = rev
      )
    }
  } else {
    bks <- sort(c(bks, mid))
    bks_p <- bks[bks > mid]
    bks_n <- bks[bks < mid]
    lbp <- length(bks_p)
    lbn <- length(bks_n)
    if (lbp > 1 && lbn > 1) {
      pal <- mf_get_pal(
        n = c(lbn, lbp) - 1,
        palette = palette,
        alpha = alpha,
        rev = rev,
        neutral = neutral
      )
    } else if (lbn == 0) {
      pal <- mf_get_pal(
        n = lbp - 1,
        palette = palette[2],
        alpha = alpha,
        rev = rev
      )
    } else if (lbp == 0) {
      pal <- mf_get_pal(
        n = lbn - 1,
        palette = palette[1],
        alpha = alpha,
        rev = rev
      )
    } else if (lbn == 1) {
      pal <- c(
        get_hex_pal(neutral, alpha[1]),
        mf_get_pal(
          n = lbp - 1,
          palette = palette[2],
          alpha = alpha,
          rev = rev
        )
      )
    } else if (lbp == 1) {
      pal <- c(
        mf_get_pal(
          n = lbn - 1,
          palette = palette[1],
          alpha = alpha,
          rev = rev
        ),
        get_hex_pal(neutral, alpha[1])
      )
    }
  }
  return(pal)
}

## mf_get_pal
expect_silent(mf_get_pal(n = 5, palette = "Dynamic"))
expect_silent(mf_get_pal(n = c(6, 5), palette = c("Mint", "Reds 2")))
expect_silent(mf_get_pal(n = c(6, 5), palette = c("Mint", "Reds 2"),
                         neutral = "grey"))
expect_equal(mf_get_pal(breaks = 1:10, mid = 1, palette = c("Mint", "Viridis")),
             mf_get_pal(9, "Viridis", alpha = 1))
expect_equal(mf_get_pal(breaks = 1:10, mid = -1, palette = c("Mint", "Viridis")),
             mf_get_pal(9, "Viridis", alpha = 1))
expect_equal(mf_get_pal(breaks = 1:10, mid = 10, palette = c("Mint", "Viridis")),
             mf_get_pal(9, "Mint", alpha = 1))
expect_equal(mf_get_pal(breaks = 1:10, mid = 11, palette = c("Mint", "Viridis")),
             mf_get_pal(9, "Mint", alpha = 1))
expect_equal(mf_get_pal(breaks = 1:10, mid = 3, palette = c("Mint", "Viridis")),
             mf_get_pal(c(2, 7), c("Mint", "Viridis"), alpha = 1))
expect_error(mf_get_pal(breaks = 1:10, mid = 3, palette = c("Mint")))
expect_equal(mf_get_pal(breaks = 1:10, mid = 3.5, palette = c("Mint", "Viridis")),
             mf_get_pal(c(2, 6), c("Mint", "Viridis"), alpha = 1, neutral = "grey80"))
expect_equal(mf_get_pal(breaks = 1:10, mid = 1.5, palette = c("Mint", "Viridis")),
             c("#CCCCCCFF", mf_get_pal(8, "Viridis", alpha = 1)))
expect_equal(mf_get_pal(breaks = 1:10, mid = 9.5, palette = c("Mint", "Viridis")),
             c(mf_get_pal(8, "Mint", alpha = 1), "#CCCCCCFF"))

#' @title Plot labels
#' @description Put labels on a map.
#' @name mf_label
#' @param x	object of class `sf`
#' @param var name of the variable to map
#' @param col labels color, it can be a single color or a vector of colors
#' @param cex labels cex, it can be a single size or a vector of sizes
#' @param ... further \link{text} arguments.
#' @param bg halo color, it can be a single color or a vector of colors
#' @param r width of the halo, it can be a single value or a vector of values
#' @param overlap if FALSE, labels are moved so they do not overlap.
#' @param halo if TRUE, a 'halo' is displayed around the text and additional
#' arguments bg and r can be modified to set the color and width of the halo.
#' @param lines if TRUE, then lines are plotted between x,y and the word,
#' for those words not covering their x,y coordinate
#' @param q quality of the non overlapping labels placement. Possible values
#' are 0 (quick results), 1 (reasonable quality and speed), 2 (better quality),
#' 3 (insane quality, can take a lot of time).
#' @return No return value, labels are displayed.
#' @export
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mtq$cex <- c(rep(.8, 8), 2, rep(.8, 25))
#' mf_label(
#'   x = mtq, var = "LIBGEO",
#'   col = "grey10", halo = TRUE, cex = mtq$cex,
#'   overlap = FALSE, lines = FALSE
#' )
mf_label <- function(x,
                     var,
                     col,
                     cex = 0.7,
                     overlap = TRUE,
                     lines = TRUE,
                     halo = FALSE,
                     bg,
                     r = 0.1,
                     q = 1,
                     ...) {
  test_cur_plot()
  # margins mgmt
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  col <- go(col, "highlight")
  bg <- go(bg, "background")


  words <- x[[var]]
  cc <- st_coordinates(st_centroid(
    x = st_geometry(x),
    of_largest_polygon = TRUE
  ))

  if (nrow(x) == 1) {
    overlap <- TRUE
  }

  if (!overlap) {
    xo <- unlist(cc[, 1])
    yo <- unlist(cc[, 2])
    lay <- wordlayout(xo, yo, words, cex, q = q)
    if (lines) {
      nlab <- length(xo)
      if (length(col) != nlab) {
        col <- rep(col[1], nlab)
      }
      for (i in seq_along(xo)) {
        xl <- lay[i, 1]
        yl <- lay[i, 2]
        w <- lay[i, 3]
        h <- lay[i, 4]
        if (xo[i] < xl || xo[i] > xl + w ||
          yo[i] < yl || yo[i] > yl + h) {
          points(xo[i], yo[i], pch = 16, col = col[i], cex = .5)
          nx <- xl + .5 * w
          ny <- yl + .5 * h
          lines(c(xo[i], nx), c(yo[i], ny), col = col[i], lwd = 1)
        }
      }
    }
    cc <- matrix(
      data = c(lay[, 1] + .5 * lay[, 3], lay[, 2] + .5 * lay[, 4]),
      ncol = 2, byrow = FALSE
    )
  }
  if (halo) {
    shadowtext(
      x = cc[, 1], y = cc[, 2], labels = words,
      cex = cex, col = col, bg = bg, r = r, ...
    )
  } else {
    text(x = cc[, 1], y = cc[, 2], labels = words, cex = cex, col = col, ...)
  }
  return(invisible(x))
}

#' @title Plot credits
#' @description Plot credits (sources, author, year...).
#' @name mf_credits
#' @param col color of the text, hex code or color name given by [colors].
#' The default color is the highlight color (see [mf_theme]).
#' @param pos position, one of 'bottomleft', 'bottomright' or 'rightbottom'
#' @param txt text of the credits, use '\\n' to add line breaks
#' @param cex cex of the credits
#' @param font font of the credits
#' @param bg background color
#' @export
#' @return No return value, credits are displayed.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_credits(txt = "Author\nSources - Year")
mf_credits <- function(txt = "Source(s) & Author(s)",
                       pos = "bottomleft",
                       col,
                       cex = .6,
                       font = 3,
                       bg = NA) {
  test_cur_plot()

  col <- go(col, "highlight")


  recordGraphics(
    {
      mf_credits_display(txt, pos, col, cex, font, bg)
    },
    list = list(
      txt = txt,
      pos = pos,
      col = col,
      cex = cex,
      font = font,
      bg = bg
    ),
    env = getNamespace("mapsf")
  )
}


mf_credits_display <- function(txt = "Source(s) & Author(s)",
                               pos = "bottomleft",
                               col,
                               cex = .6,
                               font = 3,
                               bg = NA) {
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))
  xyi <- xyinch(1)
  ratio <- xyi[2] / xyi[1]


  pd <- par("usr")
  pdp_x <- xinch(par("csi")) / 4
  pdp_y <- yinch(par("csi")) / 4


  w <- strwidth(s = txt, units = "user", cex = cex, font = font)
  h <- strheight(s = txt, units = "user", cex = cex, font = font)

  ppos <- switch(pos,
    bottomleft = {
      xr <- pd[1] + w + 2 * pdp_x
      xl <- pd[1]
      yb <- pd[3]
      yt <- pd[3] + h + 2 * pdp_y
      pd[1] <- pd[1] + pdp_x
      pd[3] <- pd[3] + pdp_y
      list(
        pd = pd, adj = c(0, 0), srt = 0,
        xr = xr, xl = xl, yb = yb, yt = yt
      )
    },
    bottomright = {
      xr <- pd[2]
      xl <- pd[2] - w - 2 * pdp_x
      yb <- pd[3]
      yt <- pd[3] + h + 2 * pdp_y
      pd[1] <- pd[2] - pdp_x
      pd[3] <- pd[3] + pdp_y
      list(
        pd = pd, adj = c(1, 0), srt = 0,
        xr = xr, xl = xl, yb = yb, yt = yt
      )
    },
    rightbottom = {
      xr <- pd[2]
      xl <- pd[2] - h / ratio - 2 * pdp_x
      yb <- pd[3]
      yt <- pd[3] + w * ratio + 2 * pdp_y
      pd[1] <- pd[2] - pdp_x
      pd[3] <- pd[3] + pdp_y
      list(
        pd = pd, adj = c(0, 0), srt = 90,
        xr = xr, xl = xl, yb = yb, yt = yt
      )
    }
  )


  rect(
    xleft = ppos$xl, ybottom = ppos$yb, xright = ppos$xr,
    ytop = ppos$yt, col = bg, border = NA, xpd = TRUE
  )

  text(
    x = ppos$pd[1], y = ppos$pd[3], labels = txt,
    cex = cex, xpd = TRUE, adj = ppos$adj,
    col = col, srt = ppos$srt, font = font
  )
}

#' @title Plot a logo on a map
#'
#' @description
#' The logo can be a PNG or JPG/JPEG file.
#'
#' @param filename filename of the logo image, PNG or JPG/JPEG format.
#' @param pos position of the logo, one of 'topleft', 'top','topright',
#' 'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two
#' coordinates in map units (c(x, y)). Use 'interactive' to choose the
#' legend position by clicking on the map.
#' @param cex amount by which the logo width should be magnified
#' or reduced relative to the default
#' @param adj adjust the position of the logo in x and y directions
#' @param resize if FALSE, the logo is displayed at its original size in pixels
#' and `cex` is not used.
#' @return No return value, a logo is displayed.
#' @export
#'
#' @examples
#' m <- mf_get_mtq()
#' mf_map(m)
#' mf_scale()
#' logo <- system.file("img", "Rlogo.png", package = "png")
#' mf_logo(logo, pos = "bottomleft", adj = c(0, 4))
#' mf_credits()
mf_logo <- function(filename, pos = "bottomright", cex = 1, adj = c(0, 0),
                    resize = TRUE) {
  test_cur_plot()
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  logo <- readimage(filename)

  if (length(pos) == 1 && pos == "interactive") {
    pos <- interleg(txt = c("logo", "Logo"))
  }

  recordGraphics(
    {
      pu <- par("usr")
      pp <- dim(logo)[2:1]
      if (resize == FALSE) {
        dev_px <- dev.size("px")
        dev_in <- dev.size("in")
        pp <- c(
          xinch(pp[1] * dev_in[1] / dev_px[1]),
          yinch(pp[2] * dev_in[2] / dev_px[2])
        )
      } else {
        pp <- pp * (xinch(1) / pp[1]) * cex
        pp[2] <- pp[2] * yinch(1) / xinch(1)
      }
      xy <- posinset(pos, pu, pp[1], pp[2], adj = adj)
      rasterImage(
        image   = logo,
        xleft   = xy[1],
        ybottom = xy[3],
        xright  = xy[2],
        ytop    = xy[4]
      )
    },
    list = list(pos = pos, logo = logo, cex = cex, adj = adj, resize = resize),
    env = getNamespace("mapsf")
  )
}

mf_map(mtq)
expect_silent(mf_logo(system.file("img/background.jpg",
                                        package = "mapsf")))
expect_silent(mf_logo(system.file("help/figures/logo.png",
                                        package = "mapsf"),
                      pos = "topright",
                      adj = c(-10,-10), cex = 1))
expect_silent(mf_logo(system.file("help/figures/logo.png",
                                  package = "mapsf"),
                      pos = c(690000, 1625355.65627547),
                      cex = 2.5))
expect_silent(mf_logo(system.file("help/figures/logo.png",
                                  package = "mapsf"),
                      pos = c(690000, 1625355.65627547),
                      resize = FALSE))
expect_error(mf_logo(system.file("help/figures/logo.png",
                                  package = "mapsf"),
                      pos = "toplaft"))

mf_map(mtq)
expect_silent(mf_logo(system.file("img/background.jpg",
                                  package = "mapsf")))
expect_silent(mf_logo(system.file("help/figures/logo.png",
                                  package = "mapsf"),
                      pos = "topright",
                      adj = c(-10,-10), cex = 1))
expect_silent(mf_logo(system.file("help/figures/logo.png",
                                  package = "mapsf"),
                      pos = c(690000, 1625355.65627547),
                      cex = 2.5))
expect_silent(mf_logo(system.file("img/background.jpg",
                                  package = "mapsf"),
                      pos = c(690000, 1625355.65627547),
                      resize = FALSE))
expect_error(mapsf:::posinset("topleeft", pusr = graphics::par("usr"),
                              wdest = 500, hdest = 500, adj = c(0,0)))

#' @title Get a border layer from polygons
#' @description This function extracts borders between contiguous polygons.
#' @name mf_get_borders
#' @param x an sf object of POLYGONS, using a projected CRS
#' @note
#' If the polygon layer contains topology errors (such as contiguous
#' polygons not sharing exactly the same boundary) the function may not return
#' all boundaries correctly. It is possible to use `st_snap()` or other
#' functions to try and correct these errors.
#' @return An sf object (MULTILINESTRING) of borders is returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mtq_b <- mf_get_borders(mtq)
#' mf_map(mtq)
#' mf_map(mtq_b, col = 1:5, lwd = 4, add = TRUE)
#' @export
mf_get_borders <- function(x) {
  if (get_geom_type(x) != "POLYGON") {
    stop(paste0('"x" should be a POLYGON sf object'), call. = FALSE)
  }
  if (st_is_longlat(x)) {
    stop("This feature does not work on unprojected (long/lat) layers.",
      call. = FALSE
    )
  }
  oag <- st_agr(x)
  x <- st_set_agr(x, "constant")
  border <- st_collection_extract(st_intersection(x, x), "LINESTRING")
  border <- st_set_agr(border, c(oag, oag))
  return(border)
}

expect_silent(mf_get_borders(mtq))
expect_error(mf_get_borders(st_transform(mtq, "EPSG:4326")))
expect_error(mf_get_borders(mtq_p))
x <- mf_get_borders(mtq)
expect_equal(nrow(x), 142)
expect_inherits(sf::st_geometry(x), "sfc_MULTILINESTRING")

#' @title Plot a map
#' @description
#' `mf_map()` is the main function of the package, it displays map layers on a
#' georeferenced plot.
#'
#' `mf_map()` has three main arguments:
#' * `x`, an sf object,
#' * `var`, the name(s) of a variable(s) to map,
#' * `type`, the map type.
#'
#' Relevant arguments and default values are different for each map type and are
#' described in dedicated help pages (see [base][mf_map_base],
#' [choro][mf_map_choro], [typo][mf_map_typo], [prop][mf_map_prop],
#' [prop_choro][mf_map_prop_choro], [prop_typo][mf_map_prop_typo],
#' [symb][mf_map_symb], [grad][mf_map_grad] or [symb_choro][mf_map_symb_choro]).
#' @family map types
#' @usage
#' mf_map(x, var, type = "base",
#'        breaks, nbreaks, pal, alpha, rev, inches, val_max, symbol, col,
#'        lwd_max, val_order, pch, cex, border, lwd, col_na, cex_na, pch_na,
#'        expandBB, extent, bg, add,
#'        leg_pos, leg_title, leg_title_cex, leg_val_cex, leg_val_rnd,
#'        leg_val_dec, leg_val_big, leg_no_data, leg_frame, leg_frame_border,
#'        leg_horiz, leg_adj, leg_bg, leg_fg, leg_size,
#'        leg_box_border, leg_box_cex, ...)
#' @param x object of class `sf`
#' @param var name(s) of the variable(s) to map
#' @param type
#' * **[base][mf_map_base]**: base maps
#' * **[choro][mf_map_choro]**: choropleth maps
#' * **[typo][mf_map_typo]**: typology maps
#' * **[prop][mf_map_prop]**: proportional symbols maps
#' * **[prop_choro][mf_map_prop_choro]**: proportional symbols with choropleth
#' coloration
#' * **[prop_typo][mf_map_prop_typo]**: proportional symbols with typology
#' coloration
#' * **[symb][mf_map_symb]**: symbols maps
#' * **[grad][mf_map_grad]**: graduated symbols maps
#' * **[symb_choro][mf_map_symb_choro]**: symbols with choropleth coloration
#' @param breaks	either a numeric vector with the actual breaks, or a
#' classification method name. The main methods are 'quantile',
#' 'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
#' See [mf_get_breaks] for details.
#' @param nbreaks number of classes
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals].
#' @param alpha `col` or `pal` opacity, in the range \[0,1\] (0 means
#' transparent and 1 means opaque). Default is set to 1.
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param inches size of the largest symbol in inches (radius for circles,
#' half width for squares)
#' @param val_max maximum value corresponding to the largest symbol or line
#' @param lwd_max width of the largest line
#' @param val_order modalities order in the legend, a character vector that
#' matches `var` modalities
#' @param symbol type of proportional symbols, either "circle" or "square"
#' @param border border color for polygons or symbols. It can be a hex
#' code or a color name given by [colors].
#' @param lwd border width of polygons, symbols or lines
#' @param pch type of symbol to use for points, see [pch]
#' @param cex symbols size, 2 means 2 times bigger
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param extent object with an `st_bbox` method to define plot extent;
#' defaults to `x`. `extent` and `x` must use the same CRS.
#' @param bg background color of the map, hex code or color name given by
#' [colors], ignored if `add = TRUE`
#' @param add whether to add the layer to an existing plot (TRUE) or not (FALSE)
#' @param col a color, hex code or color name given by [colors]
#' @param pch_na symbol to use for missing values on points, see [pch]
#' @param cex_na symbols size for missing values on points
#' @param col_na color for missing values, a hex code or
#' a color name given by [colors].
#' @param leg_pos position of the legend, one of 'topleft', 'top','topright',
#' 'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two
#' coordinates in map units (c(x, y)). Use `NA` to avoid plotting the legend,
#' use 'interactive' to choose the legend position by clicking on the map.
#' @param leg_title legend title
#' @param leg_title_cex size of the title
#' @param leg_val_cex size of the values
#' @param leg_val_rnd number of decimal places of the values displayed in the
#' legend
#' @param leg_val_dec	decimal separator
#' @param leg_val_big	thousands separator
#' @param leg_no_data label for missing values
#' @param leg_frame	whether to add a frame to the legend (TRUE) or not (FALSE)
#' @param leg_frame_border border color of the legend frame
#' @param leg_horiz	display the legend horizontally (for proportional symbols
#' and choropleth types)
#' @param leg_adj	adjust the position of the legend in x and y directions
#' @param leg_bg color of the legend background
#' @param leg_fg	color of the legend foreground
#' @param leg_size size of the legend. Combine this argument with
#' `leg_title_cex` and `leg_val_cex`.
#' @param leg_box_border border color of legend boxes (for types related to
#' choropleth and typology)
#' @param leg_box_cex	width and height size expansion of boxes
#' @param ... ignored
#'
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' # basic examples
#' # type = "base"
#' mf_map(mtq)
#' # type = "prop"
#' mf_map(mtq)
#' mf_map(mtq, var = "POP", type = "prop")
#' # type = "choro"
#' mf_map(mtq, var = "MED", type = "choro")
#' # type = "typo"
#' mf_map(mtq, "STATUS", "typo")
#' # type = "symb"
#' mf_map(mtq)
#' mf_map(mtq, "STATUS", "symb")
#' # type = "grad"
#' mf_map(mtq)
#' mf_map(mtq, var = "POP", type = "grad")
#' # type = "prop_choro"
#' mf_map(mtq)
#' mf_map(mtq, var = c("POP", "MED"), type = "prop_choro")
#' # type = "prop_typo"
#' mf_map(mtq)
#' mf_map(mtq, var = c("POP", "STATUS"), type = "prop_typo")
#' # type = "symb_choro
#' mf_map(mtq)
#' mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro")
mf_map <- function(x,
                   var,
                   type = "base",
                   breaks,
                   nbreaks,
                   pal,
                   alpha,
                   rev,
                   inches,
                   val_max,
                   symbol,
                   col,
                   lwd_max,
                   val_order,
                   pch,
                   cex,
                   border,
                   lwd,
                   col_na,
                   cex_na,
                   pch_na,
                   expandBB,
                   extent,
                   bg,
                   add,
                   leg_pos,
                   leg_title,
                   leg_title_cex,
                   leg_val_cex,
                   leg_val_rnd,
                   leg_val_dec,
                   leg_val_big,
                   leg_no_data,
                   leg_frame,
                   leg_frame_border,
                   leg_horiz,
                   leg_adj,
                   leg_bg,
                   leg_fg,
                   leg_size,
                   leg_box_border,
                   leg_box_cex,
                   ...) {
  # check args
  if (!type %in% c(
    "base", "prop", "choro", "typo", "symb", "grad",
    "prop_choro", "prop_typo", "symb_choro"
  )) {
    stop(
      paste0(
        '\'type\' should be one of "base", "prop", "choro", "typo", ',
        '"symb", "grad", "prop_choro", "prop_typo" or "symb_choro".'
      ),
      call. = FALSE
    )
  }

  cl <- inherits(x = x, what = c("sf", "sfc", "sfg"), which = TRUE) != 0
  if (cl[1] == FALSE && cl[2] == TRUE && type != "base") {
    stop(paste0("'x' should be an sf object."), call. = FALSE)
  }
  if (cl[1] == FALSE && cl[2] == FALSE && cl[3] == FALSE) {
    stop(paste0("'x' should be an sf, sfc or sfg object."), call. = FALSE)
  }

  if (!missing(var)) {
    if (type == "base") {
      message("Please use the 'type' argument to map variables.")
    } else {
      lv <- length(var)
      lin <- var %in% names(x)
      if (lv != length(lin[lin == TRUE])) {
        stop(
          paste0("It is likely that 'var' is not a valid variable name."),
          call. = FALSE
        )
      }
    }
  }

  # add mgmgt, set default add, do not add if no device is launch
  if (missing(add)) {
    add <- switch(type,
      prop = TRUE,
      choro = FALSE,
      typo = FALSE,
      symb = TRUE,
      base = FALSE,
      grad = TRUE,
      prop_choro = TRUE,
      prop_typo = TRUE,
      symb_choro = TRUE
    )
  }


  if (is.null(dev.list())) {
    add <- FALSE
  }

  argx <- as.list(match.call()[-1])
  argx <- argx[!names(argx) %in% c("type")]


  if (type != "base") {
    argx <- check_args(argx, type)
  }

  # enabling pipe without side effect
  argx$x <- eval(x)

  do.call(what = get(paste0("mf_", type)), argx, envir = parent.frame())

  return(invisible(x))
}

expect_silent(mf_map(mtq, c("POP", "MED"), type = "prop_choro", add = FALSE))
mf_map(mtq)
expect_silent(mf_map(mtq, c("POP", "MED"), type = "prop_choro"))
expect_message(mf_map(mtq, c("POP", "MED"), type = "prop_choro",
                            leg_pos = c(673434.5, 1642503.7,
                                        682727.1, 1598170.3)
))
expect_message(mf_map(mtq, c("POP", "MED"), type = "prop_choro",
                            leg_pos = c("topright",  682727.1, 1598170.3)
))


expect_message(mf_map(mtq, c("POP", "MED"), type = "prop_choro",
                            leg_pos = c(673434.5, 1642503.7, "top")
))

mf_map(mtq)
mtq[6, "MED"] <- NA
expect_silent(
  mf_map(
    x = mtq, var = c("POP", "MED"), type = "prop_choro",
    inches = .35, border = "tomato4",
    val_max = 90000, symbol = "circle", col_na = "grey", pal = "Cividis",
    breaks = "equal", nbreaks = 4, lwd = 4,
    leg_pos = c("bottomright"),
    leg_title = c("Population", "Median Income"),
    leg_title_cex = c(0.8),
    leg_val_cex = c(.7),
    leg_val_rnd = c(0, 0),
    leg_no_data = "No data",
    leg_frame = TRUE,
    add = TRUE
  )
)

expect_error(mf_map(x = mtq, var = c("POP", "MED"), type = "prop_choro",
                    leg_title = "Solo title"))

expect_message(mf_map(x = mtq_l, var = c("fij", "fij"), type = "prop_choro"))

mf_map(mtq)
expect_silent(mf_map(mtq, var = "POP", type = "prop"))
expect_silent(mf_map(mtq, var = "MED", type = "choro"))
expect_silent(mf_map(mtq, var = "STATUS", type = "typo"))
expect_silent(mf_map(mtq, var = "STATUS", type = "symb"))
expect_silent(mf_map(mtq, var = "POP", type = "grad"))
expect_silent(mf_map(mtq, var = c("POP", "MED"), type = "prop_choro"))
expect_silent(mf_map(mtq, var = c("POP", "STATUS"), type = "prop_typo"))
expect_silent(mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro"))
expect_message(mf_map(mtq, "POP"))
expect_error(mf_map(mtq, "POP", "props"))
expect_error(mf_map(mtq, "POPx", "prop"))
expect_error(mf_map("not an sf object"))
expect_error(mf_map(st_geometry(mtq), "POP", "prop"))
expect_message(mf_map(mtq, "POP", "prop", breaks = "q6"))
expect_message(mf_map(mtq, "POP", "prop", breaks = "q6", nbreaks = 8))


# test for alpha
expect_silent(mf_map(mtq, var = "POP", type = "prop", alpha = .2))
expect_silent(mf_map(mtq, var = "MED", type = "choro", alpha = .2))
expect_silent(mf_map(mtq, var = "STATUS", type = "typo", alpha = .2))
expect_silent(mf_map(mtq, var = "STATUS", type = "symb", alpha = .2))
expect_silent(mf_map(mtq, var = "POP", type = "grad", alpha = .2))
expect_silent(mf_map(mtq, var = c("POP", "MED"), type = "prop_choro", alpha = .2))
expect_silent(mf_map(mtq, var = c("POP", "STATUS"), type = "prop_typo", alpha = .2))
expect_silent(mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro", alpha = .2))

mf_map(mtq)
expect_silent(mf_map(x = mtq, var = "POP", type = "prop", inches = .4,
                     symbol = "circle",
                     val_max = 90000,
                     col = "tomato1", border = "blue", lwd = 1,
                     leg_pos = "right", leg_title = "Population",
                     leg_title_cex = 1, leg_val_cex = .8, leg_val_rnd = 0,
                     leg_frame = TRUE, add = TRUE))
expect_silent(mf_map(x = mtq, var = "POP", type = "prop", add = FALSE))
expect_silent(mf_map(x = mtq, var = "POP", type = "prop",
                     inches = .2, symbol = "circle",
                      val_max = 30000,
                      col = "tomato1", lwd = 1,
                      leg_pos = "right", leg_title = "Population",
                      leg_title_cex = 1, leg_val_cex = .8, leg_val_rnd = 0,
                      leg_frame = TRUE))

expect_silent(mf_map(x = mtq_l, var = "fij", type = "prop"))
expect_silent(mf_map(x = mtq_l, var = "fij", type = "prop", add = FALSE))
expect_silent(mf_map(x = mtq_l, var = "fij", type = "prop", val_max = 2000))



mtq$POP[1] <- 0
expect_message(mf_map(x = mtq, var = "POP", type = "prop", add = FALSE))
mtq$POP[2] <- NA
expect_message(mf_map(x = mtq, var = "POP", type = "prop", add = FALSE))
mtq$POP[3] <- Inf
expect_message(mf_map(x = mtq, var = "POP", type = "prop", add = FALSE))
mtq$POP[1:3] <- 0
expect_message(mf_map(x = mtq, var = "POP", type = "prop", add = FALSE))
mtq$POP[4:6] <- NA
expect_message(mf_map(x = mtq, var = "POP", type = "prop", add = FALSE))
mtq$POP[8:12] <- Inf
expect_message(mf_map(x = mtq, var = "POP", type = "prop", add = FALSE))
mtq$MED <- mtq$MED * -1
expect_message(mf_map(x = mtq, var = "MED", type = "prop", add = FALSE))
mtq$POP <- NA
suppressMessages(expect_error(mf_map(x = mtq, var = "POP", type = "prop",
                                     add = FALSE)))

mf_map(mtq)
expect_silent(mf_map(mtq_l, var = "fij", type = "grad"))
expect_silent(mf_map(mtq_l, var = "fij", type = "grad", add = FALSE))
expect_silent(mf_map(mtq, var = "POP", type = "grad", add = FALSE))
expect_error(mf_map(mtq_l, var = "fij", type = "grad", lwd = c(1, 2)))
expect_error(mf_map(mtq, var = "POP", type = "grad", cex = c(1, 2)))

a <- mf_get_pencil(mtq, buffer = 100)
b <- mf_get_pencil(mtq, buffer = -100, lefthanded = FALSE, clip = FALSE)
expect_true(methods::is(st_geometry(a), "sfc_MULTILINESTRING"))
expect_true(methods::is(st_geometry(b), "sfc_MULTILINESTRING"))
expect_error(mf_get_pencil(mtq, size = 10, buffer = 10000000, clip = T))
mtq$MED[1:3] <- NA
expect_silent(mf_map(mf_get_pencil(mtq,100), var = 'MED', type = 'choro'))

mf_map(mtq)
expect_silent(mf_scale(pos = c(713709.9, 1596117)))
expect_silent(mf_scale())
expect_silent(mf_scale(pos = "bottomleft", size = 5))

expect_error(mf_scale(crs_units = "pop"))
expect_error(mf_scale(scale_units = "pop"))

expect_silent(mf_scale(scale_units = "m"))
expect_silent(mf_scale(scale_units = "mi"))
expect_silent(mf_scale(scale_units = "ft"))

mtqx <- st_transform(mtq, 2264)
mf_map(mtqx)
expect_silent(mf_scale(crs_units = "ft", scale_units = "m"))
expect_silent(mf_scale(crs_units = "ft", scale_units = "km"))
expect_silent(mf_scale(crs_units = "ft", scale_units = "mi"))
expect_silent(mf_scale(crs_units = "ft", scale_units = "ft"))
expect_silent(mf_scale(x = mtqx, scale_units = "m"))

mtqy <- st_transform(mtq, 4326)
mf_map(mtqy)
expect_message(mf_scale(x = mtqy))
st_crs(mtqy) <- NA_crs_
expect_message(mf_scale(x = mtqy))

#' @title Deprecated - Plot a typology map
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "typo"`
#' instead.
#'
#'
#' Plot a typology map.
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'border',
#' 'lwd',
#' 'add' ,
#' 'col_na',
#' 'pal',
#' 'alpha',
#' 'rev',
#' 'leg_pos',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'val_order',
#' 'leg_no_data',
#' 'leg_frame',
#' 'leg_frame_border',
#' 'leg_size',
#' 'leg_box_border',
#' 'leg_box_cex',
#' 'leg_fg',
#' 'leg_bg',
#' 'leg_adj'))
#' @param cex cex cex of the symbols if x is a POINT layer
#' @param pch pch type of pch if x is a POINT layer
#' @keywords internal
#' @export
#' @return No return value, a map is displayed.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq, "STATUS", "typo")
#' mtq[6, "STATUS"] <- NA
#' mf_map(
#'   x = mtq, var = "STATUS", type = "typo",
#'   pal = c("red", "blue", "yellow"), lwd = 1.1,
#'   val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
#'   col_na = "green", border = "brown",
#'   leg_pos = "bottomleft",
#'   leg_title = "Status", leg_title_cex = 1.1,
#'   leg_val_cex = 1, leg_no_data = "No data",
#'   leg_frame = TRUE, add = FALSE
#' )
mf_typo <- function(x,
                    var,
                    pal,
                    alpha = NULL,
                    rev = FALSE,
                    val_order,
                    border,
                    pch = 21,
                    cex = 2,
                    lwd = .7,
                    col_na = "white",
                    extent = x,
                    bg,
                    expandBB = rep(.04, 4),
                    leg_pos = mf_get_leg_pos(x),
                    leg_title = var,
                    leg_title_cex = .8,
                    leg_val_cex = .6,
                    leg_no_data = "No data",
                    leg_frame = FALSE,
                    leg_frame_border,
                    leg_adj = c(0, 0),
                    leg_size = 1,
                    leg_box_border,
                    leg_box_cex = c(1, 1),
                    leg_fg,
                    leg_bg,
                    add = FALSE) {
  deprecate_direct_calls_to("mf_typo")
  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")
  pal <- go(pal, "pal_quali", "Dynamic")
  leg_box_border <- go(leg_box_border, "highlight")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )
  lwd <- lwd[[1]]
  cex <- cex[[1]]
  pch <- pch[[1]]

  # get modalities
  val_order <- get_modalities(
    x = x[[var]],
    val_order = val_order
  )

  # get color list and association
  pal <- get_the_pal(
    pal = pal, nbreaks = length(val_order),
    alpha = alpha, rev = !rev
  )
  # get color vector
  mycols <- get_col_typo(
    x = x[[var]], pal = pal,
    val_order = val_order
  )

  no_data <- FALSE
  if (max(is.na(mycols)) == 1) {
    no_data <- TRUE
  }
  mycols[is.na(mycols)] <- col_na

  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  xtype <- get_geom_type(x)
  if (xtype == "LINE") {
    plot(st_geometry(x), col = mycols, lwd = lwd, add = TRUE)
    leg(
      type = "typo",
      pos = leg_pos,
      val = val_order,
      title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex,
      col_na = col_na,
      no_data = no_data,
      no_data_txt = leg_no_data,
      box_border = NA,
      frame = leg_frame,
      frame_border = leg_frame_border,
      pal = pal,
      bg = leg_bg,
      fg = leg_fg,
      box_cex = c(1.2, .1),
      adj = leg_adj,
      size = leg_size,
    )
  }
  if (xtype == "POLYGON") {
    border <- go(border, opt = "highlight")
    plot(st_geometry(x),
      col = mycols, border = border,
      lwd = lwd, add = TRUE
    )
    leg(
      type = "typo",
      pos = leg_pos,
      val = val_order,
      title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex,
      col_na = col_na,
      no_data = no_data,
      no_data_txt = leg_no_data,
      size = leg_size,
      box_border = leg_box_border,
      box_cex = leg_box_cex,
      frame_border = leg_frame_border,
      frame = leg_frame,
      pal = pal,
      bg = leg_bg,
      fg = leg_fg,
      adj = leg_adj
    )
  }
  if (xtype == "POINT") {
    border <- go(border, "background")
    if (pch %in% 21:25) {
      mycolspt <- border
    } else {
      mycolspt <- mycols
    }
    mycolsptbg <- mycols
    plot(st_geometry(x),
      col = mycolspt, bg = mycolsptbg, cex = cex, pch = pch,
      lwd = lwd, add = TRUE
    )
    n <- length(val_order)
    leg(
      type = "symb",
      pos = leg_pos, val = val_order, title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex, col_na = col_na, no_data = no_data,
      no_data_txt = leg_no_data,
      frame = leg_frame, border = border, pal = pal, lwd = lwd,
      cex = rep(cex, n), pch = rep(pch, n), cex_na = cex,
      pch_na = pch, bg = leg_bg, fg = leg_fg, adj = leg_adj,
      size = leg_size, frame_border = leg_frame_border
    )
  }

  return(invisible(x))
}

#' @title Plot graticules
#' @description Display graticules and labels on a map.
#'
#' @param x object of class `sf`, `sfc` or `SpatRaster`
#' @param pos labels positions ("bottom", "left", "top" and / or "right")
#' @param cex labels size
#' @param col graticules and label color
#' @param lwd graticules line width
#' @param lty graticules line type
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param label whether to add labels (TRUE) or not (FALSE)
#' @param add whether to add the layer to an existing plot (TRUE) or
#' not (FALSE)
#' @param extent object with an `st_bbox` method to define plot extent;
#' defaults to `x`. `extent` and `x` must use the same CRS.
#' @param bg background color of the map, hex code or color name given by
#' [colors], ignored if `add = TRUE`
#' @section Use of graticules:
#' From [sf::st_graticule()]:
#' "In cartographic visualization, the use of graticules is not advised, unless
#' the graphical output will be used for measurement or navigation, or the
#' direction of North is important for the interpretation of the content, or
#' the content is intended to display distortions and artifacts created by
#' projection. Unnecessary use of graticules only adds visual clutter but
#' little relevant information. Use of coastlines, administrative boundaries
#' or place names permits most viewers of the output to orient themselves
#' better than a graticule."
#' @return An (invisible) layer of graticules is returned (LINESTRING).
#' @export
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq, expandBB = c(0, .1, .1, 0))
#' mf_graticule(mtq)
#'
#' mf_graticule(
#'   x = mtq,
#'   col = "coral4",
#'   lwd = 2,
#'   lty = 2,
#'   expandBB = c(.1, 0, 0, .1),
#'   label = TRUE,
#'   pos = c("right", "bottom"),
#'   cex = .8,
#'   add = FALSE
#' )
#' mf_map(mtq, add = TRUE)
mf_graticule <- function(x,
                         col,
                         lwd = 1,
                         lty = 1,
                         expandBB = rep(0, 4),
                         label = TRUE,
                         pos = c("top", "left"),
                         cex = .7,
                         extent = x,
                         bg,
                         add = TRUE) {
  bgc <- go(bg, "background")

  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  col <- go(col, "highlight")

  g <- st_graticule(st_as_sfc(st_bbox(x)))

  mf_map(
    g,
    col = col,
    lwd = lwd,
    lty = lty,
    add = TRUE
  )

  if (label == TRUE) {
    invisible(lapply(seq_len(nrow(g)), function(i) {
      if (g$type[i] == "N" &&
        g$x_start[i] - min(g$x_start) < 1000 &&
        "left" %in% pos) {
        text(
          g$x_start[i],
          g$y_start[i],
          labels = parse(text = g$degree_label[i]),
          srt = g$angle_start[i],
          pos = 2,
          cex = cex,
          xpd = TRUE,
          col = col
        )
      }
      if (g$type[i] == "E" &&
        g$y_start[i] - min(g$y_start) < 1000 &&
        "bottom" %in% pos) {
        text(
          g$x_start[i],
          g$y_start[i],
          labels = parse(text = g$degree_label[i]),
          srt = g$angle_start[i] - 90,
          pos = 1,
          cex = cex,
          xpd = TRUE,
          col = col
        )
      }
      if (g$type[i] == "N" &&
        g$x_end[i] - max(g$x_end) > -1000 && "right" %in% pos) {
        text(
          g$x_end[i],
          g$y_end[i],
          labels = parse(text = g$degree_label[i]),
          srt = g$angle_end[i],
          pos = 4,
          cex = cex,
          xpd = TRUE,
          col = col
        )
      }
      if (g$type[i] == "E" &&
        g$y_end[i] - max(g$y_end) > -1000 && "top" %in% pos) {
        text(
          g$x_end[i],
          g$y_end[i],
          labels = parse(text = g$degree_label[i]),
          srt = g$angle_end[i] - 90,
          pos = 3,
          cex = cex,
          xpd = TRUE,
          col = col
        )
      }
    }))
  }
  return(invisible(g))
}

#' @title Export a map in SVG format
#' @name mf_svg
#' @description Export a map with the extent of a spatial object in SVG format.
#'
#' SVG export is the perfect solution for editing maps with desktop vector
#' graphics software. SVG is a vector graphics file format.
#'
#' If `width` is specified, then `height` is deduced from the width/height
#' ratio of `x`. Alternatively, if `height` is specified, then `width`
#' is deduced from the width/height ratio of `x`.
#' This helps to produce maps without too much wasted space.
#'
#' Use `dev.off() to finish the export (see Examples).
#'
#' @param x object of class `sf`, `sfc` or `SpatRaster`
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param filename path to the exported file
#' @param width width of the figure (inches)
#' @param height height of the figure (inches)
#' @param svglite if TRUE, the export is done with the `svglite` package
#' if it is installed (see Details)
#' @param ... further parameters
#' @export
#' @details
#' The default driver for building SVG files, `grDevices::svg()`, has
#' limitations regarding speed, file size, editability, and font support.
#' The `svglite` package aims to solve these issues but it is not lightweight
#' in terms of dependencies, so it is not imported by `mapsf`, but rather
#' suggested.
#'
#' However, we strongly recommend its use if the aim is to edit the maps
#' after export.
#'
#' @return No return value, an SVG device is initiated.
#' @examples
#' mtq <- mf_get_mtq()
#' (filename <- tempfile(fileext = ".svg"))
#' mf_svg(mtq, filename = filename)
#' mf_map(mtq)
#' mf_title()
#' dev.off()
mf_svg <- function(x, filename = "map.svg",
                   width, height,
                   expandBB = rep(0, 4),
                   svglite = TRUE,
                   ...) {
  test_ratio_input(x)
  bb <- x_to_bb(x = x, expandBB = expandBB)
  mar <- get_mar()
  ratio <- bb_to_ratio(bb = bb, mar = mar, width = width, height = height)
  if (isTRUE(svglite)) {
    if (requireNamespace("svglite", quietly = TRUE)) {
      svglite::svglite(filename,
        width = ratio[1], height = ratio[2],
        fix_text_size = FALSE, ...
      )
    } else {
      message(
        paste0("'svglite' is not installed.")
      )
      svg(filename, width = ratio[1], height = ratio[2], ...)
    }
  } else {
    svg(filename, width = ratio[1], height = ratio[2], ...)
  }
}

#' @title Export a map in PNG format
#' @name mf_png
#' @description Export a map with the extent of a spatial object in PNG format.
#'
#' PNG is a raster graphics file format and PNG export should be used for
#' maps that do not require further modification.
#'
#' If `width` is specified, then `height` is deduced from the width/height
#' ratio of `x`. Alternatively, if `height` is specified, then `width`
#' is deduced from the width/height ratio of `x`.
#' This helps to produce maps without too much wasted space.
#'
#' Use `dev.off()` to finish the export (see Examples).
#' @param x object of class `sf`, `sfc` or `SpatRaster`
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param filename path to the exported file
#' @param width width of the figure (pixels)
#' @param height height of the figure (pixels)
#' @param res nominal resolution in ppi
#' @param ... further parameters
#' @export
#' @return No return value, a PNG device is initiated.
#' @examples
#' mtq <- mf_get_mtq()
#' (filename <- tempfile(fileext = ".png"))
#' mf_png(mtq, filename = filename)
#' mf_map(mtq)
#' mf_title()
#' dev.off()
mf_png <- function(x, filename = "map.png",
                   width, height,
                   expandBB = rep(0, 4),
                   res = 96,
                   ...) {
  test_ratio_input(x)
  bb <- x_to_bb(x = x, expandBB = expandBB)
  mar <- get_mar()
  ratio <- bb_to_ratio(bb = bb, mar = mar)

  if (missing(width) && missing(height)) {
    width <- 672
  }

  if (missing(height)) {
    height <- round(ratio[2] * width / ratio[1], 0)
  }

  if (missing(width)) {
    width <- round(ratio[1] * height / ratio[2], 0)
  }

  if (isTRUE(capabilities("cairo"))) {
    png(
      filename = filename, width = width, height = height, res = res,
      type = "cairo-png", ...
    )
  } else {
    png(filename = filename, width = width, height = height, res = res, ...)
  }
}

#' @title Plot a background image
#' @description Plot a background image on an existing plot
#' @param filename filename of the background image, PNG or JPG/JPEG format.
#' @param ... ignored
#'
#' @return No return value, a background image is displayed.
#' @export
#'
#' @examples
#' if (require("jpeg")) {
#'   mtq <- mf_get_mtq()
#'   mf_map(mtq, col = NA, border = NA)
#'   mf_background(system.file("img/background.jpg", package = "mapsf"))
#'   mf_map(mtq, lwd = 3, col = NA, border = "white", add = TRUE)
#'   mf_credits(
#'     txt = "Background photo by Noita Digital on Unsplash",
#'     col = "white"
#'   )
#' }
mf_background <- function(filename, ...) {
  test_cur_plot()
  img <- readimage(filename)
  recordGraphics(
    {
      pusr <- par("usr")
      rasterImage(
        image   = img,
        xleft   = pusr[1],
        ybottom = pusr[3],
        xright  = pusr[2],
        ytop    = pusr[4]
      )
    },
    list = list(img = img),
    env = getNamespace("mapsf")
  )
}

#' @title Get the optimal position of a legend
#'
#' @description Find the optimal for one or two legends. The optimal position is
#' a position that minimizes overlap between a spatial object and a legend.
#' @eval my_params(c("x"))
#' @param n number of positions to get (1 or 2)
#'
#' @return A vector of position is returned
#' @keywords internal
#' @export
#'
#' @examples
#' mtq <- mf_get_mtq()
#' mf_get_leg_pos(mtq)
mf_get_leg_pos <- function(x, n = 1) {
  if (is.null(dev.list())) {
    bb <- st_bbox(x)
  } else {
    p <- par("usr")
    bb <- st_bbox(
      c(
        xmin = p[1], ymin = p[3],
        xmax = p[2], ymax = p[4]
      ),
      crs = st_crs(x)
    )
  }
  g <- st_make_grid(x = st_as_sfc(bb), n = c(3, 3), crs = st_crs(x))
  g <- g[c(2, 4, 5, 6, 7, 8, 9)]
  y <- st_union(st_convex_hull(st_geometry(x)))
  z <- st_intersects(g, y)
  ind <- which(unlist(lapply(z, length)) == 0)
  if (n == 1) {
    if (length(ind) > 0) {
      pos <- max(ind)
    } else {
      pos <- which.min(st_area(st_intersection(g, y)))
    }
  }
  if (n == 2) {
    pos <- c(NA, NA)
    if (length(ind) > 1) {
      pos <- sort(ind, decreasing = TRUE)[1:2]
    } else {
      if (length(ind) == 1) {
        ii <- rep(NA, 7)
        names(ii) <- 1:7
        ii[ind] <- 0
        couv <- st_area(st_intersection(g, y))
        ii[is.na(ii)] <- couv
        pos <- as.numeric(names(sort(ii)[1:2]))
      } else {
        couv <- st_area(st_intersection(g, y))
        names(couv) <- 1:7
        pos <- as.numeric(names(sort(couv)[1:2]))
      }
    }
  }
  tpos <- c(
    "bottom",
    "left", "center", "right",
    "topleft", "top", "topright"
  )
  return(tpos[pos])
}

#' @title Get map width and height values
#' @description This function is to be used to get width and height values
#' for maps created in reports (*.Rmd, *.qmd).\cr
#' It uses the width / height ratio of a spatial object bounding box to find a
#' matching ratio for the map.\cr
#' If width is specified, then height is deduced from the width / height ratio
#' of x, figure margins and title size.\cr
#' If height is specified, then width is
#' deduced from the width / height ratio of x, figure margins and title size.
#' @param x object of class `sf`, `sfc` or `SpatRaster`
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param theme theme used for the map
#' @param width width of the figure (inches), use only one of width or height
#' @param height height of the figure (inches), use only one of width or height
#' @return Width and height are returned in inches.
#' @export
#'
#' @examples
#' mtq <- mf_get_mtq()
#' mf_get_ratio(x = mtq, width = 5)
mf_get_ratio <- function(x,
                         width, height,
                         expandBB = rep(0, 4),
                         theme = mf_theme()) {
  # input test
  test_ratio_input(x)
  bb <- x_to_bb(x = x, expandBB = expandBB)
  mar <- get_mar(theme)
  bb_to_ratio(bb = bb, mar = mar, width = width, height = height)
}


test_ratio_input <- function(x) {
  # input test
  if (!inherits(x, c("bbox", "SpatVector", "SpatRaster", "sf", "sfc", "sfg"))) {
    stop(
      paste0(
        "x should be an object of class sf, sfc, sfg, bbox, ",
        "SpatRaster or SpatVector"
      ),
      call. = FALSE
    )
  }

  if (inherits(x, c("SpatRaster", "SpatVector"))) {
    if (!requireNamespace("terra", quietly = TRUE)) {
      stop(
        paste0(
          "'terra' package is needed for this function to work. ",
          "Please install it."
        ),
        call. = FALSE
      )
    }
  }
}


x_to_bb <- function(x, expandBB) {
  if (inherits(x, c("SpatRaster", "SpatVector"))) {
    ras_proj <- terra::crs(x)
    x <- st_as_sfc(st_bbox(terra::ext(x)[c(1, 3, 2, 4)]))
    st_crs(x) <- ras_proj
    expandBB <- expandBB + c(rep(-.04, 4))
  } else {
    if (par("xaxs") == "r") {
      expandBB <- expandBB / (1 + 0.08)
    }
  }

  if (isTRUE(st_is_longlat(st_crs(x)))) {
    x <- st_as_sfc(st_bbox(x))
    lat_ts <- mean(st_bbox(x)[c(2, 4)]) # latitude of true scale
    x <- st_transform(x = x, paste0("+proj=eqc +lat_ts=", lat_ts))
  }

  bb <- st_bbox(x)
  # expandBB mgmt
  w <- bb[3] - bb[1]
  h <- bb[4] - bb[2]
  bb <- bb + (expandBB[c(2, 1, 4, 3)] * c(-w, -h, w, h))

  return(bb)
}

get_mar <- function(theme) {
  if (missing(theme)) {
    mar <- getOption("mapsf.mar")
  } else {
    old_theme <- mf_theme()
    mf_theme(theme)
    mar <- getOption("mapsf.mar")
    mf_theme(old_theme)
  }
  mar
}


bb_to_ratio <- function(bb, mar, width, height) {
  if (!missing(width) && !missing(height)) {
    return(c(width, height))
  }
  iw <- bb[3] - bb[1]
  ih <- bb[4] - bb[2]
  if (missing(width) && missing(height)) {
    width <- 7
  }
  if (!missing(width) && width > 50) {
    message(paste0(
      "It is unlikely that you really want to produce a figure",
      " with more than 50 inches of width.", " The width has been",
      " set to 7 inches."
    ))
    width <- 7
  }
  if (missing(height)) {
    wh <- iw / ih
    widthmar <- width - (0.2 * (mar[2] + mar[4]))
    height <- (widthmar / wh) + (0.2 * (mar[1] + mar[3]))
  } else {
    hw <- ih / iw
    heightmar <- height - (0.2 * (mar[1] + mar[3]))
    width <- (heightmar / hw) + (0.2 * (mar[2] + mar[4]))
  }
  return(unname(round(c(width, height), 3)))
}

expect_silent(mf_get_ratio(mtq, height = 6))
expect_silent(mf_get_ratio(mtq, width = 7))
expect_silent(mf_get_ratio(mtq))
expect_equal(mf_get_ratio(mtq, theme = "base"), c(7, 8.513))
expect_silent(mf_get_ratio(mtq, width = 3, expandBB = c(0, 0, 0, .3),
                           theme = "sol_dark"))
expect_silent(mf_get_ratio(sf::st_transform(mtq, 4326)))
r <- terra::rast(system.file("ex/elev.tif", package = "terra"))
expect_silent(mf_get_ratio(r))

my_params <- function(x) {
  params <- list(
    xfull = "x object of class \\code{sf} or \\code{sfc}",
    x = "x object of class \\code{sf}",
    var = "var name(s) of the variable(s) to plot",
    vars = "var names of the variables to plot",
    bg = paste0(
      "bg background color of the map (hex code or color name given",
      "by \\code{\\link{colors}}), ignored if \\code{add = TRUE}"
    ),
    fg = "fg foreground color",
    border = "border border color",
    lwd = "lwd border width",
    inches = paste0(
      "inches size of the biggest symbol (radius for circles,",
      " half width for squares) in inches."
    ),
    lwd_max = "lwd_max line width of the largest line",
    symbol = "symbol type of symbols, 'circle' or 'square'",
    col = "col a color, hex code or color name given by \\code{\\link{colors}}",
    leg_pos = paste0(
      "leg_pos position of the legend, one of 'topleft', 'top',",
      "'topright', 'right', 'bottomright', 'bottom', ",
      "'bottomleft', 'left' or a vector of two coordinates ",
      "in map units (c(x, y)). ",
      "Use \\code{NA} to avoid plotting the legend, use 'interactive' ",
      "to choose the ",
      "legend position by clicking on the map."
    ),
    leg_pos2 = paste0(
      "leg_pos position of the legend, two of 'topleft', 'top','topright', ",
      "'right','bottomright', 'bottom', 'bottomleft', ",
      "'left' or a vector of two ",
      "coordinates in map units (c(x, y)). leg_pos argument can be ",
      "c('position', 'position'), c('position', x2, y2), ",
      "c(x1,y1, 'position') or c(x1, y1, x2, y2). ",
      "Use \\code{NA} to avoid plotting the legend, use 'interactive' ",
      "to choose the ",
      "legend position by clicking on the map."
    ),
    leg_title = "leg_title legend title",
    leg_title_cex = "leg_title_cex size of the legend title",
    leg_val_cex = "leg_val_cex size of the values in the legend",
    leg_val_dec = "leg_val_dec decimal separator",
    leg_val_big = "leg_val_big thousands separator",
    leg_val_rnd =
      "leg_val_rnd number of decimal places of the values in the legend",
    val_order =
      "val_order values order, a character vector that matches var modalities",
    leg_frame =
      "leg_frame whether to add a frame to the legend (TRUE) or not (FALSE)",
    leg_no_data = "leg_no_data label for missing values",
    add =
      "add whether to add the layer to an existing plot (TRUE) or not (FALSE)",
    pal = paste0(
      "pal a set of colors (hex codes) or a palette name (",
      "valid palette names can be obtained with \\code{\\link{hcl.pals}})."
    ),
    rev = paste0(
      "rev if \\code{pal} is a valid palette name, ",
      "whether the ordering of the colors should be reversed (TRUE)",
      " or not (FALSE)"
    ),
    alpha = "alpha opacity, in the range 0,1",
    col_na = "col_na color for missing values",
    cex_na = "cex_na point size for NA values",
    pch_na = "pch_na point type for NA values",
    val_max = "val_max maximum value used for proportional symbols",
    breaks = paste0(
      "breaks either a numeric vector with the actual breaks, ",
      "or a classification method name ",
      "(see \\link{mf_get_breaks} and Details)"
    ),
    nbreaks = "nbreaks number of classes",
    pos = paste0(
      "pos position. It can be one of 'topleft', 'top',",
      "'topright', 'right', 'bottomright', 'bottom',",
      "'bottomleft', 'left', 'interactive' or a vector of two coordinates ",
      "in map units (c(x, y))"
    ),
    title = "title legend title",
    title_cex = "title_cex size of the legend title",
    val_cex = "val_cex size of the values in the legend",
    val_rnd = "val_rnd number of decimal places of the values in the legend",
    frame = "frame whether to add a frame to the legend (TRUE) or not (FALSE)",
    no_data_txt = "no_data_txt label for missing values",
    no_data = "no_data if TRUE a 'missing values' box is plotted",
    cex = "cex size of the legend; 2 means two times bigger",
    cexs = "cex cex (point size) for symbols",
    pch = "pch pch (point type) for symbols",
    leg_adj = "leg_adj adjust the position of the legend in x and y directions",
    leg_horiz = paste0(
      "leg_horiz display the legend horizontally ",
      "(for proportional symbols and choropleth types)"
    ),
    leg_frame_border = "leg_frame_border border color of the legend frame",
    leg_bg = "leg_bg color of the legend background",
    leg_fg = "leg_fg color of the legend foreground",
    leg_size = "leg_size size of the legend; 2 means two times bigger",
    leg_border = "leg_border symbol border color(s)",
    leg_box_border = "leg_box_border border color of legend boxes",
    leg_box_cex = paste0(
      "leg_box_cex width and height size expansion ",
      "of boxes"
    ),
    extent = paste0(
      "extent object with an st_bbox method to define ",
      "plot extent; defaults to x. extent and x must use the ",
      "same CRS."
    )
  )


  for (i in seq_along(params)) {
    params[[i]] <- paste0(
      "@param ", " ",
      params[[i]]
    )
  }

  unname(unlist(params[x]))
}

mf_map(mtq)
expect_silent(mf_label(x = mtq[1,], var = "LIBGEO", halo = FALSE, cex = 0.8,
                       overlap = FALSE, lines = TRUE))
expect_silent(mf_label(x = mtq, var = "LIBGEO", halo = TRUE, cex = 0.8,
         overlap = FALSE, lines = TRUE))
expect_silent(mf_label(x = mtq, var = "LIBGEO", halo = TRUE,
                       cex = 0.8, col = sample(1:5, nrow(mtq), replace = TRUE),
                       overlap = FALSE, lines = TRUE))
expect_silent(mf_label(x = mtq, var = "LIBGEO", halo = TRUE,
                       cex = 0.8, col = sample(1:5, nrow(mtq), replace = TRUE),
                       overlap = FALSE, lines = TRUE, pos = 3, font = 3))

display_text <- function(x, y, txt, cex, font, family, adj, col_txt,
                         offset_x, offset_y,
                         box, box_bbox, col_box, col_box_border, lwd_box,
                         halo, cex_halo, col_halo) {
  if (isTRUE(box)) {
    rect(
      xleft = box_bbox$xleft,
      ybottom = box_bbox$ybottom,
      xright = box_bbox$xright,
      ytop = box_bbox$ytop,
      col = col_box,
      border = col_box_border,
      lwd = lwd_box
    )
  }
  if (isTRUE(halo)) {
    theta <- seq(0, 2 * pi, length.out = 50)
    r <- .2 * cex_halo
    xo <- r * offset_x
    yo <- r * offset_y
    for (j in theta) {
      text(
        x = x + cos(j) * xo,
        y = y + sin(j) * yo * 1,
        labels = txt,
        cex = cex,
        font = font,
        family = family,
        adj = adj,
        col = col_halo
      )
    }
  }
  text(
    x = x,
    y = y,
    labels = txt,
    cex = cex,
    font = font,
    family = family,
    adj = adj,
    col = col_txt
  )
}


xy_pos <- function(x, y, cex, offset_x, offset_y, pos) {
  if (pos == 0) {
    return(c(x, y))
  }

  df_angle <- data.frame(
    pos = 1:8,
    angle = c(270, 225, 180, 135, 90, 45, 0, 315)
  )
  angle <- df_angle[match(pos, df_angle[, 1]), "angle"]
  return(c(
    x = x + cex * offset_x * cos((angle * pi) / 180),
    y = y + cex * offset_y * sin((angle * pi) / 180)
  ))
}

get_position_from_name <- function(pos) {
  df_pos <- data.frame(
    named_pos = c(
      "center", "bottom", "bottomleft", "left", "topleft", "top",
      "topright", "right", "bottomright"
    ),
    pos = 0:8
  )
  posr <- df_pos[match(pos, df_pos[, 1]), "pos"][[1]]
  if (is.na(posr)) {
    warning(paste0("'", pos, "' is not a correct position."), call. = FALSE)
    posr <- 0
  }
  return(posr)
}

display_line <- function(line, pos, box, x, y, xy, offset_x, offset_y,
                         lwd, col_line, arrow, cex_arrow, offset,
                         htext, wtext, clockwise) {
  if (pos == 0 || line == 0) {
    return()
  }

  if (arrow == FALSE) {
    cex_arrow <- 0
  }

  xy0 <- xy
  xy1 <- xy_pos(
    x = x, y = y, cex = 1,
    offset_x = offset_x / 2,
    offset_y = offset_y / 2, pos = pos
  )


  if (line == 2) {
    if (pos %in% c(1, 3, 5, 7)) {
      line <- 1
    } else {
      if (clockwise == TRUE) {
        if (pos %in% c(4)) {
          xy0[2] <- xy0[2] + offset_y / 2 + htext / 2
        }
        if (pos %in% c(8)) {
          xy0[2] <- xy0[2] - offset_y / 2 - htext / 2
        }
        if (pos %in% c(4, 8)) {
          segments(
            x0 = xy0[1], y0 = xy0[2], x1 = x, y1 = xy0[2],
            col = col_line, lwd = lwd
          )
          arrows(
            x0 = x, y0 = xy0[2], x1 = x, y1 = xy1[2],
            lwd = lwd, col = col_line, length = 0.05 * cex_arrow
          )
        }
        if (pos %in% c(6)) {
          xy0[1] <- xy0[1] + offset_x / 2 + wtext / 2
        }
        if (pos %in% c(2)) {
          xy0[1] <- xy0[1] - offset_x / 2 - wtext / 2
        }
        if (pos %in% c(2, 6)) {
          segments(
            x0 = xy0[1], y0 = xy0[2], x1 = xy0[1], y1 = y,
            col = col_line, lwd = lwd
          )
          arrows(
            x0 = xy0[1], y0 = y, x1 = xy1[1], y1 = y,
            lwd = lwd, col = col_line, length = 0.05 * cex_arrow
          )
        }
      } else {
        if (pos %in% c(6)) {
          xy0[2] <- xy0[2] + offset_y / 2 + htext / 2
        }
        if (pos %in% c(2)) {
          xy0[2] <- xy0[2] - offset_y / 2 - htext / 2
        }
        if (pos %in% c(2, 6)) {
          segments(
            x0 = xy0[1], y0 = xy0[2], x1 = x, y1 = xy0[2],
            col = col_line, lwd = lwd
          )
          arrows(
            x0 = x, y0 = xy0[2], x1 = x, y1 = xy1[2],
            lwd = lwd, col = col_line, length = 0.05 * cex_arrow
          )
        }
        if (pos %in% c(8)) {
          xy0[1] <- xy0[1] + offset_x / 2 + wtext / 2
        }
        if (pos %in% c(4)) {
          xy0[1] <- xy0[1] - offset_x / 2 - wtext / 2
        }
        if (pos %in% c(4, 8)) {
          segments(
            x0 = xy0[1], y0 = xy0[2], x1 = xy0[1], y1 = y,
            col = col_line, lwd = lwd
          )
          arrows(
            x0 = xy0[1], y0 = y, x1 = xy1[1], y1 = y,
            lwd = lwd, col = col_line, length = 0.05 * cex_arrow
          )
        }
      }
    }
  }

  if (line == 3) {
    if (clockwise == TRUE) {
      if (pos == 2) {
        xy0[1] <- xy0[1] - offset_x / 2 - wtext / 2
        xy1[2] <- xy1[2] + offset_y / 4
      }
      if (pos == 4) {
        xy0[2] <- xy0[2] + offset_y / 2 + htext / 2
        xy1[1] <- xy1[1] + offset_x / 4
      }
      if (pos == 6) {
        xy0[1] <- xy0[1] + offset_x / 2 + wtext / 2
        xy1[2] <- xy1[2] - offset_y / 4
      }
      if (pos == 8) {
        xy0[2] <- xy0[2] - offset_y / 2 - htext / 2
        xy1[1] <- xy1[1] - offset_x / 4
      }
    } else {
      if (pos == 2) {
        xy0[2] <- xy0[2] - offset_y / 2 - htext / 2
        xy1[1] <- xy1[1] + offset_x / 4
      }
      if (pos == 4) {
        xy0[1] <- xy0[1] - offset_x / 2 - wtext / 2
        xy1[2] <- xy1[2] - offset_y / 4
      }
      if (pos == 6) {
        xy0[2] <- xy0[2] + offset_y / 2 + htext / 2
        xy1[1] <- xy1[1] - offset_x / 4
      }
      if (pos == 8) {
        xy0[1] <- xy0[1] + offset_x / 2 + wtext / 2
        xy1[2] <- xy1[2] + offset_y / 4
      }
    }
    trio <- build_trio(
      x0 = xy0[1], y0 = xy0[2], x1 = xy1[1], y1 = xy1[2],
      pos = pos, clockwise = clockwise
    )

    x_courbe <- bernstein(trio$x, seq(0, 1, .01))
    y_courbe <- bernstein(trio$y, seq(0, 1, .01))
    points(x_courbe, y_courbe, type = "l", col = col_line, lwd = lwd)
    arrows(
      x0 = x_courbe[100], y0 = y_courbe[100],
      x1 = x_courbe[101], y1 = y_courbe[101],
      lwd = lwd, col = col_line, length = 0.05 * cex_arrow
    )
  }

  if (line == 1) {
    arrows(
      x0 = xy0[1], y0 = xy0[2], x1 = xy1[1], y1 = xy1[2],
      lwd = lwd, col = col_line, length = 0.05 * cex_arrow
    )
  }
}


bernstein <- function(beta, t = seq(0, 1, .01)) {
  n <- length(beta) - 1
  w <- choose(n, 0:n)
  b <- rep(0, length(t))
  for (v in 0:n) {
    b <- b + beta[v + 1] * w[v + 1] * t^v * (1 - t)^(n - v)
  }
  b
}

build_trio <- function(x0, y0, x1, y1, pos, clockwise = FALSE) {
  f <- yinch(1) / xinch(1)
  k <- sqrt((x1 - x0)^2 + (y1 - y0)^2) / 5
  if (pos == 1) {
    kx <- k
    ky <- 0
  }
  if (pos == 2) {
    kx <- k
    ky <- -k * f
  }
  if (pos == 3) {
    kx <- 0
    ky <- -k * f
  }
  if (pos == 4) {
    kx <- -k
    ky <- -k * f
  }
  if (pos == 5) {
    kx <- -k
    ky <- 0
  }
  if (pos == 6) {
    kx <- -k
    ky <- k * f
  }
  if (pos == 7) {
    kx <- 0
    ky <- k * f
  }
  if (pos == 8) {
    kx <- k
    ky <- k * f
  }
  s <- ifelse(isTRUE(clockwise), -1, 1)

  xt <- x0 + (x1 - x0) / 2 + (kx * s)
  yt <- y0 + (y1 - y0) / 2 + (ky * s)

  return(list(x = c(x0, xt, x1), y = c(y0, yt, y1)))
}


get_xy_for_txt <- function(x, adj, pos, offset) {
  if (inherits(x, c("sf", "sfc"))) {
    x <- st_coordinates(
      st_centroid(
        st_geometry(x[1, ]),
        of_largest_polygon = TRUE
      )
    )
  }
  if (is.numeric(x) && length(x) == 2) {
    return(list(x = x[[1]], y = x[[2]], pos = pos, offset = offset))
  }

  x_spacing <- xinch(par("csi")) / 4
  y_spacing <- yinch(par("csi")) / 4
  pu <- par("usr")
  extra <- adj * c(x_spacing, y_spacing)
  xy <- switch(x,
    bottomleft = list(
      x = pu[1],
      y = pu[3],
      pos = "topright"
    ),
    topleft = list(
      x = pu[1],
      y = pu[4],
      pos = "bottomright"
    ),
    left = list(
      x = pu[1],
      y = pu[3] + (pu[4] - pu[3]) / 2,
      pos = "right"
    ),
    top = list(
      x = pu[1] + (pu[2] - pu[1]) / 2,
      y = pu[4],
      pos = "bottom"
    ),
    bottom = list(
      x = pu[1] + (pu[2] - pu[1]) / 2,
      y = pu[3],
      pos = "top"
    ),
    bottomright = list(
      x = pu[2],
      y = pu[3],
      pos = "topleft"
    ),
    right = list(
      x = pu[2],
      y = pu[3] + (pu[4] - pu[3]) / 2,
      pos = "left"
    ),
    topright = list(
      x = pu[2],
      y = pu[4],
      pos = "bottomleft"
    )
  )
  xy$x <- xy$x + extra[1]
  xy$y <- xy$y + extra[2]
  xy$offset <- 0
  return(xy)
}

#' @title Plot a map layout
#' @description Plot a map layout (title, credits, scalebar,
#' north arrow, frame).
#'
#' This function uses [mf_title], [mf_credits],
#' [mf_scale] and [mf_arrow] with default values.
#'
#' @name mf_layout
#' @param title title of the map
#' @param credits credits
#' @param scale display a scale bar
#' @param arrow display an arrow
#' @param frame display a frame
#' @export
#' @return No return value, a map layout is displayed.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_layout()
mf_layout <- function(title = "Map Title",
                      credits = "Authors & Sources",
                      scale = TRUE, arrow = TRUE,
                      frame = FALSE) {
  test_cur_plot()
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))
  fg <- go(opt = "highlight")

  if (title != "") {
    mf_title(txt = title)
  }

  if (credits != "") {
    mf_credits(txt = credits, pos = "bottomleft")
  }
  if (arrow) {
    mf_arrow()
  }
  if (scale) {
    mf_scale()
  }
  if (frame) {
    pux <- par("usr")
    rect(pux[1], pux[3], pux[2], pux[4], border = fg, col = NA)
  }
}

#' @title Plot a point on a world map
#' @description Plot a point on a world map.
#' @param x object of class `sf` or `sfc`
#' @param lon longitude
#' @param lat latitude
#' @param water_col color of the water
#' @param land_col color of the land
#' @param border_col color of the borders
#' @param border_lwd width of the borders
#' @param ... further parameters related to the plotted point aspect
#' (cex, pch, col...)
#' @return No return value, a world map is displayed.
#' @export
#' @examples
#' mtq <- mf_get_mtq()
#' mf_worldmap(mtq)
#' mf_worldmap(lon = 24, lat = 39)
#' mf_worldmap(
#'   lon = 106, lat = 26,
#'   pch = 4, lwd = 3, cex = 2, col = "tomato4",
#'   water_col = "#232525", land_col = "#A9B7C6",
#'   border_col = "white", border_lwd = 1
#' )
mf_worldmap <- function(x, lon, lat, water_col = "lightblue",
                        land_col = "grey60", border_col = "grey40",
                        border_lwd = .8, ...) {
  ops <- list(...)
  ops$pch <- ifelse(is.null(ops$pch), 17, ops$pch)
  ops$col <- ifelse(is.null(ops$col), "red", ops$col)
  ops$x <- 0
  ops$y <- 0

  if (!missing(x)) {
    co <- st_coordinates(
      st_transform(st_centroid(st_as_sfc(st_bbox(x))), "EPSG:4326")
    )
    lon <- co[1]
    lat <- co[2]
  }
  lo <- orthomap(lon, lat)
  mf_map(lo$ocean,
    col = water_col,
    border = water_col
  )
  mf_map(lo$land,
    add = TRUE,
    col = land_col,
    border = border_col,
    lwd = border_lwd
  )

  do.call(points, ops)
  return(invisible(NULL))
}


orthomap <- function(lon, lat) {
  g <- as_s2_geography(TRUE)
  co <- s2_data_countries()
  oc <- s2_difference(g, s2_union_agg(co)) # oceans
  co <- s2_difference(co, s2_union_agg(oc)) # land

  # visible half
  b <- s2_buffer_cells(as_s2_geography(paste0("POINT(", lon, " ", lat, ")")),
    distance = 9800000
  )

  # proj
  prj <- paste0("+proj=ortho +lat_0=", lat, " +lon_0=", lon)

  # visible land
  cov <- s2_intersection(b, co)
  cov <- st_transform(st_as_sfc(cov), prj)
  cov <- cov[!st_is_empty(cov)]
  cov <- suppressWarnings(st_collection_extract(cov, "POLYGON"))
  cov <- st_cast(cov, "MULTIPOLYGON")


  # visible ocean
  ocv <- s2_intersection(b, oc)
  ocv <- st_transform(st_as_sfc(ocv), prj)
  ocv <- ocv[!st_is_empty(ocv)]
  ocv <- suppressWarnings(st_collection_extract(ocv, "POLYGON"))
  ocv <- st_cast(ocv, "MULTIPOLYGON")

  return(list(ocean = ocv, land = cov))
}

#' @title Plot a scale bar
#' @description Plot a scale bar.
#' @name mf_scale
#' @param col color of the scale bar (line and text)
#' @param size size of the scale bar in scale units (`scale_units`,
#' default to km). If size is not set, an automatic size is used.
#' @param lwd line width of the scale bar
#' @param cex size of the scale bar text
#' @param pos position. It can be one of 'bottomright', 'bottomleft',
#' 'interactive' or a vector of two coordinates in map units (c(x, y)).
#' @param crs_units units used in the CRS of the currently plotted layer.
#' Possible values are "m" and "ft" (see Details).
#' @param scale_units units used for the scale bar. Can be "mi" for miles,
#' "ft" for feet, "m" for meters, or "km" for kilometers (default).
#' @param adj adjust the position of the scale bar in x and y directions
#' @param x object of class crs, sf or sfc. If set, the CRS of x will be used
#' instead of `crs_units` to define CRS units.
#' @details Most CRS use the meter as unit. Some US CRS use feet or US survey
#' feet. If unsure of the unit used in the CRS you can use the x argument of the
#' function.
#' Alternatively, you can use
#' `sf::st_crs(zz, parameters = TRUE)$units_gdal` to see which units
#' are used in the `zz` layer.
#'
#' The scale bar cannot be displayed on unprojected (long/lat) maps or
#' on maps without documented CRS.
#'
#' @return No return value, a scale bar is displayed.
#' @export
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_scale()
#'
#' library(sf)
#' nc <- st_read(system.file("shape/nc.shp", package = "sf"))[1, ]
#'
#' nc_foot <- st_transform(nc, 2264) # NC state plane, US foot
#' mf_map(nc_foot)
#' mf_scale(size = 5, crs_units = "ft", scale_units = "mi")
#' mf_map(nc_foot)
#' mf_scale(size = 5, x = nc_foot, scale_units = "mi")
#'
#' nc_meter <- st_transform(nc, 32119) # NC state plane, m
#' mf_map(nc_meter)
#' mf_scale(size = 5, crs_units = "m", scale_units = "mi")
#' mf_scale(size = 5, crs_units = "m", scale_units = "km", pos = "bottomleft")
mf_scale <- function(size,
                     pos = "bottomright",
                     lwd = 1.5,
                     cex = 0.6,
                     col,
                     crs_units = "m",
                     scale_units = "km",
                     adj = c(0, 0),
                     x) {
  test_cur_plot()
  col <- go(col, "highlight")

  if (!missing(x)) {
    uu <- st_crs(x)$ud_unit
    u_m <- structure(1,
      units = structure(
        list(
          numerator = "m",
          denominator = character(0)
        ),
        class = "symbolic_units"
      ),
      class = "units"
    )
    u_f <- structure(1,
      units = structure(
        list(
          numerator = "foot",
          denominator = character(0)
        ),
        class = "symbolic_units"
      ),
      class = "units"
    )
    u_sf <- structure(1,
      units = structure(
        list(
          numerator = "US_survey_foot",
          denominator = character(0)
        ),
        class = "symbolic_units"
      ),
      class = "units"
    )
    u_d <- structure(1,
      units = structure(
        list(
          numerator = "\u00b0",
          denominator = character(0)
        ),
        class = "symbolic_units"
      ),
      class = "units"
    )

    if (identical(uu, u_m)) {
      crs_units <- "m"
    } else {
      if (identical(uu, u_f) || identical(uu, u_sf)) {
        crs_units <- "ft"
      } else {
        if (identical(uu, u_d)) {
          message("The scale bar cannot be displayed on unprojected (long/lat) maps.")
        } else {
          message("The scale bar cannot be displayed on maps without documented CRS.")
        }
        return(invisible(NULL))
      }
    }
  }

  if (!crs_units %in% c("m", "ft")) {
    stop("crs_units must be 'm' or 'ft'.")
  }
  if (!scale_units %in% c("km", "m", "ft", "mi")) {
    stop("scale_units must be 'km', 'm', 'ft' or 'mi'.")
  }

  if (missing(size)) {
    size <- NULL
  }

  if (diff(par("usr")[1:2]) < 360 * 2 || diff(par("usr")[3:4]) < 180 * 2) {
    message("The scale bar cannot be displayed on unprojected (long/lat) maps.")
    return(invisible(NULL))
  }

  if (length(pos) == 1 && pos == "interactive") {
    mf_scale_display(size, pos, lwd, cex, col, crs_units, scale_units, adj)
  } else {
    recordGraphics(
      {
        mf_scale_display(size, pos, lwd, cex, col, crs_units, scale_units, adj)
      },
      list = list(
        size = size,
        pos = pos,
        lwd = lwd,
        cex = cex,
        col = col,
        crs_units = crs_units,
        scale_units = scale_units,
        adj = adj
      ),
      env = getNamespace("mapsf")
    )
  }
}

mf_scale_display <- function(size,
                             pos = "bottomright",
                             lwd = 1.5,
                             cex = 0.6,
                             col,
                             crs_units = "m",
                             scale_units = "km",
                             adj = c(0, 0)) {
  # get the current plot dimensions
  pu <- par("usr")
  inset <- xinch(par("csi")) / 4
  # default scale
  if (is.null(size)) {
    size <- diff(pu[1:2]) / 12
    size <- unit_conversion(
      size = size,
      unit_in = crs_units,
      unit_out = scale_units
    )
    size_text <- pretty_scale(size, scale_units)
    size <- unit_conversion(
      size = size_text,
      unit_in = scale_units,
      unit_out = crs_units
    )
  } else {
    size_text <- as.character(size)
    size <- unit_conversion(size,
      unit_in = scale_units,
      unit_out = crs_units
    )
  }

  # label
  labelscale <- paste0(size_text, " ", scale_units)

  # xy pos
  xscale <- pu[2] - inset - size
  yscale <- pu[3] + inset

  if (!missing(pos)) {
    if (is.numeric(pos) && length(pos) == 2) {
      xscale <- pos[1]
      yscale <- pos[2]
    } else {
      if (pos == "interactive") {
        isc <- interleg(txt = c("scale bar", "Scale bar"))
        xscale <- isc[1]
        yscale <- isc[2]
      }
      if (pos == "bottomleft") {
        xscale <- pu[1] + inset
        yscale <- pu[3] + inset
      }
    }
  }

  xscale <- xscale + adj[1] * inset / 2
  yscale <- yscale + adj[2] * inset / 2

  # plot the scale bar
  segments(
    x0 = xscale,
    y0 = yscale,
    x1 = xscale + size,
    y1 = yscale,
    lwd = lwd,
    col = col,
    xpd = TRUE
  )
  # plot the scale bar label
  text(
    xscale + (size / 2),
    yscale,
    adj = c(0.5, -.5),
    labels = labelscale,
    cex = cex,
    col = col,
    xpd = TRUE
  )
}


#' Convert units
#' @param size a size
#' @param unit_in input unit
#' @param unit_out output unit
#' @noRd
unit_conversion <- function(size, unit_in, unit_out) {
  if (unit_out == "m") {
    if (unit_in == "ft") size <- size / 3.28084
    if (unit_in == "km") size <- size * 1000
    if (unit_in == "mi") size <- size * 1609.34
  }
  if (unit_out == "km") {
    if (unit_in == "m") size <- size / 1000
    if (unit_in == "ft") size <- size / 3280.84
    if (unit_in == "mi") size <- size * 1.60934
  }
  if (unit_out == "mi") {
    if (unit_in == "m") size <- size / 1609.34
    if (unit_in == "ft") size <- size / 5280
    if (unit_in == "km") size <- size / 1.60934
  }
  if (unit_out == "ft") {
    if (unit_in == "m") size <- size * 3.28084
    if (unit_in == "km") size <- size * 3280.84
    if (unit_in == "mi") size <- size * 5280
  }

  return(size)
}


pretty_scale <- function(size, scale_units) {
  if (scale_units %in% c("km", "mi")) {
    m <- matrix(data = c(
      .5, .5,
      1.5, 1,
      3.5, 2,
      7.5, 5,
      15, 10,
      35, 20,
      75, 50,
      150, 100,
      350, 200,
      750, 1000,
      Inf, 2000
    ), ncol = 2, byrow = TRUE)
  }
  if (scale_units %in% c("ft", "m")) {
    m <- matrix(data = c(
      150, 100,
      350, 200,
      750, 500,
      1500, 1000,
      3500, 2000,
      Inf, 5000
    ), ncol = 2, byrow = TRUE)
  }
  for (i in seq_along(m[, 1])) {
    if (size <= m[i, 1]) {
      return(m[i, 2])
    }
  }
}

#' @title Deprecated - Plot proportional symbols using typology coloration
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "prop_typo"`
#' instead.
#'
#'
#' Plot proportional symbols with colors based on qualitative data.
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'border',
#' 'lwd',
#' 'add' ,
#' 'lwd_max',
#' 'inches',
#' 'val_max',
#' 'symbol',
#' 'col_na',
#' 'pal',
#' 'alpha',
#' 'rev',
#' 'leg_val_rnd',
#' 'leg_val_dec',
#' 'leg_val_big',
#' 'leg_pos2',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'val_order',
#' 'leg_no_data',
#' 'leg_frame',
#' 'leg_adj',
#' 'leg_horiz'))
#'
#' @keywords internal
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_map(mtq, c("POP", "STATUS"), "prop_typo")
#'
#' mtq[6, "STATUS"] <- NA
#' mf_map(mtq)
#' mf_map(
#'   x = mtq, var = c("POP", "STATUS"), type = "prop_typo",
#'   inches = .35, border = "tomato4",
#'   val_max = 90000, symbol = "circle", col_na = "grey", pal = "Dynamic",
#'   lwd = 2,
#'   leg_pos = "bottomright",
#'   leg_title = c("Population", "Municipality\nstatus"),
#'   leg_title_cex = 0.9,
#'   leg_val_cex = .7,
#'   val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
#'   leg_no_data = "No dada",
#'   leg_frame = TRUE,
#'   add = TRUE
#' )
mf_prop_typo <- function(x, var,
                         inches = 0.3,
                         val_max,
                         symbol = "circle",
                         pal,
                         alpha = NULL,
                         rev = FALSE,
                         val_order,
                         border,
                         lwd = .7,
                         lwd_max = 15,
                         col_na = "white",
                         extent = x,
                         bg,
                         expandBB = rep(.04, 4),
                         leg_pos = mf_get_leg_pos(x, 1),
                         leg_title = var,
                         leg_title_cex = c(.8, .8),
                         leg_val_cex = c(.6, .6),
                         leg_val_rnd = 0,
                         leg_val_dec = ".",
                         leg_val_big = "",
                         leg_no_data = "No data",
                         leg_frame = c(FALSE, FALSE),
                         leg_frame_border,
                         leg_horiz = FALSE,
                         leg_adj = c(0, 0),
                         leg_fg,
                         leg_bg,
                         leg_size = 1,
                         leg_box_border,
                         leg_box_cex = c(1, 1),
                         add = TRUE) {
  deprecate_direct_calls_to("mf_prop_typo")
  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")
  pal <- go(pal, "pal_quali", "Dynamic")
  leg_box_border <- go(leg_box_border, "highlight")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )
  border <- go(border, "background")
  border <- border[[1]]

  var2 <- var[2]
  var1 <- var[1]

  xtype <- get_geom_type(x)
  # linestring special case
  if (xtype == "LINE") {
    xl <- x[!is.na(x[[var1]]), ]
    if (!missing(val_max)) {
      maxval <- val_max
    } else {
      maxval <- max(xl[[var1]])
    }
    xl$lwd <- xl[[var1]] * lwd_max / maxval
    if (add == FALSE) {
      mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
    }
    xl <- xl[xl[[var1]] != 0, ]
    # turn to positive values
    xl[[var1]] <- abs(xl[[var1]])
    # Order the dots
    xl <- xl[order(xl[[var1]], decreasing = TRUE), ]

    val_order <- get_modalities(
      x = xl[[var2]],
      val_order = val_order
    )
    # get color list and association
    pal <- get_the_pal(
      pal = pal, nbreaks = length(val_order), alpha = alpha,
      rev = !rev
    )
    # get color vector
    mycols <- get_col_typo(
      x = xl[[var2]], pal = pal,
      val_order = val_order
    )

    no_data <- FALSE
    if (max(is.na(mycols)) == 1) {
      no_data <- TRUE
    }
    mycols[is.na(mycols)] <- col_na
    op2 <- par(lend = 1)
    mf_map(xl, lwd = xl$lwd, add = TRUE, col = mycols)
    val <- seq(min(xl[[var1]]), max(xl[[var1]]), length.out = 4)
    leg_pos <- split_leg(leg_pos)

    ccol <- getOption("mapsf.highlight")

    if (length(leg_pos) == 1) {
      la1 <- list(
        type = "prop_line",
        val = val,
        title = leg_title[1],
        lwd = max(xl$lwd),
        col = ccol,
        val_rnd = leg_val_rnd[1],
        val_dec = leg_val_dec,
        val_big = leg_val_big
      )
      lg <- do.call(leg_comp, la1)
      la2 <- list(
        leg = lg,
        type = "typo",
        val = val_order,
        title = leg_title[2],
        col_na = col_na,
        no_data = no_data,
        no_data_txt = leg_no_data,
        pal = pal,
        box_cex = leg_box_cex * c(1, .5),
        box_border = pal
      )
      lg <- do.call(leg_comp, la2)
      leg_draw(lg,
        pos = leg_pos[[1]], bg = leg_bg, fg = leg_fg, size = leg_size,
        frame = leg_frame[1], title_cex = leg_title_cex[1],
        val_cex = leg_val_cex[1], mar = getOption("mapsf.mar"),
        adj = leg_adj, frame_border = leg_frame_border
      )
    } else {
      message(paste0(
        "The use of separated legends for this map type is deprecated.\n",
        "Please, use only one value for leg_pos",
        " or use mf_legend() to display two legends."
      ))
      leg(
        type = "prop_line",
        pos = leg_pos[[1]], val = val, lwd = max(xl$lwd),
        col = ccol,
        title = leg_title[1], title_cex = leg_title_cex[1],
        val_cex = leg_val_cex[1], val_rnd = leg_val_rnd,
        val_dec = leg_val_dec,
        val_big = leg_val_big,
        frame = leg_frame[1], bg = leg_bg, fg = leg_fg
      )
      leg(
        type = "typo",
        pos = leg_pos[[2]], val = val_order, title = leg_title[2],
        title_cex = leg_title_cex[2], val_cex = leg_val_cex[2],
        col_na = col_na, no_data = no_data, no_data_txt = leg_no_data,
        frame = leg_frame[2], pal = pal, bg = leg_bg, fg = leg_fg,
        box_cex = leg_box_cex * c(1, .5), box_border = pal
      )
    }
    par(op2)
    return(invisible(x))
  }

  # check merge and order
  dots <- create_dots(x = x, var = var1)

  # get modalities
  val_order <- get_modalities(
    x = dots[[var2]],
    val_order = val_order
  )
  # get color list and association
  pal <- get_the_pal(
    pal = pal, nbreaks = length(val_order),
    alpha = alpha, rev = !rev
  )
  # get color vector
  mycols <- get_col_typo(
    x = dots[[var2]], pal = pal,
    val_order = val_order
  )

  no_data <- FALSE
  if (max(is.na(mycols)) == 1) {
    no_data <- TRUE
  }
  mycols[is.na(mycols)] <- col_na

  # Default max value
  if (missing(val_max)) {
    val_max <- max(dots[[var1]])
  }

  # get sizes
  sizes <- get_size(
    var = dots[[var1]], inches = inches,
    val_max = val_max, symbol = symbol
  )

  # size and values for legend, hollow circle (fixmax case)
  size_max <- max(sizes)
  val <- seq(sqrt(min(dots[[var1]])), sqrt(max(dots[[var1]])), length.out = 4)
  val <- val * val
  if (inches <= size_max) {
    inches <- size_max
    borders <- border
  } else {
    mycols <- c(NA, mycols)
    borders <- c(NA, rep(border, nrow(dots)))
    dots <- rbind(dots[1, ], dots)
    dots[1, var1] <- val_max
    sizes <- c(inches, sizes)
  }

  # empty plot if needed
  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  # Plot the symbols
  plot_symbols(
    symbol = symbol, dots = dots, sizes = xinch(sizes),
    mycols = mycols, border = borders, lwd = lwd,
    inches = inches
  )

  if (is.null(getOption("mapsf.legacy"))) {
    ccol <- getOption("mapsf.foreground")
  } else {
    ccol <- "grey80"
  }
  if (all(leg_frame, !leg_horiz)) {
    ccol <- getOption("mapsf.foreground")
  }

  if (length(leg_pos) == 1) {
    leg_title <- test2args(leg_title)
    la1 <- list(
      type = "prop",
      val = val,
      title = leg_title[1],
      symbol = symbol,
      inches = size_max,
      col = ccol,
      val_rnd = leg_val_rnd[1],
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      border = border,
      lwd = lwd, horiz = leg_horiz,
      self_adjust = TRUE
    )
    lg <- do.call(leg_comp, la1)
    pch <- ifelse(symbol == "circle", 21, 22)
    cex <- 2
    la2 <- list(
      leg = lg,
      type = "symb",
      val = val_order,
      title = leg_title[2],
      col_na = col_na,
      no_data = no_data,
      no_data_txt = leg_no_data,
      pal = pal,
      lwd = lwd,
      border = border,
      cex = cex, pch = pch,
      cex_na = cex,
      pch_na = pch,
      box_cex = c(.75, .75)
    )
    lg <- do.call(leg_comp, la2)
    leg_draw(lg,
      pos = leg_pos[[1]], bg = leg_bg, fg = leg_fg, size = leg_size,
      frame = leg_frame[1], title_cex = leg_title_cex[1],
      val_cex = leg_val_cex[1], mar = getOption("mapsf.mar"),
      adj = leg_adj, frame_border = leg_frame_border
    )
  } else {
    message(paste0(
      "The use of separated legends for this map type is deprecated.\n",
      "Please, use only one value for leg_pos",
      " or use mf_legend() to display two legends."
    ))
    leg(
      type = "prop",
      pos = leg_pos[[1]], val = val, title = leg_title[1],
      symbol = symbol, inches = size_max, col = ccol,
      title_cex = leg_title_cex[1], val_cex = leg_val_cex[1],
      val_rnd = leg_val_rnd, val_dec = leg_val_dec,
      val_big = leg_val_big, horiz = leg_horiz,
      frame = leg_frame[1], border = border,
      lwd = lwd, bg = leg_bg, fg = leg_fg,
      self_adjust = TRUE, mar = getOption("mapsf.mar")
    )
    leg(
      type = "typo",
      pos = leg_pos[[2]], val = val_order, title = leg_title[2],
      title_cex = leg_title_cex[2], val_cex = leg_val_cex[2],
      col_na = col_na, no_data = no_data, no_data_txt = leg_no_data,
      frame = leg_frame[2], pal = pal, bg = leg_bg, fg = leg_fg,
      box_border = leg_box_border, box_cex = leg_box_cex
    )
  }
  return(invisible(x))
}

#' @title Deprecated - Plot proportional symbols using choropleth coloration
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "prop_choro"`
#' instead.
#'
#' Plot proportional symbols with colors based on a quantitative
#' data classification.
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'border',
#' 'lwd',
#' 'add' ,
#' 'inches',
#' 'val_max',
#' 'symbol',
#' 'col_na',
#' 'pal',
#' 'alpha',
#' 'rev',
#' 'breaks',
#' 'nbreaks',
#' 'leg_pos2',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'leg_val_rnd',
#' 'leg_val_dec',
#' 'leg_val_big',
#' 'leg_no_data',
#' 'leg_frame',
#' 'leg_frame_border',
#' 'leg_fg',
#' 'leg_bg',
#' 'leg_size',
#' 'leg_box_border',
#' 'leg_box_cex',
#' 'leg_adj',
#' 'leg_horiz'))
#' @details
#' Breaks defined by a numeric vector or a classification method are
#' left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\[, \[5 - 10\[, \[10 - 15\[, \[15 - 20\].
#' The "jenks" method is an exception and has to be right-closed.
#' Jenks breaks computed as \code{c(2, 5, 10, 15, 20)}
#' will be mapped as \[2 - 5\], \]5 - 10\], \]10 - 15\], \]15 - 20\].
#' @keywords internal
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_map(mtq, c("POP", "MED"), "prop_choro")
#'
#' mf_map(mtq)
#' mtq[6, "MED"] <- NA
#' mf_map(
#'   x = mtq, var = c("POP", "MED"), type = "prop_choro",
#'   inches = .35, border = "tomato4",
#'   val_max = 90000, symbol = "circle", col_na = "grey", pal = "Cividis",
#'   breaks = "equal", nbreaks = 4, lwd = 4,
#'   leg_pos = "bottomright",
#'   leg_title = c("Population", "Median Income"),
#'   leg_title_cex = 1,
#'   leg_val_cex = .9,
#'   leg_val_rnd = c(0, 0),
#'   leg_no_data = "No data",
#'   leg_frame = TRUE,
#'   add = TRUE
#' )
mf_prop_choro <- function(x,
                          var,
                          inches = 0.3,
                          val_max,
                          symbol = "circle",
                          pal,
                          alpha = NULL,
                          rev = FALSE,
                          breaks = "quantile",
                          nbreaks,
                          border,
                          lwd = .7,
                          col_na = "white",
                          extent = x,
                          bg,
                          expandBB = rep(.04, 4),
                          leg_pos = mf_get_leg_pos(x, 1),
                          leg_title = var,
                          leg_title_cex = c(.8, .8),
                          leg_val_cex = c(.6, .6),
                          leg_val_rnd = c(0, 2),
                          leg_val_dec = ".",
                          leg_val_big = "",
                          leg_no_data = "No data",
                          leg_frame = c(FALSE, FALSE),
                          leg_frame_border,
                          leg_horiz = c(FALSE, FALSE),
                          leg_adj = c(0, 0),
                          leg_fg,
                          leg_bg,
                          leg_size = 1,
                          leg_box_border,
                          leg_box_cex = c(1, 1),
                          add = TRUE) {
  deprecate_direct_calls_to("mf_prop_choro")
  xtype <- get_geom_type(x)
  # linestring special case
  if (xtype == "LINE") {
    message("This map type is not available for lines.")
    return(invisible(NULL))
  }

  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")
  pal <- go(pal, "pal_seq", "Mint")
  leg_box_border <- go(leg_box_border, "highlight")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )
  border <- go(border, "background")
  border <- border[[1]]

  var2 <- var[2]
  var1 <- var[1]
  # check merge and order
  dots <- create_dots(x = x, var = var1)

  # jenks
  jen <- FALSE
  if (any(breaks %in% "jenks")) {
    jen <- TRUE
  }
  # get the breaks
  breaks <- mf_get_breaks(
    x = dots[[var2]], nbreaks = nbreaks,
    breaks = breaks
  )
  nbreaks <- length(breaks) - 1
  # get the cols
  pal <- get_the_pal(pal = pal, nbreaks = nbreaks, alpha = alpha, rev = !rev)
  # get the color vector
  mycols <- get_col_vec(x = dots[[var2]], breaks = breaks, pal = pal, jen = jen)

  no_data <- FALSE
  if (max(is.na(mycols)) == 1) {
    no_data <- TRUE
  }
  mycols[is.na(mycols)] <- col_na

  # Default max value
  if (missing(val_max)) {
    val_max <- max(dots[[var1]])
  }

  # get sizes
  sizes <- get_size(
    var = dots[[var1]], inches = inches,
    val_max = val_max, symbol = symbol
  )

  # size and values for legend, hollow circle (fixmax case)
  size_max <- max(sizes)
  val <- seq(sqrt(min(dots[[var1]])), sqrt(max(dots[[var1]])), length.out = 4)
  val <- val * val
  if (inches <= size_max) {
    inches <- size_max
    borders <- border
  } else {
    mycols <- c(NA, mycols)
    borders <- c(NA, rep(border, nrow(dots)))
    dots <- rbind(dots[1, ], dots)
    dots[1, var] <- val_max
    sizes <- c(inches, sizes)
  }

  # empty plot if needed
  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  # Plot the symbols
  plot_symbols(
    symbol = symbol, dots = dots, sizes = xinch(sizes),
    mycols = mycols, border = borders, lwd = lwd,
    inches = inches
  )

  leg_pos <- split_leg(leg_pos)

  if (is.null(getOption("mapsf.legacy"))) {
    ccol <- getOption("mapsf.foreground")
  } else {
    ccol <- "grey80"
  }
  if (all(leg_frame, !leg_horiz)) {
    ccol <- getOption("mapsf.foreground")
  }

  if (length(leg_pos) == 1) {
    leg_title <- test2args(leg_title)
    leg_val_rnd <- test2args(leg_val_rnd)
    leg_horiz <- test2args(leg_horiz)

    la1 <- list(
      type = "prop",
      val = val,
      title = leg_title[1],
      symbol = symbol,
      inches = size_max,
      col = ccol,
      val_rnd = leg_val_rnd[1],
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      border = border,
      lwd = lwd,
      horiz = leg_horiz[1],
      self_adjust = TRUE
    )
    lg <- do.call(leg_comp, la1)
    la2 <- list(
      leg = lg,
      type = "choro_point",
      symbol = symbol,
      val = breaks,
      title = leg_title[2],
      val_rnd = leg_val_rnd[2],
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      col_na = col_na,
      no_data = no_data,
      no_data_txt = leg_no_data,
      horiz = leg_horiz[2],
      pal = pal,
      border = border
    )
    lg <- do.call(leg_comp, la2)
    leg_draw(lg,
      pos = leg_pos[[1]], bg = leg_bg, fg = leg_fg, size = leg_size,
      frame = leg_frame[1], title_cex = leg_title_cex[1],
      val_cex = leg_val_cex[1], mar = getOption("mapsf.mar"),
      adj = leg_adj, frame_border = leg_frame_border
    )
  } else {
    message(paste0(
      "The use of separated legends for this map type is deprecated.\n",
      "Please, use only one value for leg_pos",
      " or use mf_legend() to display two legends."
    ))
    leg(
      type = "prop",
      pos = leg_pos[[1]], val = val, title = leg_title[1],
      symbol = symbol, inches = size_max, col = ccol,
      title_cex = leg_title_cex[1], val_cex = leg_val_cex[1],
      val_rnd = leg_val_rnd[1], val_dec = leg_val_dec,
      val_big = leg_val_big,
      horiz = leg_horiz[1],
      frame = leg_frame[1], border = border, lwd = lwd,
      bg = leg_bg, fg = leg_fg, self_adjust = TRUE,
      mar = getOption("mapsf.mar"), size = leg_size
    )
    leg(
      type = "choro",
      pos = leg_pos[[2]], val = breaks, title = leg_title[2],
      title_cex = leg_title_cex[2], val_cex = leg_val_cex[2],
      val_rnd = leg_val_rnd[2], val_dec = leg_val_dec,
      val_big = leg_val_big, horiz = leg_horiz[2],
      col_na = col_na, no_data = no_data, no_data_txt = leg_no_data,
      frame = leg_frame[2], pal = pal, bg = leg_bg, fg = leg_fg,
      size = leg_size, box_border = leg_box_border, box_cex = leg_box_cex
    )
  }

  return(invisible(x))
}

#' @title Deprecated - Export a map
#' @name mf_export
#' @description
#' This function is deprecated. Please use \code{\link{mf_png}} or
#' \code{\link{mf_svg}} instead.
#'
#' Export a map with the extent of a spatial object.\cr
#' The map is exported in PNG or SVG format.\cr
#' If only one of \code{width} or \code{height} is set, \code{mf_export} uses
#' the width/height ratio of \code{x} bounding box to find a matching ratio for
#' the export.\cr
#' Always use \code{add = TRUE} in \code{mf_map} calls following an
#' \code{mf_export} call.\cr
#' Use \code{dev.off} to finish the export (see Examples).
#' @param x object of class \code{sf}, \code{sfc} or \code{SpatRaster}
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param filename path to the exported file. If the file extension is ".png" a
#' png graphic device is opened, if the file extension is ".svg" a svg graphic
#' device is opened.
#' @param width width of the figure (pixels for png, inches for svg)
#' @param height height of the figure (pixels for png, inches for svg)
#' @param res resolution (for png)
#' @param ... further parameters for png or svg export
#' @export
#' @keywords internal
#' @return No return value, a map file is initiated (in PNG or SVG format).
#' @examples
#' mtq <- mf_get_mtq()
#' (filename <- tempfile(fileext = ".png"))
#' mf_png(mtq, filename = filename)
#' mf_map(mtq)
#' mf_title()
#' dev.off()
mf_export <- function(x,
                      filename = "map.png",
                      width,
                      height,
                      res = 96,
                      ...,
                      expandBB = rep(0, 4)) {
  warning(
    paste0(
      "mf_export() is deprecated. ",
      "Use mf_png() or mf_svg() instead."
    ),
    call. = FALSE
  )

  # input test
  if (!inherits(x, c("bbox", "SpatVector", "SpatRaster", "sf", "sfc", "sfg"))) {
    stop(
      paste0(
        "x should be an object of class sf, sfc, sfg, bbox, ",
        "SpatRaster or SpatVector"
      ),
      call. = FALSE
    )
  }

  # file format
  nc <- nchar(filename)
  ext <- substr(filename, nc - 3, nc)
  if (!ext %in% c(".png", ".svg")) {
    stop('The filename extension should be ".png" or ".svg".', call. = FALSE)
  }
  export <- substr(ext, 2, 4)


  if (inherits(x, "SpatRaster")) {
    if (!requireNamespace("terra", quietly = TRUE)) {
      stop(
        paste0(
          "'terra' package is needed for this function to work. ",
          "Please install it."
        ),
        call. = FALSE
      )
    }
    proj <- terra::crs(x)
    bb <- terra::ext(x)[c(1, 3, 2, 4)]
    y <- st_as_sfc(st_bbox(bb))
    st_crs(y) <- proj
    mf_export(
      x = y, filename = filename, width = width, height = height, res = res,
      expandBB = c(rep(-.04, 4)) + expandBB, ...
    )
    return(invisible(NULL))
  }

  mar <- getOption("mapsf.mar")
  bgmap <- getOption("mapsf.background")

  # transform to bbox
  bb <- st_bbox(x)
  y <- st_as_sfc(bb)

  if (par("xaxs") == "r") {
    expandBB <- expandBB / (1 + 0.08)
  }
  # expandBB mgmt
  extra <- expandBB[c(2, 1, 4, 3)]
  w <- bb[3] - bb[1]
  h <- bb[4] - bb[2]
  bb <- bb + (extra * c(-w, -h, w, h))


  if (export == "png") {
    if (!missing(width) && !missing(height)) {
      fd <- c(width, height)
    } else {
      if (missing(width) && missing(height)) {
        width <- 600
      }
      fd <- get_ratio(
        x = bb,
        width = width,
        height = height,
        mar = mar,
        res = res,
        format = "png"
      )
    }
    if (isTRUE(capabilities("cairo"))) {
      png(filename,
        width = fd[1], height = fd[2], res = res,
        type = "cairo-png", ...
      )
    } else {
      png(filename, width = fd[1], height = fd[2], res = res, ...)
    }
  }
  if (export == "svg") {
    if (!missing(width) && !missing(height)) {
      fd <- c(width, height)
    } else {
      if (missing(height) && missing(width)) {
        width <- 7
      }
      if (!missing(width) && width > 50) {
        message(paste0(
          "It is unlikely that you really want to produce a figure",
          " with more than 50 inches of width.", " The width has been",
          " set to 7 inches."
        ))
        width <- 7
      }
      fd <- get_ratio(
        x = bb,
        width = width,
        height = height,
        mar = mar, res = 96, format = "svg"
      ) / 96
    }
    svg(filename = filename, width = fd[1], height = fd[2], ...)
  }

  # margins mgmt
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))
  # plot with bg and margins
  plot(y, col = NA, border = NA, expandBB = expandBB)
  pux <- par("usr")
  rect(pux[1], pux[3], pux[2], pux[4], border = NA, col = bgmap)

  return(invisible(NULL))
}


get_ratio <- function(x, width, height, mar, res, format) {
  if (isTRUE(st_is_longlat(x))) {
    x <- st_as_sfc(x)
    lat_ts <- mean(st_bbox(x)[c(2, 4)]) # latitude of true scale
    x <- st_transform(x = x, paste0("+proj=eqc +lat_ts=", lat_ts))
    x <- st_bbox(x)
  }

  iw <- x[3] - x[1]
  ih <- x[4] - x[2]
  if (missing(height)) {
    if (format == "svg") {
      width <- width * 96
    }
    wh <- iw / ih
    widthmar <- width - (0.2 * (mar[2] + mar[4]) * res)
    height <- (widthmar / wh) + (0.2 * (mar[1] + mar[3]) * res)
  } else {
    if (format == "svg") {
      height <- height * 96
    }
    hw <- ih / iw
    heightmar <- height - (0.2 * (mar[1] + mar[3]) * res)
    width <- (heightmar / hw) + (0.2 * (mar[2] + mar[4]) * res)
  }
  return(unname(floor(c(width, height))))
}

#' @title Deprecated functions in mapsf
#'
#' @description
#' These functions and features still work but will be removed in the next
#' major version of the package.
#'
#' ## `mf_map` sub-functions
#'
#' Instead of using the following deprecated functions, one can use [mf_map]
#' with the corresponding type:
#' - `mf_base()` => [mf_map_base]
#' - `mf_choro()` => [mf_map_choro]
#' - `mf_prop()` => [mf_map_prop]
#' - `mf_typo()` => [mf_map_typo]
#' - `mf_symb()` => [mf_map_symb]
#' - `mf_grad()` => [mf_map_grad]
#' - `mf_prop_typo()` => [mf_map_prop_typo]
#' - `mf_prop_choro()` => [mf_map_prop_choro]
#' - `mf_symb_choro()` => [mf_map_symb_choro]
#'
#'
#' ## `mf_init`
#'
#' `mf_init` is deprecated. It is possible to use `mf_map` instead
#' (`mf_map(x, type = "base", col = NA, border = NA)`.
#'
#' It is also possible to use the `extent` argument of [mf_map].
#'
#'
#' ## `mf_export`
#'
#' `mf_export` is deprecated, use [mf_png] or [mf_svg] instead.
#'
#' ## `mf_annotation`
#'
#' `mf_annotation` is deprecated, use [mf_text] instead.
#'
#' ## Double legends
#'
#' The use of separated legends for map types **prop_choro**, **prop_typo** and
#' **symb_choro** is deprecated. Use `leg_pos = NA` and [mf_legend] if
#' separated legends are needed.
#'
#' ## Theming system
#'
#' In [mf_theme], the following themes are deprecated: "default", "brutal",
#' "ink", "dark", "agolalight", "candy", "darkula", "iceberg", "green",
#' "nevermind", "jsk" and "barcelona".\cr
#' The following arguments are also deprecated: "bg", "fg", "tab",
#' "pos", "inner", "line", "cex" and "font".\cr
#'
#' Although the map theming system has been radically changed in version 1.0.0
#' of the package, you can still use the old themes by referencing them by name.
#' If you need to use the *pre* v1.0.0 default theme, set `x` to "default".\cr
#' If an old theme is set, only deprecated arguments are used and others are
#' ignored.\cr
#' If current and deprecated arguments are mixed, only deprecated arguments are
#' used and others are ignored.\cr
#' All references and usages of the old theming system will be removed in the
#' next major version.
#'
#' @name mapsf-deprecated
NULL

#' @title Plot a text
#'
#' @description Plot a text on the map.
#'
#' @param x
#' an sf object (the first row is used),
#' a couple of coordinates (c(x, y)),
#' a position (one of 'topleft', 'top','topright', 'right', 'bottomright',
#' 'bottom', 'bottomleft', 'left'),
#' or "interactive" for interactive placement.
#' @param txt text to display
#' @param cex text size
#' @param col_txt text color
#' @param pos position of the text relative to `x` (one of 'center',
#' 'topleft', 'top', 'topright', 'right', 'bottomright', 'bottom',
#' 'bottomleft', 'left').
#' Ignored if `x` is a position.
#' @param offset offset between the text and `x`. Ignored if `x` is a position.
#' @param align text alignement, one of "left", "right" or "center"
#' @param font text font
#' @param family text family
#' @param halo add a halo around the text
#' @param col_halo halo color
#' @param cex_halo halo width
#' @param box add a box around the text
#' @param col_box box color
#' @param col_box_border box border color
#' @param lwd_box line width of the box border
#' @param line type of the line drawn between the text and `x`.
#' 0 for no line, 1 for a straight line, 2 for a line with an angle,
#' 3 for a curved line. Ignored if `x` is a position.
#' @param clockwise direction of the curve for types 2 and 3
#' @param lwd line width
#' @param col_line line color
#' @param arrow add an arrow to the line
#' @param cex_arrow arrow size
#' @param adj adjust the text position in x and y directions
#'
#' @return No return value, an text is displayed.
#' @export
#'
#' @examples
#' library(mapsf)
#' mf_theme("base")
#' mtq <- mf_get_mtq()
#' mtq_p <- mf_get_mtq("points")
#' mf_map(mtq, expandBB = c(0, .1, 0, .3))
#' mf_map(mtq_p[c(2, 3, 17, 28), ],
#'   pch = 4, lwd = 1.5, cex = .5, col = "red",
#'   add = TRUE
#' )
#' mf_title("Title of the map", banner = TRUE)
#' mf_frame()
#' mf_text(x = "topright", txt = "x = 'topright'")
#' mf_text(
#'   x = "bottomleft",
#'   txt = "x = 'bottomleft'\nadj = c(4,4)\nalign = 'left'",
#'   adj = c(4, 4),
#'   align = "left"
#' )
#' mf_text(
#'   x = c(728000, 1625500),
#'   txt = "x = c(X, Y)",
#'   pos = "right",
#'   line = 2,
#'   offset = 5
#' )
#' mf_text(
#'   mtq[3, ],
#'   txt = "pos = 'top'\nhalo = TRUE",
#'   pos = "top",
#'   align = "center",
#'   halo = TRUE
#' )
#' mf_text(
#'   mtq[3, ],
#'   txt = "pos = 'bottomleft'\nhalo = TRUE\nalign = 'right'",
#'   pos = "bottomleft",
#'   align = "right",
#'   halo = TRUE
#' )
#' mf_text(
#'   x = mtq[17, ],
#'   txt = "pos = 'bottomright'\nline = 1\nbox = TRUE",
#'   pos = "bottomright",
#'   offset = 10,
#'   line = 1,
#'   box = TRUE
#' )
#' mf_text(
#'   x = mtq[17, ],
#'   txt = "pos = 'topright'\nline = 1\nalign = 'left'",
#'   pos = "topright",
#'   offset = 15,
#'   align = "left",
#'   line = 1
#' )
#' mf_text(
#'   x = mtq[2, ],
#'   txt = "pos = 'topleft'\nline = 2\nbox = TRUE\nclockwise = TRUE",
#'   pos = "topleft",
#'   offset = 8,
#'   clockwise = TRUE,
#'   line = 2,
#'   box = TRUE
#' )
#' mf_text(
#'   x = mtq[2, ],
#'   txt = "pos = 'bottomleft'\nline = 2\nclockwise = FALSE",
#'   pos = "bottomleft",
#'   offset = 6,
#'   clockwise = FALSE,
#'   line = 2,
#'   box = FALSE
#' )
#' mf_text(
#'   x = mtq[28, ],
#'   txt = "pos = 'topright'\nline = 3\nclockwise = FALSE",
#'   pos = "topright",
#'   offset = 10,
#'   clockwise = FALSE,
#'   line = 3,
#'   halo = TRUE,
#'   align = "left"
#' )
#' mf_text(
#'   x = mtq[28, ],
#'   txt = "pos = 'right'\nline = 3\nbox = TRUE\nclockwise = TRUE",
#'   pos = "right",
#'   offset = 10,
#'   clockwise = TRUE,
#'   line = 3,
#'   box = TRUE
#' )
mf_text <- function(
  x,
  txt = "Text",
  cex = .8,
  col_txt,
  pos = "center",
  offset = 0,
  align = "center",
  font = 1,
  family = "sans",
  halo = FALSE,
  col_halo,
  cex_halo = cex,
  box = FALSE,
  col_box,
  col_box_border,
  lwd_box = 2,
  line = 0,
  clockwise = TRUE,
  lwd = 2,
  col_line,
  arrow = TRUE,
  cex_arrow = 1,
  adj = c(0, 0)
) {
  test_cur_plot()
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  if (inherits(x, "character") && x == "interactive") {
    x <- interleg(txt = c("text", "Text"))
  }

  col_txt <- go(col_txt, "highlight")
  col_halo <- go(col_halo, "background")
  col_box <- go(col_box, "background")
  col_box_border <- go(col_box_border, "highlight")
  col_line <- go(col_line, "highlight")

  recordGraphics(
    expr = mf_text_display(
      x = x, txt = txt, cex = cex, col_txt = col_txt,
      pos = pos, offset = offset, align = align,
      font = font, family = family,
      halo = halo, col_halo = col_halo,
      cex_halo = cex_halo,
      box = box, col_box = col_box,
      col_box_border = col_box_border,
      lwd_box = lwd_box, line = line,
      clockwise = clockwise,
      lwd = lwd, col_line = col_line, arrow = arrow,
      cex_arrow = cex_arrow,
      adj = adj
    ),
    list = list(
      x = x, txt = txt, cex = cex, col_txt = col_txt,
      pos = pos, offset = offset, align = align, font = font,
      family = family,
      halo = halo, col_halo = col_halo, cex_halo = cex_halo,
      box = box, col_box = col_box, col_box_border = col_box_border,
      lwd_box = lwd_box, line = line, clockwise = clockwise,
      lwd = lwd, col_line = col_line, arrow = arrow,
      cex_arrow = cex_arrow,
      adj = adj
    ),
    env = getNamespace("mapsf")
  )


  return(invisible(NULL))
}


mf_text_display <- function(
  x,
  txt = "Text",
  cex = .8,
  col_txt,
  pos = "center",
  offset = .5,
  align = "center",
  font = 1,
  family = "sans",
  halo = FALSE,
  col_halo,
  cex_halo = cex,
  box = FALSE,
  col_box,
  col_box_border,
  lwd_box = 2,
  line = 0,
  clockwise = TRUE,
  lwd = 2,
  col_line,
  arrow = TRUE,
  cex_arrow = 1,
  adj = c(0, 0)
) {
  res <- get_xy_for_txt(x, adj = adj, pos = pos, offset = offset)
  x <- res$x
  y <- res$y
  pos <- res$pos
  offset <- res$offset

  # internal var
  offset_x <- xinch(par("csi")) / 2
  offset_y <- yinch(par("csi")) / 2
  rat <- offset_x / offset_y
  wtext <- strwidth(txt, units = "user", cex = cex, font = font, family = family)
  htext <- strheight(txt, units = "user", cex = cex, font = font, family = family)
  pos <- get_position_from_name(pos)
  xy <- xy_pos(
    x = x, y = y,
    cex = offset / 2,
    offset_x = offset_x,
    offset_y = offset_y,
    pos = pos
  )
  if (pos == 0) {
    adj_y <- .5
    xy <- c(x, y)
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(x + wtext / 2, y)
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(x - wtext / 2, y)
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(x, y)
    }
    box_bbox <- list(
      xleft   = xy[1] - (wtext / 2 + offset_x / 2),
      ybottom = xy[2] - (htext / 2 + offset_y / 2),
      xright  = xy[1] + (wtext / 2 + offset_x / 2),
      ytop    = xy[2] + (htext / 2 + offset_y / 2)
    )
  }

  if (pos == 1) {
    adj_y <- 1
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1] + wtext / 2, xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1] - wtext / 2, xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1], xy[2])
    }
    xy_text[2] <- xy_text[2] - offset_y / 2
    box_bbox <- list(
      xleft   = xy[1] - (wtext / 2 + offset_x / 2),
      ybottom = xy[2] - (htext + offset_y),
      xright  = xy[1] + (wtext / 2 + offset_x / 2),
      ytop    = xy[2]
    )
  }

  if (pos == 2) {
    adj_y <- 1
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1], xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1] - wtext, xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1] - wtext / 2, xy[2])
    }
    xy_text[1] <- xy_text[1] - offset_x / 2
    xy_text[2] <- xy_text[2] - offset_y / 2
    box_bbox <- list(
      xleft   = xy[1] - (wtext + offset_x),
      ybottom = xy[2] - (htext + offset_y),
      xright  = xy[1],
      ytop    = xy[2]
    )
  }

  if (pos == 3) {
    adj_y <- 0.5
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1], xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1] - wtext, xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1] - wtext / 2, xy[2])
    }
    xy_text[1] <- xy_text[1] - offset_x / 2
    box_bbox <- list(
      xleft   = xy[1] - (wtext + offset_x),
      ybottom = xy[2] - (htext / 2 + offset_y / 2),
      xright  = xy[1],
      ytop    = xy[2] + (htext / 2 + offset_y / 2)
    )
  }

  if (pos == 4) {
    adj_y <- 0
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1], xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1] - wtext, xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1] - wtext / 2, xy[2])
    }
    xy_text[1] <- xy_text[1] - offset_x / 2
    xy_text[2] <- xy_text[2] + offset_y / 2
    box_bbox <- list(
      xleft   = xy[1] - (wtext + offset_x),
      ybottom = xy[2],
      xright  = xy[1],
      ytop    = xy[2] + (htext + offset_y)
    )
  }

  if (pos == 5) {
    adj_y <- 0
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1] + wtext / 2, xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1] - wtext / 2, xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1], xy[2])
    }
    xy_text[2] <- xy_text[2] + offset_y / 2
    box_bbox <- list(
      xleft   = xy[1] - (wtext / 2 + offset_x / 2),
      ybottom = xy[2],
      xright  = xy[1] + (wtext / 2 + offset_x / 2),
      ytop    = xy[2] + (htext + offset_y)
    )
  }

  if (pos == 6) {
    adj_y <- 0
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1] + wtext, xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1], xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1] + wtext / 2, xy[2])
    }
    xy_text[1] <- xy_text[1] + offset_x / 2
    xy_text[2] <- xy_text[2] + offset_y / 2
    box_bbox <- list(
      xleft   = xy[1],
      ybottom = xy[2],
      xright  = xy[1] + (wtext + offset_x),
      ytop    = xy[2] + (htext + offset_y)
    )
  }

  if (pos == 7) {
    adj_y <- 0.5
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1] + wtext, xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1], xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1] + wtext / 2, xy[2])
    }
    xy_text[1] <- xy_text[1] + offset_x / 2
    box_bbox <- list(
      xleft   = xy[1],
      ybottom = xy[2] - (htext / 2 + offset_y / 2),
      xright  = xy[1] + (wtext + offset_x),
      ytop    = xy[2] + (htext / 2 + offset_y / 2)
    )
  }

  if (pos == 8) {
    adj_y <- 1
    if (align == "right") {
      adj_x <- 1
      xy_text <- c(xy[1] + wtext, xy[2])
    }
    if (align == "left") {
      adj_x <- 0
      xy_text <- c(xy[1], xy[2])
    }
    if (align == "center") {
      adj_x <- .5
      xy_text <- c(xy[1] + wtext / 2, xy[2])
    }
    xy_text[1] <- xy_text[1] + offset_x / 2
    xy_text[2] <- xy_text[2] - offset_y / 2
    box_bbox <- list(
      xleft   = xy[1],
      ybottom = xy[2] - (htext + offset_y),
      xright  = xy[1] + (wtext + offset_x),
      ytop    = xy[2]
    )
  }


  display_line(
    line = line, pos = pos, box = box, x = x, y = y, xy = xy,
    offset_x = offset_x, offset_y = offset_y, lwd = lwd,
    col_line = col_line, arrow = arrow, cex_arrow, offset = offset,
    htext = htext, wtext = wtext, clockwise = clockwise
  )

  display_text(
    x = xy_text[1], y = xy_text[2], txt = txt, col_txt = col_txt,
    cex = cex, font = font, family = family, adj = c(adj_x, adj_y),
    offset_x = offset_x, offset_y = offset_y,
    box = box, box_bbox = box_bbox, col_box = col_box,
    col_box_border = col_box_border, lwd_box = lwd_box, halo = halo,
    cex_halo = cex_halo, col_halo = col_halo
  )
}

#' @title Plot a raster
#' @description Plot a raster object (`SpatRaster` from `terra`).
#' @name mf_raster
#' @param x a `SpatRaster`
#' @param type type of raster map, one of "continuous", "classes", or
#' "interval". Default type for a numeric and categorial raster are
#' "continuous" and "classes" respectively.
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param add whether to add the layer to an existing plot (TRUE) or
#' not (FALSE)
#' @param leg_horiz display the legend horizontally
#' @param breaks either a numeric vector with the actual breaks
#' (for type = "continuous" and type = "interval"), or a
#' classification method name (for type = "interval" only; see [mf_get_breaks]
#' for details).
#' @param nbreaks number of classes (for type = "interval" only)
#' @param val_order modalities order in the legend, a character vector that
#' matches `var` modalities. Default to alphabetic order of modalities (for
#' type = "classes" only).
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals]. The default palette is the pal_quali palette for
#' type = "classes" and pal_seq otherwise (see [mf_theme]).
#' @param alpha pal` opacity, in the range \[0,1\] (0 means
#' transparent and 1 means opaque). Default is set to 1.
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param bg background color of the map, hex code or color name given by
#' [colors], ignored if `add = TRUE`
#'
#' @param leg_pos position of the legend, one of 'topleft', 'top','topright',
#' 'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two
#' coordinates in map units (c(x, y)). Use `NA` to avoid plotting the legend,
#' use 'interactive' to choose the legend position by clicking on the map.
#' @param leg_title legend title
#' @param leg_title_cex size of the title
#' @param leg_val_cex size of the values
#' @param leg_val_rnd number of decimal places of the values displayed in the
#' legend
#' @param leg_val_dec	decimal separator
#' @param leg_val_big	thousands separator
#' @param leg_frame	whether to add a frame to the legend (TRUE) or not (FALSE)
#' @param leg_frame_border border color of the legend frame
#' @param leg_horiz	display the legend horizontally (for proportional symbols
#' and choropleth types)
#' @param leg_adj	adjust the position of the legend in x and y directions
#' @param leg_bg color of the legend background
#' @param leg_fg	color of the legend foreground
#' @param leg_size size of the legend. Combine this argument with
#' `leg_title_cex` and `leg_val_cex`.
#' @param leg_box_border border color of legend boxes (for types related to
#' choropleth and typology)
#' @param leg_box_cex	width and height size expansion of boxes
#' @param ... bgalpha, smooth, maxcell or other arguments passed to
#' `terra::plotRGB` or `terra::plot`
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' if (require("terra")) {
#'   # multi band
#'   logo <- rast(system.file("ex/logo.tif", package = "terra"))
#'   mf_raster(logo)
#'
#'   # one band
#'   elev <- rast(system.file("ex/elev.tif", package = "terra"))
#'
#'   ## continuous
#'   mf_raster(elev)
#'   mf_raster(elev,
#'     pal = "Burg", expandBB = c(.2, 0, 0, 0),
#'     leg_pos = "bottom", leg_horiz = TRUE
#'   )
#'
#'   ## continuous with colors and breaks
#'   mf_raster(elev,
#'     type = "continuous",
#'     breaks = c(141, 400, 547),
#'     pal = c("darkseagreen1", "black", "red")
#'   )
#'
#'   ## interval
#'   mf_raster(elev,
#'     type = "interval",
#'     nbreaks = 5, breaks = "equal", pal = "Teal"
#'   )
#'
#'   ## classes
#'   elev2 <- classify(elev, c(140, 400, 450, 549))
#'   lev_evel <- data.frame(ID = 0:2, elevation = c("Low", "High", "Super High"))
#'   levels(elev2) <- lev_evel
#'   mf_raster(elev2)
#'   mf_raster(elev2,
#'     pal = c("salmon4", "olivedrab", "yellow3"),
#'     val_order = c("Super High", "High", "Low")
#'   )
#' }
mf_raster <- function(x,
                      type,
                      nbreaks,
                      breaks = "equal",
                      val_order,
                      pal,
                      alpha = NULL,
                      rev = FALSE,
                      expandBB = rep(0, 4),
                      bg,
                      leg_pos = "right",
                      leg_title = names(x),
                      leg_title_cex = .8,
                      leg_val_cex = .6,
                      leg_val_rnd = 1,
                      leg_val_dec = ".",
                      leg_val_big = "",
                      leg_frame = FALSE,
                      leg_frame_border,
                      leg_horiz = FALSE,
                      leg_adj = c(0, 0),
                      leg_box_border,
                      leg_box_cex = c(1, 1),
                      leg_fg,
                      leg_bg,
                      leg_size = 1,
                      add = FALSE,
                      ...) {
  # test for terra
  if (!requireNamespace("terra", quietly = TRUE)) {
    stop(paste0(
      "'terra' package is needed for this function to work.",
      "Please install it."
    ), call. = FALSE)
  }

  # input test
  if (!inherits(x, "SpatRaster")) {
    stop(paste0("x should be a SpatRaster."), call. = FALSE)
  }


  bg <- ifelse(missing(bg), getOption("mapsf.background"), bg)
  leg_box_border <- go(leg_box_border, "highlight", "#333333")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )


  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))


  # catch arguments
  ops <- list(...)
  ops$x <- x
  ops$add <- TRUE
  max_cell <- is.null(ops$maxcell) | terra::ncell(x) >= 1e6
  ops$maxcell <- ifelse(max_cell, 1e6, ops$maxcell)
  ops$bgalpha <- ifelse(is.null(ops$bgalpha), 0, ops$bgalpha)
  ops$legend <- ifelse(is.null(ops$legend), FALSE, ops$legend)
  ops$axes <- ifelse(is.null(ops$axes), FALSE, ops$axes)
  ops$box <- ifelse(is.null(ops$box), FALSE, ops$box)
  ops$mar <- NA
  ops$alpha <- alpha
  if (isTRUE(add)) {
    ops$xlim <- par("usr")[1:2]
    ops$ylim <- par("usr")[3:4]
  } else {
    ops$xlim <- terra::ext(x)[1:2]
    ops$ylim <- terra::ext(x)[3:4]
  }

  # Multiband Raster
  if (terra::nlyr(x) >= 2) {
    mf_raster_multiband(ops, expandBB, bg, add)
  }

  ops$clip <- FALSE
  # One band raster
  if (terra::nlyr(x) == 1) {
    # set the type - default to continuous for numeric raster
    if (missing(type)) {
      ops$type <- ifelse(is.null(terra::cats(x)[[1]]), "continuous", "classes")
    } else {
      if (!type %in% c("continuous", "interval", "classes")) {
        stop("'type' should be one of 'continuous', 'interval' or 'classes'.",
          call. = FALSE
        )
      }
      ops$type <- type
    }

    # common args
    ops_leg <- mget(ls(pattern = "leg_"))
    ops$smooth <- ifelse(is.null(ops$smooth), FALSE, ops$smooth)

    if (ops$type == "interval") {
      mf_raster_interval(
        ops, ops_leg, pal, breaks, nbreaks, alpha, rev, add, expandBB, bg
      )
    }

    if (ops$type == "continuous") {
      mf_raster_continuous(
        ops, ops_leg, breaks, pal, expandBB, add, alpha, rev, bg
      )
    }

    if (ops$type == "classes") {
      mf_raster_classes(
        ops, ops_leg, pal, val_order, expandBB, add, alpha, rev, bg
      )
    }
  }

  return(invisible(x))
}

#' @title Plot a legend
#' @description
#' Plot different types of legend. The "type" argument defines the
#' legend type.
#' Please note that some arguments are available for all types of legend and
#' some others are only relevant for specific legend types (see Details).
#' `mf_legend()` is a wrapper for `maplegend::leg()`.
#' @param type type of legend:
#' * **prop** for proportional symbols,
#' * **choro** for choropleth maps,
#' * **cont** for continuous maps (e.g. raster),
#' * **typo** for typology maps,
#' * **symb** for symbols maps,
#' * **prop_line** for proportional lines maps,
#' * **grad_line** for graduated lines maps,
#' * **histo** for histograms,
#' * **choro_point** for choropleth points maps,
#' * **choro_line** for choropleth lines maps,
#' * **choro_symb** for choropleth on symbols maps
#' * **typo_line** for typology lines maps.
#' @param val
#' vector of value(s) (for "prop" and "prop_line", at least c(min, max)
#' for "cont"),
#' vector of categories (for "symb", "typo", "typo_line"),
#' break labels (for "choro", "choro_point", "choro_line", "choro_symb",
#' and "grad_line"),
#' histogram parameters (for "histo").
#' @param pos position of the legend. It can be one of 'topleft', 'top',
#' 'topright', 'right', 'bottomright', 'bottom','bottomleft',
#' 'left', 'interactive' or a vector of two coordinates
#' in map units (c(x, y)).
#' @param pal a set of colors (hex codes) or a palette name (valid palette
#' names can be obtained with [hcl.pals].
#' @param alpha if `pal` is a valid palette name, the
#' alpha-transparency level in the range \[0,1\]
#' @param inches size of the largest symbol (radius for circles, half width
#' for squares) in inches
#' @param val_max maximum value corresponding to the largest symbol
#' @param border symbol border color(s)
#' @param symbol type of symbols, 'circle' or 'square'
#' @param self_adjust if TRUE values are self-adjusted to keep min, max and
#' intermediate rounded values
#' @param title title of the legend
#' @param title_cex size of the legend title
#' @param val_cex size of the values in the legend
#' @param val_rnd number of decimal places of the values in
#' the legend
#' @param val_dec decimal separator
#' @param val_big thousands separator
#' @param frame if TRUE the legend is plotted within a frame
#' @param no_data if TRUE a "missing value" box is plotted
#' @param no_data_txt label for missing values
#' @param bg background color of the legend
#' @param fg foreground color of the legend
#' @param box_border border color of legend boxes
#' @param box_cex width and height size expansion of boxes, histogram
#' circles, squares or lines
#' @param return_bbox return only bounding box of the legend.
#' No legend is plotted.
#' @param col color of the symbols (for "prop") or color of the lines (for
#' "prop_line" and "grad_line")
#' @param lwd width(s) of the symbols borders (for "prop", "symb",
#' "choro_point", "choro_symb"),
#' width of the largest line (for "prop_line"), line width (for "choro_line"
#' and "typo_line"), vector of line widths (for "grad_line")
#' @param size size of the legend; 2 means two times bigger
#' @param cex size(s) of the symbols
#' @param pch type(s) of the symbols (0:25)
#' @param col_na color for missing values
#' @param cex_na size of the symbols for missing values
#' @param pch_na type of the symbols for missing values
#' @param horiz if TRUE plot an horizontal legend
#' @param adj adjust the position of the legend in x and y directions
#' @param frame_border border color of the frame
#' @return No value is returned, a legend is displayed
#' (except if `return_bbox` is used).
#' @importFrom maplegend leg
#' @export
#' @details
#' Some arguments are available for all types of legend: `val`, `pos`, `title`,
#' `title_cex`, `val_cex`, `frame`, `bg`, `fg`, `size`, `adj`, `alpha`,
#' `return_bbox`).
#'
#' Relevant arguments for each specific legend types:
#' * `mf_legend(type = "prop", val, inches, val_max, symbol, col, lwd, border, val_rnd, val_big, val_dec, self_adjust, horiz)`
#' * `mf_legend(type = "choro", val, pal, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt, box_border, box_cex, horiz)`
#' * `mf_legend(type = "cont", val, pal, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt, box_border, box_cex, horiz)`
#' * `mf_legend(type = "typo", val, pal, col_na, no_data, no_data_txt, box_border, box_cex)`
#' * `mf_legend(type = "symb", val, pal, pch, cex, lwd, pch_na, cex_na, col_na, no_data, no_data_txt)`
#' * `mf_legend(type = "prop_line", val, col, lwd, val_rnd, val_big, val_dec)`
#' * `mf_legend(type = "grad_line", val, col, lwd, val_rnd, val_big, val_dec)`
#' * `mf_legend(type = "histo", val, pal, box_border, val_rnd, val_big, val_dec)`
#' * `mf_legend(type = "choro_point", val, pal, symbol, border, cex, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt, horiz)`
#' * `mf_legend(type = "choro_line", val, pal, lwd, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt)`
#' * `mf_legend(type = "choro_symb", val, pal, pch, lwd, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt)`
#' * `mf_legend(type = "typo_line", val, pal, lwd, col_na, no_data, no_data_txt, box_cex)`

#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_legend(type = "prop", pos = "topright", val = c(1, 5, 10), inches = .3)
#' mf_legend(
#'   type = "choro", pos = "bottomright", val = c(10, 20, 30, 40, 50),
#'   pal = hcl.colors(4, "Reds 2")
#' )
#' mf_legend(
#'   type = "typo", pos = "topleft", val = c("A", "B", "C", "D"),
#'   pal = hcl.colors(4, "Dynamic")
#' )
#' mf_legend(
#'   type = "symb", pos = "bottomleft", val = c("A", "B", "C"),
#'   pch = 21:23, cex = c(1, 2, 2),
#'   pal = hcl.colors(3, "Dynamic")
#' )
#' mf_legend(
#'   type = "grad_line", pos = "top", val = c(1, 2, 3, 4, 10, 15),
#'   lwd = c(0.2, 2, 4, 5, 10)
#' )
#' mf_legend(type = "prop_line", pos = "bottom", lwd = 20, val = c(5, 50, 100))
mf_legend <- function(type,
                      val,
                      pos = "left",
                      pal = "Inferno",
                      alpha = 1,
                      col = "tomato4",
                      inches = .3,
                      val_max = NULL,
                      symbol = "circle",
                      self_adjust = FALSE,
                      lwd = 0.7,
                      border = "#333333",
                      pch = seq_along(val),
                      cex = rep(1, length(val)),
                      title = "Legend Title",
                      title_cex = 0.8 * size,
                      val_cex = 0.6 * size,
                      val_rnd = 0,
                      val_dec = ".",
                      val_big = "",
                      col_na = "white",
                      cex_na = 1,
                      pch_na = 4,
                      no_data = FALSE,
                      no_data_txt = "No Data",
                      box_border = "#333333",
                      box_cex = c(1, 1),
                      horiz = FALSE,
                      frame_border,
                      frame = FALSE,
                      bg,
                      fg,
                      size = 1,
                      return_bbox = FALSE,
                      adj = c(0, 0)) {
  test_cur_plot()
  args <- match.call()
  args$mar <- getOption("mapsf.mar")
  args$bg <- go(bg, "foreground", getOption("mapsf.background"))
  args$fg <- go(fg, "highlight")
  args$frame_border <- go(frame_border, "foreground", getOption("mapsf.highlight"))
  args[[1]] <- quote(maplegend::leg)
  r <- eval.parent(args)
  return(invisible(r))
}

#' @title Package description
#' @name mapsf
#' @description
#' Create and integrate thematic maps in your workflow. This package
#' helps to design various cartographic representations such as proportional
#' symbols, choropleth or typology maps. It also offers several functions to
#' display layout elements that improve the graphic presentation of maps
#' (e.g. scale bar, north arrow, title, labels). `mapsf` maps `sf` objects on
#' `base` graphics.
#'
#'
#' A "Get Started" **vignette** contains commented scripts on how to create
#' various maps: `vignette(topic = "mapsf", package = "mapsf")`
#'
#' @section Symbology:
#' These functions display cartographic layers.
#' - [mf_map()] Plot a map
#' - [mf_label()] Plot labels
#' - [mf_raster()] Plot a raster
#' - [mf_graticule()] Plot graticules
#'
#' @section Map layout:
#' These functions are dedicated to the map layout design.
#' - [mf_theme()] Set a theme
#' - [mf_shadow()] Plot a shadow
#' - [mf_background()] Plot a background image
#' - [mf_annotation()] Plot an annotation
#' - [mf_arrow()] Plot a north arrow
#' - [mf_credits()] Plot credits
#' - [mf_layout()] Plot a map layout
#' - [mf_title()] Plot a title
#' - [mf_scale()] Plot a scale bar
#' - [mf_inset_on()] / [mf_inset_off()] Plot an inset
#' - [mf_worldmap()] Plot a point on a world map
#' - [mf_legend()] Plot a legend
#'
#' @section Utility functions:
#' - [mf_svg()] Export a map in SVG file format
#' - [mf_png()] Export a map in SVG file format
#' - [mf_distr()] Plot a distribution
#' - [mf_get_links()] Get a link layer from a data.frame of links
#' - [mf_get_pal()] Get color palettes
#' - [mf_get_breaks()] Get class intervals
#' - [mf_get_mtq()] Get the 'mtq' dataset
#' - [mf_get_ratio()] Get map width and height values
#' - [mf_get_pencil()] Get a pencil layer from polygons
#' - [mf_get_borders()] Get a border layer from polygons
#
#'
#' @docType package
"_PACKAGE"

#' @title Get the 'mtq' dataset
#' @name mf_get_mtq
#' @description Import the mtq dataset (Martinique municipalities).
#' @param x one of "polygons", "points", "lines"
#' @details
#' This a wrapper around `st_read(system.file("gpkg/mtq.gpkg", package = "mapsf"), layer = x, quiet = TRUE)`.
#' ## For polygons (municipalities shapes) and points (municipalities centroids)
#' - **INSEE_COM**: Municipality identifier
#' - **STATUS**: Municipality administrative status
#' - **LIBGEO**: Municipality name
#' - **POP**: Total population, 2015
#' - **MED**: Median disposable income adjusted per equivalent household member, in euros, 2015
#' - **CHOM**: Unemployed population, 2015
#' - **ACT**: Active population, 2015
#'
#' ## For lines (professional mobility flows from Fort-de-France to other municipalities)
#' - **i**: Municipality of residence identifier
#' - **j**: Municipality of workplace identifier
#' - **fij**: Flows of workers (employed population, 15 y.o. or more, 2015, only flows \> 100)
#' - **sj**: Administrative status of the workplace municipality
#' @source
#' ## For polygons (municipalities shapes) and points (municipalities centroids)
#' [Base comparateur de territoires](https://www.insee.fr/fr/statistiques/2521169)
#' (data, upload date: 2018-09-25) & ADMIN EXPRESS-COG (geometry, 2018 edition).
#'
#' Citation: Insee and IGN, 2018
#' ## For lines (professional mobility flows from Fort-de-France to other municipalities)
#' [Flux de mobilité - déplacements domicile-lieu de travail](https://www.insee.fr/fr/statistiques/3566477) (upload date: 2018-08-01)
#'
#' Citation: Insee, 2018
#'
#' @return an sf object
#' @export
#' @examples
#' mtq <- mf_get_mtq()
mf_get_mtq <- function(x = "polygons") {
  st_read(system.file("gpkg/mtq.gpkg", package = "mapsf"),
    layer = x, quiet = TRUE
  )
}

#' @title Plot a frame
#' @description Plot a frame around an existing map.
#' @param extent type of frame, either 'map' or 'figure'
#' @param col line color
#' @param lwd line width
#' @param lty line type
#' @param ... other arguments from [graphics::box()]
#'
#' @return No return value, a frame is displayed.
#' @export
#'
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_title()
#' mf_frame(extent = "map")
#' mf_map(mtq)
#' mf_title()
#' mf_frame(extent = "figure")
mf_frame <- function(extent = "map", col, lwd = 1.5, lty = 1, ...) {
  test_cur_plot()
  if (missing(col)) {
    col <- getOption("mapsf.highlight")
  }

  if (extent == "map") {
    inner <- as.numeric(!getOption("mapsf.title_inner"))
    mar <- getOption("mapsf.mar")
    line <- getOption("mapsf.title_line")
    top <- -line * inner
    mar <- mar + c(0, 0, ifelse(top > 0, 0, top), 0)
    mar[mar < 0] <- 0
    op <- par(mar = mar, no.readonly = TRUE)
    on.exit(par(op))
    box(which = "plot", col = col, lwd = lwd, lty = lty, ...)
  }
  if (extent == "figure") {
    box(which = "figure", col = col, lwd = lwd, lty = lty, ...)
  }
  return(invisible(NULL))
}

#' @title Deprecated - Plot symbols
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "symb"`
#' instead.
#'
#' Plot symbols based on qualitative data.
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'border',
#' 'lwd',
#' 'add',
#' 'pch',
#' 'cexs',
#' 'val_order',
#' 'col_na',
#' 'cex_na',
#' 'pch_na',
#' 'pal',
#' 'alpha',
#' 'rev',
#' 'leg_pos',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'leg_no_data',
#' 'leg_frame_border',
#' 'leg_fg',
#' 'leg_bg',
#' 'leg_size',
#' 'leg_frame',
#' 'leg_adj'))
#' @keywords internal
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_map(mtq, "STATUS", "symb")
#'
#' mtq[6, "STATUS"] <- NA
#' mf_map(mtq)
#' mf_map(
#'   x = mtq, var = "STATUS", type = "symb",
#'   pch = c(21:23), pal = c("red1", "tan1", "khaki1"),
#'   border = "grey20", cex = c(1.5, 1, .9), lwd = .5,
#'   val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
#'   pch_na = 24, leg_frame = TRUE
#' )
mf_symb <- function(x, var,
                    pal,
                    alpha = NULL,
                    rev = FALSE,
                    border,
                    pch,
                    cex = 2,
                    lwd = .7,
                    col_na = "grey",
                    pch_na = 4,
                    cex_na = 1,
                    val_order,
                    extent = x,
                    bg,
                    expandBB = rep(.04, 4),
                    leg_pos = mf_get_leg_pos(x),
                    leg_title = var,
                    leg_title_cex = .8,
                    leg_val_cex = .6,
                    leg_no_data = "No data",
                    leg_frame = FALSE,
                    leg_frame_border,
                    leg_adj = c(0, 0),
                    leg_fg,
                    leg_bg,
                    leg_size = 1,
                    add = TRUE) {
  deprecate_direct_calls_to("mf_symb")
  xtype <- get_geom_type(x)
  # linestring special case
  if (xtype == "LINE") {
    message("This map type is not available for lines.")
    return(invisible(NULL))
  }

  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")
  border <- go(border, "background")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )
  pal <- go(pal, "pal_quali", "Dynamic")


  xout <- x
  # Transform to point
  st_geometry(x) <- st_centroid(st_geometry(x), of_largest_polygon = TRUE)

  # get modalities
  val_order <- get_modalities(
    x = x[[var]],
    val_order = val_order
  )
  # get color list and association
  pal <- get_the_pal(
    pal = pal, nbreaks = length(val_order),
    alpha = alpha, rev = !rev
  )
  # get color vector
  mycols <- get_col_typo(
    x = x[[var]], pal = pal,
    val_order = val_order
  )

  if (missing(pch)) {
    pchs <- c(21:25, 0:20, 32:127)
    pch <- pchs[seq_along(val_order)]
  }
  if (length(pch) != length(val_order)) {
    message(paste0(
      "The length of pch does not match the number of ",
      "modalities. The first pch is used for all modalities."
    ))
    pch <- rep(pch[1], length(val_order))
  }

  if (length(cex) != length(val_order)) {
    if (length(cex) != 1) {
      message(paste0(
        "The length of cex does not match the number of ",
        "modalities. The first cex is used for all modalities."
      ))
    }
    cex <- rep(cex[1], length(val_order))
  }

  # get symbol vector
  mysym <- get_sym_typo(
    x = x[[var]],
    pch = pch,
    val_order = val_order
  )
  # get symbol vector
  mycex <- get_sym_typo(
    x = x[[var]],
    pch = cex,
    val_order = val_order
  )

  no_data <- FALSE
  if (max(is.na(mycols)) == 1) {
    no_data <- TRUE
  }
  mycols[is.na(mycols)] <- col_na
  mysym[is.na(mysym)] <- pch_na
  mycex[is.na(mycex)] <- cex_na


  border <- border[[1]]
  lwd <- lwd[[1]]

  mycolspt <- mycols
  mycolspt[mysym %in% 21:25] <- border
  mycolsptbg <- mycols
  border <- rep(border, sum(pch %in% 21:25))

  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  plot(st_geometry(x),
    col = mycolspt, bg = mycolsptbg, cex = mycex, pch = mysym,
    lwd = lwd, add = TRUE
  )

  leg(
    type = "symb",
    pos = leg_pos, val = val_order, title = leg_title,
    title_cex = leg_title_cex,
    val_cex = leg_val_cex, col_na = col_na, no_data = no_data,
    no_data_txt = leg_no_data,
    frame = leg_frame, border = border, pal = pal, lwd = lwd,
    cex = cex, pch = pch, cex_na = cex_na,
    pch_na = pch_na, bg = leg_bg, fg = leg_fg, adj = leg_adj,
    size = leg_size, frame_border = leg_frame_border
  )
  return(invisible(xout))
}

#' @title Deprecated - Plot proportional symbols
#' @description
#' This function is deprecated. Please use `mf_map()` with `type = "prop"`
#' instead.
#'
#' Plot proportional symbols.
#' @eval my_params(c(
#' 'x',
#' 'var',
#' 'col',
#' 'alpha',
#' 'border',
#' 'lwd',
#' 'add' ,
#' 'inches',
#' 'val_max',
#' 'lwd_max',
#' 'symbol',
#' 'leg_pos',
#' 'leg_title',
#' 'leg_title_cex',
#' 'leg_val_cex',
#' 'leg_val_rnd',
#' 'leg_val_dec',
#' 'leg_val_big',
#' 'leg_frame',
#' 'leg_horiz',
#' 'leg_frame_border',
#' 'leg_bg',
#' 'leg_fg',
#' 'leg_size',
#' 'leg_adj'))
#' @keywords internal
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_map(mtq, "POP", "prop")
#'
#' mf_map(mtq)
#' mf_map(
#'   x = mtq, var = "POP", type = "prop",
#'   inches = .4, symbol = "circle", val_max = 90000,
#'   col = "tomato1", border = "blue", lwd = 1,
#'   leg_pos = "right", leg_title = "Population",
#'   leg_title_cex = 1, leg_val_cex = .8, leg_val_rnd = 0,
#'   leg_frame = TRUE, add = TRUE
#' )
mf_prop <- function(x,
                    var,
                    inches = 0.3,
                    val_max,
                    lwd_max = 20,
                    symbol = "circle",
                    col,
                    alpha = NULL,
                    border,
                    lwd = .7,
                    extent = x,
                    bg,
                    expandBB = rep(.04, 4),
                    leg_pos = mf_get_leg_pos(x),
                    leg_title = var,
                    leg_title_cex = .8,
                    leg_val_cex = .6,
                    leg_val_rnd = 0,
                    leg_val_dec = ".",
                    leg_val_big = "",
                    leg_frame = FALSE,
                    leg_frame_border,
                    leg_horiz = FALSE,
                    leg_adj = c(0, 0),
                    leg_fg,
                    leg_bg,
                    leg_size = 1,
                    add = TRUE) {
  deprecate_direct_calls_to("mf_prop")
  # default
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  bgc <- go(bg, "background")
  col <- go(col, "highlight", "tomato4")
  border <- go(border, "background")
  leg_fg <- go(leg_fg, "highlight")
  leg_bg <- go(leg_bg, "foreground", getOption("mapsf.background"))
  leg_frame_border <- go(
    leg_frame_border, "foreground",
    getOption("mapsf.highlight")
  )
  border <- border[[1]]

  if (!is.null(alpha)) {
    col <- get_hex_pal(col, alpha)
  }

  xtype <- get_geom_type(x)
  # linestring special case
  if (xtype == "LINE") {
    xl <- x[!is.na(x[[var]]), ]
    if (!missing(val_max)) {
      maxval <- val_max
    } else {
      maxval <- max(xl[[var]])
    }
    xl$lwd <- xl[[var]] * lwd_max / maxval
    if (add == FALSE) {
      mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
    }
    op2 <- par(lend = 1)
    mf_map(xl, lwd = xl$lwd, add = TRUE, col = col)

    val <- seq(min(xl[[var]]), max(xl[[var]]), length.out = 4)
    leg(
      type = "prop_line",
      pos = leg_pos,
      val = val,
      lwd = max(xl$lwd),
      col = col,
      title = leg_title,
      title_cex = leg_title_cex,
      val_cex = leg_val_cex,
      val_rnd = leg_val_rnd,
      val_dec = leg_val_dec,
      val_big = leg_val_big,
      frame = leg_frame,
      bg = leg_bg,
      fg = leg_fg,
      adj = leg_adj,
      frame_border = leg_frame_border,
      size = leg_size
    )

    par(op2)

    return(invisible(x))
  }


  # check merge and order
  dots <- create_dots(x = x, var = var)

  # default col
  mycols <- rep(col, nrow(dots))

  # Default max value
  if (missing(val_max)) {
    val_max <- max(dots[[var]])
  }

  # get sizes
  sizes <- get_size(
    var = dots[[var]], inches = inches,
    val_max = val_max, symbol = symbol
  )

  # size and values for legend, hollow circle (fixmax case)
  size_max <- max(sizes)
  val <- seq(sqrt(min(dots[[var]])), sqrt(max(dots[[var]])), length.out = 4)
  val <- val * val

  if (inches <= size_max) {
    inches <- size_max
    borders <- border
  } else {
    mycols <- c(NA, mycols)
    borders <- c(NA, rep(border, nrow(dots)))
    dots <- rbind(dots[1, ], dots)
    dots[1, var] <- val_max
    sizes <- c(inches, sizes)
  }

  # empty plot if needed
  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }

  # Plot the symbols
  plot_symbols(
    symbol = symbol, dots = dots, sizes = xinch(sizes),
    mycols = mycols, border = borders, lwd = lwd,
    inches = inches
  )

  # symbols size
  leg(
    type = "prop",
    pos = leg_pos,
    val = val,
    title = leg_title,
    symbol = symbol,
    inches = size_max,
    col = col,
    title_cex = leg_title_cex,
    val_cex = leg_val_cex,
    val_rnd = leg_val_rnd,
    val_dec = leg_val_dec,
    val_big = leg_val_big,
    horiz = leg_horiz,
    frame = leg_frame,
    border = border,
    lwd = lwd,
    adj = leg_adj,
    bg = leg_bg,
    fg = leg_fg,
    self_adjust = TRUE,
    mar = getOption("mapsf.mar"),
    frame_border = leg_frame_border,
    size = leg_size
  )

  return(invisible(x))
}

#' @title Plot a north arrow
#' @description Plot a north arrow.
#' @name mf_arrow
#' @param pos position. It can be one of 'topleft', 'top','topright', 'right',
#' 'bottomright', 'bottom','bottomleft', 'left', 'interactive' or a vector of
#' two coordinates in map units (c(x, y))
#' @param col arrow color
#' @param cex arrow size
#' @param adj adjust the position of the north arrow in x and y directions
#' @param align object of class `sf` or `sfc` used to adjust the
#' arrow to the real north
#' @return No return value, a north arrow is displayed.
#' @export
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_arrow(pos = "topright")
mf_arrow <- function(pos = "topleft",
                     col,
                     cex = 1,
                     adj = c(0, 0),
                     align) {
  test_cur_plot()
  col <- go(col, "highlight")
  if (!missing(align)) {
    align <- st_crs(align)
  } else {
    align <- NULL
  }

  if (length(pos) == 1 && pos == "interactive") {
    mf_arrow_display(pos, col, cex, adj, align)
  } else {
    recordGraphics(
      {
        mf_arrow_display(pos, col, cex, adj, align)
      },
      list = list(
        pos = pos,
        col = col,
        cex = cex,
        adj = adj,
        align = align
      ),
      env = getNamespace("mapsf")
    )
  }
}


mf_arrow_display <- function(pos = "topleft",
                             col,
                             cex = 1,
                             adj = c(0, 0),
                             align) {
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))
  map_extent <- par("usr")
  xe <- map_extent[1:2]
  ye <- map_extent[3:4]
  inset_x <- xinch(par("csi")) / 2
  inset_y <- yinch(par("csi")) / 2
  n_arrow <- build_arrow(x = mean(xe), y = mean(ye), inset_x = inset_x * cex, inset_y = inset_y * cex)
  bb_n_arrow <- st_bbox(n_arrow)
  h <- bb_n_arrow[4] - bb_n_arrow[2]
  w <- bb_n_arrow[3] - bb_n_arrow[1]
  xe <- xe + c(inset_x, -inset_x) / 2
  ye <- ye + c(inset_y, -inset_y) / 2
  pos_a <- get_arrow_pos(pos, xe, ye, w, h) + adj * c(inset_x, inset_y) / 2
  north_arrow <- n_arrow + c(pos_a[1] - bb_n_arrow[1], pos_a[2] - bb_n_arrow[4])

  if (inherits(align, "crs")) {
    xcrs <- st_crs(align)
    a <- st_as_sf(
      x = data.frame(X = pos_a[1], Y = pos_a[2]), coords = c("X", "Y"),
      crs = xcrs, remove = FALSE
    )
    b <- st_as_sf(
      x = data.frame(st_coordinates(a) + c(0, -100000)),
      coords = c("X", "Y"), crs = xcrs, remove = FALSE
    )
    ap <- st_transform(a, 4326)
    cp <- st_as_sf(
      x = data.frame(st_coordinates(ap) + c(0, -2)),
      coords = c("X", "Y"), crs = 4326, remove = FALSE
    )
    cx <- st_transform(cp, xcrs)
    cx[, c("X", "Y")] <- st_coordinates(cx)
    delta_xb <- b$X - a$X
    delta_yb <- b$Y - a$Y
    delta_xc <- cx$X - a$X
    delta_yc <- cx$Y - a$Y
    v_x <- c(delta_xb, delta_yb)
    v_y <- c(delta_xc, delta_yc)
    theta <- acos(sum(v_x * v_y) / (sqrt(sum(v_x^2)) * sqrt(sum(v_y^2))))
    theta <- sign(cx$X - b$X) * theta
    rot <- function(a) matrix(c(cos(a), sin(a), -sin(a), cos(a)), 2, 2)
    n_arrow <- north_arrow * rot(-theta)
    bb_n_arrow <- st_bbox(n_arrow)
    h <- bb_n_arrow[4] - bb_n_arrow[2]
    w <- bb_n_arrow[3] - bb_n_arrow[1]
    pos_a <- get_arrow_pos(pos, xe, ye, w, h)
    north_arrow <- n_arrow + c(
      pos_a[1] - bb_n_arrow[1],
      pos_a[2] - bb_n_arrow[4]
    )
  }

  mf_map(north_arrow, col = col, border = col, add = TRUE)
}

get_arrow_pos <- function(pos, xe, ye, w, h) {
  if (is.numeric(pos) && length(pos) == 2) {
    xarrow <- pos[1]
    yarrow <- pos[2]
  } else {
    switch(pos,
      topleft = {
        xarrow <- xe[1]
        yarrow <- ye[2]
      },
      bottomright = {
        xarrow <- xe[2] - w
        yarrow <- ye[1] + h
      },
      topright = {
        xarrow <- xe[2] - w
        yarrow <- ye[2]
      },
      bottomleft = {
        xarrow <- xe[1]
        yarrow <- ye[1] + h
      },
      top = {
        xarrow <- xe[1] + diff(xe) / 2 - w * .5
        yarrow <- ye[2]
      },
      bottom = {
        xarrow <- xe[1] + diff(xe) / 2 - w
        yarrow <- ye[1] + h
      },
      left = {
        xarrow <- xe[1]
        yarrow <- ye[1] + diff(ye) / 2 + h * 0.5
      },
      right = {
        xarrow <- xe[2] - w
        yarrow <- ye[1] + diff(ye) / 2 + h * 0.5
      },
      interactive = {
        iar <- interleg(txt = c("arrow", "Arrow"))
        xarrow <- iar[1]
        yarrow <- iar[2]
      }
    )
  }
  return(c(xarrow, yarrow))
}


build_arrow <- function(x, y, inset_x, inset_y) {
  x_triangle <- c(
    x,
    x + inset_x / 2,
    x + inset_x,
    x + inset_x / 2,
    x
  )
  y_triangle <- c(
    y - inset_y,
    y,
    y - inset_y,
    y - inset_y * .9,
    y - inset_y
  )
  triangle <- st_polygon(list(matrix(
    data = c(x_triangle, y_triangle),
    nrow = 5, ncol = 2, byrow = FALSE
  )))
  x_n <- c(
    x + inset_x / 4,
    x + inset_x / 4,
    x + inset_x / 4 + inset_x / 2,
    x + inset_x / 4 + inset_x / 2
  )
  y_n <- c(
    y - inset_y - inset_y * .75 - inset_y / 3,
    y - inset_y - inset_y / 3,
    y - inset_y - inset_y * .75 - inset_y / 3,
    y - inset_y - inset_y / 3
  )
  n <- st_multilinestring(list(matrix(
    data = c(x_n, y_n),
    nrow = 4, ncol = 2,
    byrow = FALSE
  )))
  n <- st_buffer(n, inset_x * 0.05,
    endCapStyle = "SQUARE",
    joinStyle = "MITRE", mitreLimit = 1
  )

  n_arrow <- st_multipolygon(list(triangle, n))

  return(n_arrow)
}

#' @title Get class intervals
#' @name mf_get_breaks
#' @description A function to classify continuous variables.
#'
#' This function is a wrapper for
#' \code{\link[classInt:classIntervals]{classIntervals}}
#' with some additional methods.
#'
#' @param x a vector of numeric values. NA and Inf values are not used in the
#' classification.
#' @param nbreaks a number of classes
#' @param breaks a classification method; the main methods are "quantile",
#' "equal", "msd", "ckmeans" (natural breaks), "Q6" and "geom".
#' See Details for the full list.
#' @param k number of standard deviation for "msd" method (see Details)
#' @param central creation of a central class for "msd" method (see Details)
#' @param ... further arguments
#' of \code{\link[classInt:classIntervals]{classIntervals}}
#' @seealso \link[classInt:classIntervals]{classIntervals}
#' @details
#'
#' ## `classInt` methods
#' "fixed", "sd", "equal", "pretty", "quantile", "kmeans", "hclust", "bclust",
#' "fisher", "jenks", "dpih", "headtails", "maximum", and "box"
#' are \code{\link[classInt:classIntervals]{classIntervals}}
#' methods. You may need to pass additional arguments for some of them.
#'
#' ## Natural breaks method
#' The "jenks", "fisher" and "ckmeans" methods are based on the same concept of
#' **natural breaks** and and produce similar groupings. The use of "ckmeans" is
#' recommended.
#'
#' * The "jenks" method produces class boundaries falling on data points and is
#' slow.
#' * The "fisher" method produces class boundaries located more conveniently
#' between data points, and is faster than the "jenks" method.
#' * The "ckmeans" method produces exactly the same class boundaries as the
#' "fisher" method, but is much faster. It uses the optimal univariate
#' k-means method from the `Ckmeans.1d.dp` package.
#' If the "ckmeans" method is selected but the `Ckmeans.1d.dp` package is not
#' installed then the "fisher" method is used.
#'
#' The relative speeds of these three methods may vary depending on the number
#' of data points and the number of classes.
#'
#' ## Other methods
#' The "msd" method is based on the **mean** and the **standard deviation**
#' of a numeric vector.
#' The \code{nbreaks} parameter is not relevant, use \code{k} and
#' \code{central} instead. \code{k} indicates
#' the extent of each class in share of standard deviation.
#' If \code{central=TRUE} then
#' the mean value is the center of a class else the mean is a break value.\cr\cr
#' The "q6" method uses the following \code{\link[stats:quantile]{quantile}}
#' probabilities: 0, 0.05, 0.275, 0.5, 0.725, 0.95, 1.\cr\cr
#' The "Q6" method uses the following \code{\link[stats:quantile]{quantile}}
#' probabilities: 0, 0.05, 0.25, 0.5, 0.75, 0.95, 1.\cr\cr
#' The "geom" method is based on a geometric progression along
#' the variable values, all values must be strictly greater than zero.\cr\cr
#' The "arith" method is based on an arithmetic progression along
#' the variable values.\cr\cr
#' The "em" method is based on nested averages computation.
#'
#' ## Class boundaries
#' Breaks defined by a numeric vector or a classification method are
#' left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
#' will be mapped as [2 - 5[, [5 - 10[, [10 - 15[, \[15 - 20].
#'
#' @examples
#' mtq <- mf_get_mtq()
#' mf_get_breaks(x = mtq$MED, nbreaks = 6, breaks = "quantile")
#' @return A numeric vector of breaks
#' @export
mf_get_breaks <- function(x, nbreaks, breaks, k = 1, central = FALSE, ...) {
  if (is.numeric(breaks)) {
    intervals <- sort(breaks)
    return(intervals)
  }

  x <- as.vector(na.omit(x))
  x <- x[is.finite(x)]
  custom_methods <- c("geom", "arith", "q6", "Q6", "em", "msd", "ckmeans")

  # default number of classes
  if (missing(nbreaks)) {
    nbreaks <- round(1 + 3.3 * log10(length(x)), 0)
  }

  if (breaks == "ckmeans") {
    if (!requireNamespace("Ckmeans.1d.dp", quietly = TRUE)) {
      warning(
        paste0(
          "The 'Ckmeans.1d.dp' package is needed for ",
          "this classification method. Please install it.\n",
          "The 'fisher' method will be used instead."
        ),
        call. = FALSE
      )
      breaks <- "fisher"
    } else {
      bks <- Ckmeans.1d.dp::Ckmeans.1d.dp(x = x, k = nbreaks)
      intervals <- Ckmeans.1d.dp::ahist(
        x = bks, style = "midpoints",
        data = x, plot = FALSE
      )$breaks
    }
  }

  if (!breaks %in% custom_methods) {
    intervals <- classIntervals(
      var = x,
      n = nbreaks,
      style = breaks, ...
    )$brks
  }

  if (breaks == "geom") {
    intervals <- min(x)
    if (intervals <= 0) {
      stop(
        paste0(
          "All values must be strictly greater ",
          "than 0 when using the 'geom' method."
        ),
        call. = FALSE
      )
    }
    intervals <- c(intervals, max(x))
    r <- exp((log(max(x)) - log(min(x))) / nbreaks) # raison
    tmp <- min(x)
    for (i in 1:(nbreaks - 1)) {
      intervals <- c(intervals, tmp * r)
      tmp <- tmp * r
      intervals <- sort(intervals)
    }
  }

  if (breaks == "arith") {
    intervals <- min(x)
    intervals <- c(intervals, max(x))
    r <- (max(x) - min(x)) / sum(1:nbreaks) # raison
    tmp <- min(x)
    for (i in 1:(nbreaks - 1)) {
      intervals <- c(intervals, tmp + r)
      tmp <- tmp + r
      intervals <- sort(intervals)
    }
  }

  if (breaks == "q6") {
    intervals <- as.vector(
      quantile(x = x, probs = c(0, .05, .275, .5, .725, .95, 1))
    )
  }

  if (breaks == "Q6") {
    intervals <- as.vector(
      quantile(x = x, probs = c(0, .05, .25, .5, .75, .95, 1))
    )
  }

  if (breaks == "em") {
    t <- bitwAnd(nbreaks, (nbreaks - 1))
    if (t != 0) {
      stop("The number of classes must be a power of 2")
    } else {
      min_vec <- min(x)
      max_vec <- max(x)
      it <- log2(nbreaks)

      int <- c(min_vec, max_vec)

      for (a in 1:it) {
        valprv <- c()
        for (i in 1:(length(int) - 1)) {
          if (i == 1) {
            sub_vec <- x[x >= int[i] & x <= int[i + 1]]
          } else {
            sub_vec <- x[x > int[i] & x <= int[i + 1]]
          }
          valprv <- c(valprv, mean(sub_vec))
        }
        int <- c(int, valprv)
        int <- int[order(int)]
      }
      intervals <- int
    }
  }

  if (breaks == "msd") {
    min_vec <- min(x)
    max_vec <- max(x)
    avg_vec <- mean(x)
    sd_vec <- sqrt(sum((x - avg_vec)^2) / length(x))
    if (central == FALSE) {
      pose <- ceiling((max_vec - avg_vec) / (sd_vec * k))
      nege <- ceiling((avg_vec - min_vec) / (sd_vec * k))
      avg_vec + (1:pose) * (sd_vec * k)
      bks <- c(
        avg_vec - (1:nege) * (sd_vec * k),
        avg_vec,
        avg_vec + (1:pose) * (sd_vec * k)
      )
      intervals <- c(min_vec, bks[bks > min_vec & bks < max_vec], max_vec)
    } else {
      pose <- ceiling((max_vec - (avg_vec + 0.5 * sd_vec * k)) / (sd_vec * k))
      nege <- ceiling(((avg_vec - 0.5 * sd_vec * k) - min_vec) / (sd_vec * k))
      bks <- c(
        (avg_vec - 0.5 * sd_vec * k) - (1:nege) * (sd_vec * k),
        (avg_vec - 0.5 * sd_vec * k),
        (avg_vec + 0.5 * sd_vec * k),
        (avg_vec + 0.5 * sd_vec * k) + (1:pose) * (sd_vec * k)
      )
      intervals <- c(min_vec, bks[bks > min_vec & bks < max_vec], max_vec)
    }
    intervals <- intervals[order(intervals)]
  }

  return(unique(intervals))
}

set.seed(46)
x <- runif(1000)
bks <- mf_get_breaks(x, nbreaks = 5, breaks = "equal")
r <- mf_distr(x)
expect_equal(r$bw, target = 0.04413086, tolerance = 1e-7)
expect_equal(r$nbins, target = 11)
expect_silent(mf_distr(x, nbins = 11))
expect_silent(mf_distr(x, nbins = 11, bw = .1))
expect_silent(mf_distr(x))
expect_silent(mf_distr(x, bw = .1))
expect_silent(mf_distr(x, nbins = 1000))
expect_silent(mf_distr(x, nbins = 2))
expect_silent(mf_distr(x = x, breaks = bks, pal = "Reds 2"))
expect_silent(mf_distr(x, yaxt = FALSE))

#' @title Plot a title
#' @param txt title text
#' @param pos position, one of 'left', 'center', 'right'
#' @param tab if TRUE the title is displayed as a tab
#' @param bg background of the title
#' @param fg foreground of the title
#' @param cex cex of the title
#' @param font font of the title
#' @param line number of lines used for the title
#' @param inner if TRUE the title is displayed inside the plot area;
#' if FALSE the title is displayed in the top margin
#' @param banner if TRUE the title is displayed as a banner
#' @export
#' @return No return value, a title is displayed.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_title()
mf_title <- function(txt = "Map Title", pos, tab,
                     bg, fg, cex, line, font,
                     inner, banner) {
  test_cur_plot()
  tab <- go(tab, "title_tab")
  pos <- go(pos, "title_pos")
  inner <- go(inner, "title_inner")
  line <- go(line, "title_line")
  cex <- go(cex, "title_cex")
  font <- go(font, "title_font")
  fg <- go(fg, "highlight", getOption("mapsf.background"))
  banner <- go(banner, "title_banner")

  if (isFALSE(banner)) {
    bg <- go(bg, opt = "background", legacy = bg)
  } else {
    bg <- go(bg, "foreground")
  }

  # correct line space for multiplot
  mmf <- par("mfrow")
  if (mmf[1] == 2 && mmf[2] == 2) {
    line <- line * 0.83
  }
  if (max(mmf) > 2) {
    line <- line * .66
  }

  recordGraphics(
    {
      mf_title_display(txt, pos, tab, bg, fg, cex, line, font, inner, banner)
    },
    list = list(
      txt = txt,
      pos = pos,
      tab = tab,
      bg = bg,
      fg = fg,
      cex = cex,
      line = line,
      font = font,
      inner = inner,
      banner = banner
    ),
    env = getNamespace("mapsf")
  )
}

mf_title_display <- function(txt = "Map Title", pos, tab,
                             bg, fg, cex, line, font,
                             inner, banner) {
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))
  # size refs
  pu <- par("usr")
  hbox <- line * 0.2 * yinch(1)
  inset_x <- xinch(par("csi")) / 4

  wtitle <- strwidth(txt, units = "user", cex = cex, font = font)

  # compute rect coord
  pw <- pu[2] - pu[1]
  pb <- pu[c(1, 3, 2, 4)]
  pb <- switch(pos,
    left = {
      pb[3] <- pu[1] + inset_x + wtitle + inset_x
      pb
    },
    right = {
      pb[1] <- pu[2] - (inset_x + wtitle + inset_x)
      pb[3] <- pu[2]
      pb
    },
    center = {
      pb[1] <- pu[1] + (pw / 2) - (wtitle / 2) - inset_x
      pb[3] <- pb[1] + wtitle + inset_x + inset_x
      pb
    }
  )
  # adjust box coord if inner
  if (inner) {
    pb[2] <- pu[4] - hbox
    pb[4] <- pu[4]
  } else {
    pb[2] <- pu[4]
    pb[4] <- pu[4] + hbox
  }
  # title coord
  pt <- c(pb[1] + inset_x, pb[2] + (hbox) / 2)

  # adjust box
  if (tab == FALSE) {
    pb[c(1, 3)] <- pu[1:2]
  }

  # display rect
  rect(
    xleft = pb[1],
    ybottom = pb[2] - hbox * 0.02,
    xright = pb[3],
    ytop = pb[4],
    col = bg,
    xpd = TRUE,
    border = NA
  )

  # display title
  text(
    x = pt[1],
    y = pt[2],
    labels = txt, adj = c(0, 0.5),
    cex = cex, col = fg,
    font = font, xpd = TRUE
  )
  f <- getOption("mapsf.frame")
  if (f %in% c("map", "figure")) {
    mf_frame(
      extent = f,
      col = getOption("mapsf.highlight"),
      lwd = getOption("mapsf.frame_lwd"),
      lty = getOption("mapsf.frame_lty")
    )
  }

  return(invisible(NULL))
}

#' @title Plot a distribution
#' @description
#' This function displays the statistical distribution of a variable with
#' a histogram, a box plot, a strip chart and a density curve on the same plot.
#'
#' This graphic can be useful to choose an appropriate classification method for
#' choropleth maps.
#'
#' User-defined class boundaries can also be displayed on the plot.
#'
#' @param x a numeric variable
#' @param nbins number of bins in the histogram
#' @param bw bandwidth of the density curve
#' @param breaks a vector of class boundaries. If `breaks` is used, the boxplot
#' is not displayed.
#' @param pal a color, a set of colors (hex codes) or a palette name.
#' Palette names can be obtained with [hcl.pals]. The default color is either
#' the highlight color if `breaks` is not used, or the background color
#' otherwise (see [mf_theme]).
#' @param rev logical indicating whether the ordering of the colors should be
#' reversed
#' @param alpha opacity, in the range \[0,1\] (0 means
#' transparent and 1 means opaque). Default is set to 1.
#' @param main plot title
#' @param yaxt if FALSE the y axis is not displayed
#' @param ylab y axis label
#' @return The number of bins of the histogram and the bandwidth of the density
#' curve are (invisibly) returned in a list.
#' @export
#' @seealso [mf_map_choro]
#' @examples
#' (mf_distr(rnorm(1000)))
#' mf_distr(rbeta(1000, .6, 7))
#' mf_distr(rbeta(1000, 5, .6))
#' a <- rbeta(1000, .6, 7)
#' bks <- mf_get_breaks(a, nbreaks = 5, breaks = "quantile")
#' mf_distr(a, breaks = bks)
#' mf_distr(a,
#'   breaks = bks, pal = "Teal", yaxt = FALSE,
#'   main = 'Classification method : "quantile"'
#' )
mf_distr <- function(x, nbins, bw, breaks, pal, alpha = 1, rev = FALSE,
                     main = "Distribution", yaxt = TRUE, ylab = "Density") {
  x <- as.numeric(x)
  x <- x[!is.na(x)]
  x <- x[is.finite(x)]

  if (missing(bw) && missing(nbins)) {
    bw <- bw.SJ(x)
    nbins <- round((max(x) - min(x)) / (2 * bw), 0)
  }
  if (!missing(bw) && missing(nbins)) {
    nbins <- round((max(x) - min(x)) / (2 * bw), 0)
  }
  if (missing(bw) && !missing(nbins)) {
    bw <- ((max(x) - min(x)) / nbins) / 2
  }

  if (nbins > 100) {
    nbins <- 100
  }
  if (nbins < 3) {
    nbins <- 3
  }

  hbks <- seq(min(x), max(x), length.out = nbins + 1)

  # graphic basic parameters
  d <- density(x, bw = bw)
  h <- hist(x, breaks = hbks, plot = FALSE)
  b <- boxplot(x, plot = FALSE)

  # graphic dimensions and labels
  y_labels <- pretty(c(min(d$y, h$density), max(d$y, h$density)))
  y_lim <- c(min(y_labels), max(y_labels))
  dim_y_lim <- diff(y_lim)
  small_offset <- dim_y_lim / 20
  large_offset <- dim_y_lim / 7
  y_lim[1] <- y_lim[1] - large_offset - small_offset
  x_labels <- pretty(x)
  x_lim <- c(min(x_labels), max(x_labels))

  # points coords
  pts_y <- runif(d$n, min = y_lim[1], max = y_lim[1] + large_offset)

  # curve coords
  dx <- d$x[d$x >= x_lim[1] & d$x <= x_lim[2]]
  dy <- d$y[d$x >= x_lim[1] & d$x <= x_lim[2]]

  # actual plot
  # empty
  plot(
    x = x_lim,
    y = y_lim,
    col = NA,
    axes = FALSE,
    xlab = "",
    ylab = ""
  )
  # background
  rcol <- getOption("mapsf.background")
  rborder <- getOption("mapsf.background")

  recordGraphics(
    {
      plt <- par("plt")
      usr <- par("usr")
      rect(
        par("usr")[1] - plt[1] * (usr[2] - usr[1]) / (plt[2] - plt[1]),
        par("usr")[3] - plt[3] * (usr[4] - usr[3]) / (plt[4] - plt[3]),
        par("usr")[2] + (1 - plt[2]) * (usr[2] - usr[1]) / (plt[2] - plt[1]),
        par("usr")[4] + (1 - plt[4]) * (usr[4] - usr[3]) / (plt[4] - plt[3]),
        col = rcol,
        border = rborder,
        xpd = TRUE
      )
    },
    list = list(rcol = rcol, rborder = rborder),
    env = getNamespace("mapsf")
  )

  # histogram
  hist(
    x,
    breaks = h$breaks,
    freq = FALSE,
    ylim = y_lim,
    xlim = x_lim,
    lwd = 1,
    axes = FALSE,
    xlab = "",
    main = "Distribution",
    col.lab = getOption("mapsf.highlight"),
    col.main = getOption("mapsf.highlight"),
    col.axis = getOption("mapsf.highlight"),
    col = getOption("mapsf.foreground"),
    border = getOption("mapsf.background"),
    lwd = .2,
    add = TRUE
  )
  title(
    main = main,
    ylab = ifelse(yaxt, ylab, ""),
    line = 2,
    col.lab = getOption("mapsf.highlight"),
    col.main = getOption("mapsf.highlight")
  )
  # smooth line
  lines(
    x = dx,
    y = dy,
    lwd = 2,
    col = go(opt = "highlight", legacy = "#cf0000")
  )
  axis(
    side = 1,
    at = x_labels,
    col.axis = getOption("mapsf.highlight"),
    col = getOption("mapsf.highlight"),
    col.ticks = getOption("mapsf.highlight")
  )

  if (isTRUE(yaxt)) {
    axis(
      side = 2,
      pos = x_lim[1],
      at = y_labels,
      las = 2,
      col.axis = getOption("mapsf.highlight"),
      col = getOption("mapsf.highlight"),
      col.ticks = getOption("mapsf.highlight")
    )
  }

  if (!missing(breaks)) {
    if (missing(pal)) {
      pal <- getOption("mapsf.foreground")
    }
    pal <- get_the_pal(pal = pal, nbreaks = length(breaks) - 1, alpha = alpha, rev = !rev)
    mycols <- get_col_vec(x = x, breaks = breaks, pal = pal, jen = FALSE)
    points(
      x = x,
      y = pts_y,
      pch = 21,
      cex = .7,
      lwd = .8,
      bg = mycols,
      col = getOption("mapsf.background")
    )
    segments(
      x0 = breaks,
      y0 = rep(y_lim[1] - small_offset / 2, length(breaks)),
      x1 = breaks,
      y1 = rep(y_lim[1] + large_offset + small_offset / 2, length(breaks)),
      col = getOption("mapsf.highlight"),
      lwd = 2
    )
  } else {
    points(
      x = x,
      y = pts_y,
      pch = 21,
      cex = .7,
      lwd = .8,
      bg = getOption("mapsf.highlight"),
      col = getOption("mapsf.background")
    )

    box_plot(
      x = b$stats[, 1],
      y = y_lim[1],
      large_offset = large_offset,
      small_offset = small_offset,
      col = getOption("mapsf.background"),
      lwd = 4
    )
    box_plot(
      x = b$stats[, 1],
      y = y_lim[1],
      large_offset = large_offset,
      small_offset = small_offset,
      col = getOption("mapsf.highlight"),
      lwd = 1.5
    )
  }

  return(invisible(list(bw = bw, nbins = round(nbins, 0))))
}

box_plot <- function(x, y, large_offset, small_offset, col, lwd) {
  # box
  polygon(
    x = x[c(2, 4, 4, 2)],
    y = c(
      y - small_offset / 2,
      y - small_offset / 2,
      y + large_offset + small_offset / 2,
      y + large_offset + small_offset / 2
    ),
    lwd = lwd,
    border = col
  )
  # Moustaches
  y_mid <- y + large_offset / 2
  segments(
    x0 = c(x[2], x[4]),
    y0 = c(y_mid, y_mid),
    x1 = c(x[1], x[5]),
    y1 = c(y_mid, y_mid),
    lty = 1,
    lwd = lwd,
    col = col
  )
  # Median
  segments(
    x0 = x[3],
    y0 = y - small_offset / 2,
    x1 = x[3],
    y1 = y[1] + large_offset + small_offset / 2,
    lty = 1,
    lwd = lwd + .8,
    col = col
  )
}

# base ----
#' @name mf_map_base
#' @title Plot a base map
#' @description
#' `mf_map()` can be used to display geographic layers (`sf` objects), using
#' the default map type **base**.
#'
#' ## Usage
#' For polygons:
#' ```r
#' mf_map(x, col, border, lwd = 0.7, lty = 1,
#'        alpha, expandBB, extent, bg, add = FALSE)
#' ```
#' For points:
#' ```r
#' mf_map(x, col, border, pch = 20, cex = 1, lwd = 0.7,
#'        alpha, expandBB, extent, bg, add = FALSE)
#' ```
#' For lines:
#' ```r
#' mf_map(x, col, lwd = .7, lty = 1,
#'        alpha, expandBB, extent, bg, add = FALSE)
#' ```
#' @param x	object of class `sf`, `sfc` or `sfg`
#' @param col a color, hex code or color name given by [colors].
#' The default color for polygons is the foreground color, the default color for
#' points and lines the highlight color (see [mf_theme]).
#' @param border border color for polygons and points symbols, hex code or
#' color name given by [colors]. The default color for polygon is the
#' highlight color, the default color for points is the foreground color (see
#' [mf_theme]).
#' @param lwd border width for polygons and points symbols, lines width
#' @param lty type of line for polygons borders and lines
#' @param pch type of symbol to use for points, see [pch]
#' @param cex symbols size, 2 means 2 times bigger
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @return x is (invisibly) returned.
#' @seealso [mf_map()]
#' @examples
#' mtq <- mf_get_mtq()
#' pts <- mf_get_mtq("points")
#' flows <- mf_get_mtq("lines")
#' mf_map(mtq, lty = 3)
#' mf_map(pts, col = "red", border = "white", pch = 21, add = TRUE)
#' mf_map(flows, col = "coral", lwd = 2, add = TRUE)
NULL

# prop ----
#' @name mf_map_prop
#' @title Plot proportional symbols
#' @description
#' With the **prop** map type, `mf_map()` displays symbols (squares or circles)
#' with areas proportional to a quantitative variable (stocks).
#'
#' For polygons, centroids are used to plot proportional symbols.
#'
#' ## Usage
#' For polygons and points:
#' ```r
#' mf_map(x, var, type = "prop",
#'        inches = 0.3, val_max, symbol, col, border, lwd,
#'        expandBB, extent, bg, alpha, add = FALSE, leg_*)
#'
#' ```
#' For lines:
#' ```r
#' mf_map(x, var, type = "prop",
#'        val_max, lwd_max = 20, col,
#'        expandBB, extent, bg, alpha, add = FALSE, leg_*)
#' ```
#' @param x	object of class `sf`
#' @param var name of the variable to map
#' @param type "prop"
#' @param inches size of the largest symbol in inches (radius for circles,
#' half width for squares)
#' @param val_max maximum value corresponding to the largest symbol or line
#' @param lwd_max width of the largest line
#' @param symbol type of proportional symbols, either "circle" or "square"
#' @param col color of the proportional symbols or lines, a hex code or
#' a color name given by [colors].
#' The default color is the highlight color (see [mf_theme]).
#' @param border border color for proportional symbols, a hex code or
#' color name given by [colors]. The default color the background color (see
#' [mf_theme]).
#' @param lwd border width of proportional symbols
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @param leg_* legend arguments described in [mf_map]
#' @return x is (invisibly) returned.
#' @seealso [mf_map()]
#' @examples
#' mtq <- mf_get_mtq()
#' flows <- mf_get_mtq("lines")
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_map(mtq, "POP", "prop",
#'   inches = .4, leg_title = "Population",
#'   leg_pos = "topright"
#' )
#' mf_map(flows, "fij", "prop",
#'   lwd_max = 10, col = "steelblue2",
#'   leg_pos = "right"
#' )
NULL

# choro ----
#' @name mf_map_choro
#' @title Plot a choropleth map
#' @description
#' With the **choro** map type, `mf_map()` displays a choropleth map.
#'
#' In choropleth maps, areas are shaded according to the variation of a
#' quantitative variable. They are used to represent ratios or indices.
#'
#' ## Usage
#' For polygons:
#' ```r
#' mf_map(x, var, type = "choro",
#'        breaks = "quantile", nbreaks, pal, rev = FALSE,
#'        border, lwd = 0.7, col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' For points:
#' ```r
#' mf_map(x, var, type = "choro",
#'        breaks = "quantile", nbreaks, pal, rev = FALSE,
#'        border, pch = 21, cex = 2, lwd = 0.7, col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' For lines:
#' ```r
#' mf_map(x, var, type = "choro",
#'        breaks = "quantile", nbreaks, pal, rev = FALSE, lwd = .7,
#'        col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' @param x	object of class `sf`
#' @param var name of the variable to map
#' @param type "choro"
#' @param breaks	either a numeric vector with the actual breaks, or a
#' classification method name. The main methods are 'quantile',
#' 'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
#' See [mf_get_breaks] for details.
#' @param nbreaks number of classes
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals]. The default palette is the pal_seq palette
#' (see [mf_theme]).
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param border border color for polygons and symbols, a hex code or
#' color name given by [colors]. The default color for polygons is the
#' highlight color, the default color for points is the background color (see
#' [mf_theme]).
#' @param lwd border width for polygons and points symbols, lines width
#' @param pch type of symbol to use for points, see [pch] (points only)
#' @param cex symbols size, 2 means 2 times bigger (points only)
#' @param col_na color for missing values, a hex code or
#' a color name given by [colors].
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @param leg_* legend arguments described in [mf_map]
#' @return x is (invisibly) returned.
#' @seealso [mf_map()], [mf_distr()], [mf_get_breaks()], [mf_get_pal()]
#' @examples
#' mtq <- mf_get_mtq()
#' pts <- mf_get_mtq("points")
#' flows <- mf_get_mtq("lines")
#' # polygons
#' mtq[6, "MED"] <- NA
#' mf_map(
#'   x = mtq, var = "MED", type = "choro",
#'   col_na = "grey90", pal = "Cividis",
#'   breaks = "equal", nbreaks = 5, border = "white",
#'   lwd = .5, leg_pos = "topleft",
#'   leg_title = "Median Income", leg_title_cex = 1,
#'   leg_val_cex = .9, leg_val_rnd = -2, leg_no_data = "No data",
#'   leg_box_cex = c(0.5, 3), leg_box_border = NA, leg_frame = FALSE
#' )
#' # points
#' mf_map(mtq)
#' mf_map(
#'   x = pts, var = "MED", type = "choro",
#'   pch = 21, cex = 3, lwd = 1.2,
#'   pal = "Teal", border = "white",
#'   leg_horiz = FALSE, leg_val_big = " ",
#'   leg_val_rnd = -2, leg_pos = "topright",
#'   leg_frame = TRUE, add = TRUE
#' )
#' # lines
#' mf_map(mtq, extent = flows)
#' mf_map(
#'   x = flows, var = "fij", type = "choro",
#'   breaks = "equal", nbreaks = 3, add = TRUE,
#'   lwd = 5, pal = "Burg", leg_horiz = TRUE,
#'   leg_box_cex = c(.7, 1),
#'   leg_val_rnd = 0, leg_pos = "bottomleft"
#' )
NULL

# typo ----
#' @name mf_map_typo
#' @title Plot a typology map
#' @description
#' With the **typo** map type, `mf_map()` displays a typology map.
#'
#' In typology maps, areas are shaded according to the modalities of a
#' qualitative variable.
#'
#' ## Usage
#' For polygons:
#' ```r
#' mf_map(x, var, type = "typo",
#'        pal, rev = FALSE, val_order,
#'        border, lwd = 0.7, col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' For points:
#' ```r
#' mf_map(x, var, type = "typo",
#'        pal, rev = FALSE, val_order,
#'        border, pch = 21, cex = 2, lwd = 0.7, col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' For lines:
#' ```r
#' mf_map(x, var, type = "typo",
#'        pal, rev = FALSE, val_order, lwd = .7,
#'        col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' @param x	object of class `sf`
#' @param var name of the variable to map
#' @param type "choro"
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals]. The default palette is the pal_quali palette
#' (see [mf_theme]).
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param val_order modalities order in the legend, a character vector that
#' matches `var` modalities. Default to alphabetic order of modalities.
#' @param border border color for polygons and symbols, a hex code or
#' color name given by [colors]. The default color for polygons is the
#' highlight color, the default color for points is the background color (see
#' [mf_theme]).
#' @param lwd border width for polygons and points symbols, lines width
#' @param pch type of symbol to use for points, see [pch] (points only)
#' @param cex symbols size, 2 means 2 times bigger (points only)
#' @param col_na color for missing values, a hex code or
#' a color name given by [colors].
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @param leg_* legend arguments described in [mf_map]
#' @return x is (invisibly) returned.
#' @seealso [mf_map()], [mf_get_pal()]
#' @examples
#' mtq <- mf_get_mtq()
#' pts <- mf_get_mtq("points")
#' flows <- mf_get_mtq("lines")
#' # polygons
#' mtq[6, "STATUS"] <- NA
#' mf_map(
#'   x = mtq, var = "STATUS", type = "typo",
#'   col_na = "grey90", border = "white",
#'   pal = c("#FFE93F", "#00214E", "#7C7C7C"),
#'   val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
#'   lwd = .5, leg_pos = "bottomleft",
#'   leg_title = "", leg_title_cex = 1,
#'   leg_val_cex = .9, leg_no_data = "No data",
#'   leg_box_cex = c(0.5, 3), leg_box_border = NA
#' )
#' # points
#' mf_map(
#'   x = pts, var = "STATUS", type = "typo",
#'   cex = 3, pal = "Dark 3", border = "grey",
#'   leg_pos = "bottomleft"
#' )
#' # lines
#' mf_map(mtq, extent = flows)
#' mf_map(
#'   x = flows, var = "sj", type = "typo",
#'   add = TRUE,
#'   lwd = 2, pal = c("red", "blue"),
#'   leg_pos = "bottomleft"
#' )
NULL

# grad ----
#' @name mf_map_grad
#' @title Plot graduated symbols
#' @description
#' With the **grad** map type, `mf_map()` displays graduated symbols on a map.
#'
#' Graduated symbols are based on classified quantitative variables.
#'
#' For polygons, centroids are used to plot graduated symbols.
#' ## Usage
#' For polygons and points:
#' ```r
#' mf_map(x, var, type = "grad",
#'        breaks = "quantile", nbreaks = 3,
#'        col, border, lwd = 0.7, pch = 21, cex,
#'        alpha, expandBB, extent, bg, add = TRUE, leg_*)
#' ```
#' For lines:
#' ```r
#' mf_map(x, var, type = "grad",
#'        breaks = "quantile", nbreaks = 3,
#'        col, lwd,
#'        alpha, expandBB, extent, bg, add = TRUE, leg_*)
#' ```
#' @param x	object of class `sf`
#' @param var name of the variable to map
#' @param type "grad"
#' @param breaks	either a numeric vector with the actual breaks, or a
#' classification method name. The main methods are 'quantile',
#' 'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
#' See [mf_get_breaks] for details.
#' @param nbreaks number of classes
#' @param col color of the graduated symbols or lines, a hex code or
#' a color name given by [colors].
#' The default color is the highlight color (see [mf_theme]).
#' @param border border color for symbols, a hex code or
#' color name given by [colors]. The default color is the background color (see
#' [mf_theme]).
#' @param lwd border width for graduated symbols, a vector of line widths for
#' graduated lines
#' @param pch type of symbol to use for points, see [pch] (points only)
#' @param cex a vector of sizes for symbols (points only)
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @param leg_* legend arguments described in [mf_map]
#' @return x is (invisibly) returned.
#' @seealso [mf_map()]
#' @examples
#' mtq <- mf_get_mtq()
#' flows <- mf_get_mtq("lines")
#' mf_map(mtq, bg = "cornsilk2")
#' mf_map(flows, "fij", "grad",
#'   breaks = "geom", nbreaks = 3,
#'   lwd = c(1, 3, 7),
#'   leg_title = "N. commuters",
#'   leg_pos = "bottomleft", leg_val_rnd = 0
#' )
#' mf_map(mtq, "POP", "grad",
#'   breaks = c(686, 5000, 25000, 82502),
#'   cex = c(1, 2, 4), pch = 22, col = "steelblue",
#'   leg_title = "Population", leg_pos = "topright",
#'   leg_frame = TRUE
#' )
NULL

# symb ----
#' @name mf_map_symb
#' @title Plot symbols
#' @description
#' `mf_map()` can use symbols to display qualitative data, using **symb**
#' map type.
#'
#' For polygons, centroids are used to plot graduated symbols. This map type
#' is not available for lines.
#'
#' ## Usage
#' For polygons and points:
#' ```r
#' mf_map(x, var, type = "symb",
#'        pch, cex = 2, lwd = 0.7, pal, rev = FALSE, border,
#'        val_order,
#'        col_na = "grey", pch_na = 4, cex_na = 1,
#'        alpha, expandBB, extent, bg, add = TRUE, leg_*)
#' ```
#' @param x	object of class `sf` (polygons or points)
#' @param var name of the variable to map
#' @param type "symb"
#' @param pch a vector of types of symbols, see [pch].
#' The length of `pch` should match the number of modalities.
#' @param cex a vector of sizes for symbols.
#' The length of `cex` should match the number of modalities.
#' @param lwd border width of symbols
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals]. The default palette is the pal_quali palette
#' (see [mf_theme]).
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param border border color for symbols, a hex code or color name given
#' by [colors]. The default color is the background color (see
#' [mf_theme]).
#' @param val_order modalities order in the legend, a character vector that
#' matches `var` modalities. Default to alphabetic order of modalities.
#' @param pch_na type of symbol for missing values, see [pch]
#' @param cex_na size of symbol for missing values
#' @param col_na color for missing values, a hex code or
#' a color name given by [colors]
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mtq$STATUS[3] <- NA
#' mf_map(mtq)
#' mf_map(mtq, "STATUS", "symb",
#'   pal = "Berlin", border = "white", lwd = 1,
#'   cex = c(4, 3, 2), pch = c(21:23), col_na = "red",
#'   val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
#'   leg_title = ""
#' )
NULL


# prop_choro ----
#' @name mf_map_prop_choro
#' @title Plot proportional symbols with choropleth coloration
#' @description
#' `mf_map()` with **prop_choro** type creates symbols that are proportional
#' to values of a first variable and colored to reflect the classification of a
#' second variable.
#'
#' This map types uses two variables and some arguments need to be set for both
#' variables (see Details).
#'
#' For polygons, centroids are used to plot proportional symbols. This map type
#' is not available for lines.
#'
#' ## Usage
#' For polygons and points:
#' ```r
#' mf_map(x, var, type = "prop_choro",
#'        inches = 0.3, val_max,  symbol = "circle",
#'        pal, rev = FALSE, breaks = "quantile", nbreaks,
#'        border, lwd = 0.7, col_na = "white",
#'        alpha, expandBB, extent, bg, add = TRUE, leg_*)
#' ```
#'
#' @param x	object of class `sf` (polygons or points)
#' @param var names of the variables to map. The first value refers to the
#' proportional symbols, the second one to the choropleth coloration.
#' @param type "prop_choro"
#' @param inches size of the largest symbol in inches (radius for circles,
#' half width for squares)
#' @param val_max maximum value corresponding to the largest symbol or line
#' @param symbol type of proportional symbols, either "circle" or "square"
#' @param breaks	either a numeric vector with the actual breaks, or a
#' classification method name. The main methods are 'quantile',
#' 'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
#' See [mf_get_breaks] for details.
#' @param nbreaks number of classes
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals]. The default palette is the pal_seq palette
#' (see [mf_theme]).
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param border border color of proportional symbols, a hex code or
#' color name given by [colors]. The default color is the background color (see
#' [mf_theme]).
#' @param lwd border width of proportional symbols
#' @param col_na color for missing values, a hex code or
#' a color name given by [colors]
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @param leg_* legend arguments described in [mf_map]. See details for
#' arguments with two values.
#' @return x is (invisibly) returned.
#' @details
#' Legend arguments that need two values are: 'leg_title',
#' 'leg_val_rnd', and 'leg_horiz'. The first
#' values refers to the proportional symbols legend, the second one to the
#' choropleth legend.
#'
#'
#' @seealso [mf_map()], [mf_map_prop], [mf_map_choro], [mf_distr()],
#' [mf_get_breaks()], [mf_get_pal()]
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mtq[6, "MED"] <- NA
#' mf_map(
#'   x = mtq, var = c("POP", "MED"), type = "prop_choro",
#'   inches = .2,
#'   val_max = 90000, symbol = "circle",
#'   col_na = "grey90", pal = "Cividis",
#'   breaks = "msd", nbreaks = 4, lwd = 1,
#'   leg_pos = "topright",
#'   leg_title = c("Population", "Median Income"),
#'   leg_val_rnd = c(0, 1),
#'   leg_horiz = c(TRUE, FALSE),
#'   leg_title_cex = .9,
#'   leg_val_dec = ",",
#'   leg_val_cex = .8,
#'   leg_size = 1,
#'   add = TRUE
#' )
NULL

# symb-choro ----
#' @name mf_map_symb_choro
#' @title Plot symbols with choropleth coloration
#' @description
#' `mf_map()` with **symb_choro** type creates symbols that reflect modalities
#' of a first qualitative variable and colored to reflect the classification of
#' a second variable.
#'
#' This map types uses two variables and some arguments need to be set for both
#' variables (see Details).
#'
#' For polygons, centroids are used to plot symbols. This map type
#' is not available for lines.
#'
#' ## Usage
#' For polygons and points:
#' ```r
#' mf_map(x, var, type = "symb_choro",
#'        pch, cex = 2, lwd = 0.7, border, val_order,
#'        pal, rev = FALSE, breaks = "quantile", nbreaks,
#'        pch_na = 4, cex_na = 1, col_na = "white",
#'        alpha, expandBB, extent, bg, add = TRUE, leg_*)
#' ```
#' @param x	object of class `sf` (polygons or points)
#' @param var names of the variables to map. The first value refers to the
#' symbols categories, the second one to the choropleth coloration.
#' @param type "symb_choro"
#' @param pch a vector of types of symbols, see [pch].
#' The length of `pch` should match the number of modalities.
#' @param cex a vector of sizes for symbols.
#' The length of `cex` should match the number of modalities.
#' @param lwd border width of symbols
#' @param border border color for symbols, a hex code or color name given
#' by [colors]. The default color is the background color (see
#' [mf_theme]).
#' @param val_order modalities order in the legend, a character vector that
#' matches `var` modalities. Default to alphabetic order of modalities.
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals]. The default palette is the pal_seq palette
#' (see [mf_theme]).
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param breaks	either a numeric vector with the actual breaks, or a
#' classification method name. The main methods are 'quantile',
#' 'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
#' See [mf_get_breaks] for details.
#' @param nbreaks number of classes
#' @param pch_na type of symbol for missing values, see [pch]
#' @param cex_na size of symbol for missing values
#' @param col_na color for missing values, a hex code or
#' a color name given by [colors]
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @param leg_* legend arguments described in [mf_map]. See details for
#' arguments with two values.
#' @details
#' Legend arguments that need two values are: 'leg_title',
#' 'leg_no_data'. The first value refers to the symbols legend, the second
#' one to the choropleth legend.
#' @return x is (invisibly) returned.
#' @seealso [mf_map()], [mf_map_symb], [mf_map_choro], [mf_distr()],
#' [mf_get_breaks()], [mf_get_pal()]
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mtq$STATUS[4] <- NA
#' mf_map(mtq, c("STATUS", "MED"),
#'   type = "symb_choro", lwd = 1,
#'   pal = "Reds 3", breaks = "quantile", nbreaks = 4,
#'   cex = c(2, 1, 1), pch = c(20, 21, 23), pch_na = 22,
#'   leg_pos = "topright", border = "white",
#'   val_order = c("Prefecture", "Sub-prefecture", "Simple municipality")
#' )
NULL





# prop_typo ----
#' @name mf_map_prop_typo
#' @title Plot proportional symbols with typology coloration
#' @description
#' `mf_map()` with **prop_typo** type creates symbols that are proportional
#' to values of a first variable and colored to reflect the modalities of a
#' second qualitative variable.
#'
#' This map types uses two variables and some arguments need to be set for both
#' variables (see Details).
#'
#' For polygons, centroids are used to plot proportional symbols.
#' ## Usage
#' For polygons and points:
#' ```r
#' mf_map(x, var, type = "prop_typo",
#'        inches = 0.3, val_max, symbol, border,
#'        pal, rev = FALSE, val_order,
#'        border, lwd = 0.7, col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' For lines:
#' ```r
#' mf_map(x, var, type = "typo",
#'        lwd_max = 15,
#'        pal, rev = FALSE, val_order,
#'        col_na = "white",
#'        alpha, expandBB, extent, bg, add = FALSE, leg_*)
#' ```
#' @param x	object of class `sf`
#' @param var names of the variables to map. The first value refers to the
#' proportional symbols, the second one to the typology coloration.
#' @param type "prop_typo"
#' @param inches size of the largest symbol in inches (radius for circles,
#' half width for squares)
#' @param lwd_max width of the largest line
#' @param val_max maximum value corresponding to the largest symbol or line
#' @param symbol type of proportional symbols, either "circle" or "square"
#' @param border border color for proportional symbols, a hex code or
#' color name given by [colors]. The default color is the background color (see
#' [mf_theme]).
#' @param pal a set of colors (hex codes) or a palette name. Palette names can
#' be obtained with [hcl.pals]. The default palette is the pal_quali palette
#' (see [mf_theme]).
#' @param rev if `pal` is a palette name, whether the ordering of the colors
#' should be reversed (TRUE) or not (FALSE)
#' @param val_order modalities order in the legend, a character vector that
#' matches `var` modalities. Default to alphabetic order of modalities.
#' @param col_na color for missing values, a hex code or
#' a color name given by [colors].
#' @param alpha,expandBB,extent,bg,add arguments described in [mf_map]
#' @param leg_* legend arguments described in [mf_map]. See details for
#' arguments with two values.
#'
#' @details
#' 'leg_title' needs two values. The first value refers to the symbols legend,
#' the second one to the choropleth legend.
#'
#' @return x is (invisibly) returned.
#' @seealso [mf_map()], [mf_map_prop], [mf_map_typo], [mf_get_pal()]
#' @examples
#' mtq <- mf_get_mtq()
#' flows <- mf_get_mtq("lines")
#' mf_map(mtq, extent = flows, expandBB = c(0, .5, 0, 0))
#' mf_map(flows, c("fij", "sj"), "prop_typo",
#'   val_order = c("Sub-prefecture", "Simple municipality"),
#'   pal = c("steelblue", "lightblue"), lwd_max = 30,
#'   leg_pos = "topleft", leg_title = c("commuters", "destination")
#' )
#' mf_map(
#'   x = mtq, var = c("POP", "STATUS"), type = "prop_typo",
#'   inches = .2, border = "tomato4", lwd = 1,
#'   pal = c("darkblue", "steelblue", "lightblue"),
#'   val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
#'   leg_pos = "bottomleft",
#'   leg_title = c("Population", ""),
#'   leg_no_data = "No dada",
#'   add = TRUE
#' )
NULL

#' @title Get a pencil layer from polygons
#' @name mf_get_pencil
#' @description Create a pencil layer. This function transforms a POLYGON or
#' MULTIPOLYGON sf object into a MULTILINESTRING one.
#' @param x an sf object, a simple feature collection (POLYGON or MULTIPOLYGON).
#' @param size density of the penciling. Median number of points used to build
#' the MULTILINESTRING.
#' @param buffer buffer around each polygon. This buffer (in map units) is used
#' to take sample points. A negative value adds a margin between the penciling
#' and the original polygons borders
#' @param lefthanded if TRUE the penciling is done left-handed style.
#' @param clip if TRUE, the penciling is cut by the original polygon.
#' @return A MULTILINESTRING sf object is returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mtq_pencil <- mf_get_pencil(x = mtq, clip = FALSE)
#' mf_map(mtq)
#' mf_map(mtq_pencil, add = TRUE)
#' @export
mf_get_pencil <- function(x, size = 100, buffer = 0, lefthanded = TRUE,
                          clip = FALSE) {
  a <- median(st_area(st_set_crs(x, NA)))
  size <- size * size
  . <- lapply(st_geometry(x), makelines,
    size = size, buffer = buffer,
    lefthanded = lefthanded, a = a, clip = clip
  )
  . <- st_sfc(do.call(rbind, .))
  if (length(.) < nrow(x)) {
    stop(
      paste0(
        "Try a smaller value for 'buffer' ",
        "or a larger vaue for 'size'"
      ),
      call. = FALSE
    )
  }
  . <- st_sf(geometry = ., x[, , drop = TRUE], sf_column_name = "geometry")
  . <- st_set_crs(., st_crs(x))
  . <- st_cast(., "MULTILINESTRING")
  return(.)
}

makelines <- function(x, size, buffer, lefthanded, a, clip) {
  size <- round(sqrt(as.numeric(st_area(x) * size / a)), 0)
  if (size <= 10) {
    size <- 10
  }
  xbuf <- st_buffer(st_sfc(x), buffer)

  pt <- st_sample(xbuf, size = size, exact = FALSE)

  if (lefthanded) {
    pt <- st_sf(pt, x = st_coordinates(pt)[, 2] +
      st_coordinates(pt)[, 1])
  } else {
    pt <- st_sf(pt, x = st_coordinates(pt)[, 2] -
      st_coordinates(pt)[, 1])
  }
  pt <- st_combine(pt[order(pt$x), ])

  if (isTRUE(clip)) {
    pt <- st_intersection(st_cast(pt, "LINESTRING"), x)
  } else {
    pt <- st_intersection(st_cast(pt, "LINESTRING"), xbuf)
  }
  return(pt)
}

#' @title Plot an inset
#' @description This function is used to add an inset map to the current map.
#' @param x an sf object, or "worldmap" to use with \link{mf_worldmap}.
#' @param pos position, one of "bottomleft", "left", "topleft", "top", "bottom",
#' "bottomright", "right", "topright"
#' @param cex share of the map width occupied by the inset
#' @param fig coordinates of the inset region (in NDC, see in ?par())
#' @export
#' @details
#' If x is used (with pos and cex), the width/height ratio of the inset will
#' match the width/height ratio of x bounding box.\cr
#' If fig is used, coordinates (xmin, xmax, ymin, ymax) are expressed as
#' fractions of the mapping space (i.e. excluding margins).\cr
#' If map layers have to be plotted after the inset (i.e after mf_inset_off()),
#' please use add = TRUE.\cr
#' It is not possible to plot an inset within an inset.\cr
#' It is possible to plot anything (base plots) within the inset, not only map
#' layers.
#' @return No return value, an inset is initiated or closed.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_inset_on(x = mtq[1, ], cex = .2)
#' mf_map(mtq[1, ])
#' mf_inset_off()
#'
#' mf_map(mtq)
#' mf_inset_on(x = "worldmap", pos = "bottomleft")
#' mf_worldmap(x = mtq)
#' mf_inset_off()
#'
#' mf_map(mtq)
#' mf_inset_on(fig = c(0, 0.25, 0, 0.25))
#' mf_map(x = mtq)
#' mf_inset_off()
mf_inset_on <- function(x, pos = "topright", cex = .2, fig) {
  test_cur_plot()

  if (.gmapsf$inset) {
    stop(
      paste0(
        "You have already started an inset. ",
        "Use `mf_inset_off()` to close the current inset."
      ),
      call. = FALSE
    )
  }
  .gmapsf$inset <- TRUE
  pusr <- par("usr")
  .gmapsf$par <- par(no.readonly = TRUE)
  if (!missing(x)) {
    if (is.character(x)) {
      if (x == "worldmap") {
        h <- 1
        w <- 1
      }
    } else {
      bb <- st_bbox(x)
      w <- bb[3] - bb[1]
      h <- bb[4] - bb[2]
    }
    ratio <- h / w
    wdest <- (pusr[2] - pusr[1]) * cex
    hdest <- wdest * ratio
    inset_xy <- posinset(pos = pos, pusr = pusr, wdest = wdest, hdest = hdest)
    tgx <- c(
      grconvertX(inset_xy[1:2], from = "user", to = "ndc"),
      grconvertY(inset_xy[3:4], from = "user", to = "ndc")
    )
  }

  if (!missing(fig)) {
    ffig <- c(
      grconvertX(pusr[1:2], from = "user", to = "ndc"),
      grconvertY(pusr[3:4], from = "user", to = "ndc")
    )
    tgtx <- fig[1:2]
    tgty <- fig[3:4]

    ffig2 <- c(tgtx * (ffig[2] - ffig[1]), tgty * (ffig[4] - ffig[3]))
    tgx <- c(ffig[1] + ffig2[1:2], ffig[3] + ffig2[3:4])
  }

  theme <- mf_theme()
  .gmapsf$oth <- theme
  .gmapsf$mfg <- par("mfg")

  if (max(tgx) > 1 || min(tgx) < 0) {
    tgx[tgx > 1] <- 1
    tgx[tgx < 0] <- 0
  }
  par(bg = NA)
  par(
    fig = round(tgx, 3),
    mar = c(0, 0, 0, 0),
    new = TRUE
  )
  theme$mar <- c(0, 0, 0, 0)
  theme$background <- NA
  mf_theme(theme)
}


#' @name mf_inset_on
#' @export
mf_inset_off <- function() {
  .gmapsf$inset <- FALSE
  pusr <- .gmapsf$par$usr
  par(.gmapsf$par)
  par(new = TRUE, mar = .gmapsf$oth$mar)
  plot.new()
  plot.window(
    xlim = pusr[1:2], ylim = pusr[3:4],
    xaxs = "i", yaxs = "i", asp = TRUE
  )
  mf_theme(.gmapsf$oth)
  par(mar = .gmapsf$par$mar, mfg = .gmapsf$mfg)
  points(1, 1, col = NA)
}

#' @title Plot a shadow
#' @name mf_shadow
#' @description Plot the shadow of a polygon layer.
#' @param x an sf or sfc polygon object
#' @param col shadow color. The default color is the highlight color
#' (see [mf_theme]).
#' @param cex shadow extent
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @param extent object with an `st_bbox` method to define plot extent;
#' defaults to `x`. `extent` and `x` must use the same CRS.
#' @param bg background color of the map, hex code or color name given by
#' [colors], ignored if `add = TRUE`
#' @param add whether to add the layer to an existing plot (TRUE) or not (FALSE)
#' @export
#' @return x is (invisibly) returned.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_shadow(mtq)
#' mf_map(mtq, add = TRUE)
mf_shadow <- function(x, col, cex = 1, add = FALSE, extent = x, bg,
                      expandBB = rep(.04, 4)) {
  if (get_geom_type(x) != "POLYGON") {
    stop("x must be an sf or sfc POLYGON object.", call. = FALSE)
  }
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))
  bgc <- go(bg, "background")

  if (add == FALSE) {
    mf_init(x, expandBB = expandBB, extent = extent, bgc = bgc)
  }
  col <- go(col, "highlight", "grey50")
  xyi <- xyinch(1)
  ratio <- xyi[2] / xyi[1]
  d_x <- cex * strwidth("M", units = "user", cex = 1) / 3
  d_y <- d_x * ratio
  z <- st_geometry(x) + c(d_x, -d_y)
  mf_map(
    x = z, type = "base", col = col,
    border = col, lwd = .5, add = TRUE
  )
  return(invisible(x))
}

get_the_raster_pal <- function(pal, nbreaks, alpha, rev = TRUE) {
  if (length(pal) == 1) {
    if (pal %in% hcl.pals()) {
      cols <- hcl.colors(n = nbreaks, palette = pal, rev = rev)
    } else {
      stop("This is not a palette name", call. = FALSE)
    }
  } else {
    cols <- colorRampPalette(pal, alpha = TRUE)(nbreaks)
  }
  if (!is.null(alpha)) {
    cols <- get_hex_pal(cols, alpha)
  }
  return(cols)
}


get_continuous_pal <- function(breaks, pal, alpha) {
  # get a palette repartitionthat match classes size
  etendu <- max(breaks) - min(breaks)
  lb <- length(breaks)
  dd <- data.frame(from = breaks[1:(lb - 1)], to = breaks[2:lb])
  dd$diff <- dd$to - dd$from
  dd$ncol <- round(dd$diff * 1000 / etendu)
  dd$colfrom <- pal[1:(lb - 1)]
  dd$colto <- pal[2:lb]
  l <- list()
  for (i in 1:(lb - 1)) {
    l[[i]] <- colorRampPalette(c(dd$colfrom[i], dd$colto[i]), alpha = TRUE)(dd$ncol[i])
  }
  p <- do.call(c, l)
  if (!is.null(alpha)) {
    p <- get_hex_pal(p, alpha)
  }
  p
}


mf_raster_multiband <- function(ops, expandBB, bg, add) {
  ops$smooth <- ifelse(is.null(ops$smooth), TRUE, ops$smooth)
  if (add == FALSE) {
    mf_init(ops$x, expandBB = expandBB, bgc = bg)
  }
  do.call(terra::plotRGB, ops)
}

mf_raster_interval <- function(ops, ops_leg, pal, breaks, nbreaks, alpha,
                               rev, add, expandBB, bg) {
  pal <- go(pal, "pal_seq", "Dark Mint")
  # set breaks and palette
  ops$breaks <- mf_get_breaks(
    x = terra::values(ops$x), nbreaks = nbreaks,
    breaks = breaks
  )
  ops$col <- get_the_pal(
    pal = pal, nbreaks = length(ops$breaks) - 1,
    alpha = alpha, rev = !rev
  )
  # init
  if (add == FALSE) {
    mf_init(ops$x, expandBB = expandBB, bgc = bg)
  }
  ops$alpha <- NULL
  # plot
  do.call(terra::plot, ops)
  # legend
  leg(
    type = "choro",
    box_cex = ops_leg$leg_box_cex,
    val = ops$breaks,
    horiz = ops_leg$leg_horiz,
    box_border = ops_leg$leg_box_border,
    pos = ops_leg$leg_pos,
    pal = ops$col,
    title = ops_leg$leg_title,
    title_cex = ops_leg$leg_title_cex,
    val_cex = ops_leg$leg_val_cex,
    val_rnd = ops_leg$leg_val_rnd,
    val_dec = ops_leg$leg_val_dec,
    val_big = ops_leg$leg_val_big,
    frame = ops_leg$leg_frame,
    bg = ops_leg$leg_bg,
    fg = ops_leg$leg_fg,
    frame_border = ops_leg$leg_frame_border,
    adj = ops_leg$leg_adj,
    size = ops_leg$leg_size
  )
}


mf_raster_continuous <- function(ops, ops_leg, breaks, pal, expandBB, add,
                                 alpha, rev, bg) {
  pal <- go(pal, "pal_seq", "Dark Mint")
  val <- terra::values(ops$x, na.rm = TRUE)

  # with breaks
  lb <- length(breaks)
  if (lb > 1) {
    if (length(pal) != (lb)) {
      stop(paste0("'pal' should be a vector of ", lb, " colors"), call. = FALSE)
    }
    pal <- get_continuous_pal(breaks, pal, alpha)
    p_pal <- pal
    # this for vmin superior to lmin or/and vmax inferior to lmax
    # other cases are missing
    val_min <- min(val)
    val_max <- max(val)
    bks_min <- min(breaks)
    bks_max <- max(breaks)
    one_unit <- length(pal) / (bks_max - bks_min)
    d_min <- bks_min - val_min
    d_max <- bks_max - val_max
    if (d_min > 0) {
      p_pal <- c(rep(NA, round(d_min * one_unit, 0)), p_pal)
    }
    if (d_min < 0) {
      p_pal <- p_pal[-(1:round(-d_min * one_unit, 0))]
    }
    if (d_max > 0) {
      p_pal <- p_pal[1:(length(p_pal) - round(d_max * one_unit, 0))]
    }
    if (d_max < 0) {
      p_pal <- c(p_pal, rep(NA, round(-d_max * one_unit, 0)))
    }
    ops$col <- p_pal
    vv <- breaks
  } else {
    pal <- get_the_raster_pal(
      pal = pal, nbreaks = 255, alpha = alpha,
      rev = !rev
    )
    ops$col <- pal[-1]
    # For the legend
    v <- mf_get_breaks(x = val, nbreaks = 4, breaks = "pretty")
    vmin <- min(val)
    vmax <- max(val)
    lim <- (vmax - vmin) / 10
    vv <- c(vmin, v[v > vmin & v < vmax], vmax)
    lvv <- length(vv)
    if (vv[2] - vv[1] < lim) {
      vv <- vv[-2]
    }
    lvv <- length(vv)
    if ((vv[lvv] - vv[(lvv - 1)]) < lim) {
      vv <- vv[-(lvv - 1)]
    }
  }

  if (add == FALSE) {
    mf_init(ops$x, expandBB = expandBB, bgc = bg)
  }

  ops$alpha <- NULL
  do.call(terra::plot, ops)

  leg(
    type = "cont",
    box_cex = c(1.5, 2) * ops_leg$leg_box_cex,
    val = vv,
    horiz = ops_leg$leg_horiz,
    pos = ops_leg$leg_pos,
    pal = pal,
    title = ops_leg$leg_title,
    title_cex = ops_leg$leg_title_cex,
    val_cex = ops_leg$leg_val_cex,
    val_rnd = ops_leg$leg_val_rnd,
    val_dec = ops_leg$leg_val_dec,
    val_big = ops_leg$leg_val_big,
    frame = ops_leg$leg_frame,
    bg = ops_leg$leg_bg,
    fg = ops_leg$leg_fg,
    frame_border = ops_leg$leg_frame_border,
    adj = ops_leg$leg_adj,
    size = ops_leg$leg_size
  )
}


mf_raster_classes <- function(ops, ops_leg, pal, val_order, expandBB,
                              add, alpha, rev, bg) {
  modalities <- terra::cats(ops$x)[[1]]
  if (is.null(modalities)) {
    ops$x <- terra::as.factor(ops$x)
    modalities <- terra::cats(ops$x)[[1]]
  }

  pal <- go(pal, "pal_quali", "Dark 2")

  if (missing(val_order)) {
    val_order <- modalities[, 2]
  }
  pal <- get_the_pal(
    pal = pal, nbreaks = length(val_order),
    alpha = alpha, rev = rev
  )
  refcol <- data.frame(mod = val_order, col = pal)

  mod <- merge(
    x = modalities, y = refcol,
    by.x = names(modalities)[2],
    by.y = "mod", all.x = TRUE
  )
  mod <- mod[, c("ID", "col")]
  mod <- mod[order(mod$ID), "col"]

  ops$col <- mod

  if (add == FALSE) {
    mf_init(ops$x, expandBB = expandBB, bgc = bg)
  }
  ops$alpha <- NULL
  do.call(terra::plot, ops)

  leg(
    type = "typo",
    pos = ops_leg$leg_pos,
    val = refcol$mod,
    title = ops_leg$leg_title,
    title_cex = ops_leg$leg_title_cex,
    val_cex = ops_leg$leg_val_cex,
    no_data = FALSE,
    size = ops_leg$leg_size,
    box_border = ops_leg$leg_box_border,
    box_cex = ops_leg$leg_box_cex,
    frame_border = ops_leg$leg_frame_border,
    frame = ops_leg$leg_frame,
    pal = refcol$col,
    bg = ops_leg$leg_bg,
    fg = ops_leg$leg_fg,
    adj = ops_leg$leg_adj
  )
}

suppressPackageStartupMessages(library("terra"))

a <- rast(system.file("ex/elev.tif", package = "terra"))
b <- rast(system.file("ex/logo.tif", package = "terra"))
d <-  classify(a, c(140, 450, 500, 550))
levels(d) <- data.frame(ID = 0:2, elevation = c("low", "high", "super high"))


# input type
expect_error(mf_raster(x = "x"))

# multiband
expect_silent(mf_raster(b))
expect_silent(mf_raster(b, add = TRUE))

# internals
# get_the_raster_pal
expect_equal(mapsf:::get_the_raster_pal("Reds 2", 6, 1, TRUE),
             hcl.colors(6, "Reds 2", alpha = 1, rev = TRUE))
expect_error(mapsf:::get_the_raster_pal("Jaune", 6, 1, TRUE))
expect_equal(mapsf:::get_the_raster_pal(c("red", "blue"), 6, 1, TRUE),
             c("#FF0000FF", "#CC0033FF", "#990066FF", "#650099FF",
               "#3200CCFF", "#0000FFFF"))
# get_continuous_pal
pp <- mapsf:::get_continuous_pal(c(0, 10, 20), pal = c("red", "white", "blue"), alpha = 1)
expect_equal(pp[c(1, 500, 1000)], c("#FF0000FF", "#FFFFFFFF", "#0000FFFF"))

# type
expect_error(mf_raster(a, type = "coninuous"))

# interval
expect_silent(mf_raster(a, "interval"))

# continuous
expect_silent(mf_raster(a))
expect_error(mf_raster(a, breaks = c(141, 500, 547), pal = "red"))
expect_silent(mf_raster(a, breaks = c(141, 400, 547),
                        pal = c("red", "black", "yellow")))

# classes
expect_silent(mf_raster(d, "classes"))
expect_silent(mf_raster(d, "classes", pal = "Burg",
                        val_order = rev(c("low", "high", "super high"))))
expect_silent(mf_raster(a, "classes", pal = "Burg"))


# alpha
expect_silent(mf_raster(d, "classes", alpha = .5, pal = 1:3))
expect_silent(mf_raster(a, alpha = .5, pal = 1:2))
expect_silent(mf_raster(a, "interval", alpha = .4, nbreaks = 4, pal = 1:4))

expect_silent(mf_raster(d, "classes", alpha = .5, pal = "Viridis"))
expect_silent(mf_raster(a, alpha = .5, pal = "Viridis"))
expect_silent(mf_raster(a, "interval", alpha = .4, nbreaks = 4,
                        pal = "Viridis"))

expect_silent(mf_raster(d, "classes", pal = hcl.colors(3, "Viridis",
                                                       alpha = .5)))
expect_silent(mf_raster(a,  pal = hcl.colors(12, "Viridis", alpha = .5)))
expect_silent(mf_raster(a, "interval",  nbreaks = 12,
                        pal = hcl.colors(12, "Viridis", alpha = .5)))

#' @title Deprecated - Plot an annotation
#' @description
#' This function is deprecated. Please use [mf_text] instead.
#' instead.
#'
#' Plot an annotation on a map.
#' @param x an sf object with 1 row, a couple of coordinates (c(x, y)) or
#' "interactive"
#' @param txt the text to display
#' @param pos position of the text, one of "topleft", "topright", "bottomright",
#' "bottomleft" or "center"
#' @param cex size of the text
#' @param col_arrow arrow color
#' @param col_txt text color
#' @param halo add a halo around the text
#' @param bg halo color
#' @param s arrow size (min=1)
#' @param ... further \link{text} arguments.
#' @note
#' Annotations cannot be displayed on unprojected (long/lat) maps.
#' @export
#' @return No return value, an annotation is displayed.
#' @examples
#' mtq <- mf_get_mtq()
#' mf_map(mtq)
#' mf_text(
#'   x = mtq[2, ],
#'   txt = "pos = 'bottomleft'\nline = 2\nclockwise = FALSE",
#'   pos = "bottomleft",
#'   offset = 6,
#'   clockwise = FALSE,
#'   line = 2,
#'   box = FALSE
#' )
#' mf_text(
#'   x = mtq[28, ],
#'   txt = "pos = 'topright'\nline = 3\nclockwise = FALSE",
#'   pos = "topright",
#'   offset = 10,
#'   clockwise = FALSE,
#'   line = 3,
#'   halo = TRUE,
#'   align = "left"
#' )
mf_annotation <- function(x,
                          txt,
                          pos = "topright",
                          cex = 0.8,
                          col_arrow,
                          col_txt,
                          halo = FALSE,
                          bg,
                          s = 1,
                          ...) {
  warning("mf_annotation() is deprecated. Use mf_text() instead.",
    call. = FALSE
  )

  test_cur_plot()
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  if (diff(par("usr")[1:2]) < 389) {
    message("Annotations cannot be displayed on unprojected (long/lat) maps.")
    return(invisible(NULL))
  }

  col_arrow <- go(col_arrow, "highlight", "black")
  col_txt <- go(col_txt, "highlight")
  bg <- go(bg, "background")


  if (inherits(x, "character") && x == "interactive") {
    x <- interleg(txt = c("annotation", "Annotation"))
  }

  if (inherits(x, c("sf", "sfc"))) {
    xy <- st_coordinates(
      st_centroid(
        st_geometry(x[1, ]),
        of_largest_polygon = TRUE
      )
    )
  }
  if (inherits(x, "numeric") && length(x) == 2) {
    xy <- x
  }

  if (s < 1) {
    s <- 1
  }

  if (pos %in% c("topleft", "topright", "bottomright", "bottomleft")) {
    inset <- xinch(par("csi")) / 3
    radius <- 5 * s * inset
    res <- annot_pos_params(
      pos = pos, xy = xy,
      radius = radius,
      inset = inset
    )
    drawarc(
      x = res$x_arc,
      y = res$y_arc,
      radius = radius,
      deg1 = res$deg1,
      deg2 = res$deg2,
      col = col_arrow
    )
    polygon(
      x = res$x_poly,
      y = res$y_poly,
      col = col_arrow,
      border = col_arrow,
      lwd = 1.2
    )
  } else if (pos == "center") {
    res <- list(x_txt = xy[1], y_txt = xy[2])
  }

  if (halo) {
    shadowtext(
      x = res$x_txt,
      y = res$y_txt,
      labels = txt,
      col = col_txt, bg = bg,
      cex = cex,
      adj = res$adj,
      ...
    )
  } else {
    text(
      x = res$x_txt,
      y = res$y_txt,
      cex = cex,
      labels = txt,
      col = col_txt,
      adj = res$adj,
      ...
    )
  }
}


drawarc <- function(x = 1, y = NULL, radius = 1, deg1 = 0, deg2 = 45, col) {
  n <- 0.05
  angle1 <- deg1 * pi / 180
  angle2 <- deg2 * pi / 180
  lwd <- 1.2
  xylim <- par("usr")
  ymult <- 1
  devunits <- dev.size("px")
  xy <- xy.coords(x, y)
  x <- xy$x
  y <- xy$y
  a1 <- pmin(angle1, angle2)
  a2 <- pmax(angle1, angle2)
  angle1 <- a1
  angle2 <- a2
  delta_angle <- (angle2 - angle1)
  if (n != as.integer(n)) {
    n <- as.integer(1 + delta_angle / n)
  }
  delta_angle <- delta_angle / n
  angle_s <- angle1 + seq(0, length.out = n) * delta_angle
  angle_e <- c(angle_s[-1], angle2)
  if (n > 1) {
    half_lwd_user <- (lwd / 2) * (xylim[2] - xylim[1]) / devunits[1]
    adj_angle <- delta_angle * half_lwd_user / (2 * (radius + half_lwd_user))
    angle_s[2:n] <- angle_s[2:n] - adj_angle
    angle_e[1:(n - 1)] <- angle_e[1:(n - 1)] + adj_angle
  }
  p1x <- x + radius * cos(angle_s)
  p1y <- y + radius * sin(angle_s) * ymult
  p2x <- x + radius * cos(angle_e)
  p2y <- y + radius * sin(angle_e) * ymult
  segments(p1x, p1y, p2x, p2y, col = col, lwd = lwd, lend = 3)
}


annot_pos_params <- function(pos, xy, radius, inset) {
  if (pos == "topright") {
    x_arc <- xy[1] + radius
    y_arc <- xy[2] + inset
    deg1 <- 180
    deg2 <- 90
    x_poly <- c(
      xy[1] - inset / 3,
      xy[1],
      xy[1] + inset / 3,
      xy[1] - inset / 3
    )
    y_poly <- c(
      xy[2] + 2 * inset,
      xy[2] + 5 * inset / 6,
      xy[2] + 2 * inset,
      xy[2] + 2 * inset
    )
    x_txt <- xy[1] + radius + inset / 2
    y_txt <- xy[2] + radius + inset
    adj <- c(0, .5)
  }

  if (pos == "topleft") {
    x_arc <- xy[1] - radius
    y_arc <- xy[2] + inset
    deg1 <- 0
    deg2 <- 90
    x_poly <- c(
      xy[1] - inset / 3,
      xy[1],
      xy[1] + inset / 3,
      xy[1] - inset / 3
    )
    y_poly <- c(
      xy[2] + 2 * inset,
      xy[2] + 5 * inset / 6,
      xy[2] + 2 * inset,
      xy[2] + 2 * inset
    )
    x_txt <- xy[1] - radius - inset / 2
    y_txt <- xy[2] + radius + inset
    adj <- c(1, .5)
  }

  if (pos == "bottomleft") {
    x_arc <- xy[1] - radius
    y_arc <- xy[2] - inset
    deg1 <- 360
    deg2 <- 270
    x_poly <- c(
      xy[1] - inset / 3,
      xy[1],
      xy[1] + inset / 3,
      xy[1] - inset / 3
    )
    y_poly <- c(
      xy[2] - 2 * inset,
      xy[2] - 5 * inset / 6,
      xy[2] - 2 * inset,
      xy[2] - 2 * inset
    )
    x_txt <- xy[1] - radius - inset / 2
    y_txt <- xy[2] - radius - inset
    adj <- c(1, .5)
  }

  if (pos == "bottomright") {
    x_arc <- xy[1] + radius
    y_arc <- xy[2] - inset
    deg1 <- 270
    deg2 <- 180
    x_poly <- c(
      xy[1] - inset / 3,
      xy[1],
      xy[1] + inset / 3,
      xy[1] - inset / 3
    )
    y_poly <- c(
      xy[2] - 2 * inset,
      xy[2] - 5 * inset / 6,
      xy[2] - 2 * inset,
      xy[2] - 2 * inset
    )
    x_txt <- xy[1] + radius + inset / 2
    y_txt <- xy[2] - radius - inset
    adj <- c(0, .5)
  }


  return(list(
    x_arc = x_arc,
    y_arc = y_arc,
    deg1 = deg1,
    deg2 = deg2,
    x_poly = x_poly,
    y_poly = y_poly,
    x_txt = x_txt,
    y_txt = y_txt,
    adj = adj
  ))
}

#
# import stuffs
# @import graphics
# @import stats
# @import sf
# @import graphics
#' @importFrom s2 as_s2_geography s2_data_countries s2_difference s2_union_agg
#' s2_buffer_cells as_s2_geography s2_intersection
#' @importFrom maplegend leg leg_comp leg_draw
#' @importFrom grDevices colorRampPalette dev.list png svg hcl.colors
#' recordGraphics hcl.pals dev.size xy.coords col2rgb rgb
#' @importFrom stats aggregate median na.omit quantile runif sd bw.SJ density
#' @importFrom sf st_polygon st_multilinestring st_multipolygon st_buffer
#' st_coordinates st_buffer st_geometry st_crs st_bbox st_as_sf st_transform
#' st_is_longlat st_as_sfc st_centroid st_crs<- st_geometry<- st_is_empty
#' st_collection_extract st_cast st_read st_geometry_type st_agr
#' st_set_agr st_convex_hull st_make_grid st_union st_intersects
#' st_set_geometry st_sf st_area st_combine st_graticule st_intersection
#' st_sample st_set_crs st_sfc
#' @importFrom graphics arrows axis boxplot grconvertX box rasterImage
#' grconvertY hist lines locator par plot.new plot.window points polygon
#' rasterImage rect segments strheight strwidth symbols text
#' title xinch xyinch yinch
#' @importFrom classInt classIntervals


#' @importFrom utils globalVariables
.gmapsf <- new.env(parent = emptyenv())
globalVariables(".gmapsf", package = "mapsf", add = FALSE)
.gmapsf$inset <- FALSE
.gmapsf$positions <- c(
  "bottomleft", "left", "topleft", "top", "bottom",
  "bottomright", "right", "topright",
  "interactive"
)

go <- function(x, opt, legacy) {
  if (missing(x)) {
    if (is.null(getOption("mapsf.legacy")) || missing(legacy)) {
      x <- getOption(paste0("mapsf.", opt))
    } else {
      x <- legacy
    }
  }
  return(x)
}


test_cur_plot <- function() {
  if (is.null(dev.list())) {
    stop("You can only use this feature on an existing plot.", call. = FALSE)
  }
}


interleg <- function(txt = c("legend", "Legend")) {
  if (interactive()) {
    message(paste0("Click on the map to choose the ", txt[1], " position."))
    x <- unlist(locator(1))
    message(paste0(txt[2], " coordinates:\nc(", x[[1]], ", ", x[[2]], ")"))
    return(x)
  } else {
    stop('You cannot use "interactive" in a non-interactive R session.',
      call. = FALSE
    )
  }
}


# shadow around the labels
#' @name shadowtext
#' @title shadowtext
#' @description shadowtext
#' @param x lon
#' @param y lat
#' @param labels labels
#' @param col col
#' @param bg bg
#' @param theta number of iteration
#' @param r radius
#' @param ... other txt params
#' @noRd
shadowtext <- function(x, y = NULL, labels, col = "white", bg = "black",
                       theta = seq(0, 2 * pi, length.out = 50), r = 0.1, ...) {
  xo <- r * strwidth("A")
  yo <- r * strheight("A")
  for (i in theta) {
    text(x + cos(i) * xo, y + sin(i) * yo, labels, col = bg, ...)
  }
  text(x, y, labels, col = col, ...)
}


get_breaks_methods_names <- function() {
  return(
    c(
      "quantile", "equal", "msd", "ckmeans", "Q6", "geom", "fixed", "sd",
      "pretty", "kmeans", "hclust", "bclust", "fisher", "jenks", "dpih",
      "headtails", "maximum", "box", "q6", "arith", "em"
    )
  )
}


get_themes_names <- function() {
  n <- names(.gmapsf$themes)
  return(n[!n %in% c(
    "default", "brutal", "ink", "dark", "agolalight", "candy",
    "darkula", "iceberg", "green", "nevermind", "jsk", "barcelona"
  )])
}


readimage <- function(filename) {
  ex <- strsplit(basename(filename), split = "\\.")[[1]]
  ex <- tolower(ex[length(ex)])
  if (ex == "png") {
    if (!requireNamespace("png", quietly = TRUE)) {
      stop(
        "'png' is package needed for this function to work. Please install it.",
        call. = FALSE
      )
    }
    img <- png::readPNG(filename)
  }
  if (ex %in% c("jpg", "jpeg")) {
    if (!requireNamespace("jpeg", quietly = TRUE)) {
      stop(
        paste0(
          "'jpeg' is package needed for this function to work. ",
          "Please install it."
        ),
        call. = FALSE
      )
    }
    img <- jpeg::readJPEG(filename)
  }
  return(img)
}


#' xy of legend
#'
#' @param pos pos
#' @param pu pu
#' @param wdest dl
#' @param hdest dl
#'
#' @noRd
posinset <- function(pos, pusr, wdest, hdest, adj = c(0, 0)) {
  if (is.numeric(pos) && length(pos) == 2) {
    xy <- c(
      pos[1],
      pos[1] + wdest,
      pos[2] - hdest,
      pos[2]
    )
    return(xy)
  }

  posposs <- c(
    "bottomleft", "left", "topleft", "top", "bottom",
    "bottomright", "right", "topright"
  )
  if (!pos %in% posposs) {
    stop(paste0(
      "pos should be one of ", paste0(posposs, collapse = ", "),
      "."
    ), call. = FALSE)
  }

  x_spacing <- xinch(par("csi")) / 4
  y_spacing <- yinch(par("csi")) / 4
  pusr <- pusr + c(x_spacing, -x_spacing, y_spacing, -y_spacing)

  xy <- switch(pos,
    bottomleft = c(
      pusr[1],
      pusr[1] + wdest,
      pusr[3],
      pusr[3] + hdest
    ),
    topleft = c(
      pusr[1],
      pusr[1] + wdest,
      pusr[4] - hdest,
      pusr[4]
    ),
    left = c(
      pusr[1],
      pusr[1] + wdest,
      pusr[3] + (pusr[4] - pusr[3]) / 2 - (hdest) / 2,
      pusr[3] + (pusr[4] - pusr[3]) / 2 + (hdest) / 2
    ),
    top = c(
      pusr[1] + (pusr[2] - pusr[1]) / 2 - (wdest) / 2,
      pusr[1] + (pusr[2] - pusr[1]) / 2 + (wdest) / 2,
      pusr[4] - hdest,
      pusr[4]
    ),
    bottom = c(
      pusr[1] + (pusr[2] - pusr[1]) / 2 - (wdest) / 2,
      pusr[1] + (pusr[2] - pusr[1]) / 2 + (wdest) / 2,
      pusr[3],
      pusr[3] + hdest
    ),
    bottomright = c(
      pusr[2] - wdest,
      pusr[2],
      pusr[3],
      pusr[3] + hdest
    ),
    right = c(
      pusr[2] - wdest,
      pusr[2],
      pusr[3] + (pusr[4] - pusr[3]) / 2 - (hdest) / 2,
      pusr[3] + (pusr[4] - pusr[3]) / 2 + (hdest) / 2
    ),
    topright = c(
      pusr[2] - wdest,
      pusr[2],
      pusr[4] - hdest,
      pusr[4]
    )
  )
  xy <- xy + c(adj[1], adj[1], adj[2], adj[2]) *
    c(x_spacing, x_spacing, y_spacing, y_spacing)
  return(xy)
}

expect_error(test_cur_plot())
mf_map(mtq)
expect_silent(mf_legend(type = "prop", pos = "topright",
                        val = c(1, 5, 10), inches = .3))
expect_silent(mf_legend(type = "choro", pos = "bottomright",
                        val = c(10, 20, 30, 40, 50),
                        pal = hcl.colors(4, "Reds 2")))
expect_silent(mf_legend(type = "typo", pos = "topleft",
                        val = c("A", "B", "C", "D"),
                        pal = hcl.colors(4, "Dynamic")))
expect_silent(mf_legend(type = "symb", pos = "bottomleft",
                        val = c("A", "B", "C"),
                        pch = 21:23, cex = c(1, 2, 2),
                        pal = hcl.colors(3, "Dynamic")))
expect_silent(mf_legend(type = "grad_line", pos = "top",
                        val = c(1, 2, 3, 4, 10, 15),
                        lwd = c(0.2, 2, 4, 5, 10)))
expect_silent(mf_legend(type = "prop_line", pos = "bottom", lwd = 20,
                        val = c(5, 50, 100)))
expect_error(suppressWarnings(mf_legend(type = "prop_line",
                                        pos = "interactive", lwd = 20,
                       val = c(5, 50, 100))))


expect_error(mf_legend(type = "grad_line", pos = "NA",
                        val = c(1, 2, 3, 4, 10, 15),
                        lwd = c(0.2, 2, 4, 5, 10)))

expect_silent(mf_legend(type = "grad_line",
                        pos = c(737788.682080213, 1628182.17278935),
                        val = c(1, 2, 3, 4, 10, 15),
                        lwd = c(0.2, 2, 4, 5, 10), frame = TRUE))
expect_silent(mf_legend(type = "grad_line", pos = NA,
                        val = c(1, 2, 3, 4, 10, 15),
                        lwd = c(0.2, 2, 4, 5, 10)))
expect_silent(mf_legend(type = "prop_line", pos = NA,
                        lwd = 20,
                        val = c(5, 50, 100)))
expect_silent(mf_legend(type = "prop_line",
                        pos = c(737788.682080213, 1628182.17278935),
                        lwd = 20,
                        val = c(5, 50, 100),
                        frame = TRUE))

dev.off()
expect_equal(mf_get_leg_pos(mtq, 1), "topright")
expect_equal(mf_get_leg_pos(mtq[1, ], 2), c("topleft", "top"))
expect_error(mapsf:::interleg())
mf_map(mtq)
expect_equal(mf_get_leg_pos(mtq[1, ], 2), c("topright", "right"))

# Label placement
#' @name wordlayout
#' @title wordlayout
#' @description wordlayout
#' @param x long
#' @param y lat
#' @param words labels
#' @param cex cex
#' @param q quality
#' @return coords
#' @noRd
wordlayout <- function(x, y, words, cex = 1, q) {
  d <- data.frame(q = c(0, 1, 2, 3), v = c(1, .2, .1, .01))
  tstep <- rstep <- d[d$q == q, 2]
  tails <- "g|j|p|q|y"
  n <- length(words)
  sdx <- sd(x, na.rm = TRUE)
  sdy <- sd(y, na.rm = TRUE)
  if (sdx == 0) sdx <- 1
  if (sdy == 0) sdy <- 1
  if (length(cex) == 1) cex <- rep(cex, n)
  boxes <- list()
  for (i in seq_along(words)) {
    r <- 0
    theta <- runif(1, 0, 2 * pi)
    x1 <- xo <- x[i]
    y1 <- yo <- y[i]
    wid <- strwidth(words[i], cex = cex[i]) +
      0.4 * strwidth("R", cex = cex[i])
    ht <- strheight(words[i], cex = cex[i]) +
      0.4 * strheight("R", cex = cex[i])
    # mind your ps and qs
    if (grepl(tails, words[i])) {
      ht <- ht + ht * .2
    }

    while (TRUE) {
      if (noverlap(x1 - .5 * wid, y1 - .5 * ht, wid, ht, boxes)) {
        boxes[[length(boxes) + 1]] <- c(x1 - .5 * wid, y1 - .5 * ht, wid, ht)
        break
      } else {
        theta <- theta + tstep
        r <- r + rstep * tstep / (2 * pi)
        x1 <- xo + sdx * r * cos(theta)
        y1 <- yo + sdy * r * sin(theta)
      }
    }
  }
  result <- do.call(rbind, boxes)
  colnames(result) <- c("x", "y", "width", "ht")
  rownames(result) <- words
  result
}

noverlap <- function(x1, y1, sw1, sh1, boxes) {
  for (i in seq_along(boxes)) {
    bnds <- boxes[[i]]
    x2 <- bnds[1]
    y2 <- bnds[2]
    sw2 <- bnds[3]
    sh2 <- bnds[4]
    if (x1 < x2) {
      overlap <- (x1 + sw1) > x2
    } else {
      overlap <- (x2 + sw2) > x1
    }
    if (y1 < y2) {
      overlap <- overlap && (y1 + sh1) > y2
    } else {
      overlap <- overlap && (y2 + sh2) > y1
    }
    if (overlap) {
      return(FALSE)
    }
  }
  return(TRUE)
}

#' @title Deprecated - Initialize a map with a specific extent
#' @name mf_init
#' @description
#' This function is deprecated.
#' Please use \code{mf_map(x, col = NA, border = NA)} instead.
#'
#' Plot an invisible layer with the extent of a spatial object.\cr
#' Always use \code{add = TRUE} in \code{mf_map()} calls following an
#' \code{mf_init()} call.
#' This function is similar to \code{mf_map(x, col = NA, border = NA)}.
#' @param x object of class \code{sf}, \code{sfc} or \code{SpatRaster}
#' @param expandBB fractional values to expand the bounding box with, in each
#' direction (bottom, left, top, right)
#' @eval my_params(c("extent", "bgc"))
#' @export
#' @keywords internal
#' @return No return value, a map is initiated.
#' @examples
#' mtq <- mf_get_mtq()
#' target <- mtq[30, ]
#' mf_map(target, type = "base", col = NA, border = NA)
#' mf_map(mtq, add = TRUE)
#' # or
#' mf_map(mtq, extent = target)
mf_init <- function(x, expandBB = rep(0, 4), extent = x, bgc) {
  deprecate_direct_calls_to("mf_init")

  if (inherits(x, "SpatRaster")) {
    if (!requireNamespace("terra", quietly = TRUE)) {
      stop(
        paste0(
          "'terra' package is needed for this function to work. ",
          "Please install it."
        ),
        call. = FALSE
      )
    }

    proj <- terra::crs(x)
    bb <- terra::ext(x)[c(1, 3, 2, 4)]
    y <- st_as_sfc(st_bbox(bb))
    st_crs(y) <- proj
    mf_init(y,
      expandBB = c(rep(-.0399, 4)) + expandBB,
      extent = extent, bgc = bgc
    )
    return(invisible(x))
  }

  # transform to sfc bbox
  y <- st_as_sfc(st_bbox(x), crs = st_crs(x))

  if (par("xaxs") == "r") {
    expandBB <- expandBB / (1 + 0.08)
  }

  # margins mgmt
  op <- par(mar = getOption("mapsf.mar"), no.readonly = TRUE)
  on.exit(par(op))

  if (missing(bgc)) {
    bgc <- getOption("mapsf.background")
  }

  if (st_crs(x) != st_crs(extent)) {
    warning(
      paste0(
        "'x' and 'extent' must use the same CRS.",
        " 'extent' is ignored."
      ),
      call. = FALSE
    )
    extent <- y
  }
  # plot with bg and margins
  plot(y, col = NA, border = NA, expandBB = expandBB, extent = extent)

  recordGraphics(
    {
      pux <- par("usr")
      rect(pux[1],
        pux[3],
        pux[2],
        pux[4],
        border = NA,
        col = bgc
      )
    },
    list = list(bgc = bgc),
    env = getNamespace("mapsf")
  )

  f <- getOption("mapsf.frame")
  if (f %in% c("map", "figure")) {
    mf_frame(
      extent = f,
      col = getOption("mapsf.highlight"),
      lwd = getOption("mapsf.frame_lwd"),
      lty = getOption("mapsf.frame_lty")
    )
  }
  return(invisible(x))
}

deprecate_direct_calls_to <- function(m) {
  if (any(grepl(m, sys.call(1)))) {
    warning(paste0(m, "() is deprecated. Use mf_map() instead."), call. = FALSE)
  }
}

test2args <- function(x) {
  if (length(x) != 2) {
    stop(
      paste0(
        "Argument '", deparse(substitute(x)),
        "' needs 2 values."
      ),
      call. = FALSE
    )
  }
  return(x)
}

#' Title
#'
#' @param pal pal
#' @param nbreaks nbreaks
#' @param alpha alpha
#' @noRd
get_the_pal <- function(pal, nbreaks, alpha, rev = TRUE) {
  if (length(pal) == 1) {
    if (is.function(pal)) {
      cols <- pal(nbreaks)
    } else if (pal %in% hcl.pals()) {
      cols <- hcl.colors(n = nbreaks, palette = pal, rev = rev)
    } else {
      cols <- rep(pal, nbreaks)
    }
  } else {
    cols <- pal[1:nbreaks]
  }
  if (!is.null(alpha)) {
    cols <- get_hex_pal(cols, alpha)
  }

  return(cols)
}

get_hex_pal <- function(pal, alpha) {
  pal <- col2rgb(pal, alpha = FALSE)
  ffun <- function(x) {
    rgb(pal[1, x],
      pal[2, x],
      pal[3, x],
      maxColorValue = 255
    )
  }
  paste0(sapply(seq_len(ncol(pal)), ffun), get_alpha(alpha))
}

get_alpha <- function(alpha) {
  if (alpha < 0) {
    alpha <- 0
  }
  if (alpha > 1) {
    alpha <- 1
  }
  sprintf("%02X", as.integer(255.999 * alpha))
}


get_col_vec <- function(x, breaks, pal, jen = FALSE) {
  if (jen) {
    itv <- apply(array(apply(outer(x, breaks, ">"), 1, sum)), 1, max, 1)
  } else {
    itv <- findInterval(x, breaks, all.inside = FALSE, rightmost.closed = TRUE)
  }

  lb <- length(breaks)
  nout <- sum(itv %in% c(0, lb))
  if (nout > 0) {
    if (nout == 1) {
      message(paste0(
        "1 value is outside the class limits and",
        " will be classified as 'NA'."
      ))
    } else {
      message(paste0(
        nout, " values are outside the class limits and will",
        " be classified as 'NA'."
      ))
    }
  }

  itv[itv == 0] <- length(breaks)
  colvec <- pal[itv]
  return(colvec)
}


#' @name create_dots
#' @title create_dots
#' @description Create clean, sorted sf object with centroid coordinates from
#' an sf object
#' @param x x
#' @param var var
#' @return A sorted and cleaned sf object with centroid coordinates.
#' @noRd
create_dots <- function(x = x, var = var) {
  # get centroid coords
  x <- cbind(
    st_coordinates(st_centroid(
      x = st_geometry(x),
      of_largest_polygon = TRUE
    )),
    x
  )
  lx <- nrow(x)
  # remove NAs and 0 values
  x <- x[!is.na(x = x[[var]]), ]
  lxna <- nrow(x)
  x <- x[x[[var]] != 0, ]
  lx0 <- nrow(x)

  x <- x[is.finite(x[[var]]), ]
  lxinf <- nrow(x)


  nna <- lx - lxna
  nn0 <- lx - nna - lx0
  nni <- lx - nna - nn0 - lxinf

  if (nna > 0) {
    if (nna == 1) {
      message("1 'NA' value is not plotted on the map.")
    } else {
      message(paste0(nna, " 'NA' values are not plotted on the map."))
    }
  }
  if (nn0 > 0) {
    if (nn0 == 1) {
      message("1 '0' value is not plotted on the map.")
    } else {
      message(paste0(nn0, " '0' values are not plotted on the map."))
    }
  }

  if (nni > 0) {
    if (nni == 1) {
      message("1 'Infinite' value is not plotted on the map.")
    } else {
      message(paste0(nni, " 'Infinite' values are not plotted on the map."))
    }
  }

  # check if there is something to plot
  if (nrow(x) == 0) {
    stop("Nothing is plotted on the map.", call. = FALSE)
  }

  # turn to positive values
  if (min(x[[var]]) < 0) {
    message("Negative values have been transformed into positive values.")
    x[[var]] <- abs(x[[var]])
  }

  # Order the dots
  x <- x[order(x[[var]], decreasing = TRUE), ]
  return(x)
}


#' @name get_size
#' @title get_size
#' @description get a vector of radii
#' @param inches inches
#' @param var var
#' @param fixmax fixmax
#' @param symbol symbols
#' @return a vector of radii
#' @noRd
get_size <- function(var, inches, val_max, symbol) {
  switch(symbol,
    circle = {
      smax <- inches * inches * pi
      size <- sqrt((var * smax / val_max) / pi)
    },
    square = {
      smax <- inches * inches
      size <- sqrt(var * smax / val_max)
    }
  )
  return(size)
}


# Plot symbols
plot_symbols <- function(symbol, dots, sizes, mycols, border, lwd, inches) {
  if (inherits(dots, c("sf", "sfc"))) {
    xy <- st_set_geometry(dots[, 1:2], NULL)
  } else {
    xy <- dots
  }
  switch(symbol,
    circle = {
      symbols(
        x = xy[, 1],
        y = xy[, 2],
        circles = sizes,
        bg = mycols,
        fg = border,
        lwd = lwd,
        add = TRUE,
        inches = inches,
        asp = 1
      )
    },
    square = {
      symbols(
        x = xy[, 1],
        y = xy[, 2],
        squares = sizes,
        bg = mycols,
        fg = border,
        lwd = lwd,
        add = TRUE,
        inches = inches * 2,
        asp = 1
      )
    }
  )
}


check_order <- function(val_order, mod) {
  if (missing(val_order)) {
    val_order <- sort(mod)
  }
  return(val_order)
}


get_modalities <- function(x, val_order) {
  mod <- unique(x)
  mod <- mod[!is.na(mod)]
  # check val_order vs mod values
  val_order <- check_order(val_order, mod)
}

get_col_typo <- function(x, pal, val_order) {
  # get the colors
  refcol <- data.frame(
    mod = val_order,
    col = pal,
    stringsAsFactors = FALSE
  )
  mycols <- refcol[match(x, refcol[, 1]), 2]
  mycols
}

get_sym_typo <- function(x, pch, val_order) {
  # get the colors
  refsym <- data.frame(
    mod = val_order,
    pch = pch,
    stringsAsFactors = FALSE
  )
  mysym <- refsym[match(x, refsym[, 1]), 2]
  mysym
}


# split multiple legend position
split_leg <- function(x) {
  llp <- length(x)
  if (llp == 1) {
    return(list(l1 = x))
  }
  if (llp == 2) {
    lp1 <- x[1]
    lp2 <- x[2]
    if (is.numeric(c(lp1, lp2))) {
      return(list(l1 = c(lp1, lp2)))
    }
  }
  if (llp == 3) {
    tt <- tryCatch(as.numeric(x[1]), warning = function(w) w)
    if (inherits(tt, "warning")) {
      lp1 <- x[1]
      lp2 <- as.numeric(x[2:3])
    } else {
      lp1 <- as.numeric(x[1:2])
      lp2 <- x[3]
    }
  }
  if (llp == 4) {
    lp1 <- x[1:2]
    lp2 <- x[3:4]
  }
  return(list(l1 = lp1, l2 = lp2))
}

get_geom_type <- function(x) {
  a <- list(
    POINT = "POINT",
    POINT = "MULTIPOINT",
    LINE = "LINESTRING",
    LINE = "MULTILINESTRING",
    POLYGON = "POLYGON",
    POLYGON = "MULTIPOLYGON",
    other = "GEOMETRY",
    other = "GEOMETRYCOLLECTION",
    other = "CIRCULARSTRING",
    other = "COMPOUNDCURVE",
    other = "CURVEPOLYGON",
    other = "MULTICURVE",
    other = "MULTISURFACE",
    other = "CURVE",
    other = "SURFACE",
    other = "POLYHEDRALSURFACE",
    other = "TIN",
    other = "TRIANGLE"
  )
  type <- st_geometry_type(x)
  levels(type) <- a
  type <- as.character(unique(type))
  if (length(type) > 1) {
    stop("GEOMETRYCOLLECTION objects should have consistent type",
      call. = FALSE
    )
  }
  return(type)
}


# arg checking depending on type
check_args <- function(argx, type) {
  n_rel <- !names(argx) %in% names(formals(get(paste0("mf_", type))))
  s_n_rel <- sum(n_rel)
  if (s_n_rel >= 1) {
    mes <- "The following arguments are not relevant when using type = '"
    if (s_n_rel == 1) {
      mes <- "The following argument is not relevant when using type = '"
    }
    message(paste0(
      mes, type, "': ",
      paste0(names(argx[n_rel]), collapse = ", "),
      "."
    ))
    argx <- argx[!n_rel]
  }
  argx
}


get_val_rnd <- function(val, val_rnd, val_dec = getOption("OutDec"), val_big = "") {
  if (is.numeric(val)) {
    val <- round(val, val_rnd)
    if (val_rnd <= 0) {
      val_rnd <- 0
    }
    val <- format(
      x = val, scientific = FALSE, nsmall = val_rnd,
      decimal.mark = val_dec, big.mark = val_big,
      trim = TRUE
    )
  }
  return(val)
}

---
title: 'mapsf'
author: "Timothée Giraud"
date: '`r Sys.Date()`'
output: 
  rmarkdown::html_vignette:
    fig_width: 8
    toc: true
    toc_depth: 1
    number_sections: true
vignette: >
  %\VignetteIndexEntry{mapsf}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 5,
  fig.height = 6
)
```

# Introduction

The aim of `mapsf` is to obtain thematic maps with the visual quality of those build with a classical mapping or GIS software.  
`mapsf` uses [`sf`](https://CRAN.R-project.org/package=sf) objects to produce `base` graphics.  
`mapsf` functions can be classified in the following categories:

- **Symbology**\
`mf_map()` is the main function of the package, it displays map layers on a georeferenced plot. Many parameters are available to fine tune the cartographic representations. These parameters are the common ones found in GIS and automatic cartography tools (e.g. classification and color palettes used in choropleth maps, symbols sizes used in proportional symbols maps...).  

- **Map Layout**\
Along with the cartographic functions, some other functions are dedicated to layout design (e.g. legends, scale bar, north arrow, title, credits...).  

- **Utility Functions**\
    A set of functions is dedicated to the creation of spatial objects (e.g. links creation) or other utility tasks (e.g. color palette management, class intervals extraction, sample dataset import...).



# Main Features

## Symbology

`mf_map()` has three main arguments:

* `x` an sf object
* `var` the name(s) of a variable(s) to map
* `type` the map layer type

The following map types are available:

* ![](fig/mf_base.svg) **base**: displays sf objects geometries  
* ![](fig/mf_prop.svg) **prop**: displays symbols with areas proportional to a quantitative variable (stocks)  
* ![](fig/mf_choro.svg) **choro**: areas are shaded according to the variation of a quantitative variable   
* ![](fig/mf_typo.svg) **typo**: displays a typology map of a qualitative variable    
* ![](fig/mf_symb.svg) **symb**: displays the different modalities of a qualitative variable as symbols  
* ![](fig/mf_grad.svg) **grad**: displays graduated symbols   
* ![](fig/mf_prop_choro.svg) **prop_choro**: displays symbols with sizes proportional to values of a first variable and colored to reflect the classification of a second quantitative variable  
* ![](fig/mf_prop_typo.svg) **prop_typo**: displays symbols with sizes proportional to values of a first variable and and colored to reflect the modalities of a second qualitative variable   
* ![](fig/mf_symb_choro.svg) **symb_choro**: displays the different modalities of a first qualitative variable as symbols colored to reflect the classification of a second quantitative variable  


Additionaly, `mf_raster()` can be used to displays raster objects (single/multi band, continuous, classes, intervals). 





## Map Layout

These functions are dedicated to the map layout design.


![](fig/maplayout.png)

|         |     |
|--------|----------------------|
| &#9312; `mf_title()`      | Title of the map   |
| &#9313; `mf_arrow()`      | North arrow        |
| &#9314; `mf_credits()`    | Author, sources... |
| &#9315; `mf_scale()`      | Scale bar          |
| &#9316; `mf_annotation()` | Annotations        |
| &#9317; `mf_label()`      | Labels             |
| &#9318; `mf_shadow()`     | Shadow             |


## Themes

`mf_theme()` sets a map theme. A theme is a set of graphical parameters that are applied to maps created with `mapsf`. These parameters are the figure margins, background and foreground colors and `mf_title()` options. `mapsf` offers some builtin themes. It’s possible to modify an existing theme or to start a theme from scratch. Themes are persistent across maps produced by mapsf (e.g. they survive a `dev.off()` call).

&#128279; See the online vignette: [How to use themes](https://riatelab.github.io/mapsf/articles/how_to_use_themes) 

## Export

`mf_svg()` and `mf_png()` export maps in SVG and PNG formats, respectively.
The exported map width/height ratio will match the one of a spatial object.   
**SVG export** is the perfect solution for editing maps with desktop vector graphics software, such as Inkscape. SVG is a [vector graphics](https://en.wikipedia.org/wiki/Vector_graphics) file format.      
**PNG export** should be used for maps that do not require further modification. PNG is a [raster graphics](https://en.wikipedia.org/wiki/Raster_graphics) file format.   


&#128279; See the online vignette: [How to export maps](https://riatelab.github.io/mapsf/articles/how_to_export_maps) 


# Examples of thematic maps

## Base map

`mf_map(x, type = "base")` displays `sf` objects geometries.  
`mf_get_mtq()` is used to import a sample dataset as an `sf` object. Use `sf::st_read()` to import your own spatial datasets.   

Map layout elements are set with `mf_title()`, `mf_credits()`, `mf_arrow()` and `mf_scale()`).

```{r mf_basemap, message=FALSE, warning=FALSE}
library(mapsf)
# import the sample data set
mtq <- mf_get_mtq()
# plot municipalities
mf_map(mtq, type = "base")
# layout elements
credits <- paste0("Sources: IGN, 2018\n", "mapsf ", packageVersion("mapsf"))
mf_title("Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Proportional Symbols

`mf_map(x, var, type = "prop")` displays symbols with areas proportional to a quantitative variable (stocks). The `inches` argument is used to customize the symbols sizes.

```{r mf_prop, message=FALSE, warning=FALSE}
# plot municipalities
mf_map(mtq)
# plot population
mf_map(
  x = mtq,
  var = "POP",
  type = "prop",
  inches = 0.25,
  col = "brown4",
  leg_pos = "topright",
  leg_adj = c(0, -3),
  leg_title = "Total population"
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Choropleth Map

In choropleth maps, areas are shaded according to the variation of a quantitative variable. They are used to represent ratios or indices.\
`mf_map(x, var, type = "choro")` displays choropleth maps . Arguments `nbreaks`, and `breaks` allow to customize the variable classification. `mf_get_breaks()` allows to classify data outside of the function itself. Colors palettes, defined with `pal`, can be created with `mf_get_pal()` or you can use palette names from `hcl.pals()`.

```{r mf_map_c}
# population density (inhab./km2) using sf::st_area()
mtq$POPDENS <- 1e6 * mtq$POP / sf::st_area(mtq)
# plot population density
mf_map(
  x = mtq,
  var = "POPDENS",
  type = "choro",
  breaks = "geom",
  nbreaks = 5,
  pal = "Teal",
  border = "white",
  lwd = 0.5,
  leg_pos = "bottomleft",
  leg_adj = c(0, 6),
  leg_title = "Population Density\n(inh. / km2)"
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Typology Map

`mf_map(x, var, type = "typo")` displays a typology map of a qualitative variable. `val_order` is used to set the modalities order in the legend.

`mf_label()` displays labels on the map.

```{r mf_map_t}
# plot administrative status
mf_map(
  x = mtq,
  var = "STATUS",
  type = "typo",
  pal = c("aquamarine4", "yellow3", "wheat"),
  lwd = .5,
  val_order = c(
    "Prefecture",
    "Sub-prefecture",
    "Simple municipality"
  ),
  leg_pos = "topright",
  leg_adj = c(0, 1),
  leg_title = ""
)
# labels for a few  municipalities
mf_label(
  x = mtq[mtq$STATUS != "Simple municipality", ], var = "LIBGEO",
  cex = 0.9, halo = TRUE, r = 0.15
)
# layout elements
mf_title("Administrative Status")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Proportional Symbols using Choropleth Coloration

`mf_map(x, var, type = "prop_choro")` creates a map of symbols that are proportional to values of a first variable and colored to reflect the classification of a second variable.

`expandBB` argument in `mf_map()` allows to expand the map space. Here, we increase the space available on the right of the map to avoid overlaps between the legends and the map.

```{r mf_map_pc, fig.width=5}
# Plot the municipalities and expand the map space on the right
mf_map(x = mtq, expandBB = c(0, 0, 0, .15))
# Plot symbols with choropleth coloration
mf_map(
  x = mtq,
  var = c("POP", "MED"),
  type = "prop_choro",
  border = "grey50",
  lwd = 1,
  leg_pos = c("topright"),
  leg_title = c("Population", "Median Income\n(in euros)"),
  breaks = "equal",
  nbreaks = 4,
  pal = "Greens",
  leg_val_rnd = c(0, -2),
  leg_frame = FALSE
)
# layout elements
mf_title("Population & Wealth in Martinique, 2015")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Proportional Symbols using Typology Coloration

`mf_map(x, var, type = "prop_typo")` creates a map of symbols that are proportional to values of a first variable and colored to reflect the modalities of a second qualitative variable.

```{r mf_map_pt, fig.width=5}
# plot the municipalities and expand the map space on the right
mf_map(x = mtq, expandBB = c(0, 0, 0, .15))
# plot symbols with choropleth coloration
mf_map(
  x = mtq,
  var = c("POP", "STATUS"),
  type = "prop_typo",
  symbol = "square",
  border = "white",
  lwd = .5,
  leg_pos = "topright",
  leg_title = c("Population", "Administrative\nStatus"),
  val_order = c(
    "Prefecture", "Sub-prefecture",
    "Simple municipality"
  )
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Label Map

`mf_label()` is dedicated to the display of labels on a map. The `overlap = FALSE` argument displays non overlapping labels.

```{r mf_label}
# plot municipalities
mf_map(mtq, col = "#e4e9de", border = "darkseagreen4")
# plot labels
mf_label(
  x = mtq,
  var = "LIBGEO",
  cex = 0.7,
  font = 4,
  halo = TRUE,
  r = 0.1,
  overlap = FALSE,
  q = 3,
  lines = FALSE
)
# layout elements
mf_title("Municipalities of Martinique")
mf_credits(credits)
mf_arrow(pos = "topright")
mf_scale()
```

## Links Map

`mf_get_links()` creates a link layer from an `sf` object and a link `data.frame` (long format).

`mf_map(x, var, type = "grad")` displays graduated links.


```{r mf_grad}
# import the csv file embedded in mapsf
mob <- read.csv(system.file("csv/mob.csv", package = "mapsf"))
# Select links from Fort-de-France (97209))
mob_97209 <- mob[mob$i == 97209, ]
# create an sf object of links
mob_links <- mf_get_links(x = mtq, df = mob_97209)
# Plot the municipalities
mf_map(mtq)
# plot graduated links
mf_map(
  x = mob_links,
  var = "fij",
  type = "grad",
  breaks = c(100, 500, 1000, 4679.0),
  lwd = c(1, 4, 8),
  leg_pos = "topright",
  leg_title = "Nb. of\nCommuters",
  leg_val_rnd = 0
)
# layout elements
mf_title("Commuting to Fort-de-France")
mf_credits(credits)
mf_arrow()
mf_scale()
```



# Datasets

Several datasets are embedded in the package:

-   A [GeoPackage](https://www.geopackage.org/) of Martinique municipalities can be imported with `mf_get_mtq()` (or via `sf::st_read()`).
    -   Sources: [Base comparateur de territoires](https://www.insee.fr/fr/statistiques/2521169) (data, upload date: 2018-09-25) & ADMIN EXPRESS-COG (geometry, 2018 edition).
    -   Citation: Insee and IGN, 2018
    -   Fields:
        -   **INSEE_COM**: Municipality identifier
        -   **STATUS**: Municipality administrative status
        -   **LIBGEO**: Municipality name
        -   **POP**: Total population, 2015
        -   **MED**: Median disposable income adjusted per equivalent household member, in euros, 2015
        -   **CHOM**: Unemployed population, 2015
        -   **ACT**: Active population, 2015

-   A csv file of professional mobility flows between Martinique municipalities.
    -   Sources: [Flux de mobilité - déplacements domicile-lieu de travail](https://www.insee.fr/fr/statistiques/3566477) (upload date: 2018-08-01)
    -   Citation: Insee, 2018
    -   Fields:
        -   **i**: Municipality of residence identifier
        -   **j**: Municipality of workplace identifier
        -   **fij**: Flows of workers (employed population, 15 y.o. or more, 2015, only flows \> 100)
        -   **sj**: Administrative status of the workplace municipality

---
title: 'mapsf'
author: "Timothée Giraud"
date: '`r Sys.Date()`'
output: 
  rmarkdown::html_vignette:
    fig_width: 8
    toc: true
    toc_depth: 1
    number_sections: true
vignette: >
  %\VignetteIndexEntry{mapsf}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 5,
  fig.height = 6
)
```

# Introduction

The aim of `mapsf` is to obtain thematic maps with the visual quality of those build with a classical mapping or GIS software.  
`mapsf` uses [`sf`](https://CRAN.R-project.org/package=sf) objects to produce `base` graphics.  
`mapsf` functions can be classified in the following categories:

- **Symbology**\
`mf_map()` is the main function of the package, it displays map layers on a georeferenced plot. Many parameters are available to fine tune the cartographic representations. These parameters are the common ones found in GIS and automatic cartography tools (e.g. classification and color palettes used in choropleth maps, symbols sizes used in proportional symbols maps...).  

- **Map Layout**\
Along with the cartographic functions, some other functions are dedicated to layout design (e.g. legends, scale bar, north arrow, title, credits...).  

- **Utility Functions**\
    A set of functions is dedicated to the creation of spatial objects (e.g. links creation) or other utility tasks (e.g. color palette management, class intervals extraction, sample dataset import...).



# Main Features

## Symbology

`mf_map()` has three main arguments:

* `x` an sf object
* `var` the name(s) of a variable(s) to map
* `type` the map layer type

The following map types are available:

* ![](fig/mf_base.svg) **base**: displays sf objects geometries  
* ![](fig/mf_prop.svg) **prop**: displays symbols with areas proportional to a quantitative variable (stocks)  
* ![](fig/mf_choro.svg) **choro**: areas are shaded according to the variation of a quantitative variable   
* ![](fig/mf_typo.svg) **typo**: displays a typology map of a qualitative variable    
* ![](fig/mf_symb.svg) **symb**: displays the different modalities of a qualitative variable as symbols  
* ![](fig/mf_grad.svg) **grad**: displays graduated symbols   
* ![](fig/mf_prop_choro.svg) **prop_choro**: displays symbols with sizes proportional to values of a first variable and colored to reflect the classification of a second quantitative variable  
* ![](fig/mf_prop_typo.svg) **prop_typo**: displays symbols with sizes proportional to values of a first variable and and colored to reflect the modalities of a second qualitative variable   
* ![](fig/mf_symb_choro.svg) **symb_choro**: displays the different modalities of a first qualitative variable as symbols colored to reflect the classification of a second quantitative variable  


Additionaly, `mf_raster()` can be used to displays raster objects (single/multi band, continuous, classes, intervals). 





## Map Layout

These functions are dedicated to the map layout design.


![](fig/maplayout.png)

|         |     |
|--------|----------------------|
| &#9312; `mf_title()`      | Title of the map   |
| &#9313; `mf_arrow()`      | North arrow        |
| &#9314; `mf_credits()`    | Author, sources... |
| &#9315; `mf_scale()`      | Scale bar          |
| &#9316; `mf_text()`       | Annotations        |
| &#9317; `mf_label()`      | Labels             |
| &#9318; `mf_shadow()`     | Shadow             |


## Themes

`mf_theme()` sets a map theme. A theme is a set of graphical parameters that are applied to maps created with `mapsf`. These parameters are the figure margins, background and foreground colors and `mf_title()` options. `mapsf` offers some builtin themes. It’s possible to modify an existing theme or to start a theme from scratch. Themes are persistent across maps produced by mapsf (e.g. they survive a `dev.off()` call).

&#128279; See the online vignette: [How to use themes](https://riatelab.github.io/mapsf/articles/how_to_use_themes) 

## Export

`mf_svg()` and `mf_png()` export maps in SVG and PNG formats, respectively.
The exported map width/height ratio will match the one of a spatial object.   
**SVG export** is the perfect solution for editing maps with desktop vector graphics software, such as Inkscape. SVG is a [vector graphics](https://en.wikipedia.org/wiki/Vector_graphics) file format.      
**PNG export** should be used for maps that do not require further modification. PNG is a [raster graphics](https://en.wikipedia.org/wiki/Raster_graphics) file format.   


&#128279; See the online vignette: [How to export maps](https://riatelab.github.io/mapsf/articles/how_to_export_maps) 


# Examples of thematic maps

## Base map

`mf_map(x, type = "base")` displays `sf` objects geometries.  
`mf_get_mtq()` is used to import a sample dataset as an `sf` object. Use `sf::st_read()` to import your own spatial datasets.   

Map layout elements are set with `mf_title()`, `mf_credits()`, `mf_arrow()` and `mf_scale()`).

```{r mf_basemap, message=FALSE, warning=FALSE}
library(mapsf)
# import the sample data set
mtq <- mf_get_mtq()
# plot municipalities
mf_map(mtq, type = "base")
# layout elements
credits <- paste0("Sources: IGN, 2018\n", "mapsf ", packageVersion("mapsf"))
mf_title("Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Proportional Symbols

`mf_map(x, var, type = "prop")` displays symbols with areas proportional to a quantitative variable (stocks). The `inches` argument is used to customize the symbols sizes.

```{r mf_prop, message=FALSE, warning=FALSE}
# plot municipalities
mf_map(mtq)
# plot population
mf_map(
  x = mtq,
  var = "POP",
  type = "prop",
  inches = 0.25,
  col = "brown4",
  leg_pos = "topright",
  leg_adj = c(0, -3),
  leg_title = "Total population"
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Choropleth Map

In choropleth maps, areas are shaded according to the variation of a quantitative variable. They are used to represent ratios or indices.\
`mf_map(x, var, type = "choro")` displays choropleth maps . Arguments `nbreaks`, and `breaks` allow to customize the variable classification. `mf_get_breaks()` allows to classify data outside of the function itself. Colors palettes, defined with `pal`, can be created with `mf_get_pal()` or you can use palette names from `hcl.pals()`.

```{r mf_map_c}
# population density (inhab./km2) using sf::st_area()
mtq$POPDENS <- 1e6 * mtq$POP / sf::st_area(mtq)
# plot population density
mf_map(
  x = mtq,
  var = "POPDENS",
  type = "choro",
  breaks = "geom",
  nbreaks = 5,
  pal = "Teal",
  border = "white",
  lwd = 0.5,
  leg_pos = "bottomleft",
  leg_adj = c(0, 6),
  leg_title = "Population Density\n(inh. / km2)"
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Typology Map

`mf_map(x, var, type = "typo")` displays a typology map of a qualitative variable. `val_order` is used to set the modalities order in the legend.

`mf_label()` displays labels on the map.

```{r mf_map_t}
# plot administrative status
mf_map(
  x = mtq,
  var = "STATUS",
  type = "typo",
  pal = c("aquamarine4", "yellow3", "wheat"),
  lwd = .5,
  val_order = c(
    "Prefecture",
    "Sub-prefecture",
    "Simple municipality"
  ),
  leg_pos = "topright",
  leg_adj = c(0, 1),
  leg_title = ""
)
# labels for a few  municipalities
mf_label(
  x = mtq[mtq$STATUS != "Simple municipality", ], var = "LIBGEO",
  cex = 0.9, halo = TRUE, r = 0.15
)
# layout elements
mf_title("Administrative Status")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Proportional Symbols using Choropleth Coloration

`mf_map(x, var, type = "prop_choro")` creates a map of symbols that are proportional to values of a first variable and colored to reflect the classification of a second variable.

`expandBB` argument in `mf_map()` allows to expand the map space. Here, we increase the space available on the right of the map to avoid overlaps between the legends and the map.

```{r mf_map_pc, fig.width=5}
# Plot the municipalities and expand the map space on the right
mf_map(x = mtq, expandBB = c(0, 0, 0, .15))
# Plot symbols with choropleth coloration
mf_map(
  x = mtq,
  var = c("POP", "MED"),
  type = "prop_choro",
  border = "grey50",
  lwd = 1,
  leg_pos = c("topright"),
  leg_title = c("Population", "Median Income\n(in euros)"),
  breaks = "equal",
  nbreaks = 4,
  pal = "Greens",
  leg_val_rnd = c(0, -2),
  leg_frame = FALSE
)
# layout elements
mf_title("Population & Wealth in Martinique, 2015")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Proportional Symbols using Typology Coloration

`mf_map(x, var, type = "prop_typo")` creates a map of symbols that are proportional to values of a first variable and colored to reflect the modalities of a second qualitative variable.

```{r mf_map_pt, fig.width=5}
# plot the municipalities and expand the map space on the right
mf_map(x = mtq, expandBB = c(0, 0, 0, .15))
# plot symbols with choropleth coloration
mf_map(
  x = mtq,
  var = c("POP", "STATUS"),
  type = "prop_typo",
  symbol = "square",
  border = "white",
  lwd = .5,
  leg_pos = "topright",
  leg_title = c("Population", "Administrative\nStatus"),
  val_order = c(
    "Prefecture", "Sub-prefecture",
    "Simple municipality"
  )
)
# layout elements
mf_title("Population Distribution in Martinique")
mf_credits(credits)
mf_arrow()
mf_scale()
```

## Label Map

`mf_label()` is dedicated to the display of labels on a map. The `overlap = FALSE` argument displays non overlapping labels.

```{r mf_label}
# plot municipalities
mf_map(mtq, col = "#e4e9de", border = "darkseagreen4")
# plot labels
mf_label(
  x = mtq,
  var = "LIBGEO",
  cex = 0.7,
  font = 4,
  halo = TRUE,
  r = 0.1,
  overlap = FALSE,
  q = 3,
  lines = FALSE
)
# layout elements
mf_title("Municipalities of Martinique")
mf_credits(credits)
mf_arrow(pos = "topright")
mf_scale()
```

## Links Map

`mf_get_links()` creates a link layer from an `sf` object and a link `data.frame` (long format).

`mf_map(x, var, type = "grad")` displays graduated links.


```{r mf_grad}
# import the csv file embedded in mapsf
mob <- read.csv(system.file("csv/mob.csv", package = "mapsf"))
# Select links from Fort-de-France (97209))
mob_97209 <- mob[mob$i == 97209, ]
# create an sf object of links
mob_links <- mf_get_links(x = mtq, df = mob_97209)
# Plot the municipalities
mf_map(mtq)
# plot graduated links
mf_map(
  x = mob_links,
  var = "fij",
  type = "grad",
  breaks = c(100, 500, 1000, 4679.0),
  lwd = c(1, 4, 8),
  leg_pos = "topright",
  leg_title = "Nb. of\nCommuters",
  leg_val_rnd = 0
)
# layout elements
mf_title("Commuting to Fort-de-France")
mf_credits(credits)
mf_arrow()
mf_scale()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_annotation.R
\name{mf_annotation}
\alias{mf_annotation}
\title{Deprecated - Plot an annotation}
\usage{
mf_annotation(
  x,
  txt,
  pos = "topright",
  cex = 0.8,
  col_arrow,
  col_txt,
  halo = FALSE,
  bg,
  s = 1,
  ...
)
}
\arguments{
\item{x}{an sf object with 1 row, a couple of coordinates (c(x, y)) or
"interactive"}

\item{txt}{the text to display}

\item{pos}{position of the text, one of "topleft", "topright", "bottomright",
"bottomleft" or "center"}

\item{cex}{size of the text}

\item{col_arrow}{arrow color}

\item{col_txt}{text color}

\item{halo}{add a halo around the text}

\item{bg}{halo color}

\item{s}{arrow size (min=1)}

\item{...}{further \link{text} arguments.}
}
\value{
No return value, an annotation is displayed.
}
\description{
This function is deprecated. Please use \link{mf_text} instead.
instead.

Plot an annotation on a map.
}
\note{
Annotations cannot be displayed on unprojected (long/lat) maps.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_text(
  x = mtq[2, ],
  txt = "pos = 'bottomleft'\nline = 2\nclockwise = FALSE",
  pos = "bottomleft",
  offset = 6,
  clockwise = FALSE,
  line = 2,
  box = FALSE
)
mf_text(
  x = mtq[28, ],
  txt = "pos = 'topright'\nline = 3\nclockwise = FALSE",
  pos = "topright",
  offset = 10,
  clockwise = FALSE,
  line = 3,
  halo = TRUE,
  align = "left"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_base.R
\name{mf_base}
\alias{mf_base}
\title{Deprecated - Plot an sf object}
\usage{
mf_base(
  x,
  col,
  border,
  alpha = NULL,
  cex = 1,
  pch = 20,
  lwd = 0.7,
  lty = 1,
  add = FALSE,
  extent = x,
  bg,
  expandBB = rep(0.04, 4),
  ...
)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{sfg}}

\item{col}{a color, hex code or color name given by \code{\link{colors}}}

\item{border}{border color}

\item{alpha}{opacity, in the range 0,1}

\item{cex}{point size}

\item{pch}{pch (point type) for symbols}

\item{lwd}{border width}

\item{lty}{line or border type}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}

\item{...}{ignored}
}
\value{
x is (invisibly) returned.
}
\description{
This function is deprecated. Please use \code{mf_map()} with \code{type = "base"}
instead.

Plot an sf object. This is mostly a wrapper around
\code{plot(st_geometry(x), ...)}.
}
\examples{
library(sf)
mtq <- mf_get_mtq()
mf_map(mtq, type = "base")
mf_map(mtq, type = "base", col = "blue")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_prop_typo.R
\name{mf_prop_typo}
\alias{mf_prop_typo}
\title{Deprecated - Plot proportional symbols using typology coloration}
\usage{
mf_prop_typo(
  x,
  var,
  inches = 0.3,
  val_max,
  symbol = "circle",
  pal,
  alpha = NULL,
  rev = FALSE,
  val_order,
  border,
  lwd = 0.7,
  lwd_max = 15,
  col_na = "white",
  extent = x,
  bg,
  expandBB = rep(0.04, 4),
  leg_pos = mf_get_leg_pos(x, 1),
  leg_title = var,
  leg_title_cex = c(0.8, 0.8),
  leg_val_cex = c(0.6, 0.6),
  leg_val_rnd = 0,
  leg_val_dec = ".",
  leg_val_big = "",
  leg_no_data = "No data",
  leg_frame = c(FALSE, FALSE),
  leg_frame_border,
  leg_horiz = FALSE,
  leg_adj = c(0, 0),
  leg_fg,
  leg_bg,
  leg_size = 1,
  leg_box_border,
  leg_box_cex = c(1, 1),
  add = TRUE
)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name(s) of the variable(s) to plot}

\item{inches}{size of the biggest symbol (radius for circles, half width for squares) in inches.}

\item{val_max}{maximum value used for proportional symbols}

\item{symbol}{type of symbols, 'circle' or 'square'}

\item{pal}{a set of colors (hex codes) or a palette name (valid palette names can be obtained with \code{\link{hcl.pals}}).}

\item{alpha}{opacity, in the range 0,1}

\item{rev}{if \code{pal} is a valid palette name, whether the ordering of the colors should be reversed (TRUE) or not (FALSE)}

\item{val_order}{values order, a character vector that matches var modalities}

\item{border}{border color}

\item{lwd}{border width}

\item{lwd_max}{line width of the largest line}

\item{col_na}{color for missing values}

\item{leg_pos}{position of the legend, two of 'topleft', 'top','topright', 'right','bottomright', 'bottom', 'bottomleft', 'left' or a vector of two coordinates in map units (c(x, y)). leg_pos argument can be c('position', 'position'), c('position', x2, y2), c(x1,y1, 'position') or c(x1, y1, x2, y2). Use \code{NA} to avoid plotting the legend, use 'interactive' to choose the legend position by clicking on the map.}

\item{leg_title}{legend title}

\item{leg_title_cex}{size of the legend title}

\item{leg_val_cex}{size of the values in the legend}

\item{leg_val_rnd}{number of decimal places of the values in the legend}

\item{leg_val_dec}{decimal separator}

\item{leg_val_big}{thousands separator}

\item{leg_no_data}{label for missing values}

\item{leg_frame}{whether to add a frame to the legend (TRUE) or not (FALSE)}

\item{leg_horiz}{display the legend horizontally (for proportional symbols and choropleth types)}

\item{leg_adj}{adjust the position of the legend in x and y directions}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}
}
\value{
x is (invisibly) returned.
}
\description{
This function is deprecated. Please use \code{mf_map()} with \code{type = "prop_typo"}
instead.

Plot proportional symbols with colors based on qualitative data.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_map(mtq, c("POP", "STATUS"), "prop_typo")

mtq[6, "STATUS"] <- NA
mf_map(mtq)
mf_map(
  x = mtq, var = c("POP", "STATUS"), type = "prop_typo",
  inches = .35, border = "tomato4",
  val_max = 90000, symbol = "circle", col_na = "grey", pal = "Dynamic",
  lwd = 2,
  leg_pos = "bottomright",
  leg_title = c("Population", "Municipality\nstatus"),
  leg_title_cex = 0.9,
  leg_val_cex = .7,
  val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
  leg_no_data = "No dada",
  leg_frame = TRUE,
  add = TRUE
)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_prop.R
\name{mf_prop}
\alias{mf_prop}
\title{Deprecated - Plot proportional symbols}
\usage{
mf_prop(
  x,
  var,
  inches = 0.3,
  val_max,
  lwd_max = 20,
  symbol = "circle",
  col,
  alpha = NULL,
  border,
  lwd = 0.7,
  extent = x,
  bg,
  expandBB = rep(0.04, 4),
  leg_pos = mf_get_leg_pos(x),
  leg_title = var,
  leg_title_cex = 0.8,
  leg_val_cex = 0.6,
  leg_val_rnd = 0,
  leg_val_dec = ".",
  leg_val_big = "",
  leg_frame = FALSE,
  leg_frame_border,
  leg_horiz = FALSE,
  leg_adj = c(0, 0),
  leg_fg,
  leg_bg,
  leg_size = 1,
  add = TRUE
)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name(s) of the variable(s) to plot}

\item{inches}{size of the biggest symbol (radius for circles, half width for squares) in inches.}

\item{val_max}{maximum value used for proportional symbols}

\item{lwd_max}{line width of the largest line}

\item{symbol}{type of symbols, 'circle' or 'square'}

\item{col}{a color, hex code or color name given by \code{\link{colors}}}

\item{alpha}{opacity, in the range 0,1}

\item{border}{border color}

\item{lwd}{border width}

\item{leg_pos}{position of the legend, one of 'topleft', 'top','topright', 'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two coordinates in map units (c(x, y)). Use \code{NA} to avoid plotting the legend, use 'interactive' to choose the legend position by clicking on the map.}

\item{leg_title}{legend title}

\item{leg_title_cex}{size of the legend title}

\item{leg_val_cex}{size of the values in the legend}

\item{leg_val_rnd}{number of decimal places of the values in the legend}

\item{leg_val_dec}{decimal separator}

\item{leg_val_big}{thousands separator}

\item{leg_frame}{whether to add a frame to the legend (TRUE) or not (FALSE)}

\item{leg_frame_border}{border color of the legend frame}

\item{leg_horiz}{display the legend horizontally (for proportional symbols and choropleth types)}

\item{leg_adj}{adjust the position of the legend in x and y directions}

\item{leg_fg}{color of the legend foreground}

\item{leg_bg}{color of the legend background}

\item{leg_size}{size of the legend; 2 means two times bigger}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}
}
\value{
x is (invisibly) returned.
}
\description{
This function is deprecated. Please use \code{mf_map()} with \code{type = "prop"}
instead.

Plot proportional symbols.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_map(mtq, "POP", "prop")

mf_map(mtq)
mf_map(
  x = mtq, var = "POP", type = "prop",
  inches = .4, symbol = "circle", val_max = 90000,
  col = "tomato1", border = "blue", lwd = 1,
  leg_pos = "right", leg_title = "Population",
  leg_title_cex = 1, leg_val_cex = .8, leg_val_rnd = 0,
  leg_frame = TRUE, add = TRUE
)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_logo.R
\name{mf_logo}
\alias{mf_logo}
\title{Plot a logo on a map}
\usage{
mf_logo(filename, pos = "bottomright", cex = 1, adj = c(0, 0), resize = TRUE)
}
\arguments{
\item{filename}{filename of the logo image, PNG or JPG/JPEG format.}

\item{pos}{position of the logo, one of 'topleft', 'top','topright',
'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two
coordinates in map units (c(x, y)). Use 'interactive' to choose the
legend position by clicking on the map.}

\item{cex}{amount by which the logo width should be magnified
or reduced relative to the default}

\item{adj}{adjust the position of the logo in x and y directions}

\item{resize}{if FALSE, the logo is displayed at its original size in pixels
and \code{cex} is not used.}
}
\value{
No return value, a logo is displayed.
}
\description{
The logo can be a PNG or JPG/JPEG file.
}
\examples{
m <- mf_get_mtq()
mf_map(m)
mf_scale()
logo <- system.file("img", "Rlogo.png", package = "png")
mf_logo(logo, pos = "bottomleft", adj = c(0, 4))
mf_credits()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_grad.R
\name{mf_grad}
\alias{mf_grad}
\title{Deprecated - Plot graduated symbols}
\usage{
mf_grad(
  x,
  var,
  breaks = "quantile",
  nbreaks = 3,
  col,
  alpha = NULL,
  border,
  pch = 21,
  cex,
  lwd,
  extent = x,
  bg,
  expandBB = rep(0.04, 4),
  leg_pos = mf_get_leg_pos(x),
  leg_title = var,
  leg_title_cex = 0.8,
  leg_val_cex = 0.6,
  leg_val_rnd = 0,
  leg_val_dec = ".",
  leg_val_big = "",
  leg_frame = FALSE,
  leg_adj = c(0, 0),
  leg_size = 1,
  leg_box_cex = c(1, 1),
  leg_fg,
  leg_bg,
  leg_frame_border,
  add = TRUE
)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name(s) of the variable(s) to plot}

\item{breaks}{either a numeric vector with the actual breaks, or a classification method name (see \link{mf_get_breaks} and Details)}

\item{nbreaks}{number of classes}

\item{col}{a color, hex code or color name given by \code{\link{colors}}}

\item{alpha}{opacity, in the range 0,1}

\item{border}{border color}

\item{pch}{pch (point type) for symbols}

\item{cex}{cex (point size) for symbols}

\item{lwd}{border width}

\item{leg_pos}{position of the legend, one of 'topleft', 'top','topright', 'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two coordinates in map units (c(x, y)). Use \code{NA} to avoid plotting the legend, use 'interactive' to choose the legend position by clicking on the map.}

\item{leg_title}{legend title}

\item{leg_title_cex}{size of the legend title}

\item{leg_val_cex}{size of the values in the legend}

\item{leg_val_rnd}{number of decimal places of the values in the legend}

\item{leg_val_dec}{decimal separator}

\item{leg_val_big}{thousands separator}

\item{leg_frame}{whether to add a frame to the legend (TRUE) or not (FALSE)}

\item{leg_adj}{adjust the position of the legend in x and y directions}

\item{leg_size}{size of the legend; 2 means two times bigger}

\item{leg_box_cex}{width and height size expansion of boxes}

\item{leg_fg}{color of the legend foreground}

\item{leg_bg}{color of the legend background}

\item{leg_frame_border}{border color of the legend frame}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}
}
\value{
x is (invisibly) returned.
}
\description{
This function is deprecated. Please use \code{mf_map()} with \code{type = "grad"}
instead.

Plot graduated symbols based on quantitative data.
}
\details{
Breaks defined by a numeric vector or a classification method are
left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
will be mapped as [2 - 5[, [5 - 10[, [10 - 15[, [15 - 20].
The "jenks" method is an exception and has to be right-closed.
Jenks breaks computed as \code{c(2, 5, 10, 15, 20)}
will be mapped as [2 - 5], ]5 - 10], ]10 - 15], ]15 - 20].
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_map(mtq, "POP", "grad", pch = 22)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_init.R
\name{mf_init}
\alias{mf_init}
\title{Deprecated - Initialize a map with a specific extent}
\usage{
mf_init(x, expandBB = rep(0, 4), extent = x, bgc)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{SpatRaster}}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}

\item{extent}{object with an st_bbox method to define plot extent; defaults to x. extent and x must use the same CRS.}
}
\value{
No return value, a map is initiated.
}
\description{
This function is deprecated.
Please use \code{mf_map(x, col = NA, border = NA)} instead.

Plot an invisible layer with the extent of a spatial object.\cr
Always use \code{add = TRUE} in \code{mf_map()} calls following an
\code{mf_init()} call.
This function is similar to \code{mf_map(x, col = NA, border = NA)}.
}
\examples{
mtq <- mf_get_mtq()
target <- mtq[30, ]
mf_map(target, type = "base", col = NA, border = NA)
mf_map(mtq, add = TRUE)
# or
mf_map(mtq, extent = target)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_svg.R
\name{mf_svg}
\alias{mf_svg}
\title{Export a map in SVG format}
\usage{
mf_svg(
  x,
  filename = "map.svg",
  width,
  height,
  expandBB = rep(0, 4),
  svglite = TRUE,
  ...
)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{SpatRaster}}

\item{filename}{path to the exported file}

\item{width}{width of the figure (inches)}

\item{height}{height of the figure (inches)}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}

\item{svglite}{if TRUE, the export is done with the \code{svglite} package
if it is installed (see Details)}

\item{...}{further parameters}
}
\value{
No return value, an SVG device is initiated.
}
\description{
Export a map with the extent of a spatial object in SVG format.

SVG export is the perfect solution for editing maps with desktop vector
graphics software. SVG is a vector graphics file format.

If \code{width} is specified, then \code{height} is deduced from the width/height
ratio of \code{x}. Alternatively, if \code{height} is specified, then \code{width}
is deduced from the width/height ratio of \code{x}.
This helps to produce maps without too much wasted space.

Use `dev.off() to finish the export (see Examples).
}
\details{
The default driver for building SVG files, \code{grDevices::svg()}, has
limitations regarding speed, file size, editability, and font support.
The \code{svglite} package aims to solve these issues but it is not lightweight
in terms of dependencies, so it is not imported by \code{mapsf}, but rather
suggested.

However, we strongly recommend its use if the aim is to edit the maps
after export.
}
\examples{
mtq <- mf_get_mtq()
(filename <- tempfile(fileext = ".svg"))
mf_svg(mtq, filename = filename)
mf_map(mtq)
mf_title()
dev.off()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_export.R
\name{mf_export}
\alias{mf_export}
\title{Deprecated - Export a map}
\usage{
mf_export(
  x,
  filename = "map.png",
  width,
  height,
  res = 96,
  ...,
  expandBB = rep(0, 4)
)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{SpatRaster}}

\item{filename}{path to the exported file. If the file extension is ".png" a
png graphic device is opened, if the file extension is ".svg" a svg graphic
device is opened.}

\item{width}{width of the figure (pixels for png, inches for svg)}

\item{height}{height of the figure (pixels for png, inches for svg)}

\item{res}{resolution (for png)}

\item{...}{further parameters for png or svg export}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}
}
\value{
No return value, a map file is initiated (in PNG or SVG format).
}
\description{
This function is deprecated. Please use \code{\link{mf_png}} or
\code{\link{mf_svg}} instead.

Export a map with the extent of a spatial object.\cr
The map is exported in PNG or SVG format.\cr
If only one of \code{width} or \code{height} is set, \code{mf_export} uses
the width/height ratio of \code{x} bounding box to find a matching ratio for
the export.\cr
Always use \code{add = TRUE} in \code{mf_map} calls following an
\code{mf_export} call.\cr
Use \code{dev.off} to finish the export (see Examples).
}
\examples{
mtq <- mf_get_mtq()
(filename <- tempfile(fileext = ".png"))
mf_png(mtq, filename = filename)
mf_map(mtq)
mf_title()
dev.off()
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_text.R
\name{mf_text}
\alias{mf_text}
\title{Plot a text}
\usage{
mf_text(
  x,
  txt = "Text",
  cex = 0.8,
  col_txt,
  pos = "center",
  offset = 0,
  align = "center",
  font = 1,
  family = "sans",
  halo = FALSE,
  col_halo,
  cex_halo = cex,
  box = FALSE,
  col_box,
  col_box_border,
  lwd_box = 2,
  line = 0,
  clockwise = TRUE,
  lwd = 2,
  col_line,
  arrow = TRUE,
  cex_arrow = 1,
  adj = c(0, 0)
)
}
\arguments{
\item{x}{an sf object (the first row is used),
a couple of coordinates (c(x, y)),
a position (one of 'topleft', 'top','topright', 'right', 'bottomright',
'bottom', 'bottomleft', 'left'),
or "interactive" for interactive placement.}

\item{txt}{text to display}

\item{cex}{text size}

\item{col_txt}{text color}

\item{pos}{position of the text relative to \code{x} (one of 'center',
'topleft', 'top', 'topright', 'right', 'bottomright', 'bottom',
'bottomleft', 'left').
Ignored if \code{x} is a position.}

\item{offset}{offset between the text and \code{x}. Ignored if \code{x} is a position.}

\item{align}{text alignement, one of "left", "right" or "center"}

\item{font}{text font}

\item{family}{text family}

\item{halo}{add a halo around the text}

\item{col_halo}{halo color}

\item{cex_halo}{halo width}

\item{box}{add a box around the text}

\item{col_box}{box color}

\item{col_box_border}{box border color}

\item{lwd_box}{line width of the box border}

\item{line}{type of the line drawn between the text and \code{x}.
0 for no line, 1 for a straight line, 2 for a line with an angle,
3 for a curved line. Ignored if \code{x} is a position.}

\item{clockwise}{direction of the curve for types 2 and 3}

\item{lwd}{line width}

\item{col_line}{line color}

\item{arrow}{add an arrow to the line}

\item{cex_arrow}{arrow size}

\item{adj}{adjust the text position in x and y directions}
}
\value{
No return value, an text is displayed.
}
\description{
Plot a text on the map.
}
\examples{
library(mapsf)
mf_theme("base")
mtq <- mf_get_mtq()
mtq_p <- mf_get_mtq("points")
mf_map(mtq, expandBB = c(0, .1, 0, .3))
mf_map(mtq_p[c(2, 3, 17, 28), ],
  pch = 4, lwd = 1.5, cex = .5, col = "red",
  add = TRUE
)
mf_title("Title of the map", banner = TRUE)
mf_frame()
mf_text(x = "topright", txt = "x = 'topright'")
mf_text(
  x = "bottomleft",
  txt = "x = 'bottomleft'\nadj = c(4,4)\nalign = 'left'",
  adj = c(4, 4),
  align = "left"
)
mf_text(
  x = c(728000, 1625500),
  txt = "x = c(X, Y)",
  pos = "right",
  line = 2,
  offset = 5
)
mf_text(
  mtq[3, ],
  txt = "pos = 'top'\nhalo = TRUE",
  pos = "top",
  align = "center",
  halo = TRUE
)
mf_text(
  mtq[3, ],
  txt = "pos = 'bottomleft'\nhalo = TRUE\nalign = 'right'",
  pos = "bottomleft",
  align = "right",
  halo = TRUE
)
mf_text(
  x = mtq[17, ],
  txt = "pos = 'bottomright'\nline = 1\nbox = TRUE",
  pos = "bottomright",
  offset = 10,
  line = 1,
  box = TRUE
)
mf_text(
  x = mtq[17, ],
  txt = "pos = 'topright'\nline = 1\nalign = 'left'",
  pos = "topright",
  offset = 15,
  align = "left",
  line = 1
)
mf_text(
  x = mtq[2, ],
  txt = "pos = 'topleft'\nline = 2\nbox = TRUE\nclockwise = TRUE",
  pos = "topleft",
  offset = 8,
  clockwise = TRUE,
  line = 2,
  box = TRUE
)
mf_text(
  x = mtq[2, ],
  txt = "pos = 'bottomleft'\nline = 2\nclockwise = FALSE",
  pos = "bottomleft",
  offset = 6,
  clockwise = FALSE,
  line = 2,
  box = FALSE
)
mf_text(
  x = mtq[28, ],
  txt = "pos = 'topright'\nline = 3\nclockwise = FALSE",
  pos = "topright",
  offset = 10,
  clockwise = FALSE,
  line = 3,
  halo = TRUE,
  align = "left"
)
mf_text(
  x = mtq[28, ],
  txt = "pos = 'right'\nline = 3\nbox = TRUE\nclockwise = TRUE",
  pos = "right",
  offset = 10,
  clockwise = TRUE,
  line = 3,
  box = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_title.R
\name{mf_title}
\alias{mf_title}
\title{Plot a title}
\usage{
mf_title(txt = "Map Title", pos, tab, bg, fg, cex, line, font, inner, banner)
}
\arguments{
\item{txt}{title text}

\item{pos}{position, one of 'left', 'center', 'right'}

\item{tab}{if TRUE the title is displayed as a tab}

\item{bg}{background of the title}

\item{fg}{foreground of the title}

\item{cex}{cex of the title}

\item{line}{number of lines used for the title}

\item{font}{font of the title}

\item{inner}{if TRUE the title is displayed inside the plot area;
if FALSE the title is displayed in the top margin}

\item{banner}{if TRUE the title is displayed as a banner}
}
\value{
No return value, a title is displayed.
}
\description{
Plot a title
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_title()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_grad}
\alias{mf_map_grad}
\title{Plot graduated symbols}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name of the variable to map}

\item{type}{"grad"}

\item{breaks}{either a numeric vector with the actual breaks, or a
classification method name. The main methods are 'quantile',
'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
See \link{mf_get_breaks} for details.}

\item{nbreaks}{number of classes}

\item{col}{color of the graduated symbols or lines, a hex code or
a color name given by \link{colors}.
The default color is the highlight color (see \link{mf_theme}).}

\item{border}{border color for symbols, a hex code or
color name given by \link{colors}. The default color is the background color (see
\link{mf_theme}).}

\item{lwd}{border width for graduated symbols, a vector of line widths for
graduated lines}

\item{pch}{type of symbol to use for points, see \link{pch} (points only)}

\item{cex}{a vector of sizes for symbols (points only)}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}

\item{leg_*}{legend arguments described in \link{mf_map}}
}
\value{
x is (invisibly) returned.
}
\description{
With the \strong{grad} map type, \code{mf_map()} displays graduated symbols on a map.

Graduated symbols are based on classified quantitative variables.

For polygons, centroids are used to plot graduated symbols.
\subsection{Usage}{

For polygons and points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "grad",
       breaks = "quantile", nbreaks = 3,
       col, border, lwd = 0.7, pch = 21, cex,
       alpha, expandBB, extent, bg, add = TRUE, leg_*)
}\if{html}{\out{</div>}}

For lines:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "grad",
       breaks = "quantile", nbreaks = 3,
       col, lwd,
       alpha, expandBB, extent, bg, add = TRUE, leg_*)
}\if{html}{\out{</div>}}
}
}
\examples{
mtq <- mf_get_mtq()
flows <- mf_get_mtq("lines")
mf_map(mtq, bg = "cornsilk2")
mf_map(flows, "fij", "grad",
  breaks = "geom", nbreaks = 3,
  lwd = c(1, 3, 7),
  leg_title = "N. commuters",
  leg_pos = "bottomleft", leg_val_rnd = 0
)
mf_map(mtq, "POP", "grad",
  breaks = c(686, 5000, 25000, 82502),
  cex = c(1, 2, 4), pch = 22, col = "steelblue",
  leg_title = "Population", leg_pos = "topright",
  leg_frame = TRUE
)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_credits.R
\name{mf_credits}
\alias{mf_credits}
\title{Plot credits}
\usage{
mf_credits(
  txt = "Source(s) & Author(s)",
  pos = "bottomleft",
  col,
  cex = 0.6,
  font = 3,
  bg = NA
)
}
\arguments{
\item{txt}{text of the credits, use '\\n' to add line breaks}

\item{pos}{position, one of 'bottomleft', 'bottomright' or 'rightbottom'}

\item{col}{color of the text, hex code or color name given by \link{colors}.
The default color is the highlight color (see \link{mf_theme}).}

\item{cex}{cex of the credits}

\item{font}{font of the credits}

\item{bg}{background color}
}
\value{
No return value, credits are displayed.
}
\description{
Plot credits (sources, author, year...).
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_credits(txt = "Author\nSources - Year")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_prop_choro}
\alias{mf_map_prop_choro}
\title{Plot proportional symbols with choropleth coloration}
\arguments{
\item{x}{object of class \code{sf} (polygons or points)}

\item{var}{names of the variables to map. The first value refers to the
proportional symbols, the second one to the choropleth coloration.}

\item{type}{"prop_choro"}

\item{inches}{size of the largest symbol in inches (radius for circles,
half width for squares)}

\item{val_max}{maximum value corresponding to the largest symbol or line}

\item{symbol}{type of proportional symbols, either "circle" or "square"}

\item{breaks}{either a numeric vector with the actual breaks, or a
classification method name. The main methods are 'quantile',
'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
See \link{mf_get_breaks} for details.}

\item{nbreaks}{number of classes}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}. The default palette is the pal_seq palette
(see \link{mf_theme}).}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{border}{border color of proportional symbols, a hex code or
color name given by \link{colors}. The default color is the background color (see
\link{mf_theme}).}

\item{lwd}{border width of proportional symbols}

\item{col_na}{color for missing values, a hex code or
a color name given by \link{colors}}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}

\item{leg_*}{legend arguments described in \link{mf_map}. See details for
arguments with two values.}
}
\value{
x is (invisibly) returned.
}
\description{
\code{mf_map()} with \strong{prop_choro} type creates symbols that are proportional
to values of a first variable and colored to reflect the classification of a
second variable.

This map types uses two variables and some arguments need to be set for both
variables (see Details).

For polygons, centroids are used to plot proportional symbols. This map type
is not available for lines.
\subsection{Usage}{

For polygons and points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "prop_choro",
       inches = 0.3, val_max,  symbol = "circle",
       pal, rev = FALSE, breaks = "quantile", nbreaks,
       border, lwd = 0.7, col_na = "white",
       alpha, expandBB, extent, bg, add = TRUE, leg_*)
}\if{html}{\out{</div>}}
}
}
\details{
Legend arguments that need two values are: 'leg_title',
'leg_val_rnd', and 'leg_horiz'. The first
values refers to the proportional symbols legend, the second one to the
choropleth legend.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mtq[6, "MED"] <- NA
mf_map(
  x = mtq, var = c("POP", "MED"), type = "prop_choro",
  inches = .2,
  val_max = 90000, symbol = "circle",
  col_na = "grey90", pal = "Cividis",
  breaks = "msd", nbreaks = 4, lwd = 1,
  leg_pos = "topright",
  leg_title = c("Population", "Median Income"),
  leg_val_rnd = c(0, 1),
  leg_horiz = c(TRUE, FALSE),
  leg_title_cex = .9,
  leg_val_dec = ",",
  leg_val_cex = .8,
  leg_size = 1,
  add = TRUE
)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}, \link{mf_map_prop}, \link{mf_map_choro}, \code{\link[=mf_distr]{mf_distr()}},
\code{\link[=mf_get_breaks]{mf_get_breaks()}}, \code{\link[=mf_get_pal]{mf_get_pal()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_leg_pos.R
\name{mf_get_leg_pos}
\alias{mf_get_leg_pos}
\title{Get the optimal position of a legend}
\usage{
mf_get_leg_pos(x, n = 1)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{n}{number of positions to get (1 or 2)}
}
\value{
A vector of position is returned
}
\description{
Find the optimal for one or two legends. The optimal position is
a position that minimizes overlap between a spatial object and a legend.
}
\examples{
mtq <- mf_get_mtq()
mf_get_leg_pos(mtq)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_distr.R
\name{mf_distr}
\alias{mf_distr}
\title{Plot a distribution}
\usage{
mf_distr(
  x,
  nbins,
  bw,
  breaks,
  pal,
  alpha = 1,
  rev = FALSE,
  main = "Distribution",
  yaxt = TRUE,
  ylab = "Density"
)
}
\arguments{
\item{x}{a numeric variable}

\item{nbins}{number of bins in the histogram}

\item{bw}{bandwidth of the density curve}

\item{breaks}{a vector of class boundaries. If \code{breaks} is used, the boxplot
is not displayed.}

\item{pal}{a color, a set of colors (hex codes) or a palette name.
Palette names can be obtained with \link{hcl.pals}. The default color is either
the highlight color if \code{breaks} is not used, or the background color
otherwise (see \link{mf_theme}).}

\item{alpha}{opacity, in the range [0,1] (0 means
transparent and 1 means opaque). Default is set to 1.}

\item{rev}{logical indicating whether the ordering of the colors should be
reversed}

\item{main}{plot title}

\item{yaxt}{if FALSE the y axis is not displayed}

\item{ylab}{y axis label}
}
\value{
The number of bins of the histogram and the bandwidth of the density
curve are (invisibly) returned in a list.
}
\description{
This function displays the statistical distribution of a variable with
a histogram, a box plot, a strip chart and a density curve on the same plot.

This graphic can be useful to choose an appropriate classification method for
choropleth maps.

User-defined class boundaries can also be displayed on the plot.
}
\examples{
(mf_distr(rnorm(1000)))
mf_distr(rbeta(1000, .6, 7))
mf_distr(rbeta(1000, 5, .6))
a <- rbeta(1000, .6, 7)
bks <- mf_get_breaks(a, nbreaks = 5, breaks = "quantile")
mf_distr(a, breaks = bks)
mf_distr(a,
  breaks = bks, pal = "Teal", yaxt = FALSE,
  main = 'Classification method : "quantile"'
)
}
\seealso{
\link{mf_map_choro}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_base}
\alias{mf_map_base}
\title{Plot a base map}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{sfg}}

\item{col}{a color, hex code or color name given by \link{colors}.
The default color for polygons is the foreground color, the default color for
points and lines the highlight color (see \link{mf_theme}).}

\item{border}{border color for polygons and points symbols, hex code or
color name given by \link{colors}. The default color for polygon is the
highlight color, the default color for points is the foreground color (see
\link{mf_theme}).}

\item{lwd}{border width for polygons and points symbols, lines width}

\item{lty}{type of line for polygons borders and lines}

\item{pch}{type of symbol to use for points, see \link{pch}}

\item{cex}{symbols size, 2 means 2 times bigger}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}
}
\value{
x is (invisibly) returned.
}
\description{
\code{mf_map()} can be used to display geographic layers (\code{sf} objects), using
the default map type \strong{base}.
\subsection{Usage}{

For polygons:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, col, border, lwd = 0.7, lty = 1,
       alpha, expandBB, extent, bg, add = FALSE)
}\if{html}{\out{</div>}}

For points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, col, border, pch = 20, cex = 1, lwd = 0.7,
       alpha, expandBB, extent, bg, add = FALSE)
}\if{html}{\out{</div>}}

For lines:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, col, lwd = .7, lty = 1,
       alpha, expandBB, extent, bg, add = FALSE)
}\if{html}{\out{</div>}}
}
}
\examples{
mtq <- mf_get_mtq()
pts <- mf_get_mtq("points")
flows <- mf_get_mtq("lines")
mf_map(mtq, lty = 3)
mf_map(pts, col = "red", border = "white", pch = 21, add = TRUE)
mf_map(flows, col = "coral", lwd = 2, add = TRUE)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_ratio.R
\name{mf_get_ratio}
\alias{mf_get_ratio}
\title{Get map width and height values}
\usage{
mf_get_ratio(x, width, height, expandBB = rep(0, 4), theme = mf_theme())
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{SpatRaster}}

\item{width}{width of the figure (inches), use only one of width or height}

\item{height}{height of the figure (inches), use only one of width or height}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}

\item{theme}{theme used for the map}
}
\value{
Width and height are returned in inches.
}
\description{
This function is to be used to get width and height values
for maps created in reports (*.Rmd, *.qmd).\cr
It uses the width / height ratio of a spatial object bounding box to find a
matching ratio for the map.\cr
If width is specified, then height is deduced from the width / height ratio
of x, figure margins and title size.\cr
If height is specified, then width is
deduced from the width / height ratio of x, figure margins and title size.
}
\examples{
mtq <- mf_get_mtq()
mf_get_ratio(x = mtq, width = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_pencil.R
\name{mf_get_pencil}
\alias{mf_get_pencil}
\title{Get a pencil layer from polygons}
\usage{
mf_get_pencil(x, size = 100, buffer = 0, lefthanded = TRUE, clip = FALSE)
}
\arguments{
\item{x}{an sf object, a simple feature collection (POLYGON or MULTIPOLYGON).}

\item{size}{density of the penciling. Median number of points used to build
the MULTILINESTRING.}

\item{buffer}{buffer around each polygon. This buffer (in map units) is used
to take sample points. A negative value adds a margin between the penciling
and the original polygons borders}

\item{lefthanded}{if TRUE the penciling is done left-handed style.}

\item{clip}{if TRUE, the penciling is cut by the original polygon.}
}
\value{
A MULTILINESTRING sf object is returned.
}
\description{
Create a pencil layer. This function transforms a POLYGON or
MULTIPOLYGON sf object into a MULTILINESTRING one.
}
\examples{
mtq <- mf_get_mtq()
mtq_pencil <- mf_get_pencil(x = mtq, clip = FALSE)
mf_map(mtq)
mf_map(mtq_pencil, add = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_arrow.R
\name{mf_arrow}
\alias{mf_arrow}
\title{Plot a north arrow}
\usage{
mf_arrow(pos = "topleft", col, cex = 1, adj = c(0, 0), align)
}
\arguments{
\item{pos}{position. It can be one of 'topleft', 'top','topright', 'right',
'bottomright', 'bottom','bottomleft', 'left', 'interactive' or a vector of
two coordinates in map units (c(x, y))}

\item{col}{arrow color}

\item{cex}{arrow size}

\item{adj}{adjust the position of the north arrow in x and y directions}

\item{align}{object of class \code{sf} or \code{sfc} used to adjust the
arrow to the real north}
}
\value{
No return value, a north arrow is displayed.
}
\description{
Plot a north arrow.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_arrow(pos = "topright")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_worldmap.R
\name{mf_worldmap}
\alias{mf_worldmap}
\title{Plot a point on a world map}
\usage{
mf_worldmap(
  x,
  lon,
  lat,
  water_col = "lightblue",
  land_col = "grey60",
  border_col = "grey40",
  border_lwd = 0.8,
  ...
)
}
\arguments{
\item{x}{object of class \code{sf} or \code{sfc}}

\item{lon}{longitude}

\item{lat}{latitude}

\item{water_col}{color of the water}

\item{land_col}{color of the land}

\item{border_col}{color of the borders}

\item{border_lwd}{width of the borders}

\item{...}{further parameters related to the plotted point aspect
(cex, pch, col...)}
}
\value{
No return value, a world map is displayed.
}
\description{
Plot a point on a world map.
}
\examples{
mtq <- mf_get_mtq()
mf_worldmap(mtq)
mf_worldmap(lon = 24, lat = 39)
mf_worldmap(
  lon = 106, lat = 26,
  pch = 4, lwd = 3, cex = 2, col = "tomato4",
  water_col = "#232525", land_col = "#A9B7C6",
  border_col = "white", border_lwd = 1
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_links.R
\name{mf_get_links}
\alias{mf_get_links}
\title{Get a link layer from a data.frame of links}
\usage{
mf_get_links(x, df, x_id, df_id)
}
\arguments{
\item{x}{an sf object, a simple feature collection.}

\item{df}{a data.frame that contains identifiers of starting and ending
points.}

\item{x_id}{name of the identifier variable in x, default to the first
column (optional)}

\item{df_id}{names of the identifier variables in df, character vector of
length 2, default to
the two first columns. (optional)}
}
\value{
An sf object is returned, it is composed of df and the sfc
(LINESTRING) of links.
}
\description{
Create a link layer from a data.frame of links and an sf object.
}
\examples{
mtq <- mf_get_mtq()
mob <- read.csv(system.file("csv/mob.csv", package = "mapsf"))
# Select links from Fort-de-France (97209))
mob_97209 <- mob[mob$i == 97209, ]
# Create a link layer
mob_links <- mf_get_links(x = mtq, df = mob_97209)
# Plot the links
mf_map(mtq)
mf_map(mob_links, col = "red4", lwd = 2, add = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_raster.R
\name{mf_raster}
\alias{mf_raster}
\title{Plot a raster}
\usage{
mf_raster(
  x,
  type,
  nbreaks,
  breaks = "equal",
  val_order,
  pal,
  alpha = NULL,
  rev = FALSE,
  expandBB = rep(0, 4),
  bg,
  leg_pos = "right",
  leg_title = names(x),
  leg_title_cex = 0.8,
  leg_val_cex = 0.6,
  leg_val_rnd = 1,
  leg_val_dec = ".",
  leg_val_big = "",
  leg_frame = FALSE,
  leg_frame_border,
  leg_horiz = FALSE,
  leg_adj = c(0, 0),
  leg_box_border,
  leg_box_cex = c(1, 1),
  leg_fg,
  leg_bg,
  leg_size = 1,
  add = FALSE,
  ...
)
}
\arguments{
\item{x}{a \code{SpatRaster}}

\item{type}{type of raster map, one of "continuous", "classes", or
"interval". Default type for a numeric and categorial raster are
"continuous" and "classes" respectively.}

\item{nbreaks}{number of classes (for type = "interval" only)}

\item{breaks}{either a numeric vector with the actual breaks
(for type = "continuous" and type = "interval"), or a
classification method name (for type = "interval" only; see \link{mf_get_breaks}
for details).}

\item{val_order}{modalities order in the legend, a character vector that
matches \code{var} modalities. Default to alphabetic order of modalities (for
type = "classes" only).}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}. The default palette is the pal_quali palette for
type = "classes" and pal_seq otherwise (see \link{mf_theme}).}

\item{alpha}{pal` opacity, in the range [0,1] (0 means
transparent and 1 means opaque). Default is set to 1.}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}

\item{bg}{background color of the map, hex code or color name given by
\link{colors}, ignored if \code{add = TRUE}}

\item{leg_pos}{position of the legend, one of 'topleft', 'top','topright',
'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two
coordinates in map units (c(x, y)). Use \code{NA} to avoid plotting the legend,
use 'interactive' to choose the legend position by clicking on the map.}

\item{leg_title}{legend title}

\item{leg_title_cex}{size of the title}

\item{leg_val_cex}{size of the values}

\item{leg_val_rnd}{number of decimal places of the values displayed in the
legend}

\item{leg_val_dec}{decimal separator}

\item{leg_val_big}{thousands separator}

\item{leg_frame}{whether to add a frame to the legend (TRUE) or not (FALSE)}

\item{leg_frame_border}{border color of the legend frame}

\item{leg_horiz}{display the legend horizontally (for proportional symbols
and choropleth types)}

\item{leg_adj}{adjust the position of the legend in x and y directions}

\item{leg_box_border}{border color of legend boxes (for types related to
choropleth and typology)}

\item{leg_box_cex}{width and height size expansion of boxes}

\item{leg_fg}{color of the legend foreground}

\item{leg_bg}{color of the legend background}

\item{leg_size}{size of the legend. Combine this argument with
\code{leg_title_cex} and \code{leg_val_cex}.}

\item{add}{whether to add the layer to an existing plot (TRUE) or
not (FALSE)}

\item{...}{bgalpha, smooth, maxcell or other arguments passed to
\code{terra::plotRGB} or \code{terra::plot}}
}
\value{
x is (invisibly) returned.
}
\description{
Plot a raster object (\code{SpatRaster} from \code{terra}).
}
\examples{
if (require("terra")) {
  # multi band
  logo <- rast(system.file("ex/logo.tif", package = "terra"))
  mf_raster(logo)

  # one band
  elev <- rast(system.file("ex/elev.tif", package = "terra"))

  ## continuous
  mf_raster(elev)
  mf_raster(elev,
    pal = "Burg", expandBB = c(.2, 0, 0, 0),
    leg_pos = "bottom", leg_horiz = TRUE
  )

  ## continuous with colors and breaks
  mf_raster(elev,
    type = "continuous",
    breaks = c(141, 400, 547),
    pal = c("darkseagreen1", "black", "red")
  )

  ## interval
  mf_raster(elev,
    type = "interval",
    nbreaks = 5, breaks = "equal", pal = "Teal"
  )

  ## classes
  elev2 <- classify(elev, c(140, 400, 450, 549))
  lev_evel <- data.frame(ID = 0:2, elevation = c("Low", "High", "Super High"))
  levels(elev2) <- lev_evel
  mf_raster(elev2)
  mf_raster(elev2,
    pal = c("salmon4", "olivedrab", "yellow3"),
    val_order = c("Super High", "High", "Low")
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_mtq.R
\name{mf_get_mtq}
\alias{mf_get_mtq}
\title{Get the 'mtq' dataset}
\source{
\subsection{For polygons (municipalities shapes) and points (municipalities centroids)}{

\href{https://www.insee.fr/fr/statistiques/2521169}{Base comparateur de territoires}
(data, upload date: 2018-09-25) & ADMIN EXPRESS-COG (geometry, 2018 edition).

Citation: Insee and IGN, 2018
}

\subsection{For lines (professional mobility flows from Fort-de-France to other municipalities)}{

\href{https://www.insee.fr/fr/statistiques/3566477}{Flux de mobilité - déplacements domicile-lieu de travail} (upload date: 2018-08-01)

Citation: Insee, 2018
}
}
\usage{
mf_get_mtq(x = "polygons")
}
\arguments{
\item{x}{one of "polygons", "points", "lines"}
}
\value{
an sf object
}
\description{
Import the mtq dataset (Martinique municipalities).
}
\details{
This a wrapper around \code{st_read(system.file("gpkg/mtq.gpkg", package = "mapsf"), layer = x, quiet = TRUE)}.
\subsection{For polygons (municipalities shapes) and points (municipalities centroids)}{
\itemize{
\item \strong{INSEE_COM}: Municipality identifier
\item \strong{STATUS}: Municipality administrative status
\item \strong{LIBGEO}: Municipality name
\item \strong{POP}: Total population, 2015
\item \strong{MED}: Median disposable income adjusted per equivalent household member, in euros, 2015
\item \strong{CHOM}: Unemployed population, 2015
\item \strong{ACT}: Active population, 2015
}
}

\subsection{For lines (professional mobility flows from Fort-de-France to other municipalities)}{
\itemize{
\item \strong{i}: Municipality of residence identifier
\item \strong{j}: Municipality of workplace identifier
\item \strong{fij}: Flows of workers (employed population, 15 y.o. or more, 2015, only flows \> 100)
\item \strong{sj}: Administrative status of the workplace municipality
}
}
}
\examples{
mtq <- mf_get_mtq()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_symb_choro.R
\name{mf_symb_choro}
\alias{mf_symb_choro}
\title{Deprecated - Plot symbols using choropleth coloration}
\usage{
mf_symb_choro(
  x,
  var,
  pal,
  alpha = NULL,
  rev = FALSE,
  breaks = "quantile",
  nbreaks,
  border,
  pch,
  cex = 2,
  lwd = 0.7,
  pch_na = 4,
  cex_na = 1,
  col_na = "white",
  val_order,
  extent = x,
  bg,
  expandBB = rep(0.04, 4),
  leg_pos = mf_get_leg_pos(x, 1),
  leg_title = var,
  leg_title_cex = c(0.8, 0.8),
  leg_val_cex = c(0.6, 0.6),
  leg_val_rnd = 2,
  leg_val_dec = ".",
  leg_val_big = "",
  leg_no_data = c("No data", "No data"),
  leg_frame = c(FALSE, FALSE),
  leg_frame_border,
  leg_horiz = FALSE,
  leg_adj = c(0, 0),
  leg_fg,
  leg_bg,
  leg_size = 1,
  leg_box_border,
  leg_box_cex = c(1, 1),
  add = TRUE
)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name(s) of the variable(s) to plot}

\item{pal}{a set of colors (hex codes) or a palette name (valid palette names can be obtained with \code{\link{hcl.pals}}).}

\item{alpha}{opacity, in the range 0,1}

\item{rev}{if \code{pal} is a valid palette name, whether the ordering of the colors should be reversed (TRUE) or not (FALSE)}

\item{breaks}{either a numeric vector with the actual breaks, or a classification method name (see \link{mf_get_breaks} and Details)}

\item{nbreaks}{number of classes}

\item{border}{border color}

\item{pch}{pch (point type) for symbols}

\item{cex}{cex (point size) for symbols}

\item{lwd}{border width}

\item{pch_na}{point type for NA values}

\item{cex_na}{point size for NA values}

\item{col_na}{color for missing values}

\item{val_order}{values order, a character vector that matches var modalities}

\item{leg_pos}{position of the legend, two of 'topleft', 'top','topright', 'right','bottomright', 'bottom', 'bottomleft', 'left' or a vector of two coordinates in map units (c(x, y)). leg_pos argument can be c('position', 'position'), c('position', x2, y2), c(x1,y1, 'position') or c(x1, y1, x2, y2). Use \code{NA} to avoid plotting the legend, use 'interactive' to choose the legend position by clicking on the map.}

\item{leg_title}{legend title}

\item{leg_title_cex}{size of the legend title}

\item{leg_val_cex}{size of the values in the legend}

\item{leg_val_rnd}{number of decimal places of the values in the legend}

\item{leg_val_dec}{decimal separator}

\item{leg_val_big}{thousands separator}

\item{leg_no_data}{label for missing values}

\item{leg_frame}{whether to add a frame to the legend (TRUE) or not (FALSE)}

\item{leg_frame_border}{border color of the legend frame}

\item{leg_horiz}{display the legend horizontally (for proportional symbols and choropleth types)}

\item{leg_adj}{adjust the position of the legend in x and y directions}

\item{leg_fg}{color of the legend foreground}

\item{leg_bg}{color of the legend background}

\item{leg_size}{size of the legend; 2 means two times bigger}

\item{leg_box_border}{border color of legend boxes}

\item{leg_box_cex}{width and height size expansion of boxes}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}
}
\value{
x is (invisibly) returned.
}
\description{
This function is deprecated. Please use \code{mf_map()} with \code{type = "symb_choro"}
instead.

Plot symbols with colors based on a quantitative
data classification.
}
\details{
Breaks defined by a numeric vector or a classification method are
left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
will be mapped as [2 - 5[, [5 - 10[, [10 - 15[, [15 - 20].
The "jenks" method is an exception and has to be right-closed.
Jenks breaks computed as \code{c(2, 5, 10, 15, 20)}
will be mapped as [2 - 5], ]5 - 10], ]10 - 15], ]15 - 20].
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_map(mtq, c("STATUS", "MED"), "symb_choro")

mf_map(mtq)
mtq$STATUS[30] <- NA
mtq$MED[5] <- NA
mf_map(mtq, c("STATUS", "MED"),
  type = "symb_choro",
  pal = "Reds 3", breaks = "quantile", nbreaks = 4,
  pch = 21:23, cex = c(3, 2, 1),
  pch_na = 25, cex_na = 1.5, col_na = "blue",
  val_order = c(
    "Prefecture",
    "Sub-prefecture",
    "Simple municipality"
  )
)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_breaks.R
\name{mf_get_breaks}
\alias{mf_get_breaks}
\title{Get class intervals}
\usage{
mf_get_breaks(x, nbreaks, breaks, k = 1, central = FALSE, ...)
}
\arguments{
\item{x}{a vector of numeric values. NA and Inf values are not used in the
classification.}

\item{nbreaks}{a number of classes}

\item{breaks}{a classification method; the main methods are "quantile",
"equal", "msd", "ckmeans" (natural breaks), "Q6" and "geom".
See Details for the full list.}

\item{k}{number of standard deviation for "msd" method (see Details)}

\item{central}{creation of a central class for "msd" method (see Details)}

\item{...}{further arguments
of \code{\link[classInt:classIntervals]{classIntervals}}}
}
\value{
A numeric vector of breaks
}
\description{
A function to classify continuous variables.

This function is a wrapper for
\code{\link[classInt:classIntervals]{classIntervals}}
with some additional methods.
}
\details{
\subsection{\code{classInt} methods}{

"fixed", "sd", "equal", "pretty", "quantile", "kmeans", "hclust", "bclust",
"fisher", "jenks", "dpih", "headtails", "maximum", and "box"
are \code{\link[classInt:classIntervals]{classIntervals}}
methods. You may need to pass additional arguments for some of them.
}

\subsection{Natural breaks method}{

The "jenks", "fisher" and "ckmeans" methods are based on the same concept of
\strong{natural breaks} and and produce similar groupings. The use of "ckmeans" is
recommended.
\itemize{
\item The "jenks" method produces class boundaries falling on data points and is
slow.
\item The "fisher" method produces class boundaries located more conveniently
between data points, and is faster than the "jenks" method.
\item The "ckmeans" method produces exactly the same class boundaries as the
"fisher" method, but is much faster. It uses the optimal univariate
k-means method from the \code{Ckmeans.1d.dp} package.
If the "ckmeans" method is selected but the \code{Ckmeans.1d.dp} package is not
installed then the "fisher" method is used.
}

The relative speeds of these three methods may vary depending on the number
of data points and the number of classes.
}

\subsection{Other methods}{

The "msd" method is based on the \strong{mean} and the \strong{standard deviation}
of a numeric vector.
The \code{nbreaks} parameter is not relevant, use \code{k} and
\code{central} instead. \code{k} indicates
the extent of each class in share of standard deviation.
If \code{central=TRUE} then
the mean value is the center of a class else the mean is a break value.\cr\cr
The "q6" method uses the following \code{\link[stats:quantile]{quantile}}
probabilities: 0, 0.05, 0.275, 0.5, 0.725, 0.95, 1.\cr\cr
The "Q6" method uses the following \code{\link[stats:quantile]{quantile}}
probabilities: 0, 0.05, 0.25, 0.5, 0.75, 0.95, 1.\cr\cr
The "geom" method is based on a geometric progression along
the variable values, all values must be strictly greater than zero.\cr\cr
The "arith" method is based on an arithmetic progression along
the variable values.\cr\cr
The "em" method is based on nested averages computation.
}

\subsection{Class boundaries}{

Breaks defined by a numeric vector or a classification method are
left-closed: breaks defined by \code{c(2, 5, 10, 15, 20)}
will be mapped as [2 - 5[, [5 - 10[, [10 - 15[, [15 - 20].
}
}
\examples{
mtq <- mf_get_mtq()
mf_get_breaks(x = mtq$MED, nbreaks = 6, breaks = "quantile")
}
\seealso{
\link[classInt:classIntervals]{classIntervals}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_prop}
\alias{mf_map_prop}
\title{Plot proportional symbols}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name of the variable to map}

\item{type}{"prop"}

\item{inches}{size of the largest symbol in inches (radius for circles,
half width for squares)}

\item{val_max}{maximum value corresponding to the largest symbol or line}

\item{lwd_max}{width of the largest line}

\item{symbol}{type of proportional symbols, either "circle" or "square"}

\item{col}{color of the proportional symbols or lines, a hex code or
a color name given by \link{colors}.
The default color is the highlight color (see \link{mf_theme}).}

\item{border}{border color for proportional symbols, a hex code or
color name given by \link{colors}. The default color the background color (see
\link{mf_theme}).}

\item{lwd}{border width of proportional symbols}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}

\item{leg_*}{legend arguments described in \link{mf_map}}
}
\value{
x is (invisibly) returned.
}
\description{
With the \strong{prop} map type, \code{mf_map()} displays symbols (squares or circles)
with areas proportional to a quantitative variable (stocks).

For polygons, centroids are used to plot proportional symbols.
\subsection{Usage}{

For polygons and points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "prop",
       inches = 0.3, val_max, symbol, col, border, lwd,
       expandBB, extent, bg, alpha, add = FALSE, leg_*)

}\if{html}{\out{</div>}}

For lines:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "prop",
       val_max, lwd_max = 20, col,
       expandBB, extent, bg, alpha, add = FALSE, leg_*)
}\if{html}{\out{</div>}}
}
}
\examples{
mtq <- mf_get_mtq()
flows <- mf_get_mtq("lines")
mtq <- mf_get_mtq()
mf_map(mtq)
mf_map(mtq, "POP", "prop",
  inches = .4, leg_title = "Population",
  leg_pos = "topright"
)
mf_map(flows, "fij", "prop",
  lwd_max = 10, col = "steelblue2",
  leg_pos = "right"
)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_typo.R
\name{mf_typo}
\alias{mf_typo}
\title{Deprecated - Plot a typology map}
\usage{
mf_typo(
  x,
  var,
  pal,
  alpha = NULL,
  rev = FALSE,
  val_order,
  border,
  pch = 21,
  cex = 2,
  lwd = 0.7,
  col_na = "white",
  extent = x,
  bg,
  expandBB = rep(0.04, 4),
  leg_pos = mf_get_leg_pos(x),
  leg_title = var,
  leg_title_cex = 0.8,
  leg_val_cex = 0.6,
  leg_no_data = "No data",
  leg_frame = FALSE,
  leg_frame_border,
  leg_adj = c(0, 0),
  leg_size = 1,
  leg_box_border,
  leg_box_cex = c(1, 1),
  leg_fg,
  leg_bg,
  add = FALSE
)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name(s) of the variable(s) to plot}

\item{pal}{a set of colors (hex codes) or a palette name (valid palette names can be obtained with \code{\link{hcl.pals}}).}

\item{alpha}{opacity, in the range 0,1}

\item{rev}{if \code{pal} is a valid palette name, whether the ordering of the colors should be reversed (TRUE) or not (FALSE)}

\item{val_order}{values order, a character vector that matches var modalities}

\item{border}{border color}

\item{pch}{pch type of pch if x is a POINT layer}

\item{cex}{cex cex of the symbols if x is a POINT layer}

\item{lwd}{border width}

\item{col_na}{color for missing values}

\item{leg_pos}{position of the legend, one of 'topleft', 'top','topright', 'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two coordinates in map units (c(x, y)). Use \code{NA} to avoid plotting the legend, use 'interactive' to choose the legend position by clicking on the map.}

\item{leg_title}{legend title}

\item{leg_title_cex}{size of the legend title}

\item{leg_val_cex}{size of the values in the legend}

\item{leg_no_data}{label for missing values}

\item{leg_frame}{whether to add a frame to the legend (TRUE) or not (FALSE)}

\item{leg_frame_border}{border color of the legend frame}

\item{leg_adj}{adjust the position of the legend in x and y directions}

\item{leg_size}{size of the legend; 2 means two times bigger}

\item{leg_box_border}{border color of legend boxes}

\item{leg_box_cex}{width and height size expansion of boxes}

\item{leg_fg}{color of the legend foreground}

\item{leg_bg}{color of the legend background}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}
}
\value{
No return value, a map is displayed.
}
\description{
This function is deprecated. Please use \code{mf_map()} with \code{type = "typo"}
instead.

Plot a typology map.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq, "STATUS", "typo")
mtq[6, "STATUS"] <- NA
mf_map(
  x = mtq, var = "STATUS", type = "typo",
  pal = c("red", "blue", "yellow"), lwd = 1.1,
  val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
  col_na = "green", border = "brown",
  leg_pos = "bottomleft",
  leg_title = "Status", leg_title_cex = 1.1,
  leg_val_cex = 1, leg_no_data = "No data",
  leg_frame = TRUE, add = FALSE
)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_background.R
\name{mf_background}
\alias{mf_background}
\title{Plot a background image}
\usage{
mf_background(filename, ...)
}
\arguments{
\item{filename}{filename of the background image, PNG or JPG/JPEG format.}

\item{...}{ignored}
}
\value{
No return value, a background image is displayed.
}
\description{
Plot a background image on an existing plot
}
\examples{
if (require("jpeg")) {
  mtq <- mf_get_mtq()
  mf_map(mtq, col = NA, border = NA)
  mf_background(system.file("img/background.jpg", package = "mapsf"))
  mf_map(mtq, lwd = 3, col = NA, border = "white", add = TRUE)
  mf_credits(
    txt = "Background photo by Noita Digital on Unsplash",
    col = "white"
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_graticule.R
\name{mf_graticule}
\alias{mf_graticule}
\title{Plot graticules}
\usage{
mf_graticule(
  x,
  col,
  lwd = 1,
  lty = 1,
  expandBB = rep(0, 4),
  label = TRUE,
  pos = c("top", "left"),
  cex = 0.7,
  extent = x,
  bg,
  add = TRUE
)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{SpatRaster}}

\item{col}{graticules and label color}

\item{lwd}{graticules line width}

\item{lty}{graticules line type}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}

\item{label}{whether to add labels (TRUE) or not (FALSE)}

\item{pos}{labels positions ("bottom", "left", "top" and / or "right")}

\item{cex}{labels size}

\item{extent}{object with an \code{st_bbox} method to define plot extent;
defaults to \code{x}. \code{extent} and \code{x} must use the same CRS.}

\item{bg}{background color of the map, hex code or color name given by
\link{colors}, ignored if \code{add = TRUE}}

\item{add}{whether to add the layer to an existing plot (TRUE) or
not (FALSE)}
}
\value{
An (invisible) layer of graticules is returned (LINESTRING).
}
\description{
Display graticules and labels on a map.
}
\section{Use of graticules}{

From \code{\link[sf:st_graticule]{sf::st_graticule()}}:
"In cartographic visualization, the use of graticules is not advised, unless
the graphical output will be used for measurement or navigation, or the
direction of North is important for the interpretation of the content, or
the content is intended to display distortions and artifacts created by
projection. Unnecessary use of graticules only adds visual clutter but
little relevant information. Use of coastlines, administrative boundaries
or place names permits most viewers of the output to orient themselves
better than a graticule."
}

\examples{
mtq <- mf_get_mtq()
mf_map(mtq, expandBB = c(0, .1, .1, 0))
mf_graticule(mtq)

mf_graticule(
  x = mtq,
  col = "coral4",
  lwd = 2,
  lty = 2,
  expandBB = c(.1, 0, 0, .1),
  label = TRUE,
  pos = c("right", "bottom"),
  cex = .8,
  add = FALSE
)
mf_map(mtq, add = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{mapsf}
\alias{mapsf-package}
\alias{mapsf}
\title{Package description}
\description{
Create and integrate thematic maps in your workflow. This package
helps to design various cartographic representations such as proportional
symbols, choropleth or typology maps. It also offers several functions to
display layout elements that improve the graphic presentation of maps
(e.g. scale bar, north arrow, title, labels). \code{mapsf} maps \code{sf} objects on
\code{base} graphics.

A "Get Started" \strong{vignette} contains commented scripts on how to create
various maps: \code{vignette(topic = "mapsf", package = "mapsf")}
}
\section{Symbology}{

These functions display cartographic layers.
\itemize{
\item \code{\link[=mf_map]{mf_map()}} Plot a map
\item \code{\link[=mf_label]{mf_label()}} Plot labels
\item \code{\link[=mf_raster]{mf_raster()}} Plot a raster
\item \code{\link[=mf_graticule]{mf_graticule()}} Plot graticules
}
}

\section{Map layout}{

These functions are dedicated to the map layout design.
\itemize{
\item \code{\link[=mf_theme]{mf_theme()}} Set a theme
\item \code{\link[=mf_shadow]{mf_shadow()}} Plot a shadow
\item \code{\link[=mf_background]{mf_background()}} Plot a background image
\item \code{\link[=mf_annotation]{mf_annotation()}} Plot an annotation
\item \code{\link[=mf_arrow]{mf_arrow()}} Plot a north arrow
\item \code{\link[=mf_credits]{mf_credits()}} Plot credits
\item \code{\link[=mf_layout]{mf_layout()}} Plot a map layout
\item \code{\link[=mf_title]{mf_title()}} Plot a title
\item \code{\link[=mf_scale]{mf_scale()}} Plot a scale bar
\item \code{\link[=mf_inset_on]{mf_inset_on()}} / \code{\link[=mf_inset_off]{mf_inset_off()}} Plot an inset
\item \code{\link[=mf_worldmap]{mf_worldmap()}} Plot a point on a world map
\item \code{\link[=mf_legend]{mf_legend()}} Plot a legend
}
}

\section{Utility functions}{

\itemize{
\item \code{\link[=mf_svg]{mf_svg()}} Export a map in SVG file format
\item \code{\link[=mf_png]{mf_png()}} Export a map in SVG file format
\item \code{\link[=mf_distr]{mf_distr()}} Plot a distribution
\item \code{\link[=mf_get_links]{mf_get_links()}} Get a link layer from a data.frame of links
\item \code{\link[=mf_get_pal]{mf_get_pal()}} Get color palettes
\item \code{\link[=mf_get_breaks]{mf_get_breaks()}} Get class intervals
\item \code{\link[=mf_get_mtq]{mf_get_mtq()}} Get the 'mtq' dataset
\item \code{\link[=mf_get_ratio]{mf_get_ratio()}} Get map width and height values
\item \code{\link[=mf_get_pencil]{mf_get_pencil()}} Get a pencil layer from polygons
\item \code{\link[=mf_get_borders]{mf_get_borders()}} Get a border layer from polygons
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://riatelab.github.io/mapsf/}
  \item Report bugs at \url{https://github.com/riatelab/mapsf/issues/}
}

}
\author{
\strong{Maintainer}: Timothée Giraud \email{timothee.giraud@cnrs.fr} (\href{https://orcid.org/0000-0002-1932-3323}{ORCID})

Other contributors:
\itemize{
  \item Hugues Pecout (\href{https://orcid.org/0000-0002-0246-0954}{ORCID}) (Logo) [contributor]
  \item Ronan Ysebaert (\href{https://orcid.org/0000-0002-7344-5911}{ORCID}) (Cheat sheet) [contributor]
  \item Elina Marveaux (\href{https://orcid.org/0009-0000-8667-3019}{ORCID}) (Themes) [contributor]
  \item Ian Fellows (No overlap algorithm for labels, from wordcloud package) [copyright holder]
  \item Jim Lemon (Arc drawing algorithm for annotations, from plotrix package) [copyright holder]
  \item Danielle Navarro (\href{https://orcid.org/0000-0001-7648-6578}{ORCID}) (Bézier curve algorithm for text annotations) [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_legend.R
\name{mf_legend}
\alias{mf_legend}
\title{Plot a legend}
\usage{
mf_legend(
  type,
  val,
  pos = "left",
  pal = "Inferno",
  alpha = 1,
  col = "tomato4",
  inches = 0.3,
  val_max = NULL,
  symbol = "circle",
  self_adjust = FALSE,
  lwd = 0.7,
  border = "#333333",
  pch = seq_along(val),
  cex = rep(1, length(val)),
  title = "Legend Title",
  title_cex = 0.8 * size,
  val_cex = 0.6 * size,
  val_rnd = 0,
  val_dec = ".",
  val_big = "",
  col_na = "white",
  cex_na = 1,
  pch_na = 4,
  no_data = FALSE,
  no_data_txt = "No Data",
  box_border = "#333333",
  box_cex = c(1, 1),
  horiz = FALSE,
  frame_border,
  frame = FALSE,
  bg,
  fg,
  size = 1,
  return_bbox = FALSE,
  adj = c(0, 0)
)
}
\arguments{
\item{type}{type of legend:
\itemize{
\item \strong{prop} for proportional symbols,
\item \strong{choro} for choropleth maps,
\item \strong{cont} for continuous maps (e.g. raster),
\item \strong{typo} for typology maps,
\item \strong{symb} for symbols maps,
\item \strong{prop_line} for proportional lines maps,
\item \strong{grad_line} for graduated lines maps,
\item \strong{histo} for histograms,
\item \strong{choro_point} for choropleth points maps,
\item \strong{choro_line} for choropleth lines maps,
\item \strong{choro_symb} for choropleth on symbols maps
\item \strong{typo_line} for typology lines maps.
}}

\item{val}{vector of value(s) (for "prop" and "prop_line", at least c(min, max)
for "cont"),
vector of categories (for "symb", "typo", "typo_line"),
break labels (for "choro", "choro_point", "choro_line", "choro_symb",
and "grad_line"),
histogram parameters (for "histo").}

\item{pos}{position of the legend. It can be one of 'topleft', 'top',
'topright', 'right', 'bottomright', 'bottom','bottomleft',
'left', 'interactive' or a vector of two coordinates
in map units (c(x, y)).}

\item{pal}{a set of colors (hex codes) or a palette name (valid palette
names can be obtained with \link{hcl.pals}.}

\item{alpha}{if \code{pal} is a valid palette name, the
alpha-transparency level in the range [0,1]}

\item{col}{color of the symbols (for "prop") or color of the lines (for
"prop_line" and "grad_line")}

\item{inches}{size of the largest symbol (radius for circles, half width
for squares) in inches}

\item{val_max}{maximum value corresponding to the largest symbol}

\item{symbol}{type of symbols, 'circle' or 'square'}

\item{self_adjust}{if TRUE values are self-adjusted to keep min, max and
intermediate rounded values}

\item{lwd}{width(s) of the symbols borders (for "prop", "symb",
"choro_point", "choro_symb"),
width of the largest line (for "prop_line"), line width (for "choro_line"
and "typo_line"), vector of line widths (for "grad_line")}

\item{border}{symbol border color(s)}

\item{pch}{type(s) of the symbols (0:25)}

\item{cex}{size(s) of the symbols}

\item{title}{title of the legend}

\item{title_cex}{size of the legend title}

\item{val_cex}{size of the values in the legend}

\item{val_rnd}{number of decimal places of the values in
the legend}

\item{val_dec}{decimal separator}

\item{val_big}{thousands separator}

\item{col_na}{color for missing values}

\item{cex_na}{size of the symbols for missing values}

\item{pch_na}{type of the symbols for missing values}

\item{no_data}{if TRUE a "missing value" box is plotted}

\item{no_data_txt}{label for missing values}

\item{box_border}{border color of legend boxes}

\item{box_cex}{width and height size expansion of boxes, histogram
circles, squares or lines}

\item{horiz}{if TRUE plot an horizontal legend}

\item{frame_border}{border color of the frame}

\item{frame}{if TRUE the legend is plotted within a frame}

\item{bg}{background color of the legend}

\item{fg}{foreground color of the legend}

\item{size}{size of the legend; 2 means two times bigger}

\item{return_bbox}{return only bounding box of the legend.
No legend is plotted.}

\item{adj}{adjust the position of the legend in x and y directions}
}
\value{
No value is returned, a legend is displayed
(except if \code{return_bbox} is used).
}
\description{
Plot different types of legend. The "type" argument defines the
legend type.
Please note that some arguments are available for all types of legend and
some others are only relevant for specific legend types (see Details).
\code{mf_legend()} is a wrapper for \code{maplegend::leg()}.
}
\details{
Some arguments are available for all types of legend: \code{val}, \code{pos}, \code{title},
\code{title_cex}, \code{val_cex}, \code{frame}, \code{bg}, \code{fg}, \code{size}, \code{adj}, \code{alpha},
\code{return_bbox}).

Relevant arguments for each specific legend types:
\itemize{
\item \code{mf_legend(type = "prop", val, inches, val_max, symbol, col, lwd, border, val_rnd, val_big, val_dec, self_adjust, horiz)}
\item \code{mf_legend(type = "choro", val, pal, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt, box_border, box_cex, horiz)}
\item \code{mf_legend(type = "cont", val, pal, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt, box_border, box_cex, horiz)}
\item \code{mf_legend(type = "typo", val, pal, col_na, no_data, no_data_txt, box_border, box_cex)}
\item \code{mf_legend(type = "symb", val, pal, pch, cex, lwd, pch_na, cex_na, col_na, no_data, no_data_txt)}
\item \code{mf_legend(type = "prop_line", val, col, lwd, val_rnd, val_big, val_dec)}
\item \code{mf_legend(type = "grad_line", val, col, lwd, val_rnd, val_big, val_dec)}
\item \code{mf_legend(type = "histo", val, pal, box_border, val_rnd, val_big, val_dec)}
\item \code{mf_legend(type = "choro_point", val, pal, symbol, border, cex, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt, horiz)}
\item \code{mf_legend(type = "choro_line", val, pal, lwd, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt)}
\item \code{mf_legend(type = "choro_symb", val, pal, pch, lwd, val_rnd, val_big, val_dec, col_na, no_data, no_data_txt)}
\item \code{mf_legend(type = "typo_line", val, pal, lwd, col_na, no_data, no_data_txt, box_cex)}
}
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_legend(type = "prop", pos = "topright", val = c(1, 5, 10), inches = .3)
mf_legend(
  type = "choro", pos = "bottomright", val = c(10, 20, 30, 40, 50),
  pal = hcl.colors(4, "Reds 2")
)
mf_legend(
  type = "typo", pos = "topleft", val = c("A", "B", "C", "D"),
  pal = hcl.colors(4, "Dynamic")
)
mf_legend(
  type = "symb", pos = "bottomleft", val = c("A", "B", "C"),
  pch = 21:23, cex = c(1, 2, 2),
  pal = hcl.colors(3, "Dynamic")
)
mf_legend(
  type = "grad_line", pos = "top", val = c(1, 2, 3, 4, 10, 15),
  lwd = c(0.2, 2, 4, 5, 10)
)
mf_legend(type = "prop_line", pos = "bottom", lwd = 20, val = c(5, 50, 100))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_choro}
\alias{mf_map_choro}
\title{Plot a choropleth map}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name of the variable to map}

\item{type}{"choro"}

\item{breaks}{either a numeric vector with the actual breaks, or a
classification method name. The main methods are 'quantile',
'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
See \link{mf_get_breaks} for details.}

\item{nbreaks}{number of classes}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}. The default palette is the pal_seq palette
(see \link{mf_theme}).}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{border}{border color for polygons and symbols, a hex code or
color name given by \link{colors}. The default color for polygons is the
highlight color, the default color for points is the background color (see
\link{mf_theme}).}

\item{lwd}{border width for polygons and points symbols, lines width}

\item{pch}{type of symbol to use for points, see \link{pch} (points only)}

\item{cex}{symbols size, 2 means 2 times bigger (points only)}

\item{col_na}{color for missing values, a hex code or
a color name given by \link{colors}.}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}

\item{leg_*}{legend arguments described in \link{mf_map}}
}
\value{
x is (invisibly) returned.
}
\description{
With the \strong{choro} map type, \code{mf_map()} displays a choropleth map.

In choropleth maps, areas are shaded according to the variation of a
quantitative variable. They are used to represent ratios or indices.
\subsection{Usage}{

For polygons:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "choro",
       breaks = "quantile", nbreaks, pal, rev = FALSE,
       border, lwd = 0.7, col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}

For points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "choro",
       breaks = "quantile", nbreaks, pal, rev = FALSE,
       border, pch = 21, cex = 2, lwd = 0.7, col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}

For lines:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "choro",
       breaks = "quantile", nbreaks, pal, rev = FALSE, lwd = .7,
       col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}
}
}
\examples{
mtq <- mf_get_mtq()
pts <- mf_get_mtq("points")
flows <- mf_get_mtq("lines")
# polygons
mtq[6, "MED"] <- NA
mf_map(
  x = mtq, var = "MED", type = "choro",
  col_na = "grey90", pal = "Cividis",
  breaks = "equal", nbreaks = 5, border = "white",
  lwd = .5, leg_pos = "topleft",
  leg_title = "Median Income", leg_title_cex = 1,
  leg_val_cex = .9, leg_val_rnd = -2, leg_no_data = "No data",
  leg_box_cex = c(0.5, 3), leg_box_border = NA, leg_frame = FALSE
)
# points
mf_map(mtq)
mf_map(
  x = pts, var = "MED", type = "choro",
  pch = 21, cex = 3, lwd = 1.2,
  pal = "Teal", border = "white",
  leg_horiz = FALSE, leg_val_big = " ",
  leg_val_rnd = -2, leg_pos = "topright",
  leg_frame = TRUE, add = TRUE
)
# lines
mf_map(mtq, extent = flows)
mf_map(
  x = flows, var = "fij", type = "choro",
  breaks = "equal", nbreaks = 3, add = TRUE,
  lwd = 5, pal = "Burg", leg_horiz = TRUE,
  leg_box_cex = c(.7, 1),
  leg_val_rnd = 0, leg_pos = "bottomleft"
)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}, \code{\link[=mf_distr]{mf_distr()}}, \code{\link[=mf_get_breaks]{mf_get_breaks()}}, \code{\link[=mf_get_pal]{mf_get_pal()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_frame.R
\name{mf_frame}
\alias{mf_frame}
\title{Plot a frame}
\usage{
mf_frame(extent = "map", col, lwd = 1.5, lty = 1, ...)
}
\arguments{
\item{extent}{type of frame, either 'map' or 'figure'}

\item{col}{line color}

\item{lwd}{line width}

\item{lty}{line type}

\item{...}{other arguments from \code{\link[graphics:box]{graphics::box()}}}
}
\value{
No return value, a frame is displayed.
}
\description{
Plot a frame around an existing map.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_title()
mf_frame(extent = "map")
mf_map(mtq)
mf_title()
mf_frame(extent = "figure")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_png.R
\name{mf_png}
\alias{mf_png}
\title{Export a map in PNG format}
\usage{
mf_png(
  x,
  filename = "map.png",
  width,
  height,
  expandBB = rep(0, 4),
  res = 96,
  ...
)
}
\arguments{
\item{x}{object of class \code{sf}, \code{sfc} or \code{SpatRaster}}

\item{filename}{path to the exported file}

\item{width}{width of the figure (pixels)}

\item{height}{height of the figure (pixels)}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}

\item{res}{nominal resolution in ppi}

\item{...}{further parameters}
}
\value{
No return value, a PNG device is initiated.
}
\description{
Export a map with the extent of a spatial object in PNG format.

PNG is a raster graphics file format and PNG export should be used for
maps that do not require further modification.

If \code{width} is specified, then \code{height} is deduced from the width/height
ratio of \code{x}. Alternatively, if \code{height} is specified, then \code{width}
is deduced from the width/height ratio of \code{x}.
This helps to produce maps without too much wasted space.

Use \code{dev.off()} to finish the export (see Examples).
}
\examples{
mtq <- mf_get_mtq()
(filename <- tempfile(fileext = ".png"))
mf_png(mtq, filename = filename)
mf_map(mtq)
mf_title()
dev.off()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_shadow.R
\name{mf_shadow}
\alias{mf_shadow}
\title{Plot a shadow}
\usage{
mf_shadow(
  x,
  col,
  cex = 1,
  add = FALSE,
  extent = x,
  bg,
  expandBB = rep(0.04, 4)
)
}
\arguments{
\item{x}{an sf or sfc polygon object}

\item{col}{shadow color. The default color is the highlight color
(see \link{mf_theme}).}

\item{cex}{shadow extent}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}

\item{extent}{object with an \code{st_bbox} method to define plot extent;
defaults to \code{x}. \code{extent} and \code{x} must use the same CRS.}

\item{bg}{background color of the map, hex code or color name given by
\link{colors}, ignored if \code{add = TRUE}}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}
}
\value{
x is (invisibly) returned.
}
\description{
Plot the shadow of a polygon layer.
}
\examples{
mtq <- mf_get_mtq()
mf_shadow(mtq)
mf_map(mtq, add = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_inset.R
\name{mf_inset_on}
\alias{mf_inset_on}
\alias{mf_inset_off}
\title{Plot an inset}
\usage{
mf_inset_on(x, pos = "topright", cex = 0.2, fig)

mf_inset_off()
}
\arguments{
\item{x}{an sf object, or "worldmap" to use with \link{mf_worldmap}.}

\item{pos}{position, one of "bottomleft", "left", "topleft", "top", "bottom",
"bottomright", "right", "topright"}

\item{cex}{share of the map width occupied by the inset}

\item{fig}{coordinates of the inset region (in NDC, see in ?par())}
}
\value{
No return value, an inset is initiated or closed.
}
\description{
This function is used to add an inset map to the current map.
}
\details{
If x is used (with pos and cex), the width/height ratio of the inset will
match the width/height ratio of x bounding box.\cr
If fig is used, coordinates (xmin, xmax, ymin, ymax) are expressed as
fractions of the mapping space (i.e. excluding margins).\cr
If map layers have to be plotted after the inset (i.e after mf_inset_off()),
please use add = TRUE.\cr
It is not possible to plot an inset within an inset.\cr
It is possible to plot anything (base plots) within the inset, not only map
layers.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_inset_on(x = mtq[1, ], cex = .2)
mf_map(mtq[1, ])
mf_inset_off()

mf_map(mtq)
mf_inset_on(x = "worldmap", pos = "bottomleft")
mf_worldmap(x = mtq)
mf_inset_off()

mf_map(mtq)
mf_inset_on(fig = c(0, 0.25, 0, 0.25))
mf_map(x = mtq)
mf_inset_off()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_layout.R
\name{mf_layout}
\alias{mf_layout}
\title{Plot a map layout}
\usage{
mf_layout(
  title = "Map Title",
  credits = "Authors & Sources",
  scale = TRUE,
  arrow = TRUE,
  frame = FALSE
)
}
\arguments{
\item{title}{title of the map}

\item{credits}{credits}

\item{scale}{display a scale bar}

\item{arrow}{display an arrow}

\item{frame}{display a frame}
}
\value{
No return value, a map layout is displayed.
}
\description{
Plot a map layout (title, credits, scalebar,
north arrow, frame).

This function uses \link{mf_title}, \link{mf_credits},
\link{mf_scale} and \link{mf_arrow} with default values.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_layout()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_theme.R
\name{mf_theme}
\alias{mf_theme}
\title{Set a theme}
\usage{
mf_theme(
  x,
  mar,
  foreground,
  background,
  highlight,
  title_tab,
  title_pos,
  title_inner,
  title_line,
  title_cex,
  title_font,
  title_banner,
  frame,
  frame_lwd,
  frame_lty,
  pal_quali,
  pal_seq,
  ...
)
}
\arguments{
\item{x}{name of a map theme. One of 'base', 'grey', 'sol_light', 'sol_dark', 'mint', 'dracula', 'rzine', 'pistachio'.}

\item{mar}{a numeral vector of the form c(bottom, left, top, right)
which gives the margin size specified in number of lines}

\item{foreground}{foreground color}

\item{background}{background color}

\item{highlight}{highlight color}

\item{title_tab}{if TRUE the title is displayed as a 'tab'}

\item{title_pos}{title position, one of 'left', 'center', 'right'}

\item{title_inner}{if TRUE the title is displayed inside the plot area;
if FALSE the title is displayed in the top margin}

\item{title_line}{number of lines used for the title}

\item{title_cex}{cex of the title}

\item{title_font}{font of the title}

\item{title_banner}{if TRUE the title is displayed as a banner}

\item{frame}{either "none", "map" or "figure"; plot a frame around the map
or the figure.}

\item{frame_lwd}{line width for the frame}

\item{frame_lty}{line type for the frame}

\item{pal_quali}{default qualitative color palette (name or function)}

\item{pal_seq}{default sequential color palette (name or function)}

\item{...}{deprecated arguments ('bg', 'fg', 'tab', 'pos', 'inner',
'line', 'cex' and 'font'). See the Note section.}
}
\value{
\code{mf_theme} (invisibly) returns the list of current theme parameters.
}
\description{
A theme is a set of graphical parameters that are applied to maps created
with \code{mapsf}. These parameters are:
\itemize{
\item figure margins and frames,
\item background, foreground and highlight colors,
\item default sequential and qualitative palettes,
\item title options (position, size, banner...).
}

\code{mapsf} offers some builtin themes. It's possible to modify an existing theme
or to start a theme from scratch. It is also possible to set a custom theme
using a list of arguments

Themes are persistent across maps produced by \code{mapsf}
(e.g. they survive a \code{dev.off()} call).

Current theme parameters are set
in \code{mapsf} options and nammed according to the following convention:
"mapsf.mf_theme_arg_name".
Use \code{getOption()} to return the value of a specific argument of
the current theme (see examples).

Use \code{mf_theme(NULL)} or \code{mf_theme('base')} to reset to default theme
settings.
}
\note{
The following themes are deprecated: "default", "brutal",
"ink", "dark", "agolalight", "candy", "darkula", "iceberg", "green",
"nevermind", "jsk" and "barcelona".\cr
The following arguments are deprecated: "bg", "fg", "tab",
"pos", "inner", "line", "cex" and "font".\cr

Although the map theming system has been radically changed in version 1.0.0
of the package, you can still use the old themes by referencing them by name.
If you need to use the \emph{pre} v1.0.0 default theme, set \code{x} to "default".\cr
If an old theme is set, only deprecated arguments are used and others are
ignored.\cr
If current and deprecated arguments are mixed, only deprecated arguments are
used and others are ignored.\cr
All references and usages of the old theming system will be removed in the
next major version.
}
\examples{
mtq <- mf_get_mtq()

# Choosing a theme by name:
mf_theme("base")
mf_map(mtq)
mf_title()

# Specifying some values directly:
mf_theme(title_banner = TRUE)
mf_map(mtq)
mf_title()

# Using a mix of the above:
mf_theme("sol_dark", title_tab = TRUE, title_font = 1)
mf_map(mtq)
mf_title()

# Specifying a list with theme values:
theme <- list(
  mar = c(1, 1, 3, 1),
  title_tab = FALSE,
  title_pos = "left",
  title_inner = FALSE,
  title_line = 2,
  title_cex = 1.5,
  title_font = 2,
  title_banner = FALSE,
  frame = "figure",
  frame_lwd = 1,
  frame_lty = 1,
  foreground = "#fbfbfb",
  background = "grey75",
  highlight = "#0f5027",
  pal_quali = "Dark 3",
  pal_seq = "Greens"
)
mf_theme(theme)
mf_map(mtq, "MED", "choro")
mf_title()

# Obtaining a list of parameters for the current theme:
current_theme <- mf_theme()

# Obtaining individual parameters for the current theme:
getOption("mapsf.highlight")
getOption("mapsf.pal_seq")


# Use default theme:
mf_theme(NULL)
# or
mf_theme("base")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_label.R
\name{mf_label}
\alias{mf_label}
\title{Plot labels}
\usage{
mf_label(
  x,
  var,
  col,
  cex = 0.7,
  overlap = TRUE,
  lines = TRUE,
  halo = FALSE,
  bg,
  r = 0.1,
  q = 1,
  ...
)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name of the variable to map}

\item{col}{labels color, it can be a single color or a vector of colors}

\item{cex}{labels cex, it can be a single size or a vector of sizes}

\item{overlap}{if FALSE, labels are moved so they do not overlap.}

\item{lines}{if TRUE, then lines are plotted between x,y and the word,
for those words not covering their x,y coordinate}

\item{halo}{if TRUE, a 'halo' is displayed around the text and additional
arguments bg and r can be modified to set the color and width of the halo.}

\item{bg}{halo color, it can be a single color or a vector of colors}

\item{r}{width of the halo, it can be a single value or a vector of values}

\item{q}{quality of the non overlapping labels placement. Possible values
are 0 (quick results), 1 (reasonable quality and speed), 2 (better quality),
3 (insane quality, can take a lot of time).}

\item{...}{further \link{text} arguments.}
}
\value{
No return value, labels are displayed.
}
\description{
Put labels on a map.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mtq$cex <- c(rep(.8, 8), 2, rep(.8, 25))
mf_label(
  x = mtq, var = "LIBGEO",
  col = "grey10", halo = TRUE, cex = mtq$cex,
  overlap = FALSE, lines = FALSE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_borders.R
\name{mf_get_borders}
\alias{mf_get_borders}
\title{Get a border layer from polygons}
\usage{
mf_get_borders(x)
}
\arguments{
\item{x}{an sf object of POLYGONS, using a projected CRS}
}
\value{
An sf object (MULTILINESTRING) of borders is returned.
}
\description{
This function extracts borders between contiguous polygons.
}
\note{
If the polygon layer contains topology errors (such as contiguous
polygons not sharing exactly the same boundary) the function may not return
all boundaries correctly. It is possible to use \code{st_snap()} or other
functions to try and correct these errors.
}
\examples{
mtq <- mf_get_mtq()
mtq_b <- mf_get_borders(mtq)
mf_map(mtq)
mf_map(mtq_b, col = 1:5, lwd = 4, add = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map.R
\name{mf_map}
\alias{mf_map}
\title{Plot a map}
\usage{
mf_map(x, var, type = "base",
       breaks, nbreaks, pal, alpha, rev, inches, val_max, symbol, col,
       lwd_max, val_order, pch, cex, border, lwd, col_na, cex_na, pch_na,
       expandBB, extent, bg, add,
       leg_pos, leg_title, leg_title_cex, leg_val_cex, leg_val_rnd,
       leg_val_dec, leg_val_big, leg_no_data, leg_frame, leg_frame_border,
       leg_horiz, leg_adj, leg_bg, leg_fg, leg_size,
       leg_box_border, leg_box_cex, ...)
}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name(s) of the variable(s) to map}

\item{type}{\itemize{
\item \strong{\link[=mf_map_base]{base}}: base maps
\item \strong{\link[=mf_map_choro]{choro}}: choropleth maps
\item \strong{\link[=mf_map_typo]{typo}}: typology maps
\item \strong{\link[=mf_map_prop]{prop}}: proportional symbols maps
\item \strong{\link[=mf_map_prop_choro]{prop_choro}}: proportional symbols with choropleth
coloration
\item \strong{\link[=mf_map_prop_typo]{prop_typo}}: proportional symbols with typology
coloration
\item \strong{\link[=mf_map_symb]{symb}}: symbols maps
\item \strong{\link[=mf_map_grad]{grad}}: graduated symbols maps
\item \strong{\link[=mf_map_symb_choro]{symb_choro}}: symbols with choropleth coloration
}}

\item{breaks}{either a numeric vector with the actual breaks, or a
classification method name. The main methods are 'quantile',
'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
See \link{mf_get_breaks} for details.}

\item{nbreaks}{number of classes}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}.}

\item{alpha}{\code{col} or \code{pal} opacity, in the range [0,1] (0 means
transparent and 1 means opaque). Default is set to 1.}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{inches}{size of the largest symbol in inches (radius for circles,
half width for squares)}

\item{val_max}{maximum value corresponding to the largest symbol or line}

\item{symbol}{type of proportional symbols, either "circle" or "square"}

\item{col}{a color, hex code or color name given by \link{colors}}

\item{lwd_max}{width of the largest line}

\item{val_order}{modalities order in the legend, a character vector that
matches \code{var} modalities}

\item{pch}{type of symbol to use for points, see \link{pch}}

\item{cex}{symbols size, 2 means 2 times bigger}

\item{border}{border color for polygons or symbols. It can be a hex
code or a color name given by \link{colors}.}

\item{lwd}{border width of polygons, symbols or lines}

\item{col_na}{color for missing values, a hex code or
a color name given by \link{colors}.}

\item{cex_na}{symbols size for missing values on points}

\item{pch_na}{symbol to use for missing values on points, see \link{pch}}

\item{expandBB}{fractional values to expand the bounding box with, in each
direction (bottom, left, top, right)}

\item{extent}{object with an \code{st_bbox} method to define plot extent;
defaults to \code{x}. \code{extent} and \code{x} must use the same CRS.}

\item{bg}{background color of the map, hex code or color name given by
\link{colors}, ignored if \code{add = TRUE}}

\item{add}{whether to add the layer to an existing plot (TRUE) or not (FALSE)}

\item{leg_pos}{position of the legend, one of 'topleft', 'top','topright',
'right', 'bottomright', 'bottom', 'bottomleft', 'left' or a vector of two
coordinates in map units (c(x, y)). Use \code{NA} to avoid plotting the legend,
use 'interactive' to choose the legend position by clicking on the map.}

\item{leg_title}{legend title}

\item{leg_title_cex}{size of the title}

\item{leg_val_cex}{size of the values}

\item{leg_val_rnd}{number of decimal places of the values displayed in the
legend}

\item{leg_val_dec}{decimal separator}

\item{leg_val_big}{thousands separator}

\item{leg_no_data}{label for missing values}

\item{leg_frame}{whether to add a frame to the legend (TRUE) or not (FALSE)}

\item{leg_frame_border}{border color of the legend frame}

\item{leg_horiz}{display the legend horizontally (for proportional symbols
and choropleth types)}

\item{leg_adj}{adjust the position of the legend in x and y directions}

\item{leg_bg}{color of the legend background}

\item{leg_fg}{color of the legend foreground}

\item{leg_size}{size of the legend. Combine this argument with
\code{leg_title_cex} and \code{leg_val_cex}.}

\item{leg_box_border}{border color of legend boxes (for types related to
choropleth and typology)}

\item{leg_box_cex}{width and height size expansion of boxes}

\item{...}{ignored}
}
\value{
x is (invisibly) returned.
}
\description{
\code{mf_map()} is the main function of the package, it displays map layers on a
georeferenced plot.

\code{mf_map()} has three main arguments:
\itemize{
\item \code{x}, an sf object,
\item \code{var}, the name(s) of a variable(s) to map,
\item \code{type}, the map type.
}

Relevant arguments and default values are different for each map type and are
described in dedicated help pages (see \link[=mf_map_base]{base},
\link[=mf_map_choro]{choro}, \link[=mf_map_typo]{typo}, \link[=mf_map_prop]{prop},
\link[=mf_map_prop_choro]{prop_choro}, \link[=mf_map_prop_typo]{prop_typo},
\link[=mf_map_symb]{symb}, \link[=mf_map_grad]{grad} or \link[=mf_map_symb_choro]{symb_choro}).
}
\examples{
mtq <- mf_get_mtq()
# basic examples
# type = "base"
mf_map(mtq)
# type = "prop"
mf_map(mtq)
mf_map(mtq, var = "POP", type = "prop")
# type = "choro"
mf_map(mtq, var = "MED", type = "choro")
# type = "typo"
mf_map(mtq, "STATUS", "typo")
# type = "symb"
mf_map(mtq)
mf_map(mtq, "STATUS", "symb")
# type = "grad"
mf_map(mtq)
mf_map(mtq, var = "POP", type = "grad")
# type = "prop_choro"
mf_map(mtq)
mf_map(mtq, var = c("POP", "MED"), type = "prop_choro")
# type = "prop_typo"
mf_map(mtq)
mf_map(mtq, var = c("POP", "STATUS"), type = "prop_typo")
# type = "symb_choro
mf_map(mtq)
mf_map(mtq, var = c("STATUS", "MED"), type = "symb_choro")
}
\concept{map types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_get_pal.R
\name{mf_get_pal}
\alias{mf_get_pal}
\title{Get color palettes}
\usage{
mf_get_pal(
  n,
  palette,
  alpha = NULL,
  rev = c(FALSE, FALSE),
  neutral,
  breaks,
  mid
)
}
\arguments{
\item{n}{the number of colors (>= 1) to be in the palette}

\item{palette}{a valid palette name. See \link{hcl.pals} to get available
palette names. The name is matched
to the list of available palettes, ignoring upper vs. lower case, spaces,
dashes, etc. in the matching.}

\item{alpha}{opacity, in the range [0,1] (0 means
transparent and 1 means opaque). Default is set to 1.}

\item{rev}{logical indicating whether the ordering of the colors should be
reversed}

\item{neutral}{a color, if two gradients are used, the 'neutral' color can be
added between them}

\item{breaks}{a vector of class limit}

\item{mid}{a numeric value use to divide the palette in two colors}
}
\value{
A vector of colors.
}
\description{
\code{mf_get_pal} builds sequential, diverging and
qualitative color palettes.
Diverging color palettes can be dissymmetric (different number of colors in
each of the two gradients).
}
\examples{
cls <- mf_get_pal(n = c(3, 7), palette = c("Reds 2", "Greens"))
plot(1:10, rep(1, 10), bg = cls, pch = 22, cex = 4)
mtq <- mf_get_mtq()
bks <- mf_get_breaks(mtq$MED, breaks = "equal", nbreaks = 8)
pal <- mf_get_pal(
  breaks = bks, mid = 15000,
  palette = c("Dark Mint", "Burg"), neutral = "grey90"
)
mf_map(mtq, "MED", "choro", breaks = bks, pal = pal)
pal <- mf_get_pal(breaks = bks, mid = bks[4], palette = c("Dark Mint", "Burg"))
mf_map(mtq, "MED", "choro", breaks = bks, pal = pal)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_symb_choro}
\alias{mf_map_symb_choro}
\title{Plot symbols with choropleth coloration}
\arguments{
\item{x}{object of class \code{sf} (polygons or points)}

\item{var}{names of the variables to map. The first value refers to the
symbols categories, the second one to the choropleth coloration.}

\item{type}{"symb_choro"}

\item{pch}{a vector of types of symbols, see \link{pch}.
The length of \code{pch} should match the number of modalities.}

\item{cex}{a vector of sizes for symbols.
The length of \code{cex} should match the number of modalities.}

\item{lwd}{border width of symbols}

\item{border}{border color for symbols, a hex code or color name given
by \link{colors}. The default color is the background color (see
\link{mf_theme}).}

\item{val_order}{modalities order in the legend, a character vector that
matches \code{var} modalities. Default to alphabetic order of modalities.}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}. The default palette is the pal_seq palette
(see \link{mf_theme}).}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{breaks}{either a numeric vector with the actual breaks, or a
classification method name. The main methods are 'quantile',
'equal', 'msd', 'ckmeans' (natural breaks), 'Q6' and 'geom'.
See \link{mf_get_breaks} for details.}

\item{nbreaks}{number of classes}

\item{pch_na}{type of symbol for missing values, see \link{pch}}

\item{cex_na}{size of symbol for missing values}

\item{col_na}{color for missing values, a hex code or
a color name given by \link{colors}}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}

\item{leg_*}{legend arguments described in \link{mf_map}. See details for
arguments with two values.}
}
\value{
x is (invisibly) returned.
}
\description{
\code{mf_map()} with \strong{symb_choro} type creates symbols that reflect modalities
of a first qualitative variable and colored to reflect the classification of
a second variable.

This map types uses two variables and some arguments need to be set for both
variables (see Details).

For polygons, centroids are used to plot symbols. This map type
is not available for lines.
\subsection{Usage}{

For polygons and points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "symb_choro",
       pch, cex = 2, lwd = 0.7, border, val_order,
       pal, rev = FALSE, breaks = "quantile", nbreaks,
       pch_na = 4, cex_na = 1, col_na = "white",
       alpha, expandBB, extent, bg, add = TRUE, leg_*)
}\if{html}{\out{</div>}}
}
}
\details{
Legend arguments that need two values are: 'leg_title',
'leg_no_data'. The first value refers to the symbols legend, the second
one to the choropleth legend.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mtq$STATUS[4] <- NA
mf_map(mtq, c("STATUS", "MED"),
  type = "symb_choro", lwd = 1,
  pal = "Reds 3", breaks = "quantile", nbreaks = 4,
  cex = c(2, 1, 1), pch = c(20, 21, 23), pch_na = 22,
  leg_pos = "topright", border = "white",
  val_order = c("Prefecture", "Sub-prefecture", "Simple municipality")
)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}, \link{mf_map_symb}, \link{mf_map_choro}, \code{\link[=mf_distr]{mf_distr()}},
\code{\link[=mf_get_breaks]{mf_get_breaks()}}, \code{\link[=mf_get_pal]{mf_get_pal()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_scale.R
\name{mf_scale}
\alias{mf_scale}
\title{Plot a scale bar}
\usage{
mf_scale(
  size,
  pos = "bottomright",
  lwd = 1.5,
  cex = 0.6,
  col,
  crs_units = "m",
  scale_units = "km",
  adj = c(0, 0),
  x
)
}
\arguments{
\item{size}{size of the scale bar in scale units (\code{scale_units},
default to km). If size is not set, an automatic size is used.}

\item{pos}{position. It can be one of 'bottomright', 'bottomleft',
'interactive' or a vector of two coordinates in map units (c(x, y)).}

\item{lwd}{line width of the scale bar}

\item{cex}{size of the scale bar text}

\item{col}{color of the scale bar (line and text)}

\item{crs_units}{units used in the CRS of the currently plotted layer.
Possible values are "m" and "ft" (see Details).}

\item{scale_units}{units used for the scale bar. Can be "mi" for miles,
"ft" for feet, "m" for meters, or "km" for kilometers (default).}

\item{adj}{adjust the position of the scale bar in x and y directions}

\item{x}{object of class crs, sf or sfc. If set, the CRS of x will be used
instead of \code{crs_units} to define CRS units.}
}
\value{
No return value, a scale bar is displayed.
}
\description{
Plot a scale bar.
}
\details{
Most CRS use the meter as unit. Some US CRS use feet or US survey
feet. If unsure of the unit used in the CRS you can use the x argument of the
function.
Alternatively, you can use
\code{sf::st_crs(zz, parameters = TRUE)$units_gdal} to see which units
are used in the \code{zz} layer.

The scale bar cannot be displayed on unprojected (long/lat) maps or
on maps without documented CRS.
}
\examples{
mtq <- mf_get_mtq()
mf_map(mtq)
mf_scale()

library(sf)
nc <- st_read(system.file("shape/nc.shp", package = "sf"))[1, ]

nc_foot <- st_transform(nc, 2264) # NC state plane, US foot
mf_map(nc_foot)
mf_scale(size = 5, crs_units = "ft", scale_units = "mi")
mf_map(nc_foot)
mf_scale(size = 5, x = nc_foot, scale_units = "mi")

nc_meter <- st_transform(nc, 32119) # NC state plane, m
mf_map(nc_meter)
mf_scale(size = 5, crs_units = "m", scale_units = "mi")
mf_scale(size = 5, crs_units = "m", scale_units = "km", pos = "bottomleft")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_prop_typo}
\alias{mf_map_prop_typo}
\title{Plot proportional symbols with typology coloration}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{names of the variables to map. The first value refers to the
proportional symbols, the second one to the typology coloration.}

\item{type}{"prop_typo"}

\item{inches}{size of the largest symbol in inches (radius for circles,
half width for squares)}

\item{lwd_max}{width of the largest line}

\item{val_max}{maximum value corresponding to the largest symbol or line}

\item{symbol}{type of proportional symbols, either "circle" or "square"}

\item{border}{border color for proportional symbols, a hex code or
color name given by \link{colors}. The default color is the background color (see
\link{mf_theme}).}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}. The default palette is the pal_quali palette
(see \link{mf_theme}).}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{val_order}{modalities order in the legend, a character vector that
matches \code{var} modalities. Default to alphabetic order of modalities.}

\item{col_na}{color for missing values, a hex code or
a color name given by \link{colors}.}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}

\item{leg_*}{legend arguments described in \link{mf_map}. See details for
arguments with two values.}
}
\value{
x is (invisibly) returned.
}
\description{
\code{mf_map()} with \strong{prop_typo} type creates symbols that are proportional
to values of a first variable and colored to reflect the modalities of a
second qualitative variable.

This map types uses two variables and some arguments need to be set for both
variables (see Details).

For polygons, centroids are used to plot proportional symbols.
\subsection{Usage}{

For polygons and points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "prop_typo",
       inches = 0.3, val_max, symbol, border,
       pal, rev = FALSE, val_order,
       border, lwd = 0.7, col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}

For lines:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "typo",
       lwd_max = 15,
       pal, rev = FALSE, val_order,
       col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}
}
}
\details{
'leg_title' needs two values. The first value refers to the symbols legend,
the second one to the choropleth legend.
}
\examples{
mtq <- mf_get_mtq()
flows <- mf_get_mtq("lines")
mf_map(mtq, extent = flows, expandBB = c(0, .5, 0, 0))
mf_map(flows, c("fij", "sj"), "prop_typo",
  val_order = c("Sub-prefecture", "Simple municipality"),
  pal = c("steelblue", "lightblue"), lwd_max = 30,
  leg_pos = "topleft", leg_title = c("commuters", "destination")
)
mf_map(
  x = mtq, var = c("POP", "STATUS"), type = "prop_typo",
  inches = .2, border = "tomato4", lwd = 1,
  pal = c("darkblue", "steelblue", "lightblue"),
  val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
  leg_pos = "bottomleft",
  leg_title = c("Population", ""),
  leg_no_data = "No dada",
  add = TRUE
)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}, \link{mf_map_prop}, \link{mf_map_typo}, \code{\link[=mf_get_pal]{mf_get_pal()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_typo}
\alias{mf_map_typo}
\title{Plot a typology map}
\arguments{
\item{x}{object of class \code{sf}}

\item{var}{name of the variable to map}

\item{type}{"choro"}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}. The default palette is the pal_quali palette
(see \link{mf_theme}).}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{val_order}{modalities order in the legend, a character vector that
matches \code{var} modalities. Default to alphabetic order of modalities.}

\item{border}{border color for polygons and symbols, a hex code or
color name given by \link{colors}. The default color for polygons is the
highlight color, the default color for points is the background color (see
\link{mf_theme}).}

\item{lwd}{border width for polygons and points symbols, lines width}

\item{pch}{type of symbol to use for points, see \link{pch} (points only)}

\item{cex}{symbols size, 2 means 2 times bigger (points only)}

\item{col_na}{color for missing values, a hex code or
a color name given by \link{colors}.}

\item{alpha, expandBB, extent, bg, add}{arguments described in \link{mf_map}}

\item{leg_*}{legend arguments described in \link{mf_map}}
}
\value{
x is (invisibly) returned.
}
\description{
With the \strong{typo} map type, \code{mf_map()} displays a typology map.

In typology maps, areas are shaded according to the modalities of a
qualitative variable.
\subsection{Usage}{

For polygons:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "typo",
       pal, rev = FALSE, val_order,
       border, lwd = 0.7, col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}

For points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "typo",
       pal, rev = FALSE, val_order,
       border, pch = 21, cex = 2, lwd = 0.7, col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}

For lines:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "typo",
       pal, rev = FALSE, val_order, lwd = .7,
       col_na = "white",
       alpha, expandBB, extent, bg, add = FALSE, leg_*)
}\if{html}{\out{</div>}}
}
}
\examples{
mtq <- mf_get_mtq()
pts <- mf_get_mtq("points")
flows <- mf_get_mtq("lines")
# polygons
mtq[6, "STATUS"] <- NA
mf_map(
  x = mtq, var = "STATUS", type = "typo",
  col_na = "grey90", border = "white",
  pal = c("#FFE93F", "#00214E", "#7C7C7C"),
  val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
  lwd = .5, leg_pos = "bottomleft",
  leg_title = "", leg_title_cex = 1,
  leg_val_cex = .9, leg_no_data = "No data",
  leg_box_cex = c(0.5, 3), leg_box_border = NA
)
# points
mf_map(
  x = pts, var = "STATUS", type = "typo",
  cex = 3, pal = "Dark 3", border = "grey",
  leg_pos = "bottomleft"
)
# lines
mf_map(mtq, extent = flows)
mf_map(
  x = flows, var = "sj", type = "typo",
  add = TRUE,
  lwd = 2, pal = c("red", "blue"),
  leg_pos = "bottomleft"
)
}
\seealso{
\code{\link[=mf_map]{mf_map()}}, \code{\link[=mf_get_pal]{mf_get_pal()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{mapsf-deprecated}
\alias{mapsf-deprecated}
\title{Deprecated functions in mapsf}
\description{
These functions and features still work but will be removed in the next
major version of the package.
\subsection{\code{mf_map} sub-functions}{

Instead of using the following deprecated functions, one can use \link{mf_map}
with the corresponding type:
\itemize{
\item \code{mf_base()} => \link{mf_map_base}
\item \code{mf_choro()} => \link{mf_map_choro}
\item \code{mf_prop()} => \link{mf_map_prop}
\item \code{mf_typo()} => \link{mf_map_typo}
\item \code{mf_symb()} => \link{mf_map_symb}
\item \code{mf_grad()} => \link{mf_map_grad}
\item \code{mf_prop_typo()} => \link{mf_map_prop_typo}
\item \code{mf_prop_choro()} => \link{mf_map_prop_choro}
\item \code{mf_symb_choro()} => \link{mf_map_symb_choro}
}
}

\subsection{\code{mf_init}}{

\code{mf_init} is deprecated. It is possible to use \code{mf_map} instead
(\code{mf_map(x, type = "base", col = NA, border = NA)}.

It is also possible to use the \code{extent} argument of \link{mf_map}.
}

\subsection{\code{mf_export}}{

\code{mf_export} is deprecated, use \link{mf_png} or \link{mf_svg} instead.
}

\subsection{\code{mf_annotation}}{

\code{mf_annotation} is deprecated, use \link{mf_text} instead.
}

\subsection{Double legends}{

The use of separated legends for map types \strong{prop_choro}, \strong{prop_typo} and
\strong{symb_choro} is deprecated. Use \code{leg_pos = NA} and \link{mf_legend} if
separated legends are needed.
}

\subsection{Theming system}{

In \link{mf_theme}, the following themes are deprecated: "default", "brutal",
"ink", "dark", "agolalight", "candy", "darkula", "iceberg", "green",
"nevermind", "jsk" and "barcelona".\cr
The following arguments are also deprecated: "bg", "fg", "tab",
"pos", "inner", "line", "cex" and "font".\cr

Although the map theming system has been radically changed in version 1.0.0
of the package, you can still use the old themes by referencing them by name.
If you need to use the \emph{pre} v1.0.0 default theme, set \code{x} to "default".\cr
If an old theme is set, only deprecated arguments are used and others are
ignored.\cr
If current and deprecated arguments are mixed, only deprecated arguments are
used and others are ignored.\cr
All references and usages of the old theming system will be removed in the
next major version.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mf_map_help.R
\name{mf_map_symb}
\alias{mf_map_symb}
\title{Plot symbols}
\arguments{
\item{x}{object of class \code{sf} (polygons or points)}

\item{var}{name of the variable to map}

\item{type}{"symb"}

\item{pch}{a vector of types of symbols, see \link{pch}.
The length of \code{pch} should match the number of modalities.}

\item{cex}{a vector of sizes for symbols.
The length of \code{cex} should match the number of modalities.}

\item{lwd}{border width of symbols}

\item{pal}{a set of colors (hex codes) or a palette name. Palette names can
be obtained with \link{hcl.pals}. The default palette is the pal_quali palette
(see \link{mf_theme}).}

\item{rev}{if \code{pal} is a palette name, whether the ordering of the colors
should be reversed (TRUE) or not (FALSE)}

\item{border}{border color for symbols, a hex code or color name given
by \link{colors}. The default color is the background color (see
\link{mf_theme}).}

\item{val_order}{modalities order in the legend, a character vector that
matches \code{var} modalities. Default to alphabetic order of modalities.}

\item{pch_na}{type of symbol for missing values, see \link{pch}}

\item{cex_na}{size of symbol for missing values}

\item{col_na}{color for missing values, a hex code or
a color name given by \link{colors}}
}
\value{
x is (invisibly) returned.
}
\description{
\code{mf_map()} can use symbols to display qualitative data, using \strong{symb}
map type.

For polygons, centroids are used to plot graduated symbols. This map type
is not available for lines.
\subsection{Usage}{

For polygons and points:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{mf_map(x, var, type = "symb",
       pch, cex = 2, lwd = 0.7, pal, rev = FALSE, border,
       val_order,
       col_na = "grey", pch_na = 4, cex_na = 1,
       alpha, expandBB, extent, bg, add = TRUE, leg_*)
}\if{html}{\out{</div>}}
}
}
\examples{
mtq <- mf_get_mtq()
mtq$STATUS[3] <- NA
mf_map(mtq)
mf_map(mtq, "STATUS", "symb",
  pal = "Berlin", border = "white", lwd = 1,
  cex = c(4, 3, 2), pch = c(21:23), col_na = "red",
  val_order = c("Prefecture", "Sub-prefecture", "Simple municipality"),
  leg_title = ""
)
}
